// Copyright lowRISC contributors.
// Licensed under the Apache License, Version 2.0, see LICENSE for details.
// SPDX-License-Identifier: Apache-2.0
#include "sw/device/lib/crypto/drivers/kmac_test_vectors.h"

// These test vectors are taken from SHA-3 of CAVP
// https://csrc.nist.gov/Projects/Cryptographic-Algorithm-Validation-Program/Secure-Hashing

// SHAKE128ShortMsg_msg.rsp Len = 56
static uint8_t shake128_msg1[] = {
    0x72, 0x16, 0xa8, 0x25, 0x02, 0x9d, 0xa1,
};
static uint8_t shake128_digest1[] = {
    0x9d, 0xe6, 0xff, 0xac, 0xf3, 0xe5, 0x96, 0x93,
    0xa3, 0xde, 0x81, 0xb0, 0x2f, 0x7d, 0xb7, 0x7a,
};
static kmac_test_vector_t shake128_t1 = {
    .operation = kSHAKE128,
    .input_str = (uint8_t *)shake128_msg1,
    .input_str_len = sizeof(shake128_msg1),
    .digest = (uint8_t *)shake128_digest1,
    .digest_len = sizeof(shake128_digest1),
};

// SHAKE128ShortMsg.rsp Len = 128
static uint8_t shake128_msg2[] = {
    0xd4, 0xd6, 0x7b, 0x00, 0xca, 0x51, 0x39, 0x77,
    0x91, 0xb8, 0x12, 0x05, 0xd5, 0x58, 0x2c, 0x0a,
};
static uint8_t shake128_digest2[] = {
    0xd0, 0xac, 0xfb, 0x2a, 0x14, 0x92, 0x8c, 0xaf,
    0x8c, 0x16, 0x8a, 0xe5, 0x14, 0x92, 0x5e, 0x4e,
};
static kmac_test_vector_t shake128_t2 = {
    .operation = kSHAKE128,
    .input_str = (uint8_t *)shake128_msg2,
    .input_str_len = sizeof(shake128_msg2),
    .digest = (uint8_t *)shake128_digest2,
    .digest_len = sizeof(shake128_digest2),
};

// SHAKE128ShortMsg.rsp Len = 1336
static uint8_t shake128_msg3[] = {
    0x18, 0x63, 0x6f, 0x70, 0x2f, 0x21, 0x6b, 0x1b, 0x93, 0x02, 0xe5, 0x9d,
    0x82, 0x19, 0x2f, 0x4e, 0x00, 0x2f, 0x82, 0xd5, 0x26, 0xc3, 0xf0, 0x4c,
    0xbd, 0x4f, 0x9b, 0x9f, 0x0b, 0xcd, 0x25, 0x35, 0xed, 0x7a, 0x67, 0xd3,
    0x26, 0xda, 0x66, 0xbd, 0xf7, 0xfc, 0x82, 0x1e, 0xf0, 0xff, 0xf1, 0xa9,
    0x05, 0xd5, 0x6c, 0x81, 0xe4, 0x47, 0x28, 0x56, 0x86, 0x39, 0x08, 0xd1,
    0x04, 0x30, 0x11, 0x33, 0xad, 0x11, 0x1e, 0x39, 0x55, 0x2c, 0xd5, 0x42,
    0xef, 0x78, 0xd9, 0xb3, 0x5f, 0x20, 0x41, 0x9b, 0x89, 0x3f, 0x4a, 0x93,
    0xae, 0xe8, 0x48, 0xe9, 0xf8, 0x6a, 0xe3, 0xfd, 0x53, 0xd2, 0x7f, 0xea,
    0x7f, 0xb1, 0xfc, 0x69, 0x63, 0x1f, 0xa0, 0xf3, 0xa5, 0xff, 0x51, 0x26,
    0x77, 0x85, 0x08, 0x6a, 0xb4, 0xf6, 0x82, 0xd4, 0x2b, 0xaf, 0x39, 0x4b,
    0x3b, 0x69, 0x92, 0xe9, 0xa0, 0xbb, 0x58, 0xa3, 0x8c, 0xe0, 0x69, 0x2d,
    0xf9, 0xbb, 0xaf, 0x18, 0x3e, 0x18, 0x52, 0x3e, 0xe1, 0x35, 0x2c, 0x5f,
    0xad, 0x81, 0x7e, 0x0c, 0x04, 0xa3, 0xe1, 0xc4, 0x76, 0xbe, 0x7f, 0x5e,
    0x92, 0xf4, 0x82, 0xa6, 0xfb, 0x29, 0xcd, 0x4b, 0xbf, 0x09, 0xea,
};
static uint8_t shake128_digest3[] = {
    0xb7, 0xb9, 0xdb, 0x48, 0x18, 0x98, 0xf8, 0x88,
    0xe5, 0xee, 0x4e, 0xd6, 0x29, 0x85, 0x98, 0x44,
};
static kmac_test_vector_t shake128_t3 = {
    .operation = kSHAKE128,
    .input_str = (uint8_t *)shake128_msg3,
    .input_str_len = sizeof(shake128_msg3),
    .digest = (uint8_t *)shake128_digest3,
    .digest_len = sizeof(shake128_digest3),
};

// SHAKE256ShortMsg.rsp Len = 312
static uint8_t shake256_msg1[] = {
    0xf5, 0x32, 0xd0, 0xec, 0xe5, 0x83, 0xbe, 0xba, 0xec, 0x55,
    0xd0, 0x30, 0xc9, 0x3f, 0xcd, 0x43, 0xad, 0x7c, 0x96, 0xa4,
    0x93, 0xef, 0x15, 0xe7, 0xd0, 0x64, 0x70, 0xe5, 0xeb, 0x19,
    0x82, 0x5f, 0xae, 0x7d, 0x8b, 0xe4, 0x7d, 0x37, 0x26,
};
static uint8_t shake256_digest1[] = {
    0x22, 0x5a, 0xce, 0x4d, 0x51, 0x40, 0xa3, 0xd7, 0x59, 0x93, 0xca,
    0x8c, 0x7b, 0x99, 0x36, 0x62, 0x03, 0x0d, 0x04, 0x6f, 0xa0, 0x20,
    0x1e, 0xe9, 0xc5, 0xc3, 0x35, 0xd9, 0x2b, 0x4d, 0x58, 0x01,
};
static kmac_test_vector_t shake256_t1 = {
    .operation = kSHAKE256,
    .input_str = (uint8_t *)shake256_msg1,
    .input_str_len = sizeof(shake256_msg1),
    .digest = (uint8_t *)shake256_digest1,
    .digest_len = sizeof(shake256_digest1),
};

// SHAKE256VariableOut.rsp Count = 1245 OutputLen = 2000
static uint8_t shake256_msg2[] = {
    0x8d, 0x80, 0x01, 0xe2, 0xc0, 0x96, 0xf1, 0xb8, 0x8e, 0x7c, 0x92,
    0x24, 0xa0, 0x86, 0xef, 0xd4, 0x79, 0x7f, 0xbf, 0x74, 0xa8, 0x03,
    0x3a, 0x2d, 0x42, 0x2a, 0x2b, 0x6b, 0x8f, 0x67, 0x47, 0xe4,
};
static uint8_t shake256_digest2[] = {
    0x2e, 0x97, 0x5f, 0x6a, 0x8a, 0x14, 0xf0, 0x70, 0x4d, 0x51, 0xb1, 0x36,
    0x67, 0xd8, 0x19, 0x5c, 0x21, 0x9f, 0x71, 0xe6, 0x34, 0x56, 0x96, 0xc4,
    0x9f, 0xa4, 0xb9, 0xd0, 0x8e, 0x92, 0x25, 0xd3, 0xd3, 0x93, 0x93, 0x42,
    0x51, 0x52, 0xc9, 0x7e, 0x71, 0xdd, 0x24, 0x60, 0x1c, 0x11, 0xab, 0xcf,
    0xa0, 0xf1, 0x2f, 0x53, 0xc6, 0x80, 0xbd, 0x3a, 0xe7, 0x57, 0xb8, 0x13,
    0x4a, 0x9c, 0x10, 0xd4, 0x29, 0x61, 0x58, 0x69, 0x21, 0x7f, 0xdd, 0x58,
    0x85, 0xc4, 0xdb, 0x17, 0x49, 0x85, 0x70, 0x3a, 0x6d, 0x6d, 0xe9, 0x4a,
    0x66, 0x7e, 0xac, 0x30, 0x23, 0x44, 0x3a, 0x83, 0x37, 0xae, 0x1b, 0xc6,
    0x01, 0xb7, 0x6d, 0x7d, 0x38, 0xec, 0x3c, 0x34, 0x46, 0x31, 0x05, 0xf0,
    0xd3, 0x94, 0x9d, 0x78, 0xe5, 0x62, 0xa0, 0x39, 0xe4, 0x46, 0x95, 0x48,
    0xb6, 0x09, 0x39, 0x5d, 0xe5, 0xa4, 0xfd, 0x43, 0xc4, 0x6c, 0xa9, 0xfd,
    0x6e, 0xe2, 0x9a, 0xda, 0x5e, 0xfc, 0x07, 0xd8, 0x4d, 0x55, 0x32, 0x49,
    0x45, 0x0d, 0xab, 0x4a, 0x49, 0xc4, 0x83, 0xde, 0xd2, 0x50, 0xc9, 0x33,
    0x8f, 0x85, 0xcd, 0x93, 0x7a, 0xe6, 0x6b, 0xb4, 0x36, 0xf3, 0xb4, 0x02,
    0x6e, 0x85, 0x9f, 0xda, 0x1c, 0xa5, 0x71, 0x43, 0x2f, 0x3b, 0xfc, 0x09,
    0xe7, 0xc0, 0x3c, 0xa4, 0xd1, 0x83, 0xb7, 0x41, 0x11, 0x1c, 0xa0, 0x48,
    0x3d, 0x0e, 0xda, 0xbc, 0x03, 0xfe, 0xb2, 0x3b, 0x17, 0xee, 0x48, 0xe8,
    0x44, 0xba, 0x24, 0x08, 0xd9, 0xdc, 0xfd, 0x01, 0x39, 0xd2, 0xe8, 0xc7,
    0x31, 0x01, 0x25, 0xae, 0xe8, 0x01, 0xc6, 0x1a, 0xb7, 0x90, 0x0d, 0x1e,
    0xfc, 0x47, 0xc0, 0x78, 0x28, 0x17, 0x66, 0xf3, 0x61, 0xc5, 0xe6, 0x11,
    0x13, 0x46, 0x23, 0x5e, 0x1d, 0xc3, 0x83, 0x25, 0x66, 0x6c,
};
static kmac_test_vector_t shake256_t2 = {
    .operation = kSHAKE256,
    .input_str = (uint8_t *)shake256_msg2,
    .input_str_len = sizeof(shake256_msg2),
    .digest = (uint8_t *)shake256_digest2,
    .digest_len = sizeof(shake256_digest2),
};

// SHA3_224ShortMsg.rsp Len = 8
static uint8_t sha3_224_msg1[] = {
    0x01,
};
static uint8_t sha3_224_digest1[] = {
    0x48, 0x82, 0x86, 0xd9, 0xd3, 0x27, 0x16, 0xe5, 0x88, 0x1e,
    0xa1, 0xee, 0x51, 0xf3, 0x6d, 0x36, 0x60, 0xd7, 0x0f, 0x0d,
    0xb0, 0x3b, 0x3f, 0x61, 0x2c, 0xe9, 0xed, 0xa4,
};
static kmac_test_vector_t sha3_224_t1 = {
    .operation = kSHA3_224,
    .input_str = (uint8_t *)sha3_224_msg1,
    .input_str_len = sizeof(sha3_224_msg1),
    .digest = (uint8_t *)sha3_224_digest1,
    .digest_len = sizeof(sha3_224_digest1),
};

// SHA3_256ShortMsg.rsp Len = 152
static uint8_t sha3_256_msg1[] = {
    0xd7, 0x51, 0xcc, 0xd2, 0xcd, 0x65, 0xf2, 0x7d, 0xb5, 0x39,
    0x17, 0x69, 0x20, 0xa7, 0x00, 0x57, 0xa0, 0x8a, 0x6b,
};
static uint8_t sha3_256_digest1[] = {
    0x7a, 0xac, 0xa8, 0x0d, 0xbe, 0xb8, 0xdc, 0x36, 0x77, 0xd1, 0x8b,
    0x84, 0x79, 0x59, 0x85, 0x46, 0x36, 0x50, 0xd7, 0x2f, 0x25, 0x43,
    0xe0, 0xec, 0x70, 0x9c, 0x9e, 0x70, 0xb8, 0xcd, 0x7b, 0x79,
};
static kmac_test_vector_t sha3_256_t1 = {
    .operation = kSHA3_256,
    .input_str = (uint8_t *)sha3_256_msg1,
    .input_str_len = sizeof(sha3_256_msg1),
    .digest = (uint8_t *)sha3_256_digest1,
    .digest_len = sizeof(sha3_256_digest1),
};

// SHA3_384LongMsg.rsp Len = 1672
static uint8_t sha3_384_msg1[] = {
    0x5f, 0xe3, 0x59, 0x23, 0xb4, 0xe0, 0xaf, 0x7d, 0xd2, 0x49, 0x71, 0x81,
    0x2a, 0x58, 0x42, 0x55, 0x19, 0x85, 0x0a, 0x50, 0x6d, 0xfa, 0x9b, 0x0d,
    0x25, 0x47, 0x95, 0xbe, 0x78, 0x57, 0x86, 0xc3, 0x19, 0xa2, 0x56, 0x7c,
    0xba, 0xa5, 0xe3, 0x5b, 0xcf, 0x8f, 0xe8, 0x3d, 0x94, 0x3e, 0x23, 0xfa,
    0x51, 0x69, 0xb7, 0x3a, 0xdc, 0x1f, 0xcf, 0x8b, 0x60, 0x70, 0x84, 0xb1,
    0x5e, 0x6a, 0x01, 0x3d, 0xf1, 0x47, 0xe4, 0x62, 0x56, 0xe4, 0xe8, 0x03,
    0xab, 0x75, 0xc1, 0x10, 0xf7, 0x78, 0x48, 0x13, 0x6b, 0xe7, 0xd8, 0x06,
    0xe8, 0xb2, 0xf8, 0x68, 0xc1, 0x6c, 0x3a, 0x90, 0xc1, 0x44, 0x63, 0x40,
    0x70, 0x38, 0xcb, 0x7d, 0x92, 0x85, 0x07, 0x9e, 0xf1, 0x62, 0xc6, 0xa4,
    0x5c, 0xed, 0xf9, 0xc9, 0xf0, 0x66, 0x37, 0x5c, 0x96, 0x9b, 0x5f, 0xcb,
    0xcd, 0xa3, 0x7f, 0x02, 0xaa, 0xcf, 0xf4, 0xf3, 0x1c, 0xde, 0xd3, 0x76,
    0x75, 0x70, 0x88, 0x54, 0x26, 0xbe, 0xbd, 0x9e, 0xca, 0x87, 0x7e, 0x44,
    0x67, 0x4e, 0x9a, 0xe2, 0xf0, 0xc2, 0x4c, 0xdd, 0x0e, 0x7e, 0x1a, 0xaf,
    0x1f, 0xf2, 0xfe, 0x7f, 0x80, 0xa1, 0xc4, 0xf5, 0x07, 0x8e, 0xb3, 0x4c,
    0xd4, 0xf0, 0x6f, 0xa9, 0x4a, 0x2d, 0x1e, 0xab, 0x58, 0x06, 0xca, 0x43,
    0xfd, 0x0f, 0x06, 0xc6, 0x0b, 0x63, 0xd5, 0x40, 0x2b, 0x95, 0xc7, 0x0c,
    0x21, 0xea, 0x65, 0xa1, 0x51, 0xc5, 0xcf, 0xaf, 0x82, 0x62, 0xa4, 0x6b,
    0xe3, 0xc7, 0x22, 0x26, 0x4b,
};
static uint8_t sha3_384_digest1[] = {
    0x30, 0x54, 0xd2, 0x49, 0xf9, 0x16, 0xa6, 0x03, 0x9b, 0x2a, 0x9c, 0x3e,
    0xbe, 0xc1, 0x41, 0x87, 0x91, 0xa0, 0x60, 0x8a, 0x17, 0x0e, 0x6d, 0x36,
    0x48, 0x60, 0x35, 0xe5, 0xf9, 0x26, 0x35, 0xea, 0xba, 0x98, 0x07, 0x2a,
    0x85, 0x37, 0x3c, 0xb5, 0x4e, 0x2a, 0xe3, 0xf9, 0x82, 0xce, 0x13, 0x2b,
};
static kmac_test_vector_t sha3_384_t1 = {
    .operation = kSHA3_384,
    .input_str = (uint8_t *)sha3_384_msg1,
    .input_str_len = sizeof(sha3_384_msg1),
    .digest = (uint8_t *)sha3_384_digest1,
    .digest_len = sizeof(sha3_384_digest1),
};

// SHA3_512ShortMsg.rsp Len = 296
static uint8_t sha3_512_msg1[] = {
    0xf3, 0x4d, 0x10, 0x02, 0x69, 0xae, 0xe3, 0xea, 0xd1, 0x56,
    0x89, 0x5e, 0x86, 0x44, 0xd4, 0x74, 0x94, 0x64, 0xd5, 0x92,
    0x1d, 0x61, 0x57, 0xdf, 0xfc, 0xbb, 0xad, 0xf7, 0xa7, 0x19,
    0xae, 0xe3, 0x5a, 0xe0, 0xfd, 0x48, 0x72,
};
static uint8_t sha3_512_digest1[] = {
    0x56, 0x5a, 0x1d, 0xd9, 0xd4, 0x9f, 0x8d, 0xde, 0xfb, 0x79, 0xa3,
    0xc7, 0xa2, 0x09, 0xf5, 0x3f, 0x0b, 0xc9, 0xf5, 0x39, 0x62, 0x69,
    0xb1, 0xce, 0x2a, 0x2b, 0x28, 0x3a, 0x3c, 0xb4, 0x5e, 0xe3, 0xae,
    0x65, 0x2e, 0x4c, 0xa1, 0x0b, 0x26, 0xce, 0xd7, 0xe5, 0x23, 0x62,
    0x27, 0x00, 0x6c, 0x94, 0xa3, 0x75, 0x53, 0xdb, 0x1b, 0x6f, 0xe5,
    0xc0, 0xc2, 0xed, 0xed, 0x75, 0x6c, 0x89, 0x6b, 0xb1,
};
static kmac_test_vector_t sha3_512_t1 = {
    .operation = kSHA3_512,
    .input_str = (uint8_t *)sha3_512_msg1,
    .input_str_len = sizeof(sha3_512_msg1),
    .digest = (uint8_t *)sha3_512_digest1,
    .digest_len = sizeof(sha3_512_digest1),
};

kmac_test_vector_t *nist_kmac_vectors[] = {
    &shake128_t1, &shake128_t2, &shake128_t3, &shake256_t1, &shake256_t2,
    &sha3_224_t1, &sha3_256_t1, &sha3_384_t1, &sha3_512_t1,
};

size_t nist_kmac_nr_of_vectors =
    sizeof(nist_kmac_vectors) / sizeof(kmac_test_vector_t *);
