// Copyright lowRISC contributors.
// Licensed under the Apache License, Version 2.0, see LICENSE for details.
// SPDX-License-Identifier: Apache-2.0
//
// Register Top module auto-generated by `reggen`

`include "prim_assert.sv"

module flash_ctrl_core_reg_top (
  input clk_i,
  input rst_ni,
  input rst_shadowed_ni,
  input  tlul_pkg::tl_h2d_t tl_i,
  output tlul_pkg::tl_d2h_t tl_o,

  // Output port for window
  output tlul_pkg::tl_h2d_t tl_win_o  [2],
  input  tlul_pkg::tl_d2h_t tl_win_i  [2],

  // To HW
  output flash_ctrl_reg_pkg::flash_ctrl_core_reg2hw_t reg2hw, // Write
  input  flash_ctrl_reg_pkg::flash_ctrl_core_hw2reg_t hw2reg, // Read

  output logic shadowed_storage_err_o,
  output logic shadowed_update_err_o,

  // Integrity check errors
  output logic intg_err_o,

  // Config
  input devmode_i // If 1, explicit error return for unmapped register access
);

  import flash_ctrl_reg_pkg::* ;

  localparam int AW = 9;
  localparam int DW = 32;
  localparam int DBW = DW/8;                    // Byte Width

  // register signals
  logic           reg_we;
  logic           reg_re;
  logic [AW-1:0]  reg_addr;
  logic [DW-1:0]  reg_wdata;
  logic [DBW-1:0] reg_be;
  logic [DW-1:0]  reg_rdata;
  logic           reg_error;

  logic          addrmiss, wr_err;

  logic [DW-1:0] reg_rdata_next;
  logic reg_busy;

  tlul_pkg::tl_h2d_t tl_reg_h2d;
  tlul_pkg::tl_d2h_t tl_reg_d2h;


  // incoming payload check
  logic intg_err;
  tlul_cmd_intg_chk u_chk (
    .tl_i(tl_i),
    .err_o(intg_err)
  );

  // also check for spurious write enables
  logic reg_we_err;
  logic [106:0] reg_we_check;
  prim_reg_we_check #(
    .OneHotWidth(107)
  ) u_prim_reg_we_check (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .oh_i  (reg_we_check),
    .en_i  (reg_we && !addrmiss),
    .err_o (reg_we_err)
  );

  logic err_q;
  always_ff @(posedge clk_i or negedge rst_ni) begin
    if (!rst_ni) begin
      err_q <= '0;
    end else if (intg_err || reg_we_err) begin
      err_q <= 1'b1;
    end
  end

  // integrity error output is permanent and should be used for alert generation
  // register errors are transactional
  assign intg_err_o = err_q | intg_err | reg_we_err;

  // outgoing integrity generation
  tlul_pkg::tl_d2h_t tl_o_pre;
  tlul_rsp_intg_gen #(
    .EnableRspIntgGen(1),
    .EnableDataIntgGen(0)
  ) u_rsp_intg_gen (
    .tl_i(tl_o_pre),
    .tl_o(tl_o)
  );

  tlul_pkg::tl_h2d_t tl_socket_h2d [3];
  tlul_pkg::tl_d2h_t tl_socket_d2h [3];

  logic [1:0] reg_steer;

  // socket_1n connection
  assign tl_reg_h2d = tl_socket_h2d[2];
  assign tl_socket_d2h[2] = tl_reg_d2h;

  assign tl_win_o[0] = tl_socket_h2d[0];
  assign tl_socket_d2h[0] = tl_win_i[0];
  assign tl_win_o[1] = tl_socket_h2d[1];
  assign tl_socket_d2h[1] = tl_win_i[1];

  // Create Socket_1n
  tlul_socket_1n #(
    .N            (3),
    .HReqPass     (1'b1),
    .HRspPass     (1'b1),
    .DReqPass     ({3{1'b1}}),
    .DRspPass     ({3{1'b1}}),
    .HReqDepth    (4'h0),
    .HRspDepth    (4'h0),
    .DReqDepth    ({3{4'h0}}),
    .DRspDepth    ({3{4'h0}}),
    .ExplicitErrs (1'b0)
  ) u_socket (
    .clk_i  (clk_i),
    .rst_ni (rst_ni),
    .tl_h_i (tl_i),
    .tl_h_o (tl_o_pre),
    .tl_d_o (tl_socket_h2d),
    .tl_d_i (tl_socket_d2h),
    .dev_select_i (reg_steer)
  );

  // Create steering logic
  always_comb begin
    unique case (tl_i.a_address[AW-1:0]) inside
      [428:431]: begin
        reg_steer = 0;
      end
      [432:435]: begin
        reg_steer = 1;
      end
      default: begin
        // Default set to register
        reg_steer = 2;
      end
    endcase

    // Override this in case of an integrity error
    if (intg_err) begin
      reg_steer = 2;
    end
  end

  tlul_adapter_reg #(
    .RegAw(AW),
    .RegDw(DW),
    .EnableDataIntgGen(1)
  ) u_reg_if (
    .clk_i  (clk_i),
    .rst_ni (rst_ni),

    .tl_i (tl_reg_h2d),
    .tl_o (tl_reg_d2h),

    .en_ifetch_i(prim_mubi_pkg::MuBi4False),
    .intg_error_o(),

    .we_o    (reg_we),
    .re_o    (reg_re),
    .addr_o  (reg_addr),
    .wdata_o (reg_wdata),
    .be_o    (reg_be),
    .busy_i  (reg_busy),
    .rdata_i (reg_rdata),
    .error_i (reg_error)
  );

  // cdc oversampling signals

  assign reg_rdata = reg_rdata_next ;
  assign reg_error = (devmode_i & addrmiss) | wr_err | intg_err;

  // Define SW related signals
  // Format: <reg>_<field>_{wd|we|qs}
  //        or <reg>_{wd|we|qs} if field == 1 or 0
  logic intr_state_we;
  logic intr_state_prog_empty_qs;
  logic intr_state_prog_empty_wd;
  logic intr_state_prog_lvl_qs;
  logic intr_state_prog_lvl_wd;
  logic intr_state_rd_full_qs;
  logic intr_state_rd_full_wd;
  logic intr_state_rd_lvl_qs;
  logic intr_state_rd_lvl_wd;
  logic intr_state_op_done_qs;
  logic intr_state_op_done_wd;
  logic intr_state_corr_err_qs;
  logic intr_state_corr_err_wd;
  logic intr_enable_we;
  logic intr_enable_prog_empty_qs;
  logic intr_enable_prog_empty_wd;
  logic intr_enable_prog_lvl_qs;
  logic intr_enable_prog_lvl_wd;
  logic intr_enable_rd_full_qs;
  logic intr_enable_rd_full_wd;
  logic intr_enable_rd_lvl_qs;
  logic intr_enable_rd_lvl_wd;
  logic intr_enable_op_done_qs;
  logic intr_enable_op_done_wd;
  logic intr_enable_corr_err_qs;
  logic intr_enable_corr_err_wd;
  logic intr_test_we;
  logic intr_test_prog_empty_wd;
  logic intr_test_prog_lvl_wd;
  logic intr_test_rd_full_wd;
  logic intr_test_rd_lvl_wd;
  logic intr_test_op_done_wd;
  logic intr_test_corr_err_wd;
  logic alert_test_we;
  logic alert_test_recov_err_wd;
  logic alert_test_fatal_std_err_wd;
  logic alert_test_fatal_err_wd;
  logic alert_test_fatal_prim_flash_alert_wd;
  logic alert_test_recov_prim_flash_alert_wd;
  logic dis_we;
  logic [3:0] dis_qs;
  logic [3:0] dis_wd;
  logic exec_we;
  logic [31:0] exec_qs;
  logic [31:0] exec_wd;
  logic init_we;
  logic init_qs;
  logic init_wd;
  logic ctrl_regwen_re;
  logic ctrl_regwen_qs;
  logic control_we;
  logic control_start_qs;
  logic control_start_wd;
  logic [1:0] control_op_qs;
  logic [1:0] control_op_wd;
  logic control_prog_sel_qs;
  logic control_prog_sel_wd;
  logic control_erase_sel_qs;
  logic control_erase_sel_wd;
  logic control_partition_sel_qs;
  logic control_partition_sel_wd;
  logic [1:0] control_info_sel_qs;
  logic [1:0] control_info_sel_wd;
  logic [11:0] control_num_qs;
  logic [11:0] control_num_wd;
  logic addr_we;
  logic [19:0] addr_qs;
  logic [19:0] addr_wd;
  logic prog_type_en_we;
  logic prog_type_en_normal_qs;
  logic prog_type_en_normal_wd;
  logic prog_type_en_repair_qs;
  logic prog_type_en_repair_wd;
  logic erase_suspend_we;
  logic erase_suspend_qs;
  logic erase_suspend_wd;
  logic region_cfg_regwen_0_we;
  logic region_cfg_regwen_0_qs;
  logic region_cfg_regwen_0_wd;
  logic region_cfg_regwen_1_we;
  logic region_cfg_regwen_1_qs;
  logic region_cfg_regwen_1_wd;
  logic region_cfg_regwen_2_we;
  logic region_cfg_regwen_2_qs;
  logic region_cfg_regwen_2_wd;
  logic region_cfg_regwen_3_we;
  logic region_cfg_regwen_3_qs;
  logic region_cfg_regwen_3_wd;
  logic region_cfg_regwen_4_we;
  logic region_cfg_regwen_4_qs;
  logic region_cfg_regwen_4_wd;
  logic region_cfg_regwen_5_we;
  logic region_cfg_regwen_5_qs;
  logic region_cfg_regwen_5_wd;
  logic region_cfg_regwen_6_we;
  logic region_cfg_regwen_6_qs;
  logic region_cfg_regwen_6_wd;
  logic region_cfg_regwen_7_we;
  logic region_cfg_regwen_7_qs;
  logic region_cfg_regwen_7_wd;
  logic mp_region_cfg_0_we;
  logic [3:0] mp_region_cfg_0_en_0_qs;
  logic [3:0] mp_region_cfg_0_en_0_wd;
  logic [3:0] mp_region_cfg_0_rd_en_0_qs;
  logic [3:0] mp_region_cfg_0_rd_en_0_wd;
  logic [3:0] mp_region_cfg_0_prog_en_0_qs;
  logic [3:0] mp_region_cfg_0_prog_en_0_wd;
  logic [3:0] mp_region_cfg_0_erase_en_0_qs;
  logic [3:0] mp_region_cfg_0_erase_en_0_wd;
  logic [3:0] mp_region_cfg_0_scramble_en_0_qs;
  logic [3:0] mp_region_cfg_0_scramble_en_0_wd;
  logic [3:0] mp_region_cfg_0_ecc_en_0_qs;
  logic [3:0] mp_region_cfg_0_ecc_en_0_wd;
  logic [3:0] mp_region_cfg_0_he_en_0_qs;
  logic [3:0] mp_region_cfg_0_he_en_0_wd;
  logic mp_region_cfg_1_we;
  logic [3:0] mp_region_cfg_1_en_1_qs;
  logic [3:0] mp_region_cfg_1_en_1_wd;
  logic [3:0] mp_region_cfg_1_rd_en_1_qs;
  logic [3:0] mp_region_cfg_1_rd_en_1_wd;
  logic [3:0] mp_region_cfg_1_prog_en_1_qs;
  logic [3:0] mp_region_cfg_1_prog_en_1_wd;
  logic [3:0] mp_region_cfg_1_erase_en_1_qs;
  logic [3:0] mp_region_cfg_1_erase_en_1_wd;
  logic [3:0] mp_region_cfg_1_scramble_en_1_qs;
  logic [3:0] mp_region_cfg_1_scramble_en_1_wd;
  logic [3:0] mp_region_cfg_1_ecc_en_1_qs;
  logic [3:0] mp_region_cfg_1_ecc_en_1_wd;
  logic [3:0] mp_region_cfg_1_he_en_1_qs;
  logic [3:0] mp_region_cfg_1_he_en_1_wd;
  logic mp_region_cfg_2_we;
  logic [3:0] mp_region_cfg_2_en_2_qs;
  logic [3:0] mp_region_cfg_2_en_2_wd;
  logic [3:0] mp_region_cfg_2_rd_en_2_qs;
  logic [3:0] mp_region_cfg_2_rd_en_2_wd;
  logic [3:0] mp_region_cfg_2_prog_en_2_qs;
  logic [3:0] mp_region_cfg_2_prog_en_2_wd;
  logic [3:0] mp_region_cfg_2_erase_en_2_qs;
  logic [3:0] mp_region_cfg_2_erase_en_2_wd;
  logic [3:0] mp_region_cfg_2_scramble_en_2_qs;
  logic [3:0] mp_region_cfg_2_scramble_en_2_wd;
  logic [3:0] mp_region_cfg_2_ecc_en_2_qs;
  logic [3:0] mp_region_cfg_2_ecc_en_2_wd;
  logic [3:0] mp_region_cfg_2_he_en_2_qs;
  logic [3:0] mp_region_cfg_2_he_en_2_wd;
  logic mp_region_cfg_3_we;
  logic [3:0] mp_region_cfg_3_en_3_qs;
  logic [3:0] mp_region_cfg_3_en_3_wd;
  logic [3:0] mp_region_cfg_3_rd_en_3_qs;
  logic [3:0] mp_region_cfg_3_rd_en_3_wd;
  logic [3:0] mp_region_cfg_3_prog_en_3_qs;
  logic [3:0] mp_region_cfg_3_prog_en_3_wd;
  logic [3:0] mp_region_cfg_3_erase_en_3_qs;
  logic [3:0] mp_region_cfg_3_erase_en_3_wd;
  logic [3:0] mp_region_cfg_3_scramble_en_3_qs;
  logic [3:0] mp_region_cfg_3_scramble_en_3_wd;
  logic [3:0] mp_region_cfg_3_ecc_en_3_qs;
  logic [3:0] mp_region_cfg_3_ecc_en_3_wd;
  logic [3:0] mp_region_cfg_3_he_en_3_qs;
  logic [3:0] mp_region_cfg_3_he_en_3_wd;
  logic mp_region_cfg_4_we;
  logic [3:0] mp_region_cfg_4_en_4_qs;
  logic [3:0] mp_region_cfg_4_en_4_wd;
  logic [3:0] mp_region_cfg_4_rd_en_4_qs;
  logic [3:0] mp_region_cfg_4_rd_en_4_wd;
  logic [3:0] mp_region_cfg_4_prog_en_4_qs;
  logic [3:0] mp_region_cfg_4_prog_en_4_wd;
  logic [3:0] mp_region_cfg_4_erase_en_4_qs;
  logic [3:0] mp_region_cfg_4_erase_en_4_wd;
  logic [3:0] mp_region_cfg_4_scramble_en_4_qs;
  logic [3:0] mp_region_cfg_4_scramble_en_4_wd;
  logic [3:0] mp_region_cfg_4_ecc_en_4_qs;
  logic [3:0] mp_region_cfg_4_ecc_en_4_wd;
  logic [3:0] mp_region_cfg_4_he_en_4_qs;
  logic [3:0] mp_region_cfg_4_he_en_4_wd;
  logic mp_region_cfg_5_we;
  logic [3:0] mp_region_cfg_5_en_5_qs;
  logic [3:0] mp_region_cfg_5_en_5_wd;
  logic [3:0] mp_region_cfg_5_rd_en_5_qs;
  logic [3:0] mp_region_cfg_5_rd_en_5_wd;
  logic [3:0] mp_region_cfg_5_prog_en_5_qs;
  logic [3:0] mp_region_cfg_5_prog_en_5_wd;
  logic [3:0] mp_region_cfg_5_erase_en_5_qs;
  logic [3:0] mp_region_cfg_5_erase_en_5_wd;
  logic [3:0] mp_region_cfg_5_scramble_en_5_qs;
  logic [3:0] mp_region_cfg_5_scramble_en_5_wd;
  logic [3:0] mp_region_cfg_5_ecc_en_5_qs;
  logic [3:0] mp_region_cfg_5_ecc_en_5_wd;
  logic [3:0] mp_region_cfg_5_he_en_5_qs;
  logic [3:0] mp_region_cfg_5_he_en_5_wd;
  logic mp_region_cfg_6_we;
  logic [3:0] mp_region_cfg_6_en_6_qs;
  logic [3:0] mp_region_cfg_6_en_6_wd;
  logic [3:0] mp_region_cfg_6_rd_en_6_qs;
  logic [3:0] mp_region_cfg_6_rd_en_6_wd;
  logic [3:0] mp_region_cfg_6_prog_en_6_qs;
  logic [3:0] mp_region_cfg_6_prog_en_6_wd;
  logic [3:0] mp_region_cfg_6_erase_en_6_qs;
  logic [3:0] mp_region_cfg_6_erase_en_6_wd;
  logic [3:0] mp_region_cfg_6_scramble_en_6_qs;
  logic [3:0] mp_region_cfg_6_scramble_en_6_wd;
  logic [3:0] mp_region_cfg_6_ecc_en_6_qs;
  logic [3:0] mp_region_cfg_6_ecc_en_6_wd;
  logic [3:0] mp_region_cfg_6_he_en_6_qs;
  logic [3:0] mp_region_cfg_6_he_en_6_wd;
  logic mp_region_cfg_7_we;
  logic [3:0] mp_region_cfg_7_en_7_qs;
  logic [3:0] mp_region_cfg_7_en_7_wd;
  logic [3:0] mp_region_cfg_7_rd_en_7_qs;
  logic [3:0] mp_region_cfg_7_rd_en_7_wd;
  logic [3:0] mp_region_cfg_7_prog_en_7_qs;
  logic [3:0] mp_region_cfg_7_prog_en_7_wd;
  logic [3:0] mp_region_cfg_7_erase_en_7_qs;
  logic [3:0] mp_region_cfg_7_erase_en_7_wd;
  logic [3:0] mp_region_cfg_7_scramble_en_7_qs;
  logic [3:0] mp_region_cfg_7_scramble_en_7_wd;
  logic [3:0] mp_region_cfg_7_ecc_en_7_qs;
  logic [3:0] mp_region_cfg_7_ecc_en_7_wd;
  logic [3:0] mp_region_cfg_7_he_en_7_qs;
  logic [3:0] mp_region_cfg_7_he_en_7_wd;
  logic mp_region_0_we;
  logic [8:0] mp_region_0_base_0_qs;
  logic [8:0] mp_region_0_base_0_wd;
  logic [9:0] mp_region_0_size_0_qs;
  logic [9:0] mp_region_0_size_0_wd;
  logic mp_region_1_we;
  logic [8:0] mp_region_1_base_1_qs;
  logic [8:0] mp_region_1_base_1_wd;
  logic [9:0] mp_region_1_size_1_qs;
  logic [9:0] mp_region_1_size_1_wd;
  logic mp_region_2_we;
  logic [8:0] mp_region_2_base_2_qs;
  logic [8:0] mp_region_2_base_2_wd;
  logic [9:0] mp_region_2_size_2_qs;
  logic [9:0] mp_region_2_size_2_wd;
  logic mp_region_3_we;
  logic [8:0] mp_region_3_base_3_qs;
  logic [8:0] mp_region_3_base_3_wd;
  logic [9:0] mp_region_3_size_3_qs;
  logic [9:0] mp_region_3_size_3_wd;
  logic mp_region_4_we;
  logic [8:0] mp_region_4_base_4_qs;
  logic [8:0] mp_region_4_base_4_wd;
  logic [9:0] mp_region_4_size_4_qs;
  logic [9:0] mp_region_4_size_4_wd;
  logic mp_region_5_we;
  logic [8:0] mp_region_5_base_5_qs;
  logic [8:0] mp_region_5_base_5_wd;
  logic [9:0] mp_region_5_size_5_qs;
  logic [9:0] mp_region_5_size_5_wd;
  logic mp_region_6_we;
  logic [8:0] mp_region_6_base_6_qs;
  logic [8:0] mp_region_6_base_6_wd;
  logic [9:0] mp_region_6_size_6_qs;
  logic [9:0] mp_region_6_size_6_wd;
  logic mp_region_7_we;
  logic [8:0] mp_region_7_base_7_qs;
  logic [8:0] mp_region_7_base_7_wd;
  logic [9:0] mp_region_7_size_7_qs;
  logic [9:0] mp_region_7_size_7_wd;
  logic default_region_we;
  logic [3:0] default_region_rd_en_qs;
  logic [3:0] default_region_rd_en_wd;
  logic [3:0] default_region_prog_en_qs;
  logic [3:0] default_region_prog_en_wd;
  logic [3:0] default_region_erase_en_qs;
  logic [3:0] default_region_erase_en_wd;
  logic [3:0] default_region_scramble_en_qs;
  logic [3:0] default_region_scramble_en_wd;
  logic [3:0] default_region_ecc_en_qs;
  logic [3:0] default_region_ecc_en_wd;
  logic [3:0] default_region_he_en_qs;
  logic [3:0] default_region_he_en_wd;
  logic bank0_info0_regwen_0_we;
  logic bank0_info0_regwen_0_qs;
  logic bank0_info0_regwen_0_wd;
  logic bank0_info0_regwen_1_we;
  logic bank0_info0_regwen_1_qs;
  logic bank0_info0_regwen_1_wd;
  logic bank0_info0_regwen_2_we;
  logic bank0_info0_regwen_2_qs;
  logic bank0_info0_regwen_2_wd;
  logic bank0_info0_regwen_3_we;
  logic bank0_info0_regwen_3_qs;
  logic bank0_info0_regwen_3_wd;
  logic bank0_info0_regwen_4_we;
  logic bank0_info0_regwen_4_qs;
  logic bank0_info0_regwen_4_wd;
  logic bank0_info0_regwen_5_we;
  logic bank0_info0_regwen_5_qs;
  logic bank0_info0_regwen_5_wd;
  logic bank0_info0_regwen_6_we;
  logic bank0_info0_regwen_6_qs;
  logic bank0_info0_regwen_6_wd;
  logic bank0_info0_regwen_7_we;
  logic bank0_info0_regwen_7_qs;
  logic bank0_info0_regwen_7_wd;
  logic bank0_info0_regwen_8_we;
  logic bank0_info0_regwen_8_qs;
  logic bank0_info0_regwen_8_wd;
  logic bank0_info0_regwen_9_we;
  logic bank0_info0_regwen_9_qs;
  logic bank0_info0_regwen_9_wd;
  logic bank0_info0_page_cfg_0_we;
  logic [3:0] bank0_info0_page_cfg_0_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_rd_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_rd_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_prog_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_prog_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_erase_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_erase_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_scramble_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_scramble_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_ecc_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_ecc_en_0_wd;
  logic [3:0] bank0_info0_page_cfg_0_he_en_0_qs;
  logic [3:0] bank0_info0_page_cfg_0_he_en_0_wd;
  logic bank0_info0_page_cfg_1_we;
  logic [3:0] bank0_info0_page_cfg_1_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_rd_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_rd_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_prog_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_prog_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_erase_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_erase_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_scramble_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_scramble_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_ecc_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_ecc_en_1_wd;
  logic [3:0] bank0_info0_page_cfg_1_he_en_1_qs;
  logic [3:0] bank0_info0_page_cfg_1_he_en_1_wd;
  logic bank0_info0_page_cfg_2_we;
  logic [3:0] bank0_info0_page_cfg_2_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_rd_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_rd_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_prog_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_prog_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_erase_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_erase_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_scramble_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_scramble_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_ecc_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_ecc_en_2_wd;
  logic [3:0] bank0_info0_page_cfg_2_he_en_2_qs;
  logic [3:0] bank0_info0_page_cfg_2_he_en_2_wd;
  logic bank0_info0_page_cfg_3_we;
  logic [3:0] bank0_info0_page_cfg_3_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_rd_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_rd_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_prog_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_prog_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_erase_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_erase_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_scramble_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_scramble_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_ecc_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_ecc_en_3_wd;
  logic [3:0] bank0_info0_page_cfg_3_he_en_3_qs;
  logic [3:0] bank0_info0_page_cfg_3_he_en_3_wd;
  logic bank0_info0_page_cfg_4_we;
  logic [3:0] bank0_info0_page_cfg_4_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_rd_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_rd_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_prog_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_prog_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_erase_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_erase_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_scramble_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_scramble_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_ecc_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_ecc_en_4_wd;
  logic [3:0] bank0_info0_page_cfg_4_he_en_4_qs;
  logic [3:0] bank0_info0_page_cfg_4_he_en_4_wd;
  logic bank0_info0_page_cfg_5_we;
  logic [3:0] bank0_info0_page_cfg_5_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_rd_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_rd_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_prog_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_prog_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_erase_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_erase_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_scramble_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_scramble_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_ecc_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_ecc_en_5_wd;
  logic [3:0] bank0_info0_page_cfg_5_he_en_5_qs;
  logic [3:0] bank0_info0_page_cfg_5_he_en_5_wd;
  logic bank0_info0_page_cfg_6_we;
  logic [3:0] bank0_info0_page_cfg_6_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_rd_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_rd_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_prog_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_prog_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_erase_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_erase_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_scramble_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_scramble_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_ecc_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_ecc_en_6_wd;
  logic [3:0] bank0_info0_page_cfg_6_he_en_6_qs;
  logic [3:0] bank0_info0_page_cfg_6_he_en_6_wd;
  logic bank0_info0_page_cfg_7_we;
  logic [3:0] bank0_info0_page_cfg_7_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_rd_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_rd_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_prog_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_prog_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_erase_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_erase_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_scramble_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_scramble_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_ecc_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_ecc_en_7_wd;
  logic [3:0] bank0_info0_page_cfg_7_he_en_7_qs;
  logic [3:0] bank0_info0_page_cfg_7_he_en_7_wd;
  logic bank0_info0_page_cfg_8_we;
  logic [3:0] bank0_info0_page_cfg_8_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_rd_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_rd_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_prog_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_prog_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_erase_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_erase_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_scramble_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_scramble_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_ecc_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_ecc_en_8_wd;
  logic [3:0] bank0_info0_page_cfg_8_he_en_8_qs;
  logic [3:0] bank0_info0_page_cfg_8_he_en_8_wd;
  logic bank0_info0_page_cfg_9_we;
  logic [3:0] bank0_info0_page_cfg_9_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_rd_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_rd_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_prog_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_prog_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_erase_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_erase_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_scramble_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_scramble_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_ecc_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_ecc_en_9_wd;
  logic [3:0] bank0_info0_page_cfg_9_he_en_9_qs;
  logic [3:0] bank0_info0_page_cfg_9_he_en_9_wd;
  logic bank0_info1_regwen_we;
  logic bank0_info1_regwen_qs;
  logic bank0_info1_regwen_wd;
  logic bank0_info1_page_cfg_we;
  logic [3:0] bank0_info1_page_cfg_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_rd_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_rd_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_prog_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_prog_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_erase_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_erase_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_scramble_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_scramble_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_ecc_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_ecc_en_0_wd;
  logic [3:0] bank0_info1_page_cfg_he_en_0_qs;
  logic [3:0] bank0_info1_page_cfg_he_en_0_wd;
  logic bank0_info2_regwen_0_we;
  logic bank0_info2_regwen_0_qs;
  logic bank0_info2_regwen_0_wd;
  logic bank0_info2_regwen_1_we;
  logic bank0_info2_regwen_1_qs;
  logic bank0_info2_regwen_1_wd;
  logic bank0_info2_page_cfg_0_we;
  logic [3:0] bank0_info2_page_cfg_0_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_rd_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_rd_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_prog_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_prog_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_erase_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_erase_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_scramble_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_scramble_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_ecc_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_ecc_en_0_wd;
  logic [3:0] bank0_info2_page_cfg_0_he_en_0_qs;
  logic [3:0] bank0_info2_page_cfg_0_he_en_0_wd;
  logic bank0_info2_page_cfg_1_we;
  logic [3:0] bank0_info2_page_cfg_1_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_rd_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_rd_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_prog_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_prog_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_erase_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_erase_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_scramble_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_scramble_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_ecc_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_ecc_en_1_wd;
  logic [3:0] bank0_info2_page_cfg_1_he_en_1_qs;
  logic [3:0] bank0_info2_page_cfg_1_he_en_1_wd;
  logic bank1_info0_regwen_0_we;
  logic bank1_info0_regwen_0_qs;
  logic bank1_info0_regwen_0_wd;
  logic bank1_info0_regwen_1_we;
  logic bank1_info0_regwen_1_qs;
  logic bank1_info0_regwen_1_wd;
  logic bank1_info0_regwen_2_we;
  logic bank1_info0_regwen_2_qs;
  logic bank1_info0_regwen_2_wd;
  logic bank1_info0_regwen_3_we;
  logic bank1_info0_regwen_3_qs;
  logic bank1_info0_regwen_3_wd;
  logic bank1_info0_regwen_4_we;
  logic bank1_info0_regwen_4_qs;
  logic bank1_info0_regwen_4_wd;
  logic bank1_info0_regwen_5_we;
  logic bank1_info0_regwen_5_qs;
  logic bank1_info0_regwen_5_wd;
  logic bank1_info0_regwen_6_we;
  logic bank1_info0_regwen_6_qs;
  logic bank1_info0_regwen_6_wd;
  logic bank1_info0_regwen_7_we;
  logic bank1_info0_regwen_7_qs;
  logic bank1_info0_regwen_7_wd;
  logic bank1_info0_regwen_8_we;
  logic bank1_info0_regwen_8_qs;
  logic bank1_info0_regwen_8_wd;
  logic bank1_info0_regwen_9_we;
  logic bank1_info0_regwen_9_qs;
  logic bank1_info0_regwen_9_wd;
  logic bank1_info0_page_cfg_0_we;
  logic [3:0] bank1_info0_page_cfg_0_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_rd_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_rd_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_prog_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_prog_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_erase_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_erase_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_scramble_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_scramble_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_ecc_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_ecc_en_0_wd;
  logic [3:0] bank1_info0_page_cfg_0_he_en_0_qs;
  logic [3:0] bank1_info0_page_cfg_0_he_en_0_wd;
  logic bank1_info0_page_cfg_1_we;
  logic [3:0] bank1_info0_page_cfg_1_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_rd_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_rd_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_prog_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_prog_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_erase_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_erase_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_scramble_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_scramble_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_ecc_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_ecc_en_1_wd;
  logic [3:0] bank1_info0_page_cfg_1_he_en_1_qs;
  logic [3:0] bank1_info0_page_cfg_1_he_en_1_wd;
  logic bank1_info0_page_cfg_2_we;
  logic [3:0] bank1_info0_page_cfg_2_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_rd_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_rd_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_prog_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_prog_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_erase_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_erase_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_scramble_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_scramble_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_ecc_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_ecc_en_2_wd;
  logic [3:0] bank1_info0_page_cfg_2_he_en_2_qs;
  logic [3:0] bank1_info0_page_cfg_2_he_en_2_wd;
  logic bank1_info0_page_cfg_3_we;
  logic [3:0] bank1_info0_page_cfg_3_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_rd_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_rd_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_prog_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_prog_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_erase_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_erase_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_scramble_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_scramble_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_ecc_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_ecc_en_3_wd;
  logic [3:0] bank1_info0_page_cfg_3_he_en_3_qs;
  logic [3:0] bank1_info0_page_cfg_3_he_en_3_wd;
  logic bank1_info0_page_cfg_4_we;
  logic [3:0] bank1_info0_page_cfg_4_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_rd_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_rd_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_prog_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_prog_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_erase_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_erase_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_scramble_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_scramble_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_ecc_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_ecc_en_4_wd;
  logic [3:0] bank1_info0_page_cfg_4_he_en_4_qs;
  logic [3:0] bank1_info0_page_cfg_4_he_en_4_wd;
  logic bank1_info0_page_cfg_5_we;
  logic [3:0] bank1_info0_page_cfg_5_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_rd_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_rd_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_prog_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_prog_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_erase_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_erase_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_scramble_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_scramble_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_ecc_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_ecc_en_5_wd;
  logic [3:0] bank1_info0_page_cfg_5_he_en_5_qs;
  logic [3:0] bank1_info0_page_cfg_5_he_en_5_wd;
  logic bank1_info0_page_cfg_6_we;
  logic [3:0] bank1_info0_page_cfg_6_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_rd_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_rd_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_prog_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_prog_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_erase_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_erase_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_scramble_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_scramble_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_ecc_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_ecc_en_6_wd;
  logic [3:0] bank1_info0_page_cfg_6_he_en_6_qs;
  logic [3:0] bank1_info0_page_cfg_6_he_en_6_wd;
  logic bank1_info0_page_cfg_7_we;
  logic [3:0] bank1_info0_page_cfg_7_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_rd_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_rd_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_prog_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_prog_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_erase_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_erase_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_scramble_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_scramble_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_ecc_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_ecc_en_7_wd;
  logic [3:0] bank1_info0_page_cfg_7_he_en_7_qs;
  logic [3:0] bank1_info0_page_cfg_7_he_en_7_wd;
  logic bank1_info0_page_cfg_8_we;
  logic [3:0] bank1_info0_page_cfg_8_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_rd_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_rd_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_prog_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_prog_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_erase_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_erase_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_scramble_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_scramble_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_ecc_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_ecc_en_8_wd;
  logic [3:0] bank1_info0_page_cfg_8_he_en_8_qs;
  logic [3:0] bank1_info0_page_cfg_8_he_en_8_wd;
  logic bank1_info0_page_cfg_9_we;
  logic [3:0] bank1_info0_page_cfg_9_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_rd_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_rd_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_prog_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_prog_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_erase_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_erase_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_scramble_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_scramble_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_ecc_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_ecc_en_9_wd;
  logic [3:0] bank1_info0_page_cfg_9_he_en_9_qs;
  logic [3:0] bank1_info0_page_cfg_9_he_en_9_wd;
  logic bank1_info1_regwen_we;
  logic bank1_info1_regwen_qs;
  logic bank1_info1_regwen_wd;
  logic bank1_info1_page_cfg_we;
  logic [3:0] bank1_info1_page_cfg_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_rd_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_rd_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_prog_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_prog_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_erase_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_erase_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_scramble_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_scramble_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_ecc_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_ecc_en_0_wd;
  logic [3:0] bank1_info1_page_cfg_he_en_0_qs;
  logic [3:0] bank1_info1_page_cfg_he_en_0_wd;
  logic bank1_info2_regwen_0_we;
  logic bank1_info2_regwen_0_qs;
  logic bank1_info2_regwen_0_wd;
  logic bank1_info2_regwen_1_we;
  logic bank1_info2_regwen_1_qs;
  logic bank1_info2_regwen_1_wd;
  logic bank1_info2_page_cfg_0_we;
  logic [3:0] bank1_info2_page_cfg_0_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_rd_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_rd_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_prog_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_prog_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_erase_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_erase_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_scramble_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_scramble_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_ecc_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_ecc_en_0_wd;
  logic [3:0] bank1_info2_page_cfg_0_he_en_0_qs;
  logic [3:0] bank1_info2_page_cfg_0_he_en_0_wd;
  logic bank1_info2_page_cfg_1_we;
  logic [3:0] bank1_info2_page_cfg_1_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_rd_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_rd_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_prog_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_prog_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_erase_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_erase_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_scramble_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_scramble_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_ecc_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_ecc_en_1_wd;
  logic [3:0] bank1_info2_page_cfg_1_he_en_1_qs;
  logic [3:0] bank1_info2_page_cfg_1_he_en_1_wd;
  logic bank_cfg_regwen_we;
  logic bank_cfg_regwen_qs;
  logic bank_cfg_regwen_wd;
  logic mp_bank_cfg_shadowed_re;
  logic mp_bank_cfg_shadowed_we;
  logic mp_bank_cfg_shadowed_erase_en_0_qs;
  logic mp_bank_cfg_shadowed_erase_en_0_wd;
  logic mp_bank_cfg_shadowed_erase_en_0_storage_err;
  logic mp_bank_cfg_shadowed_erase_en_0_update_err;
  logic mp_bank_cfg_shadowed_erase_en_1_qs;
  logic mp_bank_cfg_shadowed_erase_en_1_wd;
  logic mp_bank_cfg_shadowed_erase_en_1_storage_err;
  logic mp_bank_cfg_shadowed_erase_en_1_update_err;
  logic op_status_we;
  logic op_status_done_qs;
  logic op_status_done_wd;
  logic op_status_err_qs;
  logic op_status_err_wd;
  logic status_rd_full_qs;
  logic status_rd_empty_qs;
  logic status_prog_full_qs;
  logic status_prog_empty_qs;
  logic status_init_wip_qs;
  logic debug_state_re;
  logic [10:0] debug_state_qs;
  logic err_code_we;
  logic err_code_op_err_qs;
  logic err_code_op_err_wd;
  logic err_code_mp_err_qs;
  logic err_code_mp_err_wd;
  logic err_code_rd_err_qs;
  logic err_code_rd_err_wd;
  logic err_code_prog_err_qs;
  logic err_code_prog_err_wd;
  logic err_code_prog_win_err_qs;
  logic err_code_prog_win_err_wd;
  logic err_code_prog_type_err_qs;
  logic err_code_prog_type_err_wd;
  logic err_code_update_err_qs;
  logic err_code_update_err_wd;
  logic err_code_macro_err_qs;
  logic err_code_macro_err_wd;
  logic std_fault_status_reg_intg_err_qs;
  logic std_fault_status_prog_intg_err_qs;
  logic std_fault_status_lcmgr_err_qs;
  logic std_fault_status_lcmgr_intg_err_qs;
  logic std_fault_status_arb_fsm_err_qs;
  logic std_fault_status_storage_err_qs;
  logic std_fault_status_phy_fsm_err_qs;
  logic std_fault_status_ctrl_cnt_err_qs;
  logic std_fault_status_fifo_err_qs;
  logic fault_status_op_err_qs;
  logic fault_status_mp_err_qs;
  logic fault_status_rd_err_qs;
  logic fault_status_prog_err_qs;
  logic fault_status_prog_win_err_qs;
  logic fault_status_prog_type_err_qs;
  logic fault_status_seed_err_qs;
  logic fault_status_phy_relbl_err_qs;
  logic fault_status_phy_storage_err_qs;
  logic fault_status_spurious_ack_qs;
  logic fault_status_arb_err_qs;
  logic fault_status_host_gnt_err_qs;
  logic [19:0] err_addr_qs;
  logic ecc_single_err_cnt_we;
  logic [7:0] ecc_single_err_cnt_ecc_single_err_cnt_0_qs;
  logic [7:0] ecc_single_err_cnt_ecc_single_err_cnt_0_wd;
  logic [7:0] ecc_single_err_cnt_ecc_single_err_cnt_1_qs;
  logic [7:0] ecc_single_err_cnt_ecc_single_err_cnt_1_wd;
  logic [19:0] ecc_single_err_addr_0_qs;
  logic [19:0] ecc_single_err_addr_1_qs;
  logic phy_alert_cfg_we;
  logic phy_alert_cfg_alert_ack_qs;
  logic phy_alert_cfg_alert_ack_wd;
  logic phy_alert_cfg_alert_trig_qs;
  logic phy_alert_cfg_alert_trig_wd;
  logic phy_status_init_wip_qs;
  logic phy_status_prog_normal_avail_qs;
  logic phy_status_prog_repair_avail_qs;
  logic scratch_we;
  logic [31:0] scratch_qs;
  logic [31:0] scratch_wd;
  logic fifo_lvl_we;
  logic [4:0] fifo_lvl_prog_qs;
  logic [4:0] fifo_lvl_prog_wd;
  logic [4:0] fifo_lvl_rd_qs;
  logic [4:0] fifo_lvl_rd_wd;
  logic fifo_rst_we;
  logic fifo_rst_qs;
  logic fifo_rst_wd;
  logic curr_fifo_lvl_re;
  logic [4:0] curr_fifo_lvl_prog_qs;
  logic [4:0] curr_fifo_lvl_rd_qs;

  // Register instances
  // R[intr_state]: V(False)
  //   F[prog_empty]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_prog_empty (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_prog_empty_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.prog_empty.de),
    .d      (hw2reg.intr_state.prog_empty.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.prog_empty.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_prog_empty_qs)
  );

  //   F[prog_lvl]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_prog_lvl (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_prog_lvl_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.prog_lvl.de),
    .d      (hw2reg.intr_state.prog_lvl.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.prog_lvl.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_prog_lvl_qs)
  );

  //   F[rd_full]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_rd_full (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_rd_full_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.rd_full.de),
    .d      (hw2reg.intr_state.rd_full.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.rd_full.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_rd_full_qs)
  );

  //   F[rd_lvl]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_rd_lvl (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_rd_lvl_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.rd_lvl.de),
    .d      (hw2reg.intr_state.rd_lvl.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.rd_lvl.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_rd_lvl_qs)
  );

  //   F[op_done]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_op_done (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_op_done_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.op_done.de),
    .d      (hw2reg.intr_state.op_done.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.op_done.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_op_done_qs)
  );

  //   F[corr_err]: 5:5
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_corr_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_corr_err_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.corr_err.de),
    .d      (hw2reg.intr_state.corr_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.corr_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_corr_err_qs)
  );


  // R[intr_enable]: V(False)
  //   F[prog_empty]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_prog_empty (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_prog_empty_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.prog_empty.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_prog_empty_qs)
  );

  //   F[prog_lvl]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_prog_lvl (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_prog_lvl_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.prog_lvl.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_prog_lvl_qs)
  );

  //   F[rd_full]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_rd_full (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_rd_full_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.rd_full.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_rd_full_qs)
  );

  //   F[rd_lvl]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_rd_lvl (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_rd_lvl_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.rd_lvl.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_rd_lvl_qs)
  );

  //   F[op_done]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_op_done (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_op_done_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.op_done.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_op_done_qs)
  );

  //   F[corr_err]: 5:5
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_corr_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_corr_err_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.corr_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_corr_err_qs)
  );


  // R[intr_test]: V(True)
  logic intr_test_qe;
  logic [5:0] intr_test_flds_we;
  assign intr_test_qe = &intr_test_flds_we;
  //   F[prog_empty]: 0:0
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_prog_empty (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_prog_empty_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[0]),
    .q      (reg2hw.intr_test.prog_empty.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.prog_empty.qe = intr_test_qe;

  //   F[prog_lvl]: 1:1
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_prog_lvl (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_prog_lvl_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[1]),
    .q      (reg2hw.intr_test.prog_lvl.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.prog_lvl.qe = intr_test_qe;

  //   F[rd_full]: 2:2
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_rd_full (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_rd_full_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[2]),
    .q      (reg2hw.intr_test.rd_full.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.rd_full.qe = intr_test_qe;

  //   F[rd_lvl]: 3:3
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_rd_lvl (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_rd_lvl_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[3]),
    .q      (reg2hw.intr_test.rd_lvl.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.rd_lvl.qe = intr_test_qe;

  //   F[op_done]: 4:4
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_op_done (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_op_done_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[4]),
    .q      (reg2hw.intr_test.op_done.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.op_done.qe = intr_test_qe;

  //   F[corr_err]: 5:5
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_corr_err (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_corr_err_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[5]),
    .q      (reg2hw.intr_test.corr_err.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.corr_err.qe = intr_test_qe;


  // R[alert_test]: V(True)
  logic alert_test_qe;
  logic [4:0] alert_test_flds_we;
  assign alert_test_qe = &alert_test_flds_we;
  //   F[recov_err]: 0:0
  prim_subreg_ext #(
    .DW    (1)
  ) u_alert_test_recov_err (
    .re     (1'b0),
    .we     (alert_test_we),
    .wd     (alert_test_recov_err_wd),
    .d      ('0),
    .qre    (),
    .qe     (alert_test_flds_we[0]),
    .q      (reg2hw.alert_test.recov_err.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.alert_test.recov_err.qe = alert_test_qe;

  //   F[fatal_std_err]: 1:1
  prim_subreg_ext #(
    .DW    (1)
  ) u_alert_test_fatal_std_err (
    .re     (1'b0),
    .we     (alert_test_we),
    .wd     (alert_test_fatal_std_err_wd),
    .d      ('0),
    .qre    (),
    .qe     (alert_test_flds_we[1]),
    .q      (reg2hw.alert_test.fatal_std_err.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.alert_test.fatal_std_err.qe = alert_test_qe;

  //   F[fatal_err]: 2:2
  prim_subreg_ext #(
    .DW    (1)
  ) u_alert_test_fatal_err (
    .re     (1'b0),
    .we     (alert_test_we),
    .wd     (alert_test_fatal_err_wd),
    .d      ('0),
    .qre    (),
    .qe     (alert_test_flds_we[2]),
    .q      (reg2hw.alert_test.fatal_err.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.alert_test.fatal_err.qe = alert_test_qe;

  //   F[fatal_prim_flash_alert]: 3:3
  prim_subreg_ext #(
    .DW    (1)
  ) u_alert_test_fatal_prim_flash_alert (
    .re     (1'b0),
    .we     (alert_test_we),
    .wd     (alert_test_fatal_prim_flash_alert_wd),
    .d      ('0),
    .qre    (),
    .qe     (alert_test_flds_we[3]),
    .q      (reg2hw.alert_test.fatal_prim_flash_alert.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.alert_test.fatal_prim_flash_alert.qe = alert_test_qe;

  //   F[recov_prim_flash_alert]: 4:4
  prim_subreg_ext #(
    .DW    (1)
  ) u_alert_test_recov_prim_flash_alert (
    .re     (1'b0),
    .we     (alert_test_we),
    .wd     (alert_test_recov_prim_flash_alert_wd),
    .d      ('0),
    .qre    (),
    .qe     (alert_test_flds_we[4]),
    .q      (reg2hw.alert_test.recov_prim_flash_alert.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.alert_test.recov_prim_flash_alert.qe = alert_test_qe;


  // R[dis]: V(False)
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (4'h9)
  ) u_dis (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (dis_we),
    .wd     (dis_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.dis.q),
    .ds     (),

    // to register interface (read)
    .qs     (dis_qs)
  );


  // R[exec]: V(False)
  prim_subreg #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_exec (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (exec_we),
    .wd     (exec_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.exec.q),
    .ds     (),

    // to register interface (read)
    .qs     (exec_qs)
  );


  // R[init]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1S),
    .RESVAL  (1'h0)
  ) u_init (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (init_we),
    .wd     (init_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.init.q),
    .ds     (),

    // to register interface (read)
    .qs     (init_qs)
  );


  // R[ctrl_regwen]: V(True)
  prim_subreg_ext #(
    .DW    (1)
  ) u_ctrl_regwen (
    .re     (ctrl_regwen_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.ctrl_regwen.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (ctrl_regwen_qs)
  );


  // R[control]: V(False)
  // Create REGWEN-gated WE signal
  logic control_gated_we;
  assign control_gated_we = control_we & ctrl_regwen_qs;
  //   F[start]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_control_start (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_start_wd),

    // from internal hardware
    .de     (hw2reg.control.start.de),
    .d      (hw2reg.control.start.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.start.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_start_qs)
  );

  //   F[op]: 5:4
  prim_subreg #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_control_op (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_op_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.op.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_op_qs)
  );

  //   F[prog_sel]: 6:6
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_control_prog_sel (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_prog_sel_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.prog_sel.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_prog_sel_qs)
  );

  //   F[erase_sel]: 7:7
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_control_erase_sel (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_erase_sel_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.erase_sel.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_erase_sel_qs)
  );

  //   F[partition_sel]: 8:8
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_control_partition_sel (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_partition_sel_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.partition_sel.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_partition_sel_qs)
  );

  //   F[info_sel]: 10:9
  prim_subreg #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_control_info_sel (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_info_sel_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.info_sel.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_info_sel_qs)
  );

  //   F[num]: 27:16
  prim_subreg #(
    .DW      (12),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (12'h0)
  ) u_control_num (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (control_gated_we),
    .wd     (control_num_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.control.num.q),
    .ds     (),

    // to register interface (read)
    .qs     (control_num_qs)
  );


  // R[addr]: V(False)
  // Create REGWEN-gated WE signal
  logic addr_gated_we;
  assign addr_gated_we = addr_we & ctrl_regwen_qs;
  prim_subreg #(
    .DW      (20),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (20'h0)
  ) u_addr (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (addr_gated_we),
    .wd     (addr_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.addr.q),
    .ds     (),

    // to register interface (read)
    .qs     (addr_qs)
  );


  // R[prog_type_en]: V(False)
  // Create REGWEN-gated WE signal
  logic prog_type_en_gated_we;
  assign prog_type_en_gated_we = prog_type_en_we & ctrl_regwen_qs;
  //   F[normal]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_prog_type_en_normal (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (prog_type_en_gated_we),
    .wd     (prog_type_en_normal_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.prog_type_en.normal.q),
    .ds     (),

    // to register interface (read)
    .qs     (prog_type_en_normal_qs)
  );

  //   F[repair]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_prog_type_en_repair (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (prog_type_en_gated_we),
    .wd     (prog_type_en_repair_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.prog_type_en.repair.q),
    .ds     (),

    // to register interface (read)
    .qs     (prog_type_en_repair_qs)
  );


  // R[erase_suspend]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_erase_suspend (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (erase_suspend_we),
    .wd     (erase_suspend_wd),

    // from internal hardware
    .de     (hw2reg.erase_suspend.de),
    .d      (hw2reg.erase_suspend.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.erase_suspend.q),
    .ds     (),

    // to register interface (read)
    .qs     (erase_suspend_qs)
  );


  // Subregister 0 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_0_we),
    .wd     (region_cfg_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_0_qs)
  );


  // Subregister 1 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_1_we),
    .wd     (region_cfg_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_1_qs)
  );


  // Subregister 2 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_2_we),
    .wd     (region_cfg_regwen_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_2_qs)
  );


  // Subregister 3 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_3_we),
    .wd     (region_cfg_regwen_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_3_qs)
  );


  // Subregister 4 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_4_we),
    .wd     (region_cfg_regwen_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_4_qs)
  );


  // Subregister 5 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_5_we),
    .wd     (region_cfg_regwen_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_5_qs)
  );


  // Subregister 6 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_6_we),
    .wd     (region_cfg_regwen_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_6_qs)
  );


  // Subregister 7 of Multireg region_cfg_regwen
  // R[region_cfg_regwen_7]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_region_cfg_regwen_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (region_cfg_regwen_7_we),
    .wd     (region_cfg_regwen_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (region_cfg_regwen_7_qs)
  );


  // Subregister 0 of Multireg mp_region_cfg
  // R[mp_region_cfg_0]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_0_gated_we;
  assign mp_region_cfg_0_gated_we = mp_region_cfg_0_we & region_cfg_regwen_0_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_0_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_0_gated_we),
    .wd     (mp_region_cfg_0_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_0_he_en_0_qs)
  );


  // Subregister 1 of Multireg mp_region_cfg
  // R[mp_region_cfg_1]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_1_gated_we;
  assign mp_region_cfg_1_gated_we = mp_region_cfg_1_we & region_cfg_regwen_1_qs;
  //   F[en_1]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_en_1_qs)
  );

  //   F[rd_en_1]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_rd_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_rd_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_rd_en_1_qs)
  );

  //   F[prog_en_1]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_prog_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_prog_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_prog_en_1_qs)
  );

  //   F[erase_en_1]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_erase_en_1_qs)
  );

  //   F[scramble_en_1]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_scramble_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_scramble_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_scramble_en_1_qs)
  );

  //   F[ecc_en_1]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_ecc_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_ecc_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_ecc_en_1_qs)
  );

  //   F[he_en_1]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_1_he_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_1_gated_we),
    .wd     (mp_region_cfg_1_he_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[1].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_1_he_en_1_qs)
  );


  // Subregister 2 of Multireg mp_region_cfg
  // R[mp_region_cfg_2]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_2_gated_we;
  assign mp_region_cfg_2_gated_we = mp_region_cfg_2_we & region_cfg_regwen_2_qs;
  //   F[en_2]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_en_2_qs)
  );

  //   F[rd_en_2]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_rd_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_rd_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_rd_en_2_qs)
  );

  //   F[prog_en_2]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_prog_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_prog_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_prog_en_2_qs)
  );

  //   F[erase_en_2]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_erase_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_erase_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_erase_en_2_qs)
  );

  //   F[scramble_en_2]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_scramble_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_scramble_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_scramble_en_2_qs)
  );

  //   F[ecc_en_2]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_ecc_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_ecc_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_ecc_en_2_qs)
  );

  //   F[he_en_2]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_2_he_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_2_gated_we),
    .wd     (mp_region_cfg_2_he_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[2].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_2_he_en_2_qs)
  );


  // Subregister 3 of Multireg mp_region_cfg
  // R[mp_region_cfg_3]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_3_gated_we;
  assign mp_region_cfg_3_gated_we = mp_region_cfg_3_we & region_cfg_regwen_3_qs;
  //   F[en_3]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_en_3_qs)
  );

  //   F[rd_en_3]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_rd_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_rd_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_rd_en_3_qs)
  );

  //   F[prog_en_3]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_prog_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_prog_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_prog_en_3_qs)
  );

  //   F[erase_en_3]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_erase_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_erase_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_erase_en_3_qs)
  );

  //   F[scramble_en_3]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_scramble_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_scramble_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_scramble_en_3_qs)
  );

  //   F[ecc_en_3]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_ecc_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_ecc_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_ecc_en_3_qs)
  );

  //   F[he_en_3]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_3_he_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_3_gated_we),
    .wd     (mp_region_cfg_3_he_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[3].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_3_he_en_3_qs)
  );


  // Subregister 4 of Multireg mp_region_cfg
  // R[mp_region_cfg_4]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_4_gated_we;
  assign mp_region_cfg_4_gated_we = mp_region_cfg_4_we & region_cfg_regwen_4_qs;
  //   F[en_4]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_en_4_qs)
  );

  //   F[rd_en_4]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_rd_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_rd_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_rd_en_4_qs)
  );

  //   F[prog_en_4]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_prog_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_prog_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_prog_en_4_qs)
  );

  //   F[erase_en_4]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_erase_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_erase_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_erase_en_4_qs)
  );

  //   F[scramble_en_4]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_scramble_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_scramble_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_scramble_en_4_qs)
  );

  //   F[ecc_en_4]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_ecc_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_ecc_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_ecc_en_4_qs)
  );

  //   F[he_en_4]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_4_he_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_4_gated_we),
    .wd     (mp_region_cfg_4_he_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[4].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_4_he_en_4_qs)
  );


  // Subregister 5 of Multireg mp_region_cfg
  // R[mp_region_cfg_5]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_5_gated_we;
  assign mp_region_cfg_5_gated_we = mp_region_cfg_5_we & region_cfg_regwen_5_qs;
  //   F[en_5]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_en_5_qs)
  );

  //   F[rd_en_5]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_rd_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_rd_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_rd_en_5_qs)
  );

  //   F[prog_en_5]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_prog_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_prog_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_prog_en_5_qs)
  );

  //   F[erase_en_5]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_erase_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_erase_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_erase_en_5_qs)
  );

  //   F[scramble_en_5]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_scramble_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_scramble_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_scramble_en_5_qs)
  );

  //   F[ecc_en_5]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_ecc_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_ecc_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_ecc_en_5_qs)
  );

  //   F[he_en_5]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_5_he_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_5_gated_we),
    .wd     (mp_region_cfg_5_he_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[5].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_5_he_en_5_qs)
  );


  // Subregister 6 of Multireg mp_region_cfg
  // R[mp_region_cfg_6]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_6_gated_we;
  assign mp_region_cfg_6_gated_we = mp_region_cfg_6_we & region_cfg_regwen_6_qs;
  //   F[en_6]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_en_6_qs)
  );

  //   F[rd_en_6]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_rd_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_rd_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_rd_en_6_qs)
  );

  //   F[prog_en_6]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_prog_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_prog_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_prog_en_6_qs)
  );

  //   F[erase_en_6]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_erase_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_erase_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_erase_en_6_qs)
  );

  //   F[scramble_en_6]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_scramble_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_scramble_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_scramble_en_6_qs)
  );

  //   F[ecc_en_6]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_ecc_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_ecc_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_ecc_en_6_qs)
  );

  //   F[he_en_6]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_6_he_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_6_gated_we),
    .wd     (mp_region_cfg_6_he_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[6].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_6_he_en_6_qs)
  );


  // Subregister 7 of Multireg mp_region_cfg
  // R[mp_region_cfg_7]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_cfg_7_gated_we;
  assign mp_region_cfg_7_gated_we = mp_region_cfg_7_we & region_cfg_regwen_7_qs;
  //   F[en_7]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_en_7_qs)
  );

  //   F[rd_en_7]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_rd_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_rd_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_rd_en_7_qs)
  );

  //   F[prog_en_7]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_prog_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_prog_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_prog_en_7_qs)
  );

  //   F[erase_en_7]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_erase_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_erase_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_erase_en_7_qs)
  );

  //   F[scramble_en_7]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_scramble_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_scramble_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_scramble_en_7_qs)
  );

  //   F[ecc_en_7]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_ecc_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_ecc_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_ecc_en_7_qs)
  );

  //   F[he_en_7]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_mp_region_cfg_7_he_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_cfg_7_gated_we),
    .wd     (mp_region_cfg_7_he_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region_cfg[7].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_cfg_7_he_en_7_qs)
  );


  // Subregister 0 of Multireg mp_region
  // R[mp_region_0]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_0_gated_we;
  assign mp_region_0_gated_we = mp_region_0_we & region_cfg_regwen_0_qs;
  //   F[base_0]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_0_base_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_0_gated_we),
    .wd     (mp_region_0_base_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[0].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_0_base_0_qs)
  );

  //   F[size_0]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_0_size_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_0_gated_we),
    .wd     (mp_region_0_size_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[0].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_0_size_0_qs)
  );


  // Subregister 1 of Multireg mp_region
  // R[mp_region_1]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_1_gated_we;
  assign mp_region_1_gated_we = mp_region_1_we & region_cfg_regwen_1_qs;
  //   F[base_1]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_1_base_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_1_gated_we),
    .wd     (mp_region_1_base_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[1].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_1_base_1_qs)
  );

  //   F[size_1]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_1_size_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_1_gated_we),
    .wd     (mp_region_1_size_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[1].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_1_size_1_qs)
  );


  // Subregister 2 of Multireg mp_region
  // R[mp_region_2]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_2_gated_we;
  assign mp_region_2_gated_we = mp_region_2_we & region_cfg_regwen_2_qs;
  //   F[base_2]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_2_base_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_2_gated_we),
    .wd     (mp_region_2_base_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[2].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_2_base_2_qs)
  );

  //   F[size_2]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_2_size_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_2_gated_we),
    .wd     (mp_region_2_size_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[2].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_2_size_2_qs)
  );


  // Subregister 3 of Multireg mp_region
  // R[mp_region_3]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_3_gated_we;
  assign mp_region_3_gated_we = mp_region_3_we & region_cfg_regwen_3_qs;
  //   F[base_3]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_3_base_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_3_gated_we),
    .wd     (mp_region_3_base_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[3].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_3_base_3_qs)
  );

  //   F[size_3]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_3_size_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_3_gated_we),
    .wd     (mp_region_3_size_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[3].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_3_size_3_qs)
  );


  // Subregister 4 of Multireg mp_region
  // R[mp_region_4]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_4_gated_we;
  assign mp_region_4_gated_we = mp_region_4_we & region_cfg_regwen_4_qs;
  //   F[base_4]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_4_base_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_4_gated_we),
    .wd     (mp_region_4_base_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[4].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_4_base_4_qs)
  );

  //   F[size_4]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_4_size_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_4_gated_we),
    .wd     (mp_region_4_size_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[4].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_4_size_4_qs)
  );


  // Subregister 5 of Multireg mp_region
  // R[mp_region_5]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_5_gated_we;
  assign mp_region_5_gated_we = mp_region_5_we & region_cfg_regwen_5_qs;
  //   F[base_5]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_5_base_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_5_gated_we),
    .wd     (mp_region_5_base_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[5].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_5_base_5_qs)
  );

  //   F[size_5]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_5_size_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_5_gated_we),
    .wd     (mp_region_5_size_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[5].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_5_size_5_qs)
  );


  // Subregister 6 of Multireg mp_region
  // R[mp_region_6]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_6_gated_we;
  assign mp_region_6_gated_we = mp_region_6_we & region_cfg_regwen_6_qs;
  //   F[base_6]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_6_base_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_6_gated_we),
    .wd     (mp_region_6_base_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[6].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_6_base_6_qs)
  );

  //   F[size_6]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_6_size_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_6_gated_we),
    .wd     (mp_region_6_size_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[6].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_6_size_6_qs)
  );


  // Subregister 7 of Multireg mp_region
  // R[mp_region_7]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_region_7_gated_we;
  assign mp_region_7_gated_we = mp_region_7_we & region_cfg_regwen_7_qs;
  //   F[base_7]: 8:0
  prim_subreg #(
    .DW      (9),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (9'h0)
  ) u_mp_region_7_base_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_7_gated_we),
    .wd     (mp_region_7_base_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[7].base.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_7_base_7_qs)
  );

  //   F[size_7]: 18:9
  prim_subreg #(
    .DW      (10),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (10'h0)
  ) u_mp_region_7_size_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (mp_region_7_gated_we),
    .wd     (mp_region_7_size_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_region[7].size.q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_region_7_size_7_qs)
  );


  // R[default_region]: V(False)
  //   F[rd_en]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_rd_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_rd_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_rd_en_qs)
  );

  //   F[prog_en]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_prog_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_prog_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_prog_en_qs)
  );

  //   F[erase_en]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_erase_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_erase_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_erase_en_qs)
  );

  //   F[scramble_en]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_scramble_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_scramble_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_scramble_en_qs)
  );

  //   F[ecc_en]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_ecc_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_ecc_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_ecc_en_qs)
  );

  //   F[he_en]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_default_region_he_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (default_region_we),
    .wd     (default_region_he_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.default_region.he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (default_region_he_en_qs)
  );


  // Subregister 0 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_0_we),
    .wd     (bank0_info0_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_0_qs)
  );


  // Subregister 1 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_1_we),
    .wd     (bank0_info0_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_1_qs)
  );


  // Subregister 2 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_2_we),
    .wd     (bank0_info0_regwen_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_2_qs)
  );


  // Subregister 3 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_3_we),
    .wd     (bank0_info0_regwen_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_3_qs)
  );


  // Subregister 4 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_4_we),
    .wd     (bank0_info0_regwen_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_4_qs)
  );


  // Subregister 5 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_5_we),
    .wd     (bank0_info0_regwen_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_5_qs)
  );


  // Subregister 6 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_6_we),
    .wd     (bank0_info0_regwen_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_6_qs)
  );


  // Subregister 7 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_7]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_7_we),
    .wd     (bank0_info0_regwen_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_7_qs)
  );


  // Subregister 8 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_8]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_8_we),
    .wd     (bank0_info0_regwen_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_8_qs)
  );


  // Subregister 9 of Multireg bank0_info0_regwen
  // R[bank0_info0_regwen_9]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info0_regwen_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_regwen_9_we),
    .wd     (bank0_info0_regwen_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_regwen_9_qs)
  );


  // Subregister 0 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_0]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_0_gated_we;
  assign bank0_info0_page_cfg_0_gated_we = bank0_info0_page_cfg_0_we & bank0_info0_regwen_0_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_0_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_0_gated_we),
    .wd     (bank0_info0_page_cfg_0_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_0_he_en_0_qs)
  );


  // Subregister 1 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_1]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_1_gated_we;
  assign bank0_info0_page_cfg_1_gated_we = bank0_info0_page_cfg_1_we & bank0_info0_regwen_1_qs;
  //   F[en_1]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_en_1_qs)
  );

  //   F[rd_en_1]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_rd_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_rd_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_rd_en_1_qs)
  );

  //   F[prog_en_1]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_prog_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_prog_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_prog_en_1_qs)
  );

  //   F[erase_en_1]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_erase_en_1_qs)
  );

  //   F[scramble_en_1]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_scramble_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_scramble_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_scramble_en_1_qs)
  );

  //   F[ecc_en_1]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_ecc_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_ecc_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_ecc_en_1_qs)
  );

  //   F[he_en_1]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_1_he_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_1_gated_we),
    .wd     (bank0_info0_page_cfg_1_he_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[1].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_1_he_en_1_qs)
  );


  // Subregister 2 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_2]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_2_gated_we;
  assign bank0_info0_page_cfg_2_gated_we = bank0_info0_page_cfg_2_we & bank0_info0_regwen_2_qs;
  //   F[en_2]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_en_2_qs)
  );

  //   F[rd_en_2]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_rd_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_rd_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_rd_en_2_qs)
  );

  //   F[prog_en_2]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_prog_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_prog_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_prog_en_2_qs)
  );

  //   F[erase_en_2]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_erase_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_erase_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_erase_en_2_qs)
  );

  //   F[scramble_en_2]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_scramble_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_scramble_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_scramble_en_2_qs)
  );

  //   F[ecc_en_2]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_ecc_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_ecc_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_ecc_en_2_qs)
  );

  //   F[he_en_2]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_2_he_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_2_gated_we),
    .wd     (bank0_info0_page_cfg_2_he_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[2].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_2_he_en_2_qs)
  );


  // Subregister 3 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_3]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_3_gated_we;
  assign bank0_info0_page_cfg_3_gated_we = bank0_info0_page_cfg_3_we & bank0_info0_regwen_3_qs;
  //   F[en_3]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_en_3_qs)
  );

  //   F[rd_en_3]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_rd_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_rd_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_rd_en_3_qs)
  );

  //   F[prog_en_3]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_prog_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_prog_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_prog_en_3_qs)
  );

  //   F[erase_en_3]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_erase_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_erase_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_erase_en_3_qs)
  );

  //   F[scramble_en_3]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_scramble_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_scramble_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_scramble_en_3_qs)
  );

  //   F[ecc_en_3]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_ecc_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_ecc_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_ecc_en_3_qs)
  );

  //   F[he_en_3]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_3_he_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_3_gated_we),
    .wd     (bank0_info0_page_cfg_3_he_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[3].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_3_he_en_3_qs)
  );


  // Subregister 4 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_4]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_4_gated_we;
  assign bank0_info0_page_cfg_4_gated_we = bank0_info0_page_cfg_4_we & bank0_info0_regwen_4_qs;
  //   F[en_4]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_en_4_qs)
  );

  //   F[rd_en_4]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_rd_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_rd_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_rd_en_4_qs)
  );

  //   F[prog_en_4]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_prog_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_prog_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_prog_en_4_qs)
  );

  //   F[erase_en_4]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_erase_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_erase_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_erase_en_4_qs)
  );

  //   F[scramble_en_4]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_scramble_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_scramble_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_scramble_en_4_qs)
  );

  //   F[ecc_en_4]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_ecc_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_ecc_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_ecc_en_4_qs)
  );

  //   F[he_en_4]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_4_he_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_4_gated_we),
    .wd     (bank0_info0_page_cfg_4_he_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[4].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_4_he_en_4_qs)
  );


  // Subregister 5 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_5]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_5_gated_we;
  assign bank0_info0_page_cfg_5_gated_we = bank0_info0_page_cfg_5_we & bank0_info0_regwen_5_qs;
  //   F[en_5]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_en_5_qs)
  );

  //   F[rd_en_5]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_rd_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_rd_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_rd_en_5_qs)
  );

  //   F[prog_en_5]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_prog_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_prog_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_prog_en_5_qs)
  );

  //   F[erase_en_5]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_erase_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_erase_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_erase_en_5_qs)
  );

  //   F[scramble_en_5]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_scramble_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_scramble_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_scramble_en_5_qs)
  );

  //   F[ecc_en_5]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_ecc_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_ecc_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_ecc_en_5_qs)
  );

  //   F[he_en_5]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_5_he_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_5_gated_we),
    .wd     (bank0_info0_page_cfg_5_he_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[5].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_5_he_en_5_qs)
  );


  // Subregister 6 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_6]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_6_gated_we;
  assign bank0_info0_page_cfg_6_gated_we = bank0_info0_page_cfg_6_we & bank0_info0_regwen_6_qs;
  //   F[en_6]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_en_6_qs)
  );

  //   F[rd_en_6]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_rd_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_rd_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_rd_en_6_qs)
  );

  //   F[prog_en_6]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_prog_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_prog_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_prog_en_6_qs)
  );

  //   F[erase_en_6]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_erase_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_erase_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_erase_en_6_qs)
  );

  //   F[scramble_en_6]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_scramble_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_scramble_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_scramble_en_6_qs)
  );

  //   F[ecc_en_6]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_ecc_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_ecc_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_ecc_en_6_qs)
  );

  //   F[he_en_6]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_6_he_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_6_gated_we),
    .wd     (bank0_info0_page_cfg_6_he_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[6].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_6_he_en_6_qs)
  );


  // Subregister 7 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_7]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_7_gated_we;
  assign bank0_info0_page_cfg_7_gated_we = bank0_info0_page_cfg_7_we & bank0_info0_regwen_7_qs;
  //   F[en_7]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_en_7_qs)
  );

  //   F[rd_en_7]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_rd_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_rd_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_rd_en_7_qs)
  );

  //   F[prog_en_7]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_prog_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_prog_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_prog_en_7_qs)
  );

  //   F[erase_en_7]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_erase_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_erase_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_erase_en_7_qs)
  );

  //   F[scramble_en_7]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_scramble_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_scramble_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_scramble_en_7_qs)
  );

  //   F[ecc_en_7]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_ecc_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_ecc_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_ecc_en_7_qs)
  );

  //   F[he_en_7]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_7_he_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_7_gated_we),
    .wd     (bank0_info0_page_cfg_7_he_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[7].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_7_he_en_7_qs)
  );


  // Subregister 8 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_8]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_8_gated_we;
  assign bank0_info0_page_cfg_8_gated_we = bank0_info0_page_cfg_8_we & bank0_info0_regwen_8_qs;
  //   F[en_8]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_en_8_qs)
  );

  //   F[rd_en_8]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_rd_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_rd_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_rd_en_8_qs)
  );

  //   F[prog_en_8]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_prog_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_prog_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_prog_en_8_qs)
  );

  //   F[erase_en_8]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_erase_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_erase_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_erase_en_8_qs)
  );

  //   F[scramble_en_8]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_scramble_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_scramble_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_scramble_en_8_qs)
  );

  //   F[ecc_en_8]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_ecc_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_ecc_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_ecc_en_8_qs)
  );

  //   F[he_en_8]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_8_he_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_8_gated_we),
    .wd     (bank0_info0_page_cfg_8_he_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[8].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_8_he_en_8_qs)
  );


  // Subregister 9 of Multireg bank0_info0_page_cfg
  // R[bank0_info0_page_cfg_9]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info0_page_cfg_9_gated_we;
  assign bank0_info0_page_cfg_9_gated_we = bank0_info0_page_cfg_9_we & bank0_info0_regwen_9_qs;
  //   F[en_9]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_en_9_qs)
  );

  //   F[rd_en_9]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_rd_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_rd_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_rd_en_9_qs)
  );

  //   F[prog_en_9]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_prog_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_prog_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_prog_en_9_qs)
  );

  //   F[erase_en_9]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_erase_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_erase_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_erase_en_9_qs)
  );

  //   F[scramble_en_9]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_scramble_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_scramble_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_scramble_en_9_qs)
  );

  //   F[ecc_en_9]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_ecc_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_ecc_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_ecc_en_9_qs)
  );

  //   F[he_en_9]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info0_page_cfg_9_he_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info0_page_cfg_9_gated_we),
    .wd     (bank0_info0_page_cfg_9_he_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info0_page_cfg[9].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info0_page_cfg_9_he_en_9_qs)
  );


  // Subregister 0 of Multireg bank0_info1_regwen
  // R[bank0_info1_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info1_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_regwen_we),
    .wd     (bank0_info1_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_regwen_qs)
  );


  // Subregister 0 of Multireg bank0_info1_page_cfg
  // R[bank0_info1_page_cfg]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info1_page_cfg_gated_we;
  assign bank0_info1_page_cfg_gated_we = bank0_info1_page_cfg_we & bank0_info1_regwen_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info1_page_cfg_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info1_page_cfg_gated_we),
    .wd     (bank0_info1_page_cfg_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info1_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info1_page_cfg_he_en_0_qs)
  );


  // Subregister 0 of Multireg bank0_info2_regwen
  // R[bank0_info2_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info2_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_regwen_0_we),
    .wd     (bank0_info2_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_regwen_0_qs)
  );


  // Subregister 1 of Multireg bank0_info2_regwen
  // R[bank0_info2_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank0_info2_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_regwen_1_we),
    .wd     (bank0_info2_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_regwen_1_qs)
  );


  // Subregister 0 of Multireg bank0_info2_page_cfg
  // R[bank0_info2_page_cfg_0]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info2_page_cfg_0_gated_we;
  assign bank0_info2_page_cfg_0_gated_we = bank0_info2_page_cfg_0_we & bank0_info2_regwen_0_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_0_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_0_gated_we),
    .wd     (bank0_info2_page_cfg_0_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_0_he_en_0_qs)
  );


  // Subregister 1 of Multireg bank0_info2_page_cfg
  // R[bank0_info2_page_cfg_1]: V(False)
  // Create REGWEN-gated WE signal
  logic bank0_info2_page_cfg_1_gated_we;
  assign bank0_info2_page_cfg_1_gated_we = bank0_info2_page_cfg_1_we & bank0_info2_regwen_1_qs;
  //   F[en_1]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_en_1_qs)
  );

  //   F[rd_en_1]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_rd_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_rd_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_rd_en_1_qs)
  );

  //   F[prog_en_1]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_prog_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_prog_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_prog_en_1_qs)
  );

  //   F[erase_en_1]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_erase_en_1_qs)
  );

  //   F[scramble_en_1]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_scramble_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_scramble_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_scramble_en_1_qs)
  );

  //   F[ecc_en_1]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_ecc_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_ecc_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_ecc_en_1_qs)
  );

  //   F[he_en_1]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank0_info2_page_cfg_1_he_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank0_info2_page_cfg_1_gated_we),
    .wd     (bank0_info2_page_cfg_1_he_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank0_info2_page_cfg[1].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank0_info2_page_cfg_1_he_en_1_qs)
  );


  // Subregister 0 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_0_we),
    .wd     (bank1_info0_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_0_qs)
  );


  // Subregister 1 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_1_we),
    .wd     (bank1_info0_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_1_qs)
  );


  // Subregister 2 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_2_we),
    .wd     (bank1_info0_regwen_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_2_qs)
  );


  // Subregister 3 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_3_we),
    .wd     (bank1_info0_regwen_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_3_qs)
  );


  // Subregister 4 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_4_we),
    .wd     (bank1_info0_regwen_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_4_qs)
  );


  // Subregister 5 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_5_we),
    .wd     (bank1_info0_regwen_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_5_qs)
  );


  // Subregister 6 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_6_we),
    .wd     (bank1_info0_regwen_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_6_qs)
  );


  // Subregister 7 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_7]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_7_we),
    .wd     (bank1_info0_regwen_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_7_qs)
  );


  // Subregister 8 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_8]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_8_we),
    .wd     (bank1_info0_regwen_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_8_qs)
  );


  // Subregister 9 of Multireg bank1_info0_regwen
  // R[bank1_info0_regwen_9]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info0_regwen_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_regwen_9_we),
    .wd     (bank1_info0_regwen_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_regwen_9_qs)
  );


  // Subregister 0 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_0]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_0_gated_we;
  assign bank1_info0_page_cfg_0_gated_we = bank1_info0_page_cfg_0_we & bank1_info0_regwen_0_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_0_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_0_gated_we),
    .wd     (bank1_info0_page_cfg_0_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_0_he_en_0_qs)
  );


  // Subregister 1 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_1]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_1_gated_we;
  assign bank1_info0_page_cfg_1_gated_we = bank1_info0_page_cfg_1_we & bank1_info0_regwen_1_qs;
  //   F[en_1]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_en_1_qs)
  );

  //   F[rd_en_1]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_rd_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_rd_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_rd_en_1_qs)
  );

  //   F[prog_en_1]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_prog_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_prog_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_prog_en_1_qs)
  );

  //   F[erase_en_1]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_erase_en_1_qs)
  );

  //   F[scramble_en_1]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_scramble_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_scramble_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_scramble_en_1_qs)
  );

  //   F[ecc_en_1]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_ecc_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_ecc_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_ecc_en_1_qs)
  );

  //   F[he_en_1]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_1_he_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_1_gated_we),
    .wd     (bank1_info0_page_cfg_1_he_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[1].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_1_he_en_1_qs)
  );


  // Subregister 2 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_2]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_2_gated_we;
  assign bank1_info0_page_cfg_2_gated_we = bank1_info0_page_cfg_2_we & bank1_info0_regwen_2_qs;
  //   F[en_2]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_en_2_qs)
  );

  //   F[rd_en_2]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_rd_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_rd_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_rd_en_2_qs)
  );

  //   F[prog_en_2]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_prog_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_prog_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_prog_en_2_qs)
  );

  //   F[erase_en_2]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_erase_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_erase_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_erase_en_2_qs)
  );

  //   F[scramble_en_2]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_scramble_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_scramble_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_scramble_en_2_qs)
  );

  //   F[ecc_en_2]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_ecc_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_ecc_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_ecc_en_2_qs)
  );

  //   F[he_en_2]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_2_he_en_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_2_gated_we),
    .wd     (bank1_info0_page_cfg_2_he_en_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[2].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_2_he_en_2_qs)
  );


  // Subregister 3 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_3]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_3_gated_we;
  assign bank1_info0_page_cfg_3_gated_we = bank1_info0_page_cfg_3_we & bank1_info0_regwen_3_qs;
  //   F[en_3]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_en_3_qs)
  );

  //   F[rd_en_3]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_rd_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_rd_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_rd_en_3_qs)
  );

  //   F[prog_en_3]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_prog_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_prog_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_prog_en_3_qs)
  );

  //   F[erase_en_3]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_erase_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_erase_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_erase_en_3_qs)
  );

  //   F[scramble_en_3]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_scramble_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_scramble_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_scramble_en_3_qs)
  );

  //   F[ecc_en_3]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_ecc_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_ecc_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_ecc_en_3_qs)
  );

  //   F[he_en_3]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_3_he_en_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_3_gated_we),
    .wd     (bank1_info0_page_cfg_3_he_en_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[3].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_3_he_en_3_qs)
  );


  // Subregister 4 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_4]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_4_gated_we;
  assign bank1_info0_page_cfg_4_gated_we = bank1_info0_page_cfg_4_we & bank1_info0_regwen_4_qs;
  //   F[en_4]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_en_4_qs)
  );

  //   F[rd_en_4]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_rd_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_rd_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_rd_en_4_qs)
  );

  //   F[prog_en_4]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_prog_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_prog_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_prog_en_4_qs)
  );

  //   F[erase_en_4]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_erase_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_erase_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_erase_en_4_qs)
  );

  //   F[scramble_en_4]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_scramble_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_scramble_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_scramble_en_4_qs)
  );

  //   F[ecc_en_4]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_ecc_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_ecc_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_ecc_en_4_qs)
  );

  //   F[he_en_4]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_4_he_en_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_4_gated_we),
    .wd     (bank1_info0_page_cfg_4_he_en_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[4].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_4_he_en_4_qs)
  );


  // Subregister 5 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_5]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_5_gated_we;
  assign bank1_info0_page_cfg_5_gated_we = bank1_info0_page_cfg_5_we & bank1_info0_regwen_5_qs;
  //   F[en_5]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_en_5_qs)
  );

  //   F[rd_en_5]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_rd_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_rd_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_rd_en_5_qs)
  );

  //   F[prog_en_5]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_prog_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_prog_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_prog_en_5_qs)
  );

  //   F[erase_en_5]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_erase_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_erase_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_erase_en_5_qs)
  );

  //   F[scramble_en_5]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_scramble_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_scramble_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_scramble_en_5_qs)
  );

  //   F[ecc_en_5]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_ecc_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_ecc_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_ecc_en_5_qs)
  );

  //   F[he_en_5]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_5_he_en_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_5_gated_we),
    .wd     (bank1_info0_page_cfg_5_he_en_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[5].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_5_he_en_5_qs)
  );


  // Subregister 6 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_6]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_6_gated_we;
  assign bank1_info0_page_cfg_6_gated_we = bank1_info0_page_cfg_6_we & bank1_info0_regwen_6_qs;
  //   F[en_6]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_en_6_qs)
  );

  //   F[rd_en_6]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_rd_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_rd_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_rd_en_6_qs)
  );

  //   F[prog_en_6]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_prog_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_prog_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_prog_en_6_qs)
  );

  //   F[erase_en_6]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_erase_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_erase_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_erase_en_6_qs)
  );

  //   F[scramble_en_6]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_scramble_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_scramble_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_scramble_en_6_qs)
  );

  //   F[ecc_en_6]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_ecc_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_ecc_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_ecc_en_6_qs)
  );

  //   F[he_en_6]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_6_he_en_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_6_gated_we),
    .wd     (bank1_info0_page_cfg_6_he_en_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[6].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_6_he_en_6_qs)
  );


  // Subregister 7 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_7]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_7_gated_we;
  assign bank1_info0_page_cfg_7_gated_we = bank1_info0_page_cfg_7_we & bank1_info0_regwen_7_qs;
  //   F[en_7]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_en_7_qs)
  );

  //   F[rd_en_7]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_rd_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_rd_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_rd_en_7_qs)
  );

  //   F[prog_en_7]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_prog_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_prog_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_prog_en_7_qs)
  );

  //   F[erase_en_7]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_erase_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_erase_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_erase_en_7_qs)
  );

  //   F[scramble_en_7]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_scramble_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_scramble_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_scramble_en_7_qs)
  );

  //   F[ecc_en_7]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_ecc_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_ecc_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_ecc_en_7_qs)
  );

  //   F[he_en_7]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_7_he_en_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_7_gated_we),
    .wd     (bank1_info0_page_cfg_7_he_en_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[7].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_7_he_en_7_qs)
  );


  // Subregister 8 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_8]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_8_gated_we;
  assign bank1_info0_page_cfg_8_gated_we = bank1_info0_page_cfg_8_we & bank1_info0_regwen_8_qs;
  //   F[en_8]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_en_8_qs)
  );

  //   F[rd_en_8]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_rd_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_rd_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_rd_en_8_qs)
  );

  //   F[prog_en_8]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_prog_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_prog_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_prog_en_8_qs)
  );

  //   F[erase_en_8]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_erase_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_erase_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_erase_en_8_qs)
  );

  //   F[scramble_en_8]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_scramble_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_scramble_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_scramble_en_8_qs)
  );

  //   F[ecc_en_8]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_ecc_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_ecc_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_ecc_en_8_qs)
  );

  //   F[he_en_8]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_8_he_en_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_8_gated_we),
    .wd     (bank1_info0_page_cfg_8_he_en_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[8].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_8_he_en_8_qs)
  );


  // Subregister 9 of Multireg bank1_info0_page_cfg
  // R[bank1_info0_page_cfg_9]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info0_page_cfg_9_gated_we;
  assign bank1_info0_page_cfg_9_gated_we = bank1_info0_page_cfg_9_we & bank1_info0_regwen_9_qs;
  //   F[en_9]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_en_9_qs)
  );

  //   F[rd_en_9]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_rd_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_rd_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_rd_en_9_qs)
  );

  //   F[prog_en_9]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_prog_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_prog_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_prog_en_9_qs)
  );

  //   F[erase_en_9]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_erase_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_erase_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_erase_en_9_qs)
  );

  //   F[scramble_en_9]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_scramble_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_scramble_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_scramble_en_9_qs)
  );

  //   F[ecc_en_9]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_ecc_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_ecc_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_ecc_en_9_qs)
  );

  //   F[he_en_9]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info0_page_cfg_9_he_en_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info0_page_cfg_9_gated_we),
    .wd     (bank1_info0_page_cfg_9_he_en_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info0_page_cfg[9].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info0_page_cfg_9_he_en_9_qs)
  );


  // Subregister 0 of Multireg bank1_info1_regwen
  // R[bank1_info1_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info1_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_regwen_we),
    .wd     (bank1_info1_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_regwen_qs)
  );


  // Subregister 0 of Multireg bank1_info1_page_cfg
  // R[bank1_info1_page_cfg]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info1_page_cfg_gated_we;
  assign bank1_info1_page_cfg_gated_we = bank1_info1_page_cfg_we & bank1_info1_regwen_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info1_page_cfg_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info1_page_cfg_gated_we),
    .wd     (bank1_info1_page_cfg_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info1_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info1_page_cfg_he_en_0_qs)
  );


  // Subregister 0 of Multireg bank1_info2_regwen
  // R[bank1_info2_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info2_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_regwen_0_we),
    .wd     (bank1_info2_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_regwen_0_qs)
  );


  // Subregister 1 of Multireg bank1_info2_regwen
  // R[bank1_info2_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank1_info2_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_regwen_1_we),
    .wd     (bank1_info2_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_regwen_1_qs)
  );


  // Subregister 0 of Multireg bank1_info2_page_cfg
  // R[bank1_info2_page_cfg_0]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info2_page_cfg_0_gated_we;
  assign bank1_info2_page_cfg_0_gated_we = bank1_info2_page_cfg_0_we & bank1_info2_regwen_0_qs;
  //   F[en_0]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_en_0_qs)
  );

  //   F[rd_en_0]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_rd_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_rd_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_rd_en_0_qs)
  );

  //   F[prog_en_0]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_prog_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_prog_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_prog_en_0_qs)
  );

  //   F[erase_en_0]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_erase_en_0_qs)
  );

  //   F[scramble_en_0]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_scramble_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_scramble_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_scramble_en_0_qs)
  );

  //   F[ecc_en_0]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_ecc_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_ecc_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_ecc_en_0_qs)
  );

  //   F[he_en_0]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_0_he_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_0_gated_we),
    .wd     (bank1_info2_page_cfg_0_he_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[0].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_0_he_en_0_qs)
  );


  // Subregister 1 of Multireg bank1_info2_page_cfg
  // R[bank1_info2_page_cfg_1]: V(False)
  // Create REGWEN-gated WE signal
  logic bank1_info2_page_cfg_1_gated_we;
  assign bank1_info2_page_cfg_1_gated_we = bank1_info2_page_cfg_1_we & bank1_info2_regwen_1_qs;
  //   F[en_1]: 3:0
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_en_1_qs)
  );

  //   F[rd_en_1]: 7:4
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_rd_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_rd_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].rd_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_rd_en_1_qs)
  );

  //   F[prog_en_1]: 11:8
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_prog_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_prog_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].prog_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_prog_en_1_qs)
  );

  //   F[erase_en_1]: 15:12
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].erase_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_erase_en_1_qs)
  );

  //   F[scramble_en_1]: 19:16
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_scramble_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_scramble_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].scramble_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_scramble_en_1_qs)
  );

  //   F[ecc_en_1]: 23:20
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_ecc_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_ecc_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].ecc_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_ecc_en_1_qs)
  );

  //   F[he_en_1]: 27:24
  prim_subreg #(
    .DW      (4),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (4'h9)
  ) u_bank1_info2_page_cfg_1_he_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank1_info2_page_cfg_1_gated_we),
    .wd     (bank1_info2_page_cfg_1_he_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.bank1_info2_page_cfg[1].he_en.q),
    .ds     (),

    // to register interface (read)
    .qs     (bank1_info2_page_cfg_1_he_en_1_qs)
  );


  // R[bank_cfg_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_bank_cfg_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (bank_cfg_regwen_we),
    .wd     (bank_cfg_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (bank_cfg_regwen_qs)
  );


  // Subregister 0 of Multireg mp_bank_cfg_shadowed
  // R[mp_bank_cfg_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic mp_bank_cfg_shadowed_gated_we;
  assign mp_bank_cfg_shadowed_gated_we = mp_bank_cfg_shadowed_we & bank_cfg_regwen_qs;
  //   F[erase_en_0]: 0:0
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_mp_bank_cfg_shadowed_erase_en_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (mp_bank_cfg_shadowed_re),
    .we     (mp_bank_cfg_shadowed_gated_we),
    .wd     (mp_bank_cfg_shadowed_erase_en_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_bank_cfg_shadowed[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_bank_cfg_shadowed_erase_en_0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (mp_bank_cfg_shadowed_erase_en_0_update_err),
    .err_storage (mp_bank_cfg_shadowed_erase_en_0_storage_err)
  );

  //   F[erase_en_1]: 1:1
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_mp_bank_cfg_shadowed_erase_en_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (mp_bank_cfg_shadowed_re),
    .we     (mp_bank_cfg_shadowed_gated_we),
    .wd     (mp_bank_cfg_shadowed_erase_en_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.mp_bank_cfg_shadowed[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (mp_bank_cfg_shadowed_erase_en_1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (mp_bank_cfg_shadowed_erase_en_1_update_err),
    .err_storage (mp_bank_cfg_shadowed_erase_en_1_storage_err)
  );


  // R[op_status]: V(False)
  //   F[done]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_op_status_done (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (op_status_we),
    .wd     (op_status_done_wd),

    // from internal hardware
    .de     (hw2reg.op_status.done.de),
    .d      (hw2reg.op_status.done.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (op_status_done_qs)
  );

  //   F[err]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_op_status_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (op_status_we),
    .wd     (op_status_err_wd),

    // from internal hardware
    .de     (hw2reg.op_status.err.de),
    .d      (hw2reg.op_status.err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (op_status_err_qs)
  );


  // R[status]: V(False)
  //   F[rd_full]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_status_rd_full (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.status.rd_full.de),
    .d      (hw2reg.status.rd_full.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (status_rd_full_qs)
  );

  //   F[rd_empty]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h1)
  ) u_status_rd_empty (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.status.rd_empty.de),
    .d      (hw2reg.status.rd_empty.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (status_rd_empty_qs)
  );

  //   F[prog_full]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_status_prog_full (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.status.prog_full.de),
    .d      (hw2reg.status.prog_full.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (status_prog_full_qs)
  );

  //   F[prog_empty]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h1)
  ) u_status_prog_empty (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.status.prog_empty.de),
    .d      (hw2reg.status.prog_empty.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (status_prog_empty_qs)
  );

  //   F[init_wip]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_status_init_wip (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.status.init_wip.de),
    .d      (hw2reg.status.init_wip.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (status_init_wip_qs)
  );


  // R[debug_state]: V(True)
  prim_subreg_ext #(
    .DW    (11)
  ) u_debug_state (
    .re     (debug_state_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.debug_state.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (debug_state_qs)
  );


  // R[err_code]: V(False)
  //   F[op_err]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_op_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_op_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.op_err.de),
    .d      (hw2reg.err_code.op_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_op_err_qs)
  );

  //   F[mp_err]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_mp_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_mp_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.mp_err.de),
    .d      (hw2reg.err_code.mp_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_mp_err_qs)
  );

  //   F[rd_err]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_rd_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_rd_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.rd_err.de),
    .d      (hw2reg.err_code.rd_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_rd_err_qs)
  );

  //   F[prog_err]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_prog_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_prog_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.prog_err.de),
    .d      (hw2reg.err_code.prog_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_prog_err_qs)
  );

  //   F[prog_win_err]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_prog_win_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_prog_win_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.prog_win_err.de),
    .d      (hw2reg.err_code.prog_win_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_prog_win_err_qs)
  );

  //   F[prog_type_err]: 5:5
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_prog_type_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_prog_type_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.prog_type_err.de),
    .d      (hw2reg.err_code.prog_type_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_prog_type_err_qs)
  );

  //   F[update_err]: 6:6
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_update_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_update_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.update_err.de),
    .d      (hw2reg.err_code.update_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_update_err_qs)
  );

  //   F[macro_err]: 7:7
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_err_code_macro_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (err_code_we),
    .wd     (err_code_macro_err_wd),

    // from internal hardware
    .de     (hw2reg.err_code.macro_err.de),
    .d      (hw2reg.err_code.macro_err.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_code_macro_err_qs)
  );


  // R[std_fault_status]: V(False)
  //   F[reg_intg_err]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_reg_intg_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.reg_intg_err.de),
    .d      (hw2reg.std_fault_status.reg_intg_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.reg_intg_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_reg_intg_err_qs)
  );

  //   F[prog_intg_err]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_prog_intg_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.prog_intg_err.de),
    .d      (hw2reg.std_fault_status.prog_intg_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.prog_intg_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_prog_intg_err_qs)
  );

  //   F[lcmgr_err]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_lcmgr_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.lcmgr_err.de),
    .d      (hw2reg.std_fault_status.lcmgr_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.lcmgr_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_lcmgr_err_qs)
  );

  //   F[lcmgr_intg_err]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_lcmgr_intg_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.lcmgr_intg_err.de),
    .d      (hw2reg.std_fault_status.lcmgr_intg_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.lcmgr_intg_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_lcmgr_intg_err_qs)
  );

  //   F[arb_fsm_err]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_arb_fsm_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.arb_fsm_err.de),
    .d      (hw2reg.std_fault_status.arb_fsm_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.arb_fsm_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_arb_fsm_err_qs)
  );

  //   F[storage_err]: 5:5
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_storage_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.storage_err.de),
    .d      (hw2reg.std_fault_status.storage_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.storage_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_storage_err_qs)
  );

  //   F[phy_fsm_err]: 6:6
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_phy_fsm_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.phy_fsm_err.de),
    .d      (hw2reg.std_fault_status.phy_fsm_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.phy_fsm_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_phy_fsm_err_qs)
  );

  //   F[ctrl_cnt_err]: 7:7
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_ctrl_cnt_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.ctrl_cnt_err.de),
    .d      (hw2reg.std_fault_status.ctrl_cnt_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.ctrl_cnt_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_ctrl_cnt_err_qs)
  );

  //   F[fifo_err]: 8:8
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_std_fault_status_fifo_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.std_fault_status.fifo_err.de),
    .d      (hw2reg.std_fault_status.fifo_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.std_fault_status.fifo_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (std_fault_status_fifo_err_qs)
  );


  // R[fault_status]: V(False)
  //   F[op_err]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_op_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.op_err.de),
    .d      (hw2reg.fault_status.op_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.op_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_op_err_qs)
  );

  //   F[mp_err]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_mp_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.mp_err.de),
    .d      (hw2reg.fault_status.mp_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.mp_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_mp_err_qs)
  );

  //   F[rd_err]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_rd_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.rd_err.de),
    .d      (hw2reg.fault_status.rd_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.rd_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_rd_err_qs)
  );

  //   F[prog_err]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_prog_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.prog_err.de),
    .d      (hw2reg.fault_status.prog_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.prog_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_prog_err_qs)
  );

  //   F[prog_win_err]: 4:4
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_prog_win_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.prog_win_err.de),
    .d      (hw2reg.fault_status.prog_win_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.prog_win_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_prog_win_err_qs)
  );

  //   F[prog_type_err]: 5:5
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_prog_type_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.prog_type_err.de),
    .d      (hw2reg.fault_status.prog_type_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.prog_type_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_prog_type_err_qs)
  );

  //   F[seed_err]: 6:6
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_seed_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.seed_err.de),
    .d      (hw2reg.fault_status.seed_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.seed_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_seed_err_qs)
  );

  //   F[phy_relbl_err]: 7:7
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_phy_relbl_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.phy_relbl_err.de),
    .d      (hw2reg.fault_status.phy_relbl_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.phy_relbl_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_phy_relbl_err_qs)
  );

  //   F[phy_storage_err]: 8:8
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_phy_storage_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.phy_storage_err.de),
    .d      (hw2reg.fault_status.phy_storage_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.phy_storage_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_phy_storage_err_qs)
  );

  //   F[spurious_ack]: 9:9
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_spurious_ack (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.spurious_ack.de),
    .d      (hw2reg.fault_status.spurious_ack.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.spurious_ack.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_spurious_ack_qs)
  );

  //   F[arb_err]: 10:10
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_arb_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.arb_err.de),
    .d      (hw2reg.fault_status.arb_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.arb_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_arb_err_qs)
  );

  //   F[host_gnt_err]: 11:11
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_fault_status_host_gnt_err (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.fault_status.host_gnt_err.de),
    .d      (hw2reg.fault_status.host_gnt_err.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fault_status.host_gnt_err.q),
    .ds     (),

    // to register interface (read)
    .qs     (fault_status_host_gnt_err_qs)
  );


  // R[err_addr]: V(False)
  prim_subreg #(
    .DW      (20),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (20'h0)
  ) u_err_addr (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.err_addr.de),
    .d      (hw2reg.err_addr.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (err_addr_qs)
  );


  // Subregister 0 of Multireg ecc_single_err_cnt
  // R[ecc_single_err_cnt]: V(False)
  //   F[ecc_single_err_cnt_0]: 7:0
  prim_subreg #(
    .DW      (8),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (8'h0)
  ) u_ecc_single_err_cnt_ecc_single_err_cnt_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (ecc_single_err_cnt_we),
    .wd     (ecc_single_err_cnt_ecc_single_err_cnt_0_wd),

    // from internal hardware
    .de     (hw2reg.ecc_single_err_cnt[0].de),
    .d      (hw2reg.ecc_single_err_cnt[0].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.ecc_single_err_cnt[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (ecc_single_err_cnt_ecc_single_err_cnt_0_qs)
  );

  //   F[ecc_single_err_cnt_1]: 15:8
  prim_subreg #(
    .DW      (8),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (8'h0)
  ) u_ecc_single_err_cnt_ecc_single_err_cnt_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (ecc_single_err_cnt_we),
    .wd     (ecc_single_err_cnt_ecc_single_err_cnt_1_wd),

    // from internal hardware
    .de     (hw2reg.ecc_single_err_cnt[1].de),
    .d      (hw2reg.ecc_single_err_cnt[1].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.ecc_single_err_cnt[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (ecc_single_err_cnt_ecc_single_err_cnt_1_qs)
  );


  // Subregister 0 of Multireg ecc_single_err_addr
  // R[ecc_single_err_addr_0]: V(False)
  prim_subreg #(
    .DW      (20),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (20'h0)
  ) u_ecc_single_err_addr_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.ecc_single_err_addr[0].de),
    .d      (hw2reg.ecc_single_err_addr[0].d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (ecc_single_err_addr_0_qs)
  );


  // Subregister 1 of Multireg ecc_single_err_addr
  // R[ecc_single_err_addr_1]: V(False)
  prim_subreg #(
    .DW      (20),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (20'h0)
  ) u_ecc_single_err_addr_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.ecc_single_err_addr[1].de),
    .d      (hw2reg.ecc_single_err_addr[1].d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (ecc_single_err_addr_1_qs)
  );


  // R[phy_alert_cfg]: V(False)
  //   F[alert_ack]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_phy_alert_cfg_alert_ack (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (phy_alert_cfg_we),
    .wd     (phy_alert_cfg_alert_ack_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.phy_alert_cfg.alert_ack.q),
    .ds     (),

    // to register interface (read)
    .qs     (phy_alert_cfg_alert_ack_qs)
  );

  //   F[alert_trig]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_phy_alert_cfg_alert_trig (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (phy_alert_cfg_we),
    .wd     (phy_alert_cfg_alert_trig_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.phy_alert_cfg.alert_trig.q),
    .ds     (),

    // to register interface (read)
    .qs     (phy_alert_cfg_alert_trig_qs)
  );


  // R[phy_status]: V(False)
  //   F[init_wip]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h0)
  ) u_phy_status_init_wip (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.phy_status.init_wip.de),
    .d      (hw2reg.phy_status.init_wip.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (phy_status_init_wip_qs)
  );

  //   F[prog_normal_avail]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h1)
  ) u_phy_status_prog_normal_avail (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.phy_status.prog_normal_avail.de),
    .d      (hw2reg.phy_status.prog_normal_avail.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (phy_status_prog_normal_avail_qs)
  );

  //   F[prog_repair_avail]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRO),
    .RESVAL  (1'h1)
  ) u_phy_status_prog_repair_avail (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (1'b0),
    .wd     ('0),

    // from internal hardware
    .de     (hw2reg.phy_status.prog_repair_avail.de),
    .d      (hw2reg.phy_status.prog_repair_avail.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (phy_status_prog_repair_avail_qs)
  );


  // R[scratch]: V(False)
  prim_subreg #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_scratch (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (scratch_we),
    .wd     (scratch_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.scratch.q),
    .ds     (),

    // to register interface (read)
    .qs     (scratch_qs)
  );


  // R[fifo_lvl]: V(False)
  //   F[prog]: 4:0
  prim_subreg #(
    .DW      (5),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (5'hf)
  ) u_fifo_lvl_prog (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (fifo_lvl_we),
    .wd     (fifo_lvl_prog_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fifo_lvl.prog.q),
    .ds     (),

    // to register interface (read)
    .qs     (fifo_lvl_prog_qs)
  );

  //   F[rd]: 12:8
  prim_subreg #(
    .DW      (5),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (5'hf)
  ) u_fifo_lvl_rd (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (fifo_lvl_we),
    .wd     (fifo_lvl_rd_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fifo_lvl.rd.q),
    .ds     (),

    // to register interface (read)
    .qs     (fifo_lvl_rd_qs)
  );


  // R[fifo_rst]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_fifo_rst (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (fifo_rst_we),
    .wd     (fifo_rst_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.fifo_rst.q),
    .ds     (),

    // to register interface (read)
    .qs     (fifo_rst_qs)
  );


  // R[curr_fifo_lvl]: V(True)
  //   F[prog]: 4:0
  prim_subreg_ext #(
    .DW    (5)
  ) u_curr_fifo_lvl_prog (
    .re     (curr_fifo_lvl_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.curr_fifo_lvl.prog.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (curr_fifo_lvl_prog_qs)
  );

  //   F[rd]: 12:8
  prim_subreg_ext #(
    .DW    (5)
  ) u_curr_fifo_lvl_rd (
    .re     (curr_fifo_lvl_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.curr_fifo_lvl.rd.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (curr_fifo_lvl_rd_qs)
  );



  logic [106:0] addr_hit;
  always_comb begin
    addr_hit = '0;
    addr_hit[  0] = (reg_addr == FLASH_CTRL_INTR_STATE_OFFSET);
    addr_hit[  1] = (reg_addr == FLASH_CTRL_INTR_ENABLE_OFFSET);
    addr_hit[  2] = (reg_addr == FLASH_CTRL_INTR_TEST_OFFSET);
    addr_hit[  3] = (reg_addr == FLASH_CTRL_ALERT_TEST_OFFSET);
    addr_hit[  4] = (reg_addr == FLASH_CTRL_DIS_OFFSET);
    addr_hit[  5] = (reg_addr == FLASH_CTRL_EXEC_OFFSET);
    addr_hit[  6] = (reg_addr == FLASH_CTRL_INIT_OFFSET);
    addr_hit[  7] = (reg_addr == FLASH_CTRL_CTRL_REGWEN_OFFSET);
    addr_hit[  8] = (reg_addr == FLASH_CTRL_CONTROL_OFFSET);
    addr_hit[  9] = (reg_addr == FLASH_CTRL_ADDR_OFFSET);
    addr_hit[ 10] = (reg_addr == FLASH_CTRL_PROG_TYPE_EN_OFFSET);
    addr_hit[ 11] = (reg_addr == FLASH_CTRL_ERASE_SUSPEND_OFFSET);
    addr_hit[ 12] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_0_OFFSET);
    addr_hit[ 13] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_1_OFFSET);
    addr_hit[ 14] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_2_OFFSET);
    addr_hit[ 15] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_3_OFFSET);
    addr_hit[ 16] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_4_OFFSET);
    addr_hit[ 17] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_5_OFFSET);
    addr_hit[ 18] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_6_OFFSET);
    addr_hit[ 19] = (reg_addr == FLASH_CTRL_REGION_CFG_REGWEN_7_OFFSET);
    addr_hit[ 20] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_0_OFFSET);
    addr_hit[ 21] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_1_OFFSET);
    addr_hit[ 22] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_2_OFFSET);
    addr_hit[ 23] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_3_OFFSET);
    addr_hit[ 24] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_4_OFFSET);
    addr_hit[ 25] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_5_OFFSET);
    addr_hit[ 26] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_6_OFFSET);
    addr_hit[ 27] = (reg_addr == FLASH_CTRL_MP_REGION_CFG_7_OFFSET);
    addr_hit[ 28] = (reg_addr == FLASH_CTRL_MP_REGION_0_OFFSET);
    addr_hit[ 29] = (reg_addr == FLASH_CTRL_MP_REGION_1_OFFSET);
    addr_hit[ 30] = (reg_addr == FLASH_CTRL_MP_REGION_2_OFFSET);
    addr_hit[ 31] = (reg_addr == FLASH_CTRL_MP_REGION_3_OFFSET);
    addr_hit[ 32] = (reg_addr == FLASH_CTRL_MP_REGION_4_OFFSET);
    addr_hit[ 33] = (reg_addr == FLASH_CTRL_MP_REGION_5_OFFSET);
    addr_hit[ 34] = (reg_addr == FLASH_CTRL_MP_REGION_6_OFFSET);
    addr_hit[ 35] = (reg_addr == FLASH_CTRL_MP_REGION_7_OFFSET);
    addr_hit[ 36] = (reg_addr == FLASH_CTRL_DEFAULT_REGION_OFFSET);
    addr_hit[ 37] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_0_OFFSET);
    addr_hit[ 38] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_1_OFFSET);
    addr_hit[ 39] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_2_OFFSET);
    addr_hit[ 40] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_3_OFFSET);
    addr_hit[ 41] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_4_OFFSET);
    addr_hit[ 42] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_5_OFFSET);
    addr_hit[ 43] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_6_OFFSET);
    addr_hit[ 44] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_7_OFFSET);
    addr_hit[ 45] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_8_OFFSET);
    addr_hit[ 46] = (reg_addr == FLASH_CTRL_BANK0_INFO0_REGWEN_9_OFFSET);
    addr_hit[ 47] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_0_OFFSET);
    addr_hit[ 48] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_1_OFFSET);
    addr_hit[ 49] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_2_OFFSET);
    addr_hit[ 50] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_3_OFFSET);
    addr_hit[ 51] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_4_OFFSET);
    addr_hit[ 52] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_5_OFFSET);
    addr_hit[ 53] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_6_OFFSET);
    addr_hit[ 54] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_7_OFFSET);
    addr_hit[ 55] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_8_OFFSET);
    addr_hit[ 56] = (reg_addr == FLASH_CTRL_BANK0_INFO0_PAGE_CFG_9_OFFSET);
    addr_hit[ 57] = (reg_addr == FLASH_CTRL_BANK0_INFO1_REGWEN_OFFSET);
    addr_hit[ 58] = (reg_addr == FLASH_CTRL_BANK0_INFO1_PAGE_CFG_OFFSET);
    addr_hit[ 59] = (reg_addr == FLASH_CTRL_BANK0_INFO2_REGWEN_0_OFFSET);
    addr_hit[ 60] = (reg_addr == FLASH_CTRL_BANK0_INFO2_REGWEN_1_OFFSET);
    addr_hit[ 61] = (reg_addr == FLASH_CTRL_BANK0_INFO2_PAGE_CFG_0_OFFSET);
    addr_hit[ 62] = (reg_addr == FLASH_CTRL_BANK0_INFO2_PAGE_CFG_1_OFFSET);
    addr_hit[ 63] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_0_OFFSET);
    addr_hit[ 64] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_1_OFFSET);
    addr_hit[ 65] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_2_OFFSET);
    addr_hit[ 66] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_3_OFFSET);
    addr_hit[ 67] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_4_OFFSET);
    addr_hit[ 68] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_5_OFFSET);
    addr_hit[ 69] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_6_OFFSET);
    addr_hit[ 70] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_7_OFFSET);
    addr_hit[ 71] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_8_OFFSET);
    addr_hit[ 72] = (reg_addr == FLASH_CTRL_BANK1_INFO0_REGWEN_9_OFFSET);
    addr_hit[ 73] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_0_OFFSET);
    addr_hit[ 74] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_1_OFFSET);
    addr_hit[ 75] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_2_OFFSET);
    addr_hit[ 76] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_3_OFFSET);
    addr_hit[ 77] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_4_OFFSET);
    addr_hit[ 78] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_5_OFFSET);
    addr_hit[ 79] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_6_OFFSET);
    addr_hit[ 80] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_7_OFFSET);
    addr_hit[ 81] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_8_OFFSET);
    addr_hit[ 82] = (reg_addr == FLASH_CTRL_BANK1_INFO0_PAGE_CFG_9_OFFSET);
    addr_hit[ 83] = (reg_addr == FLASH_CTRL_BANK1_INFO1_REGWEN_OFFSET);
    addr_hit[ 84] = (reg_addr == FLASH_CTRL_BANK1_INFO1_PAGE_CFG_OFFSET);
    addr_hit[ 85] = (reg_addr == FLASH_CTRL_BANK1_INFO2_REGWEN_0_OFFSET);
    addr_hit[ 86] = (reg_addr == FLASH_CTRL_BANK1_INFO2_REGWEN_1_OFFSET);
    addr_hit[ 87] = (reg_addr == FLASH_CTRL_BANK1_INFO2_PAGE_CFG_0_OFFSET);
    addr_hit[ 88] = (reg_addr == FLASH_CTRL_BANK1_INFO2_PAGE_CFG_1_OFFSET);
    addr_hit[ 89] = (reg_addr == FLASH_CTRL_BANK_CFG_REGWEN_OFFSET);
    addr_hit[ 90] = (reg_addr == FLASH_CTRL_MP_BANK_CFG_SHADOWED_OFFSET);
    addr_hit[ 91] = (reg_addr == FLASH_CTRL_OP_STATUS_OFFSET);
    addr_hit[ 92] = (reg_addr == FLASH_CTRL_STATUS_OFFSET);
    addr_hit[ 93] = (reg_addr == FLASH_CTRL_DEBUG_STATE_OFFSET);
    addr_hit[ 94] = (reg_addr == FLASH_CTRL_ERR_CODE_OFFSET);
    addr_hit[ 95] = (reg_addr == FLASH_CTRL_STD_FAULT_STATUS_OFFSET);
    addr_hit[ 96] = (reg_addr == FLASH_CTRL_FAULT_STATUS_OFFSET);
    addr_hit[ 97] = (reg_addr == FLASH_CTRL_ERR_ADDR_OFFSET);
    addr_hit[ 98] = (reg_addr == FLASH_CTRL_ECC_SINGLE_ERR_CNT_OFFSET);
    addr_hit[ 99] = (reg_addr == FLASH_CTRL_ECC_SINGLE_ERR_ADDR_0_OFFSET);
    addr_hit[100] = (reg_addr == FLASH_CTRL_ECC_SINGLE_ERR_ADDR_1_OFFSET);
    addr_hit[101] = (reg_addr == FLASH_CTRL_PHY_ALERT_CFG_OFFSET);
    addr_hit[102] = (reg_addr == FLASH_CTRL_PHY_STATUS_OFFSET);
    addr_hit[103] = (reg_addr == FLASH_CTRL_SCRATCH_OFFSET);
    addr_hit[104] = (reg_addr == FLASH_CTRL_FIFO_LVL_OFFSET);
    addr_hit[105] = (reg_addr == FLASH_CTRL_FIFO_RST_OFFSET);
    addr_hit[106] = (reg_addr == FLASH_CTRL_CURR_FIFO_LVL_OFFSET);
  end

  assign addrmiss = (reg_re || reg_we) ? ~|addr_hit : 1'b0 ;

  // Check sub-word write is permitted
  always_comb begin
    wr_err = (reg_we &
              ((addr_hit[  0] & (|(FLASH_CTRL_CORE_PERMIT[  0] & ~reg_be))) |
               (addr_hit[  1] & (|(FLASH_CTRL_CORE_PERMIT[  1] & ~reg_be))) |
               (addr_hit[  2] & (|(FLASH_CTRL_CORE_PERMIT[  2] & ~reg_be))) |
               (addr_hit[  3] & (|(FLASH_CTRL_CORE_PERMIT[  3] & ~reg_be))) |
               (addr_hit[  4] & (|(FLASH_CTRL_CORE_PERMIT[  4] & ~reg_be))) |
               (addr_hit[  5] & (|(FLASH_CTRL_CORE_PERMIT[  5] & ~reg_be))) |
               (addr_hit[  6] & (|(FLASH_CTRL_CORE_PERMIT[  6] & ~reg_be))) |
               (addr_hit[  7] & (|(FLASH_CTRL_CORE_PERMIT[  7] & ~reg_be))) |
               (addr_hit[  8] & (|(FLASH_CTRL_CORE_PERMIT[  8] & ~reg_be))) |
               (addr_hit[  9] & (|(FLASH_CTRL_CORE_PERMIT[  9] & ~reg_be))) |
               (addr_hit[ 10] & (|(FLASH_CTRL_CORE_PERMIT[ 10] & ~reg_be))) |
               (addr_hit[ 11] & (|(FLASH_CTRL_CORE_PERMIT[ 11] & ~reg_be))) |
               (addr_hit[ 12] & (|(FLASH_CTRL_CORE_PERMIT[ 12] & ~reg_be))) |
               (addr_hit[ 13] & (|(FLASH_CTRL_CORE_PERMIT[ 13] & ~reg_be))) |
               (addr_hit[ 14] & (|(FLASH_CTRL_CORE_PERMIT[ 14] & ~reg_be))) |
               (addr_hit[ 15] & (|(FLASH_CTRL_CORE_PERMIT[ 15] & ~reg_be))) |
               (addr_hit[ 16] & (|(FLASH_CTRL_CORE_PERMIT[ 16] & ~reg_be))) |
               (addr_hit[ 17] & (|(FLASH_CTRL_CORE_PERMIT[ 17] & ~reg_be))) |
               (addr_hit[ 18] & (|(FLASH_CTRL_CORE_PERMIT[ 18] & ~reg_be))) |
               (addr_hit[ 19] & (|(FLASH_CTRL_CORE_PERMIT[ 19] & ~reg_be))) |
               (addr_hit[ 20] & (|(FLASH_CTRL_CORE_PERMIT[ 20] & ~reg_be))) |
               (addr_hit[ 21] & (|(FLASH_CTRL_CORE_PERMIT[ 21] & ~reg_be))) |
               (addr_hit[ 22] & (|(FLASH_CTRL_CORE_PERMIT[ 22] & ~reg_be))) |
               (addr_hit[ 23] & (|(FLASH_CTRL_CORE_PERMIT[ 23] & ~reg_be))) |
               (addr_hit[ 24] & (|(FLASH_CTRL_CORE_PERMIT[ 24] & ~reg_be))) |
               (addr_hit[ 25] & (|(FLASH_CTRL_CORE_PERMIT[ 25] & ~reg_be))) |
               (addr_hit[ 26] & (|(FLASH_CTRL_CORE_PERMIT[ 26] & ~reg_be))) |
               (addr_hit[ 27] & (|(FLASH_CTRL_CORE_PERMIT[ 27] & ~reg_be))) |
               (addr_hit[ 28] & (|(FLASH_CTRL_CORE_PERMIT[ 28] & ~reg_be))) |
               (addr_hit[ 29] & (|(FLASH_CTRL_CORE_PERMIT[ 29] & ~reg_be))) |
               (addr_hit[ 30] & (|(FLASH_CTRL_CORE_PERMIT[ 30] & ~reg_be))) |
               (addr_hit[ 31] & (|(FLASH_CTRL_CORE_PERMIT[ 31] & ~reg_be))) |
               (addr_hit[ 32] & (|(FLASH_CTRL_CORE_PERMIT[ 32] & ~reg_be))) |
               (addr_hit[ 33] & (|(FLASH_CTRL_CORE_PERMIT[ 33] & ~reg_be))) |
               (addr_hit[ 34] & (|(FLASH_CTRL_CORE_PERMIT[ 34] & ~reg_be))) |
               (addr_hit[ 35] & (|(FLASH_CTRL_CORE_PERMIT[ 35] & ~reg_be))) |
               (addr_hit[ 36] & (|(FLASH_CTRL_CORE_PERMIT[ 36] & ~reg_be))) |
               (addr_hit[ 37] & (|(FLASH_CTRL_CORE_PERMIT[ 37] & ~reg_be))) |
               (addr_hit[ 38] & (|(FLASH_CTRL_CORE_PERMIT[ 38] & ~reg_be))) |
               (addr_hit[ 39] & (|(FLASH_CTRL_CORE_PERMIT[ 39] & ~reg_be))) |
               (addr_hit[ 40] & (|(FLASH_CTRL_CORE_PERMIT[ 40] & ~reg_be))) |
               (addr_hit[ 41] & (|(FLASH_CTRL_CORE_PERMIT[ 41] & ~reg_be))) |
               (addr_hit[ 42] & (|(FLASH_CTRL_CORE_PERMIT[ 42] & ~reg_be))) |
               (addr_hit[ 43] & (|(FLASH_CTRL_CORE_PERMIT[ 43] & ~reg_be))) |
               (addr_hit[ 44] & (|(FLASH_CTRL_CORE_PERMIT[ 44] & ~reg_be))) |
               (addr_hit[ 45] & (|(FLASH_CTRL_CORE_PERMIT[ 45] & ~reg_be))) |
               (addr_hit[ 46] & (|(FLASH_CTRL_CORE_PERMIT[ 46] & ~reg_be))) |
               (addr_hit[ 47] & (|(FLASH_CTRL_CORE_PERMIT[ 47] & ~reg_be))) |
               (addr_hit[ 48] & (|(FLASH_CTRL_CORE_PERMIT[ 48] & ~reg_be))) |
               (addr_hit[ 49] & (|(FLASH_CTRL_CORE_PERMIT[ 49] & ~reg_be))) |
               (addr_hit[ 50] & (|(FLASH_CTRL_CORE_PERMIT[ 50] & ~reg_be))) |
               (addr_hit[ 51] & (|(FLASH_CTRL_CORE_PERMIT[ 51] & ~reg_be))) |
               (addr_hit[ 52] & (|(FLASH_CTRL_CORE_PERMIT[ 52] & ~reg_be))) |
               (addr_hit[ 53] & (|(FLASH_CTRL_CORE_PERMIT[ 53] & ~reg_be))) |
               (addr_hit[ 54] & (|(FLASH_CTRL_CORE_PERMIT[ 54] & ~reg_be))) |
               (addr_hit[ 55] & (|(FLASH_CTRL_CORE_PERMIT[ 55] & ~reg_be))) |
               (addr_hit[ 56] & (|(FLASH_CTRL_CORE_PERMIT[ 56] & ~reg_be))) |
               (addr_hit[ 57] & (|(FLASH_CTRL_CORE_PERMIT[ 57] & ~reg_be))) |
               (addr_hit[ 58] & (|(FLASH_CTRL_CORE_PERMIT[ 58] & ~reg_be))) |
               (addr_hit[ 59] & (|(FLASH_CTRL_CORE_PERMIT[ 59] & ~reg_be))) |
               (addr_hit[ 60] & (|(FLASH_CTRL_CORE_PERMIT[ 60] & ~reg_be))) |
               (addr_hit[ 61] & (|(FLASH_CTRL_CORE_PERMIT[ 61] & ~reg_be))) |
               (addr_hit[ 62] & (|(FLASH_CTRL_CORE_PERMIT[ 62] & ~reg_be))) |
               (addr_hit[ 63] & (|(FLASH_CTRL_CORE_PERMIT[ 63] & ~reg_be))) |
               (addr_hit[ 64] & (|(FLASH_CTRL_CORE_PERMIT[ 64] & ~reg_be))) |
               (addr_hit[ 65] & (|(FLASH_CTRL_CORE_PERMIT[ 65] & ~reg_be))) |
               (addr_hit[ 66] & (|(FLASH_CTRL_CORE_PERMIT[ 66] & ~reg_be))) |
               (addr_hit[ 67] & (|(FLASH_CTRL_CORE_PERMIT[ 67] & ~reg_be))) |
               (addr_hit[ 68] & (|(FLASH_CTRL_CORE_PERMIT[ 68] & ~reg_be))) |
               (addr_hit[ 69] & (|(FLASH_CTRL_CORE_PERMIT[ 69] & ~reg_be))) |
               (addr_hit[ 70] & (|(FLASH_CTRL_CORE_PERMIT[ 70] & ~reg_be))) |
               (addr_hit[ 71] & (|(FLASH_CTRL_CORE_PERMIT[ 71] & ~reg_be))) |
               (addr_hit[ 72] & (|(FLASH_CTRL_CORE_PERMIT[ 72] & ~reg_be))) |
               (addr_hit[ 73] & (|(FLASH_CTRL_CORE_PERMIT[ 73] & ~reg_be))) |
               (addr_hit[ 74] & (|(FLASH_CTRL_CORE_PERMIT[ 74] & ~reg_be))) |
               (addr_hit[ 75] & (|(FLASH_CTRL_CORE_PERMIT[ 75] & ~reg_be))) |
               (addr_hit[ 76] & (|(FLASH_CTRL_CORE_PERMIT[ 76] & ~reg_be))) |
               (addr_hit[ 77] & (|(FLASH_CTRL_CORE_PERMIT[ 77] & ~reg_be))) |
               (addr_hit[ 78] & (|(FLASH_CTRL_CORE_PERMIT[ 78] & ~reg_be))) |
               (addr_hit[ 79] & (|(FLASH_CTRL_CORE_PERMIT[ 79] & ~reg_be))) |
               (addr_hit[ 80] & (|(FLASH_CTRL_CORE_PERMIT[ 80] & ~reg_be))) |
               (addr_hit[ 81] & (|(FLASH_CTRL_CORE_PERMIT[ 81] & ~reg_be))) |
               (addr_hit[ 82] & (|(FLASH_CTRL_CORE_PERMIT[ 82] & ~reg_be))) |
               (addr_hit[ 83] & (|(FLASH_CTRL_CORE_PERMIT[ 83] & ~reg_be))) |
               (addr_hit[ 84] & (|(FLASH_CTRL_CORE_PERMIT[ 84] & ~reg_be))) |
               (addr_hit[ 85] & (|(FLASH_CTRL_CORE_PERMIT[ 85] & ~reg_be))) |
               (addr_hit[ 86] & (|(FLASH_CTRL_CORE_PERMIT[ 86] & ~reg_be))) |
               (addr_hit[ 87] & (|(FLASH_CTRL_CORE_PERMIT[ 87] & ~reg_be))) |
               (addr_hit[ 88] & (|(FLASH_CTRL_CORE_PERMIT[ 88] & ~reg_be))) |
               (addr_hit[ 89] & (|(FLASH_CTRL_CORE_PERMIT[ 89] & ~reg_be))) |
               (addr_hit[ 90] & (|(FLASH_CTRL_CORE_PERMIT[ 90] & ~reg_be))) |
               (addr_hit[ 91] & (|(FLASH_CTRL_CORE_PERMIT[ 91] & ~reg_be))) |
               (addr_hit[ 92] & (|(FLASH_CTRL_CORE_PERMIT[ 92] & ~reg_be))) |
               (addr_hit[ 93] & (|(FLASH_CTRL_CORE_PERMIT[ 93] & ~reg_be))) |
               (addr_hit[ 94] & (|(FLASH_CTRL_CORE_PERMIT[ 94] & ~reg_be))) |
               (addr_hit[ 95] & (|(FLASH_CTRL_CORE_PERMIT[ 95] & ~reg_be))) |
               (addr_hit[ 96] & (|(FLASH_CTRL_CORE_PERMIT[ 96] & ~reg_be))) |
               (addr_hit[ 97] & (|(FLASH_CTRL_CORE_PERMIT[ 97] & ~reg_be))) |
               (addr_hit[ 98] & (|(FLASH_CTRL_CORE_PERMIT[ 98] & ~reg_be))) |
               (addr_hit[ 99] & (|(FLASH_CTRL_CORE_PERMIT[ 99] & ~reg_be))) |
               (addr_hit[100] & (|(FLASH_CTRL_CORE_PERMIT[100] & ~reg_be))) |
               (addr_hit[101] & (|(FLASH_CTRL_CORE_PERMIT[101] & ~reg_be))) |
               (addr_hit[102] & (|(FLASH_CTRL_CORE_PERMIT[102] & ~reg_be))) |
               (addr_hit[103] & (|(FLASH_CTRL_CORE_PERMIT[103] & ~reg_be))) |
               (addr_hit[104] & (|(FLASH_CTRL_CORE_PERMIT[104] & ~reg_be))) |
               (addr_hit[105] & (|(FLASH_CTRL_CORE_PERMIT[105] & ~reg_be))) |
               (addr_hit[106] & (|(FLASH_CTRL_CORE_PERMIT[106] & ~reg_be)))));
  end

  // Generate write-enables
  assign intr_state_we = addr_hit[0] & reg_we & !reg_error;

  assign intr_state_prog_empty_wd = reg_wdata[0];

  assign intr_state_prog_lvl_wd = reg_wdata[1];

  assign intr_state_rd_full_wd = reg_wdata[2];

  assign intr_state_rd_lvl_wd = reg_wdata[3];

  assign intr_state_op_done_wd = reg_wdata[4];

  assign intr_state_corr_err_wd = reg_wdata[5];
  assign intr_enable_we = addr_hit[1] & reg_we & !reg_error;

  assign intr_enable_prog_empty_wd = reg_wdata[0];

  assign intr_enable_prog_lvl_wd = reg_wdata[1];

  assign intr_enable_rd_full_wd = reg_wdata[2];

  assign intr_enable_rd_lvl_wd = reg_wdata[3];

  assign intr_enable_op_done_wd = reg_wdata[4];

  assign intr_enable_corr_err_wd = reg_wdata[5];
  assign intr_test_we = addr_hit[2] & reg_we & !reg_error;

  assign intr_test_prog_empty_wd = reg_wdata[0];

  assign intr_test_prog_lvl_wd = reg_wdata[1];

  assign intr_test_rd_full_wd = reg_wdata[2];

  assign intr_test_rd_lvl_wd = reg_wdata[3];

  assign intr_test_op_done_wd = reg_wdata[4];

  assign intr_test_corr_err_wd = reg_wdata[5];
  assign alert_test_we = addr_hit[3] & reg_we & !reg_error;

  assign alert_test_recov_err_wd = reg_wdata[0];

  assign alert_test_fatal_std_err_wd = reg_wdata[1];

  assign alert_test_fatal_err_wd = reg_wdata[2];

  assign alert_test_fatal_prim_flash_alert_wd = reg_wdata[3];

  assign alert_test_recov_prim_flash_alert_wd = reg_wdata[4];
  assign dis_we = addr_hit[4] & reg_we & !reg_error;

  assign dis_wd = reg_wdata[3:0];
  assign exec_we = addr_hit[5] & reg_we & !reg_error;

  assign exec_wd = reg_wdata[31:0];
  assign init_we = addr_hit[6] & reg_we & !reg_error;

  assign init_wd = reg_wdata[0];
  assign ctrl_regwen_re = addr_hit[7] & reg_re & !reg_error;
  assign control_we = addr_hit[8] & reg_we & !reg_error;

  assign control_start_wd = reg_wdata[0];

  assign control_op_wd = reg_wdata[5:4];

  assign control_prog_sel_wd = reg_wdata[6];

  assign control_erase_sel_wd = reg_wdata[7];

  assign control_partition_sel_wd = reg_wdata[8];

  assign control_info_sel_wd = reg_wdata[10:9];

  assign control_num_wd = reg_wdata[27:16];
  assign addr_we = addr_hit[9] & reg_we & !reg_error;

  assign addr_wd = reg_wdata[19:0];
  assign prog_type_en_we = addr_hit[10] & reg_we & !reg_error;

  assign prog_type_en_normal_wd = reg_wdata[0];

  assign prog_type_en_repair_wd = reg_wdata[1];
  assign erase_suspend_we = addr_hit[11] & reg_we & !reg_error;

  assign erase_suspend_wd = reg_wdata[0];
  assign region_cfg_regwen_0_we = addr_hit[12] & reg_we & !reg_error;

  assign region_cfg_regwen_0_wd = reg_wdata[0];
  assign region_cfg_regwen_1_we = addr_hit[13] & reg_we & !reg_error;

  assign region_cfg_regwen_1_wd = reg_wdata[0];
  assign region_cfg_regwen_2_we = addr_hit[14] & reg_we & !reg_error;

  assign region_cfg_regwen_2_wd = reg_wdata[0];
  assign region_cfg_regwen_3_we = addr_hit[15] & reg_we & !reg_error;

  assign region_cfg_regwen_3_wd = reg_wdata[0];
  assign region_cfg_regwen_4_we = addr_hit[16] & reg_we & !reg_error;

  assign region_cfg_regwen_4_wd = reg_wdata[0];
  assign region_cfg_regwen_5_we = addr_hit[17] & reg_we & !reg_error;

  assign region_cfg_regwen_5_wd = reg_wdata[0];
  assign region_cfg_regwen_6_we = addr_hit[18] & reg_we & !reg_error;

  assign region_cfg_regwen_6_wd = reg_wdata[0];
  assign region_cfg_regwen_7_we = addr_hit[19] & reg_we & !reg_error;

  assign region_cfg_regwen_7_wd = reg_wdata[0];
  assign mp_region_cfg_0_we = addr_hit[20] & reg_we & !reg_error;

  assign mp_region_cfg_0_en_0_wd = reg_wdata[3:0];

  assign mp_region_cfg_0_rd_en_0_wd = reg_wdata[7:4];

  assign mp_region_cfg_0_prog_en_0_wd = reg_wdata[11:8];

  assign mp_region_cfg_0_erase_en_0_wd = reg_wdata[15:12];

  assign mp_region_cfg_0_scramble_en_0_wd = reg_wdata[19:16];

  assign mp_region_cfg_0_ecc_en_0_wd = reg_wdata[23:20];

  assign mp_region_cfg_0_he_en_0_wd = reg_wdata[27:24];
  assign mp_region_cfg_1_we = addr_hit[21] & reg_we & !reg_error;

  assign mp_region_cfg_1_en_1_wd = reg_wdata[3:0];

  assign mp_region_cfg_1_rd_en_1_wd = reg_wdata[7:4];

  assign mp_region_cfg_1_prog_en_1_wd = reg_wdata[11:8];

  assign mp_region_cfg_1_erase_en_1_wd = reg_wdata[15:12];

  assign mp_region_cfg_1_scramble_en_1_wd = reg_wdata[19:16];

  assign mp_region_cfg_1_ecc_en_1_wd = reg_wdata[23:20];

  assign mp_region_cfg_1_he_en_1_wd = reg_wdata[27:24];
  assign mp_region_cfg_2_we = addr_hit[22] & reg_we & !reg_error;

  assign mp_region_cfg_2_en_2_wd = reg_wdata[3:0];

  assign mp_region_cfg_2_rd_en_2_wd = reg_wdata[7:4];

  assign mp_region_cfg_2_prog_en_2_wd = reg_wdata[11:8];

  assign mp_region_cfg_2_erase_en_2_wd = reg_wdata[15:12];

  assign mp_region_cfg_2_scramble_en_2_wd = reg_wdata[19:16];

  assign mp_region_cfg_2_ecc_en_2_wd = reg_wdata[23:20];

  assign mp_region_cfg_2_he_en_2_wd = reg_wdata[27:24];
  assign mp_region_cfg_3_we = addr_hit[23] & reg_we & !reg_error;

  assign mp_region_cfg_3_en_3_wd = reg_wdata[3:0];

  assign mp_region_cfg_3_rd_en_3_wd = reg_wdata[7:4];

  assign mp_region_cfg_3_prog_en_3_wd = reg_wdata[11:8];

  assign mp_region_cfg_3_erase_en_3_wd = reg_wdata[15:12];

  assign mp_region_cfg_3_scramble_en_3_wd = reg_wdata[19:16];

  assign mp_region_cfg_3_ecc_en_3_wd = reg_wdata[23:20];

  assign mp_region_cfg_3_he_en_3_wd = reg_wdata[27:24];
  assign mp_region_cfg_4_we = addr_hit[24] & reg_we & !reg_error;

  assign mp_region_cfg_4_en_4_wd = reg_wdata[3:0];

  assign mp_region_cfg_4_rd_en_4_wd = reg_wdata[7:4];

  assign mp_region_cfg_4_prog_en_4_wd = reg_wdata[11:8];

  assign mp_region_cfg_4_erase_en_4_wd = reg_wdata[15:12];

  assign mp_region_cfg_4_scramble_en_4_wd = reg_wdata[19:16];

  assign mp_region_cfg_4_ecc_en_4_wd = reg_wdata[23:20];

  assign mp_region_cfg_4_he_en_4_wd = reg_wdata[27:24];
  assign mp_region_cfg_5_we = addr_hit[25] & reg_we & !reg_error;

  assign mp_region_cfg_5_en_5_wd = reg_wdata[3:0];

  assign mp_region_cfg_5_rd_en_5_wd = reg_wdata[7:4];

  assign mp_region_cfg_5_prog_en_5_wd = reg_wdata[11:8];

  assign mp_region_cfg_5_erase_en_5_wd = reg_wdata[15:12];

  assign mp_region_cfg_5_scramble_en_5_wd = reg_wdata[19:16];

  assign mp_region_cfg_5_ecc_en_5_wd = reg_wdata[23:20];

  assign mp_region_cfg_5_he_en_5_wd = reg_wdata[27:24];
  assign mp_region_cfg_6_we = addr_hit[26] & reg_we & !reg_error;

  assign mp_region_cfg_6_en_6_wd = reg_wdata[3:0];

  assign mp_region_cfg_6_rd_en_6_wd = reg_wdata[7:4];

  assign mp_region_cfg_6_prog_en_6_wd = reg_wdata[11:8];

  assign mp_region_cfg_6_erase_en_6_wd = reg_wdata[15:12];

  assign mp_region_cfg_6_scramble_en_6_wd = reg_wdata[19:16];

  assign mp_region_cfg_6_ecc_en_6_wd = reg_wdata[23:20];

  assign mp_region_cfg_6_he_en_6_wd = reg_wdata[27:24];
  assign mp_region_cfg_7_we = addr_hit[27] & reg_we & !reg_error;

  assign mp_region_cfg_7_en_7_wd = reg_wdata[3:0];

  assign mp_region_cfg_7_rd_en_7_wd = reg_wdata[7:4];

  assign mp_region_cfg_7_prog_en_7_wd = reg_wdata[11:8];

  assign mp_region_cfg_7_erase_en_7_wd = reg_wdata[15:12];

  assign mp_region_cfg_7_scramble_en_7_wd = reg_wdata[19:16];

  assign mp_region_cfg_7_ecc_en_7_wd = reg_wdata[23:20];

  assign mp_region_cfg_7_he_en_7_wd = reg_wdata[27:24];
  assign mp_region_0_we = addr_hit[28] & reg_we & !reg_error;

  assign mp_region_0_base_0_wd = reg_wdata[8:0];

  assign mp_region_0_size_0_wd = reg_wdata[18:9];
  assign mp_region_1_we = addr_hit[29] & reg_we & !reg_error;

  assign mp_region_1_base_1_wd = reg_wdata[8:0];

  assign mp_region_1_size_1_wd = reg_wdata[18:9];
  assign mp_region_2_we = addr_hit[30] & reg_we & !reg_error;

  assign mp_region_2_base_2_wd = reg_wdata[8:0];

  assign mp_region_2_size_2_wd = reg_wdata[18:9];
  assign mp_region_3_we = addr_hit[31] & reg_we & !reg_error;

  assign mp_region_3_base_3_wd = reg_wdata[8:0];

  assign mp_region_3_size_3_wd = reg_wdata[18:9];
  assign mp_region_4_we = addr_hit[32] & reg_we & !reg_error;

  assign mp_region_4_base_4_wd = reg_wdata[8:0];

  assign mp_region_4_size_4_wd = reg_wdata[18:9];
  assign mp_region_5_we = addr_hit[33] & reg_we & !reg_error;

  assign mp_region_5_base_5_wd = reg_wdata[8:0];

  assign mp_region_5_size_5_wd = reg_wdata[18:9];
  assign mp_region_6_we = addr_hit[34] & reg_we & !reg_error;

  assign mp_region_6_base_6_wd = reg_wdata[8:0];

  assign mp_region_6_size_6_wd = reg_wdata[18:9];
  assign mp_region_7_we = addr_hit[35] & reg_we & !reg_error;

  assign mp_region_7_base_7_wd = reg_wdata[8:0];

  assign mp_region_7_size_7_wd = reg_wdata[18:9];
  assign default_region_we = addr_hit[36] & reg_we & !reg_error;

  assign default_region_rd_en_wd = reg_wdata[3:0];

  assign default_region_prog_en_wd = reg_wdata[7:4];

  assign default_region_erase_en_wd = reg_wdata[11:8];

  assign default_region_scramble_en_wd = reg_wdata[15:12];

  assign default_region_ecc_en_wd = reg_wdata[19:16];

  assign default_region_he_en_wd = reg_wdata[23:20];
  assign bank0_info0_regwen_0_we = addr_hit[37] & reg_we & !reg_error;

  assign bank0_info0_regwen_0_wd = reg_wdata[0];
  assign bank0_info0_regwen_1_we = addr_hit[38] & reg_we & !reg_error;

  assign bank0_info0_regwen_1_wd = reg_wdata[0];
  assign bank0_info0_regwen_2_we = addr_hit[39] & reg_we & !reg_error;

  assign bank0_info0_regwen_2_wd = reg_wdata[0];
  assign bank0_info0_regwen_3_we = addr_hit[40] & reg_we & !reg_error;

  assign bank0_info0_regwen_3_wd = reg_wdata[0];
  assign bank0_info0_regwen_4_we = addr_hit[41] & reg_we & !reg_error;

  assign bank0_info0_regwen_4_wd = reg_wdata[0];
  assign bank0_info0_regwen_5_we = addr_hit[42] & reg_we & !reg_error;

  assign bank0_info0_regwen_5_wd = reg_wdata[0];
  assign bank0_info0_regwen_6_we = addr_hit[43] & reg_we & !reg_error;

  assign bank0_info0_regwen_6_wd = reg_wdata[0];
  assign bank0_info0_regwen_7_we = addr_hit[44] & reg_we & !reg_error;

  assign bank0_info0_regwen_7_wd = reg_wdata[0];
  assign bank0_info0_regwen_8_we = addr_hit[45] & reg_we & !reg_error;

  assign bank0_info0_regwen_8_wd = reg_wdata[0];
  assign bank0_info0_regwen_9_we = addr_hit[46] & reg_we & !reg_error;

  assign bank0_info0_regwen_9_wd = reg_wdata[0];
  assign bank0_info0_page_cfg_0_we = addr_hit[47] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_0_en_0_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_0_rd_en_0_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_0_prog_en_0_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_0_erase_en_0_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_0_scramble_en_0_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_0_ecc_en_0_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_0_he_en_0_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_1_we = addr_hit[48] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_1_en_1_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_1_rd_en_1_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_1_prog_en_1_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_1_erase_en_1_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_1_scramble_en_1_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_1_ecc_en_1_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_1_he_en_1_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_2_we = addr_hit[49] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_2_en_2_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_2_rd_en_2_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_2_prog_en_2_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_2_erase_en_2_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_2_scramble_en_2_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_2_ecc_en_2_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_2_he_en_2_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_3_we = addr_hit[50] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_3_en_3_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_3_rd_en_3_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_3_prog_en_3_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_3_erase_en_3_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_3_scramble_en_3_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_3_ecc_en_3_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_3_he_en_3_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_4_we = addr_hit[51] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_4_en_4_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_4_rd_en_4_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_4_prog_en_4_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_4_erase_en_4_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_4_scramble_en_4_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_4_ecc_en_4_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_4_he_en_4_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_5_we = addr_hit[52] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_5_en_5_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_5_rd_en_5_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_5_prog_en_5_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_5_erase_en_5_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_5_scramble_en_5_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_5_ecc_en_5_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_5_he_en_5_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_6_we = addr_hit[53] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_6_en_6_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_6_rd_en_6_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_6_prog_en_6_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_6_erase_en_6_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_6_scramble_en_6_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_6_ecc_en_6_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_6_he_en_6_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_7_we = addr_hit[54] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_7_en_7_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_7_rd_en_7_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_7_prog_en_7_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_7_erase_en_7_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_7_scramble_en_7_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_7_ecc_en_7_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_7_he_en_7_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_8_we = addr_hit[55] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_8_en_8_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_8_rd_en_8_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_8_prog_en_8_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_8_erase_en_8_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_8_scramble_en_8_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_8_ecc_en_8_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_8_he_en_8_wd = reg_wdata[27:24];
  assign bank0_info0_page_cfg_9_we = addr_hit[56] & reg_we & !reg_error;

  assign bank0_info0_page_cfg_9_en_9_wd = reg_wdata[3:0];

  assign bank0_info0_page_cfg_9_rd_en_9_wd = reg_wdata[7:4];

  assign bank0_info0_page_cfg_9_prog_en_9_wd = reg_wdata[11:8];

  assign bank0_info0_page_cfg_9_erase_en_9_wd = reg_wdata[15:12];

  assign bank0_info0_page_cfg_9_scramble_en_9_wd = reg_wdata[19:16];

  assign bank0_info0_page_cfg_9_ecc_en_9_wd = reg_wdata[23:20];

  assign bank0_info0_page_cfg_9_he_en_9_wd = reg_wdata[27:24];
  assign bank0_info1_regwen_we = addr_hit[57] & reg_we & !reg_error;

  assign bank0_info1_regwen_wd = reg_wdata[0];
  assign bank0_info1_page_cfg_we = addr_hit[58] & reg_we & !reg_error;

  assign bank0_info1_page_cfg_en_0_wd = reg_wdata[3:0];

  assign bank0_info1_page_cfg_rd_en_0_wd = reg_wdata[7:4];

  assign bank0_info1_page_cfg_prog_en_0_wd = reg_wdata[11:8];

  assign bank0_info1_page_cfg_erase_en_0_wd = reg_wdata[15:12];

  assign bank0_info1_page_cfg_scramble_en_0_wd = reg_wdata[19:16];

  assign bank0_info1_page_cfg_ecc_en_0_wd = reg_wdata[23:20];

  assign bank0_info1_page_cfg_he_en_0_wd = reg_wdata[27:24];
  assign bank0_info2_regwen_0_we = addr_hit[59] & reg_we & !reg_error;

  assign bank0_info2_regwen_0_wd = reg_wdata[0];
  assign bank0_info2_regwen_1_we = addr_hit[60] & reg_we & !reg_error;

  assign bank0_info2_regwen_1_wd = reg_wdata[0];
  assign bank0_info2_page_cfg_0_we = addr_hit[61] & reg_we & !reg_error;

  assign bank0_info2_page_cfg_0_en_0_wd = reg_wdata[3:0];

  assign bank0_info2_page_cfg_0_rd_en_0_wd = reg_wdata[7:4];

  assign bank0_info2_page_cfg_0_prog_en_0_wd = reg_wdata[11:8];

  assign bank0_info2_page_cfg_0_erase_en_0_wd = reg_wdata[15:12];

  assign bank0_info2_page_cfg_0_scramble_en_0_wd = reg_wdata[19:16];

  assign bank0_info2_page_cfg_0_ecc_en_0_wd = reg_wdata[23:20];

  assign bank0_info2_page_cfg_0_he_en_0_wd = reg_wdata[27:24];
  assign bank0_info2_page_cfg_1_we = addr_hit[62] & reg_we & !reg_error;

  assign bank0_info2_page_cfg_1_en_1_wd = reg_wdata[3:0];

  assign bank0_info2_page_cfg_1_rd_en_1_wd = reg_wdata[7:4];

  assign bank0_info2_page_cfg_1_prog_en_1_wd = reg_wdata[11:8];

  assign bank0_info2_page_cfg_1_erase_en_1_wd = reg_wdata[15:12];

  assign bank0_info2_page_cfg_1_scramble_en_1_wd = reg_wdata[19:16];

  assign bank0_info2_page_cfg_1_ecc_en_1_wd = reg_wdata[23:20];

  assign bank0_info2_page_cfg_1_he_en_1_wd = reg_wdata[27:24];
  assign bank1_info0_regwen_0_we = addr_hit[63] & reg_we & !reg_error;

  assign bank1_info0_regwen_0_wd = reg_wdata[0];
  assign bank1_info0_regwen_1_we = addr_hit[64] & reg_we & !reg_error;

  assign bank1_info0_regwen_1_wd = reg_wdata[0];
  assign bank1_info0_regwen_2_we = addr_hit[65] & reg_we & !reg_error;

  assign bank1_info0_regwen_2_wd = reg_wdata[0];
  assign bank1_info0_regwen_3_we = addr_hit[66] & reg_we & !reg_error;

  assign bank1_info0_regwen_3_wd = reg_wdata[0];
  assign bank1_info0_regwen_4_we = addr_hit[67] & reg_we & !reg_error;

  assign bank1_info0_regwen_4_wd = reg_wdata[0];
  assign bank1_info0_regwen_5_we = addr_hit[68] & reg_we & !reg_error;

  assign bank1_info0_regwen_5_wd = reg_wdata[0];
  assign bank1_info0_regwen_6_we = addr_hit[69] & reg_we & !reg_error;

  assign bank1_info0_regwen_6_wd = reg_wdata[0];
  assign bank1_info0_regwen_7_we = addr_hit[70] & reg_we & !reg_error;

  assign bank1_info0_regwen_7_wd = reg_wdata[0];
  assign bank1_info0_regwen_8_we = addr_hit[71] & reg_we & !reg_error;

  assign bank1_info0_regwen_8_wd = reg_wdata[0];
  assign bank1_info0_regwen_9_we = addr_hit[72] & reg_we & !reg_error;

  assign bank1_info0_regwen_9_wd = reg_wdata[0];
  assign bank1_info0_page_cfg_0_we = addr_hit[73] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_0_en_0_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_0_rd_en_0_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_0_prog_en_0_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_0_erase_en_0_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_0_scramble_en_0_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_0_ecc_en_0_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_0_he_en_0_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_1_we = addr_hit[74] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_1_en_1_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_1_rd_en_1_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_1_prog_en_1_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_1_erase_en_1_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_1_scramble_en_1_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_1_ecc_en_1_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_1_he_en_1_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_2_we = addr_hit[75] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_2_en_2_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_2_rd_en_2_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_2_prog_en_2_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_2_erase_en_2_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_2_scramble_en_2_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_2_ecc_en_2_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_2_he_en_2_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_3_we = addr_hit[76] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_3_en_3_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_3_rd_en_3_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_3_prog_en_3_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_3_erase_en_3_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_3_scramble_en_3_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_3_ecc_en_3_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_3_he_en_3_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_4_we = addr_hit[77] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_4_en_4_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_4_rd_en_4_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_4_prog_en_4_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_4_erase_en_4_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_4_scramble_en_4_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_4_ecc_en_4_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_4_he_en_4_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_5_we = addr_hit[78] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_5_en_5_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_5_rd_en_5_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_5_prog_en_5_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_5_erase_en_5_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_5_scramble_en_5_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_5_ecc_en_5_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_5_he_en_5_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_6_we = addr_hit[79] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_6_en_6_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_6_rd_en_6_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_6_prog_en_6_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_6_erase_en_6_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_6_scramble_en_6_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_6_ecc_en_6_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_6_he_en_6_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_7_we = addr_hit[80] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_7_en_7_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_7_rd_en_7_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_7_prog_en_7_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_7_erase_en_7_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_7_scramble_en_7_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_7_ecc_en_7_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_7_he_en_7_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_8_we = addr_hit[81] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_8_en_8_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_8_rd_en_8_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_8_prog_en_8_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_8_erase_en_8_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_8_scramble_en_8_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_8_ecc_en_8_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_8_he_en_8_wd = reg_wdata[27:24];
  assign bank1_info0_page_cfg_9_we = addr_hit[82] & reg_we & !reg_error;

  assign bank1_info0_page_cfg_9_en_9_wd = reg_wdata[3:0];

  assign bank1_info0_page_cfg_9_rd_en_9_wd = reg_wdata[7:4];

  assign bank1_info0_page_cfg_9_prog_en_9_wd = reg_wdata[11:8];

  assign bank1_info0_page_cfg_9_erase_en_9_wd = reg_wdata[15:12];

  assign bank1_info0_page_cfg_9_scramble_en_9_wd = reg_wdata[19:16];

  assign bank1_info0_page_cfg_9_ecc_en_9_wd = reg_wdata[23:20];

  assign bank1_info0_page_cfg_9_he_en_9_wd = reg_wdata[27:24];
  assign bank1_info1_regwen_we = addr_hit[83] & reg_we & !reg_error;

  assign bank1_info1_regwen_wd = reg_wdata[0];
  assign bank1_info1_page_cfg_we = addr_hit[84] & reg_we & !reg_error;

  assign bank1_info1_page_cfg_en_0_wd = reg_wdata[3:0];

  assign bank1_info1_page_cfg_rd_en_0_wd = reg_wdata[7:4];

  assign bank1_info1_page_cfg_prog_en_0_wd = reg_wdata[11:8];

  assign bank1_info1_page_cfg_erase_en_0_wd = reg_wdata[15:12];

  assign bank1_info1_page_cfg_scramble_en_0_wd = reg_wdata[19:16];

  assign bank1_info1_page_cfg_ecc_en_0_wd = reg_wdata[23:20];

  assign bank1_info1_page_cfg_he_en_0_wd = reg_wdata[27:24];
  assign bank1_info2_regwen_0_we = addr_hit[85] & reg_we & !reg_error;

  assign bank1_info2_regwen_0_wd = reg_wdata[0];
  assign bank1_info2_regwen_1_we = addr_hit[86] & reg_we & !reg_error;

  assign bank1_info2_regwen_1_wd = reg_wdata[0];
  assign bank1_info2_page_cfg_0_we = addr_hit[87] & reg_we & !reg_error;

  assign bank1_info2_page_cfg_0_en_0_wd = reg_wdata[3:0];

  assign bank1_info2_page_cfg_0_rd_en_0_wd = reg_wdata[7:4];

  assign bank1_info2_page_cfg_0_prog_en_0_wd = reg_wdata[11:8];

  assign bank1_info2_page_cfg_0_erase_en_0_wd = reg_wdata[15:12];

  assign bank1_info2_page_cfg_0_scramble_en_0_wd = reg_wdata[19:16];

  assign bank1_info2_page_cfg_0_ecc_en_0_wd = reg_wdata[23:20];

  assign bank1_info2_page_cfg_0_he_en_0_wd = reg_wdata[27:24];
  assign bank1_info2_page_cfg_1_we = addr_hit[88] & reg_we & !reg_error;

  assign bank1_info2_page_cfg_1_en_1_wd = reg_wdata[3:0];

  assign bank1_info2_page_cfg_1_rd_en_1_wd = reg_wdata[7:4];

  assign bank1_info2_page_cfg_1_prog_en_1_wd = reg_wdata[11:8];

  assign bank1_info2_page_cfg_1_erase_en_1_wd = reg_wdata[15:12];

  assign bank1_info2_page_cfg_1_scramble_en_1_wd = reg_wdata[19:16];

  assign bank1_info2_page_cfg_1_ecc_en_1_wd = reg_wdata[23:20];

  assign bank1_info2_page_cfg_1_he_en_1_wd = reg_wdata[27:24];
  assign bank_cfg_regwen_we = addr_hit[89] & reg_we & !reg_error;

  assign bank_cfg_regwen_wd = reg_wdata[0];
  assign mp_bank_cfg_shadowed_re = addr_hit[90] & reg_re & !reg_error;
  assign mp_bank_cfg_shadowed_we = addr_hit[90] & reg_we & !reg_error;

  assign mp_bank_cfg_shadowed_erase_en_0_wd = reg_wdata[0];

  assign mp_bank_cfg_shadowed_erase_en_1_wd = reg_wdata[1];
  assign op_status_we = addr_hit[91] & reg_we & !reg_error;

  assign op_status_done_wd = reg_wdata[0];

  assign op_status_err_wd = reg_wdata[1];
  assign debug_state_re = addr_hit[93] & reg_re & !reg_error;
  assign err_code_we = addr_hit[94] & reg_we & !reg_error;

  assign err_code_op_err_wd = reg_wdata[0];

  assign err_code_mp_err_wd = reg_wdata[1];

  assign err_code_rd_err_wd = reg_wdata[2];

  assign err_code_prog_err_wd = reg_wdata[3];

  assign err_code_prog_win_err_wd = reg_wdata[4];

  assign err_code_prog_type_err_wd = reg_wdata[5];

  assign err_code_update_err_wd = reg_wdata[6];

  assign err_code_macro_err_wd = reg_wdata[7];
  assign ecc_single_err_cnt_we = addr_hit[98] & reg_we & !reg_error;

  assign ecc_single_err_cnt_ecc_single_err_cnt_0_wd = reg_wdata[7:0];

  assign ecc_single_err_cnt_ecc_single_err_cnt_1_wd = reg_wdata[15:8];
  assign phy_alert_cfg_we = addr_hit[101] & reg_we & !reg_error;

  assign phy_alert_cfg_alert_ack_wd = reg_wdata[0];

  assign phy_alert_cfg_alert_trig_wd = reg_wdata[1];
  assign scratch_we = addr_hit[103] & reg_we & !reg_error;

  assign scratch_wd = reg_wdata[31:0];
  assign fifo_lvl_we = addr_hit[104] & reg_we & !reg_error;

  assign fifo_lvl_prog_wd = reg_wdata[4:0];

  assign fifo_lvl_rd_wd = reg_wdata[12:8];
  assign fifo_rst_we = addr_hit[105] & reg_we & !reg_error;

  assign fifo_rst_wd = reg_wdata[0];
  assign curr_fifo_lvl_re = addr_hit[106] & reg_re & !reg_error;

  // Assign write-enables to checker logic vector.
  always_comb begin
    reg_we_check = '0;
    reg_we_check[0] = intr_state_we;
    reg_we_check[1] = intr_enable_we;
    reg_we_check[2] = intr_test_we;
    reg_we_check[3] = alert_test_we;
    reg_we_check[4] = dis_we;
    reg_we_check[5] = exec_we;
    reg_we_check[6] = init_we;
    reg_we_check[7] = 1'b0;
    reg_we_check[8] = control_gated_we;
    reg_we_check[9] = addr_gated_we;
    reg_we_check[10] = prog_type_en_gated_we;
    reg_we_check[11] = erase_suspend_we;
    reg_we_check[12] = region_cfg_regwen_0_we;
    reg_we_check[13] = region_cfg_regwen_1_we;
    reg_we_check[14] = region_cfg_regwen_2_we;
    reg_we_check[15] = region_cfg_regwen_3_we;
    reg_we_check[16] = region_cfg_regwen_4_we;
    reg_we_check[17] = region_cfg_regwen_5_we;
    reg_we_check[18] = region_cfg_regwen_6_we;
    reg_we_check[19] = region_cfg_regwen_7_we;
    reg_we_check[20] = mp_region_cfg_0_gated_we;
    reg_we_check[21] = mp_region_cfg_1_gated_we;
    reg_we_check[22] = mp_region_cfg_2_gated_we;
    reg_we_check[23] = mp_region_cfg_3_gated_we;
    reg_we_check[24] = mp_region_cfg_4_gated_we;
    reg_we_check[25] = mp_region_cfg_5_gated_we;
    reg_we_check[26] = mp_region_cfg_6_gated_we;
    reg_we_check[27] = mp_region_cfg_7_gated_we;
    reg_we_check[28] = mp_region_0_gated_we;
    reg_we_check[29] = mp_region_1_gated_we;
    reg_we_check[30] = mp_region_2_gated_we;
    reg_we_check[31] = mp_region_3_gated_we;
    reg_we_check[32] = mp_region_4_gated_we;
    reg_we_check[33] = mp_region_5_gated_we;
    reg_we_check[34] = mp_region_6_gated_we;
    reg_we_check[35] = mp_region_7_gated_we;
    reg_we_check[36] = default_region_we;
    reg_we_check[37] = bank0_info0_regwen_0_we;
    reg_we_check[38] = bank0_info0_regwen_1_we;
    reg_we_check[39] = bank0_info0_regwen_2_we;
    reg_we_check[40] = bank0_info0_regwen_3_we;
    reg_we_check[41] = bank0_info0_regwen_4_we;
    reg_we_check[42] = bank0_info0_regwen_5_we;
    reg_we_check[43] = bank0_info0_regwen_6_we;
    reg_we_check[44] = bank0_info0_regwen_7_we;
    reg_we_check[45] = bank0_info0_regwen_8_we;
    reg_we_check[46] = bank0_info0_regwen_9_we;
    reg_we_check[47] = bank0_info0_page_cfg_0_gated_we;
    reg_we_check[48] = bank0_info0_page_cfg_1_gated_we;
    reg_we_check[49] = bank0_info0_page_cfg_2_gated_we;
    reg_we_check[50] = bank0_info0_page_cfg_3_gated_we;
    reg_we_check[51] = bank0_info0_page_cfg_4_gated_we;
    reg_we_check[52] = bank0_info0_page_cfg_5_gated_we;
    reg_we_check[53] = bank0_info0_page_cfg_6_gated_we;
    reg_we_check[54] = bank0_info0_page_cfg_7_gated_we;
    reg_we_check[55] = bank0_info0_page_cfg_8_gated_we;
    reg_we_check[56] = bank0_info0_page_cfg_9_gated_we;
    reg_we_check[57] = bank0_info1_regwen_we;
    reg_we_check[58] = bank0_info1_page_cfg_gated_we;
    reg_we_check[59] = bank0_info2_regwen_0_we;
    reg_we_check[60] = bank0_info2_regwen_1_we;
    reg_we_check[61] = bank0_info2_page_cfg_0_gated_we;
    reg_we_check[62] = bank0_info2_page_cfg_1_gated_we;
    reg_we_check[63] = bank1_info0_regwen_0_we;
    reg_we_check[64] = bank1_info0_regwen_1_we;
    reg_we_check[65] = bank1_info0_regwen_2_we;
    reg_we_check[66] = bank1_info0_regwen_3_we;
    reg_we_check[67] = bank1_info0_regwen_4_we;
    reg_we_check[68] = bank1_info0_regwen_5_we;
    reg_we_check[69] = bank1_info0_regwen_6_we;
    reg_we_check[70] = bank1_info0_regwen_7_we;
    reg_we_check[71] = bank1_info0_regwen_8_we;
    reg_we_check[72] = bank1_info0_regwen_9_we;
    reg_we_check[73] = bank1_info0_page_cfg_0_gated_we;
    reg_we_check[74] = bank1_info0_page_cfg_1_gated_we;
    reg_we_check[75] = bank1_info0_page_cfg_2_gated_we;
    reg_we_check[76] = bank1_info0_page_cfg_3_gated_we;
    reg_we_check[77] = bank1_info0_page_cfg_4_gated_we;
    reg_we_check[78] = bank1_info0_page_cfg_5_gated_we;
    reg_we_check[79] = bank1_info0_page_cfg_6_gated_we;
    reg_we_check[80] = bank1_info0_page_cfg_7_gated_we;
    reg_we_check[81] = bank1_info0_page_cfg_8_gated_we;
    reg_we_check[82] = bank1_info0_page_cfg_9_gated_we;
    reg_we_check[83] = bank1_info1_regwen_we;
    reg_we_check[84] = bank1_info1_page_cfg_gated_we;
    reg_we_check[85] = bank1_info2_regwen_0_we;
    reg_we_check[86] = bank1_info2_regwen_1_we;
    reg_we_check[87] = bank1_info2_page_cfg_0_gated_we;
    reg_we_check[88] = bank1_info2_page_cfg_1_gated_we;
    reg_we_check[89] = bank_cfg_regwen_we;
    reg_we_check[90] = mp_bank_cfg_shadowed_gated_we;
    reg_we_check[91] = op_status_we;
    reg_we_check[92] = 1'b0;
    reg_we_check[93] = 1'b0;
    reg_we_check[94] = err_code_we;
    reg_we_check[95] = 1'b0;
    reg_we_check[96] = 1'b0;
    reg_we_check[97] = 1'b0;
    reg_we_check[98] = ecc_single_err_cnt_we;
    reg_we_check[99] = 1'b0;
    reg_we_check[100] = 1'b0;
    reg_we_check[101] = phy_alert_cfg_we;
    reg_we_check[102] = 1'b0;
    reg_we_check[103] = scratch_we;
    reg_we_check[104] = fifo_lvl_we;
    reg_we_check[105] = fifo_rst_we;
    reg_we_check[106] = 1'b0;
  end

  // Read data return
  always_comb begin
    reg_rdata_next = '0;
    unique case (1'b1)
      addr_hit[0]: begin
        reg_rdata_next[0] = intr_state_prog_empty_qs;
        reg_rdata_next[1] = intr_state_prog_lvl_qs;
        reg_rdata_next[2] = intr_state_rd_full_qs;
        reg_rdata_next[3] = intr_state_rd_lvl_qs;
        reg_rdata_next[4] = intr_state_op_done_qs;
        reg_rdata_next[5] = intr_state_corr_err_qs;
      end

      addr_hit[1]: begin
        reg_rdata_next[0] = intr_enable_prog_empty_qs;
        reg_rdata_next[1] = intr_enable_prog_lvl_qs;
        reg_rdata_next[2] = intr_enable_rd_full_qs;
        reg_rdata_next[3] = intr_enable_rd_lvl_qs;
        reg_rdata_next[4] = intr_enable_op_done_qs;
        reg_rdata_next[5] = intr_enable_corr_err_qs;
      end

      addr_hit[2]: begin
        reg_rdata_next[0] = '0;
        reg_rdata_next[1] = '0;
        reg_rdata_next[2] = '0;
        reg_rdata_next[3] = '0;
        reg_rdata_next[4] = '0;
        reg_rdata_next[5] = '0;
      end

      addr_hit[3]: begin
        reg_rdata_next[0] = '0;
        reg_rdata_next[1] = '0;
        reg_rdata_next[2] = '0;
        reg_rdata_next[3] = '0;
        reg_rdata_next[4] = '0;
      end

      addr_hit[4]: begin
        reg_rdata_next[3:0] = dis_qs;
      end

      addr_hit[5]: begin
        reg_rdata_next[31:0] = exec_qs;
      end

      addr_hit[6]: begin
        reg_rdata_next[0] = init_qs;
      end

      addr_hit[7]: begin
        reg_rdata_next[0] = ctrl_regwen_qs;
      end

      addr_hit[8]: begin
        reg_rdata_next[0] = control_start_qs;
        reg_rdata_next[5:4] = control_op_qs;
        reg_rdata_next[6] = control_prog_sel_qs;
        reg_rdata_next[7] = control_erase_sel_qs;
        reg_rdata_next[8] = control_partition_sel_qs;
        reg_rdata_next[10:9] = control_info_sel_qs;
        reg_rdata_next[27:16] = control_num_qs;
      end

      addr_hit[9]: begin
        reg_rdata_next[19:0] = addr_qs;
      end

      addr_hit[10]: begin
        reg_rdata_next[0] = prog_type_en_normal_qs;
        reg_rdata_next[1] = prog_type_en_repair_qs;
      end

      addr_hit[11]: begin
        reg_rdata_next[0] = erase_suspend_qs;
      end

      addr_hit[12]: begin
        reg_rdata_next[0] = region_cfg_regwen_0_qs;
      end

      addr_hit[13]: begin
        reg_rdata_next[0] = region_cfg_regwen_1_qs;
      end

      addr_hit[14]: begin
        reg_rdata_next[0] = region_cfg_regwen_2_qs;
      end

      addr_hit[15]: begin
        reg_rdata_next[0] = region_cfg_regwen_3_qs;
      end

      addr_hit[16]: begin
        reg_rdata_next[0] = region_cfg_regwen_4_qs;
      end

      addr_hit[17]: begin
        reg_rdata_next[0] = region_cfg_regwen_5_qs;
      end

      addr_hit[18]: begin
        reg_rdata_next[0] = region_cfg_regwen_6_qs;
      end

      addr_hit[19]: begin
        reg_rdata_next[0] = region_cfg_regwen_7_qs;
      end

      addr_hit[20]: begin
        reg_rdata_next[3:0] = mp_region_cfg_0_en_0_qs;
        reg_rdata_next[7:4] = mp_region_cfg_0_rd_en_0_qs;
        reg_rdata_next[11:8] = mp_region_cfg_0_prog_en_0_qs;
        reg_rdata_next[15:12] = mp_region_cfg_0_erase_en_0_qs;
        reg_rdata_next[19:16] = mp_region_cfg_0_scramble_en_0_qs;
        reg_rdata_next[23:20] = mp_region_cfg_0_ecc_en_0_qs;
        reg_rdata_next[27:24] = mp_region_cfg_0_he_en_0_qs;
      end

      addr_hit[21]: begin
        reg_rdata_next[3:0] = mp_region_cfg_1_en_1_qs;
        reg_rdata_next[7:4] = mp_region_cfg_1_rd_en_1_qs;
        reg_rdata_next[11:8] = mp_region_cfg_1_prog_en_1_qs;
        reg_rdata_next[15:12] = mp_region_cfg_1_erase_en_1_qs;
        reg_rdata_next[19:16] = mp_region_cfg_1_scramble_en_1_qs;
        reg_rdata_next[23:20] = mp_region_cfg_1_ecc_en_1_qs;
        reg_rdata_next[27:24] = mp_region_cfg_1_he_en_1_qs;
      end

      addr_hit[22]: begin
        reg_rdata_next[3:0] = mp_region_cfg_2_en_2_qs;
        reg_rdata_next[7:4] = mp_region_cfg_2_rd_en_2_qs;
        reg_rdata_next[11:8] = mp_region_cfg_2_prog_en_2_qs;
        reg_rdata_next[15:12] = mp_region_cfg_2_erase_en_2_qs;
        reg_rdata_next[19:16] = mp_region_cfg_2_scramble_en_2_qs;
        reg_rdata_next[23:20] = mp_region_cfg_2_ecc_en_2_qs;
        reg_rdata_next[27:24] = mp_region_cfg_2_he_en_2_qs;
      end

      addr_hit[23]: begin
        reg_rdata_next[3:0] = mp_region_cfg_3_en_3_qs;
        reg_rdata_next[7:4] = mp_region_cfg_3_rd_en_3_qs;
        reg_rdata_next[11:8] = mp_region_cfg_3_prog_en_3_qs;
        reg_rdata_next[15:12] = mp_region_cfg_3_erase_en_3_qs;
        reg_rdata_next[19:16] = mp_region_cfg_3_scramble_en_3_qs;
        reg_rdata_next[23:20] = mp_region_cfg_3_ecc_en_3_qs;
        reg_rdata_next[27:24] = mp_region_cfg_3_he_en_3_qs;
      end

      addr_hit[24]: begin
        reg_rdata_next[3:0] = mp_region_cfg_4_en_4_qs;
        reg_rdata_next[7:4] = mp_region_cfg_4_rd_en_4_qs;
        reg_rdata_next[11:8] = mp_region_cfg_4_prog_en_4_qs;
        reg_rdata_next[15:12] = mp_region_cfg_4_erase_en_4_qs;
        reg_rdata_next[19:16] = mp_region_cfg_4_scramble_en_4_qs;
        reg_rdata_next[23:20] = mp_region_cfg_4_ecc_en_4_qs;
        reg_rdata_next[27:24] = mp_region_cfg_4_he_en_4_qs;
      end

      addr_hit[25]: begin
        reg_rdata_next[3:0] = mp_region_cfg_5_en_5_qs;
        reg_rdata_next[7:4] = mp_region_cfg_5_rd_en_5_qs;
        reg_rdata_next[11:8] = mp_region_cfg_5_prog_en_5_qs;
        reg_rdata_next[15:12] = mp_region_cfg_5_erase_en_5_qs;
        reg_rdata_next[19:16] = mp_region_cfg_5_scramble_en_5_qs;
        reg_rdata_next[23:20] = mp_region_cfg_5_ecc_en_5_qs;
        reg_rdata_next[27:24] = mp_region_cfg_5_he_en_5_qs;
      end

      addr_hit[26]: begin
        reg_rdata_next[3:0] = mp_region_cfg_6_en_6_qs;
        reg_rdata_next[7:4] = mp_region_cfg_6_rd_en_6_qs;
        reg_rdata_next[11:8] = mp_region_cfg_6_prog_en_6_qs;
        reg_rdata_next[15:12] = mp_region_cfg_6_erase_en_6_qs;
        reg_rdata_next[19:16] = mp_region_cfg_6_scramble_en_6_qs;
        reg_rdata_next[23:20] = mp_region_cfg_6_ecc_en_6_qs;
        reg_rdata_next[27:24] = mp_region_cfg_6_he_en_6_qs;
      end

      addr_hit[27]: begin
        reg_rdata_next[3:0] = mp_region_cfg_7_en_7_qs;
        reg_rdata_next[7:4] = mp_region_cfg_7_rd_en_7_qs;
        reg_rdata_next[11:8] = mp_region_cfg_7_prog_en_7_qs;
        reg_rdata_next[15:12] = mp_region_cfg_7_erase_en_7_qs;
        reg_rdata_next[19:16] = mp_region_cfg_7_scramble_en_7_qs;
        reg_rdata_next[23:20] = mp_region_cfg_7_ecc_en_7_qs;
        reg_rdata_next[27:24] = mp_region_cfg_7_he_en_7_qs;
      end

      addr_hit[28]: begin
        reg_rdata_next[8:0] = mp_region_0_base_0_qs;
        reg_rdata_next[18:9] = mp_region_0_size_0_qs;
      end

      addr_hit[29]: begin
        reg_rdata_next[8:0] = mp_region_1_base_1_qs;
        reg_rdata_next[18:9] = mp_region_1_size_1_qs;
      end

      addr_hit[30]: begin
        reg_rdata_next[8:0] = mp_region_2_base_2_qs;
        reg_rdata_next[18:9] = mp_region_2_size_2_qs;
      end

      addr_hit[31]: begin
        reg_rdata_next[8:0] = mp_region_3_base_3_qs;
        reg_rdata_next[18:9] = mp_region_3_size_3_qs;
      end

      addr_hit[32]: begin
        reg_rdata_next[8:0] = mp_region_4_base_4_qs;
        reg_rdata_next[18:9] = mp_region_4_size_4_qs;
      end

      addr_hit[33]: begin
        reg_rdata_next[8:0] = mp_region_5_base_5_qs;
        reg_rdata_next[18:9] = mp_region_5_size_5_qs;
      end

      addr_hit[34]: begin
        reg_rdata_next[8:0] = mp_region_6_base_6_qs;
        reg_rdata_next[18:9] = mp_region_6_size_6_qs;
      end

      addr_hit[35]: begin
        reg_rdata_next[8:0] = mp_region_7_base_7_qs;
        reg_rdata_next[18:9] = mp_region_7_size_7_qs;
      end

      addr_hit[36]: begin
        reg_rdata_next[3:0] = default_region_rd_en_qs;
        reg_rdata_next[7:4] = default_region_prog_en_qs;
        reg_rdata_next[11:8] = default_region_erase_en_qs;
        reg_rdata_next[15:12] = default_region_scramble_en_qs;
        reg_rdata_next[19:16] = default_region_ecc_en_qs;
        reg_rdata_next[23:20] = default_region_he_en_qs;
      end

      addr_hit[37]: begin
        reg_rdata_next[0] = bank0_info0_regwen_0_qs;
      end

      addr_hit[38]: begin
        reg_rdata_next[0] = bank0_info0_regwen_1_qs;
      end

      addr_hit[39]: begin
        reg_rdata_next[0] = bank0_info0_regwen_2_qs;
      end

      addr_hit[40]: begin
        reg_rdata_next[0] = bank0_info0_regwen_3_qs;
      end

      addr_hit[41]: begin
        reg_rdata_next[0] = bank0_info0_regwen_4_qs;
      end

      addr_hit[42]: begin
        reg_rdata_next[0] = bank0_info0_regwen_5_qs;
      end

      addr_hit[43]: begin
        reg_rdata_next[0] = bank0_info0_regwen_6_qs;
      end

      addr_hit[44]: begin
        reg_rdata_next[0] = bank0_info0_regwen_7_qs;
      end

      addr_hit[45]: begin
        reg_rdata_next[0] = bank0_info0_regwen_8_qs;
      end

      addr_hit[46]: begin
        reg_rdata_next[0] = bank0_info0_regwen_9_qs;
      end

      addr_hit[47]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_0_en_0_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_0_rd_en_0_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_0_prog_en_0_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_0_erase_en_0_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_0_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_0_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_0_he_en_0_qs;
      end

      addr_hit[48]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_1_en_1_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_1_rd_en_1_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_1_prog_en_1_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_1_erase_en_1_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_1_scramble_en_1_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_1_ecc_en_1_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_1_he_en_1_qs;
      end

      addr_hit[49]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_2_en_2_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_2_rd_en_2_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_2_prog_en_2_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_2_erase_en_2_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_2_scramble_en_2_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_2_ecc_en_2_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_2_he_en_2_qs;
      end

      addr_hit[50]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_3_en_3_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_3_rd_en_3_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_3_prog_en_3_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_3_erase_en_3_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_3_scramble_en_3_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_3_ecc_en_3_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_3_he_en_3_qs;
      end

      addr_hit[51]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_4_en_4_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_4_rd_en_4_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_4_prog_en_4_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_4_erase_en_4_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_4_scramble_en_4_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_4_ecc_en_4_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_4_he_en_4_qs;
      end

      addr_hit[52]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_5_en_5_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_5_rd_en_5_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_5_prog_en_5_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_5_erase_en_5_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_5_scramble_en_5_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_5_ecc_en_5_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_5_he_en_5_qs;
      end

      addr_hit[53]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_6_en_6_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_6_rd_en_6_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_6_prog_en_6_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_6_erase_en_6_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_6_scramble_en_6_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_6_ecc_en_6_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_6_he_en_6_qs;
      end

      addr_hit[54]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_7_en_7_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_7_rd_en_7_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_7_prog_en_7_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_7_erase_en_7_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_7_scramble_en_7_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_7_ecc_en_7_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_7_he_en_7_qs;
      end

      addr_hit[55]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_8_en_8_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_8_rd_en_8_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_8_prog_en_8_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_8_erase_en_8_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_8_scramble_en_8_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_8_ecc_en_8_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_8_he_en_8_qs;
      end

      addr_hit[56]: begin
        reg_rdata_next[3:0] = bank0_info0_page_cfg_9_en_9_qs;
        reg_rdata_next[7:4] = bank0_info0_page_cfg_9_rd_en_9_qs;
        reg_rdata_next[11:8] = bank0_info0_page_cfg_9_prog_en_9_qs;
        reg_rdata_next[15:12] = bank0_info0_page_cfg_9_erase_en_9_qs;
        reg_rdata_next[19:16] = bank0_info0_page_cfg_9_scramble_en_9_qs;
        reg_rdata_next[23:20] = bank0_info0_page_cfg_9_ecc_en_9_qs;
        reg_rdata_next[27:24] = bank0_info0_page_cfg_9_he_en_9_qs;
      end

      addr_hit[57]: begin
        reg_rdata_next[0] = bank0_info1_regwen_qs;
      end

      addr_hit[58]: begin
        reg_rdata_next[3:0] = bank0_info1_page_cfg_en_0_qs;
        reg_rdata_next[7:4] = bank0_info1_page_cfg_rd_en_0_qs;
        reg_rdata_next[11:8] = bank0_info1_page_cfg_prog_en_0_qs;
        reg_rdata_next[15:12] = bank0_info1_page_cfg_erase_en_0_qs;
        reg_rdata_next[19:16] = bank0_info1_page_cfg_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank0_info1_page_cfg_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank0_info1_page_cfg_he_en_0_qs;
      end

      addr_hit[59]: begin
        reg_rdata_next[0] = bank0_info2_regwen_0_qs;
      end

      addr_hit[60]: begin
        reg_rdata_next[0] = bank0_info2_regwen_1_qs;
      end

      addr_hit[61]: begin
        reg_rdata_next[3:0] = bank0_info2_page_cfg_0_en_0_qs;
        reg_rdata_next[7:4] = bank0_info2_page_cfg_0_rd_en_0_qs;
        reg_rdata_next[11:8] = bank0_info2_page_cfg_0_prog_en_0_qs;
        reg_rdata_next[15:12] = bank0_info2_page_cfg_0_erase_en_0_qs;
        reg_rdata_next[19:16] = bank0_info2_page_cfg_0_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank0_info2_page_cfg_0_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank0_info2_page_cfg_0_he_en_0_qs;
      end

      addr_hit[62]: begin
        reg_rdata_next[3:0] = bank0_info2_page_cfg_1_en_1_qs;
        reg_rdata_next[7:4] = bank0_info2_page_cfg_1_rd_en_1_qs;
        reg_rdata_next[11:8] = bank0_info2_page_cfg_1_prog_en_1_qs;
        reg_rdata_next[15:12] = bank0_info2_page_cfg_1_erase_en_1_qs;
        reg_rdata_next[19:16] = bank0_info2_page_cfg_1_scramble_en_1_qs;
        reg_rdata_next[23:20] = bank0_info2_page_cfg_1_ecc_en_1_qs;
        reg_rdata_next[27:24] = bank0_info2_page_cfg_1_he_en_1_qs;
      end

      addr_hit[63]: begin
        reg_rdata_next[0] = bank1_info0_regwen_0_qs;
      end

      addr_hit[64]: begin
        reg_rdata_next[0] = bank1_info0_regwen_1_qs;
      end

      addr_hit[65]: begin
        reg_rdata_next[0] = bank1_info0_regwen_2_qs;
      end

      addr_hit[66]: begin
        reg_rdata_next[0] = bank1_info0_regwen_3_qs;
      end

      addr_hit[67]: begin
        reg_rdata_next[0] = bank1_info0_regwen_4_qs;
      end

      addr_hit[68]: begin
        reg_rdata_next[0] = bank1_info0_regwen_5_qs;
      end

      addr_hit[69]: begin
        reg_rdata_next[0] = bank1_info0_regwen_6_qs;
      end

      addr_hit[70]: begin
        reg_rdata_next[0] = bank1_info0_regwen_7_qs;
      end

      addr_hit[71]: begin
        reg_rdata_next[0] = bank1_info0_regwen_8_qs;
      end

      addr_hit[72]: begin
        reg_rdata_next[0] = bank1_info0_regwen_9_qs;
      end

      addr_hit[73]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_0_en_0_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_0_rd_en_0_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_0_prog_en_0_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_0_erase_en_0_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_0_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_0_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_0_he_en_0_qs;
      end

      addr_hit[74]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_1_en_1_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_1_rd_en_1_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_1_prog_en_1_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_1_erase_en_1_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_1_scramble_en_1_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_1_ecc_en_1_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_1_he_en_1_qs;
      end

      addr_hit[75]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_2_en_2_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_2_rd_en_2_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_2_prog_en_2_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_2_erase_en_2_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_2_scramble_en_2_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_2_ecc_en_2_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_2_he_en_2_qs;
      end

      addr_hit[76]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_3_en_3_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_3_rd_en_3_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_3_prog_en_3_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_3_erase_en_3_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_3_scramble_en_3_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_3_ecc_en_3_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_3_he_en_3_qs;
      end

      addr_hit[77]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_4_en_4_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_4_rd_en_4_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_4_prog_en_4_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_4_erase_en_4_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_4_scramble_en_4_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_4_ecc_en_4_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_4_he_en_4_qs;
      end

      addr_hit[78]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_5_en_5_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_5_rd_en_5_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_5_prog_en_5_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_5_erase_en_5_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_5_scramble_en_5_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_5_ecc_en_5_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_5_he_en_5_qs;
      end

      addr_hit[79]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_6_en_6_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_6_rd_en_6_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_6_prog_en_6_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_6_erase_en_6_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_6_scramble_en_6_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_6_ecc_en_6_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_6_he_en_6_qs;
      end

      addr_hit[80]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_7_en_7_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_7_rd_en_7_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_7_prog_en_7_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_7_erase_en_7_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_7_scramble_en_7_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_7_ecc_en_7_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_7_he_en_7_qs;
      end

      addr_hit[81]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_8_en_8_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_8_rd_en_8_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_8_prog_en_8_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_8_erase_en_8_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_8_scramble_en_8_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_8_ecc_en_8_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_8_he_en_8_qs;
      end

      addr_hit[82]: begin
        reg_rdata_next[3:0] = bank1_info0_page_cfg_9_en_9_qs;
        reg_rdata_next[7:4] = bank1_info0_page_cfg_9_rd_en_9_qs;
        reg_rdata_next[11:8] = bank1_info0_page_cfg_9_prog_en_9_qs;
        reg_rdata_next[15:12] = bank1_info0_page_cfg_9_erase_en_9_qs;
        reg_rdata_next[19:16] = bank1_info0_page_cfg_9_scramble_en_9_qs;
        reg_rdata_next[23:20] = bank1_info0_page_cfg_9_ecc_en_9_qs;
        reg_rdata_next[27:24] = bank1_info0_page_cfg_9_he_en_9_qs;
      end

      addr_hit[83]: begin
        reg_rdata_next[0] = bank1_info1_regwen_qs;
      end

      addr_hit[84]: begin
        reg_rdata_next[3:0] = bank1_info1_page_cfg_en_0_qs;
        reg_rdata_next[7:4] = bank1_info1_page_cfg_rd_en_0_qs;
        reg_rdata_next[11:8] = bank1_info1_page_cfg_prog_en_0_qs;
        reg_rdata_next[15:12] = bank1_info1_page_cfg_erase_en_0_qs;
        reg_rdata_next[19:16] = bank1_info1_page_cfg_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank1_info1_page_cfg_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank1_info1_page_cfg_he_en_0_qs;
      end

      addr_hit[85]: begin
        reg_rdata_next[0] = bank1_info2_regwen_0_qs;
      end

      addr_hit[86]: begin
        reg_rdata_next[0] = bank1_info2_regwen_1_qs;
      end

      addr_hit[87]: begin
        reg_rdata_next[3:0] = bank1_info2_page_cfg_0_en_0_qs;
        reg_rdata_next[7:4] = bank1_info2_page_cfg_0_rd_en_0_qs;
        reg_rdata_next[11:8] = bank1_info2_page_cfg_0_prog_en_0_qs;
        reg_rdata_next[15:12] = bank1_info2_page_cfg_0_erase_en_0_qs;
        reg_rdata_next[19:16] = bank1_info2_page_cfg_0_scramble_en_0_qs;
        reg_rdata_next[23:20] = bank1_info2_page_cfg_0_ecc_en_0_qs;
        reg_rdata_next[27:24] = bank1_info2_page_cfg_0_he_en_0_qs;
      end

      addr_hit[88]: begin
        reg_rdata_next[3:0] = bank1_info2_page_cfg_1_en_1_qs;
        reg_rdata_next[7:4] = bank1_info2_page_cfg_1_rd_en_1_qs;
        reg_rdata_next[11:8] = bank1_info2_page_cfg_1_prog_en_1_qs;
        reg_rdata_next[15:12] = bank1_info2_page_cfg_1_erase_en_1_qs;
        reg_rdata_next[19:16] = bank1_info2_page_cfg_1_scramble_en_1_qs;
        reg_rdata_next[23:20] = bank1_info2_page_cfg_1_ecc_en_1_qs;
        reg_rdata_next[27:24] = bank1_info2_page_cfg_1_he_en_1_qs;
      end

      addr_hit[89]: begin
        reg_rdata_next[0] = bank_cfg_regwen_qs;
      end

      addr_hit[90]: begin
        reg_rdata_next[0] = mp_bank_cfg_shadowed_erase_en_0_qs;
        reg_rdata_next[1] = mp_bank_cfg_shadowed_erase_en_1_qs;
      end

      addr_hit[91]: begin
        reg_rdata_next[0] = op_status_done_qs;
        reg_rdata_next[1] = op_status_err_qs;
      end

      addr_hit[92]: begin
        reg_rdata_next[0] = status_rd_full_qs;
        reg_rdata_next[1] = status_rd_empty_qs;
        reg_rdata_next[2] = status_prog_full_qs;
        reg_rdata_next[3] = status_prog_empty_qs;
        reg_rdata_next[4] = status_init_wip_qs;
      end

      addr_hit[93]: begin
        reg_rdata_next[10:0] = debug_state_qs;
      end

      addr_hit[94]: begin
        reg_rdata_next[0] = err_code_op_err_qs;
        reg_rdata_next[1] = err_code_mp_err_qs;
        reg_rdata_next[2] = err_code_rd_err_qs;
        reg_rdata_next[3] = err_code_prog_err_qs;
        reg_rdata_next[4] = err_code_prog_win_err_qs;
        reg_rdata_next[5] = err_code_prog_type_err_qs;
        reg_rdata_next[6] = err_code_update_err_qs;
        reg_rdata_next[7] = err_code_macro_err_qs;
      end

      addr_hit[95]: begin
        reg_rdata_next[0] = std_fault_status_reg_intg_err_qs;
        reg_rdata_next[1] = std_fault_status_prog_intg_err_qs;
        reg_rdata_next[2] = std_fault_status_lcmgr_err_qs;
        reg_rdata_next[3] = std_fault_status_lcmgr_intg_err_qs;
        reg_rdata_next[4] = std_fault_status_arb_fsm_err_qs;
        reg_rdata_next[5] = std_fault_status_storage_err_qs;
        reg_rdata_next[6] = std_fault_status_phy_fsm_err_qs;
        reg_rdata_next[7] = std_fault_status_ctrl_cnt_err_qs;
        reg_rdata_next[8] = std_fault_status_fifo_err_qs;
      end

      addr_hit[96]: begin
        reg_rdata_next[0] = fault_status_op_err_qs;
        reg_rdata_next[1] = fault_status_mp_err_qs;
        reg_rdata_next[2] = fault_status_rd_err_qs;
        reg_rdata_next[3] = fault_status_prog_err_qs;
        reg_rdata_next[4] = fault_status_prog_win_err_qs;
        reg_rdata_next[5] = fault_status_prog_type_err_qs;
        reg_rdata_next[6] = fault_status_seed_err_qs;
        reg_rdata_next[7] = fault_status_phy_relbl_err_qs;
        reg_rdata_next[8] = fault_status_phy_storage_err_qs;
        reg_rdata_next[9] = fault_status_spurious_ack_qs;
        reg_rdata_next[10] = fault_status_arb_err_qs;
        reg_rdata_next[11] = fault_status_host_gnt_err_qs;
      end

      addr_hit[97]: begin
        reg_rdata_next[19:0] = err_addr_qs;
      end

      addr_hit[98]: begin
        reg_rdata_next[7:0] = ecc_single_err_cnt_ecc_single_err_cnt_0_qs;
        reg_rdata_next[15:8] = ecc_single_err_cnt_ecc_single_err_cnt_1_qs;
      end

      addr_hit[99]: begin
        reg_rdata_next[19:0] = ecc_single_err_addr_0_qs;
      end

      addr_hit[100]: begin
        reg_rdata_next[19:0] = ecc_single_err_addr_1_qs;
      end

      addr_hit[101]: begin
        reg_rdata_next[0] = phy_alert_cfg_alert_ack_qs;
        reg_rdata_next[1] = phy_alert_cfg_alert_trig_qs;
      end

      addr_hit[102]: begin
        reg_rdata_next[0] = phy_status_init_wip_qs;
        reg_rdata_next[1] = phy_status_prog_normal_avail_qs;
        reg_rdata_next[2] = phy_status_prog_repair_avail_qs;
      end

      addr_hit[103]: begin
        reg_rdata_next[31:0] = scratch_qs;
      end

      addr_hit[104]: begin
        reg_rdata_next[4:0] = fifo_lvl_prog_qs;
        reg_rdata_next[12:8] = fifo_lvl_rd_qs;
      end

      addr_hit[105]: begin
        reg_rdata_next[0] = fifo_rst_qs;
      end

      addr_hit[106]: begin
        reg_rdata_next[4:0] = curr_fifo_lvl_prog_qs;
        reg_rdata_next[12:8] = curr_fifo_lvl_rd_qs;
      end

      default: begin
        reg_rdata_next = '1;
      end
    endcase
  end

  // shadow busy
  logic shadow_busy;
  logic rst_done;
  logic shadow_rst_done;
  always_ff @(posedge clk_i or negedge rst_ni) begin
    if (!rst_ni) begin
      rst_done <= '0;
    end else begin
      rst_done <= 1'b1;
    end
  end

  always_ff @(posedge clk_i or negedge rst_shadowed_ni) begin
    if (!rst_shadowed_ni) begin
      shadow_rst_done <= '0;
    end else begin
      shadow_rst_done <= 1'b1;
    end
  end

  // both shadow and normal resets have been released
  assign shadow_busy = ~(rst_done & shadow_rst_done);

  // Collect up storage and update errors
  assign shadowed_storage_err_o = |{
    mp_bank_cfg_shadowed_erase_en_0_storage_err,
    mp_bank_cfg_shadowed_erase_en_1_storage_err
  };
  assign shadowed_update_err_o = |{
    mp_bank_cfg_shadowed_erase_en_0_update_err,
    mp_bank_cfg_shadowed_erase_en_1_update_err
  };

  // register busy
  assign reg_busy = shadow_busy;

  // Unused signal tieoff

  // wdata / byte enable are not always fully used
  // add a blanket unused statement to handle lint waivers
  logic unused_wdata;
  logic unused_be;
  assign unused_wdata = ^reg_wdata;
  assign unused_be = ^reg_be;

  // Assertions for Register Interface
  `ASSERT_PULSE(wePulse, reg_we, clk_i, !rst_ni)
  `ASSERT_PULSE(rePulse, reg_re, clk_i, !rst_ni)

  `ASSERT(reAfterRv, $rose(reg_re || reg_we) |=> tl_o_pre.d_valid, clk_i, !rst_ni)

  `ASSERT(en2addrHit, (reg_we || reg_re) |-> $onehot0(addr_hit), clk_i, !rst_ni)

  // this is formulated as an assumption such that the FPV testbenches do disprove this
  // property by mistake
  //`ASSUME(reqParity, tl_reg_h2d.a_valid |-> tl_reg_h2d.a_user.chk_en == tlul_pkg::CheckDis)

endmodule
