// Copyright lowRISC contributors.
// Licensed under the Apache License, Version 2.0, see LICENSE for details.
// SPDX-License-Identifier: Apache-2.0
//
// Register Top module auto-generated by `reggen`

`include "prim_assert.sv"

module alert_handler_reg_top (
  input clk_i,
  input rst_ni,
  input rst_shadowed_ni,
  input  tlul_pkg::tl_h2d_t tl_i,
  output tlul_pkg::tl_d2h_t tl_o,
  // To HW
  output alert_handler_reg_pkg::alert_handler_reg2hw_t reg2hw, // Write
  input  alert_handler_reg_pkg::alert_handler_hw2reg_t hw2reg, // Read

  output logic shadowed_storage_err_o,
  output logic shadowed_update_err_o,

  // Integrity check errors
  output logic intg_err_o,

  // Config
  input devmode_i // If 1, explicit error return for unmapped register access
);

  import alert_handler_reg_pkg::* ;

  localparam int AW = 11;
  localparam int DW = 32;
  localparam int DBW = DW/8;                    // Byte Width

  // register signals
  logic           reg_we;
  logic           reg_re;
  logic [AW-1:0]  reg_addr;
  logic [DW-1:0]  reg_wdata;
  logic [DBW-1:0] reg_be;
  logic [DW-1:0]  reg_rdata;
  logic           reg_error;

  logic          addrmiss, wr_err;

  logic [DW-1:0] reg_rdata_next;
  logic reg_busy;

  tlul_pkg::tl_h2d_t tl_reg_h2d;
  tlul_pkg::tl_d2h_t tl_reg_d2h;


  // incoming payload check
  logic intg_err;
  tlul_cmd_intg_chk u_chk (
    .tl_i(tl_i),
    .err_o(intg_err)
  );

  // also check for spurious write enables
  logic reg_we_err;
  logic [349:0] reg_we_check;
  prim_reg_we_check #(
    .OneHotWidth(350)
  ) u_prim_reg_we_check (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .oh_i  (reg_we_check),
    .en_i  (reg_we && !addrmiss),
    .err_o (reg_we_err)
  );

  logic err_q;
  always_ff @(posedge clk_i or negedge rst_ni) begin
    if (!rst_ni) begin
      err_q <= '0;
    end else if (intg_err || reg_we_err) begin
      err_q <= 1'b1;
    end
  end

  // integrity error output is permanent and should be used for alert generation
  // register errors are transactional
  assign intg_err_o = err_q | intg_err | reg_we_err;

  // outgoing integrity generation
  tlul_pkg::tl_d2h_t tl_o_pre;
  tlul_rsp_intg_gen #(
    .EnableRspIntgGen(1),
    .EnableDataIntgGen(1)
  ) u_rsp_intg_gen (
    .tl_i(tl_o_pre),
    .tl_o(tl_o)
  );

  assign tl_reg_h2d = tl_i;
  assign tl_o_pre   = tl_reg_d2h;

  tlul_adapter_reg #(
    .RegAw(AW),
    .RegDw(DW),
    .EnableDataIntgGen(0)
  ) u_reg_if (
    .clk_i  (clk_i),
    .rst_ni (rst_ni),

    .tl_i (tl_reg_h2d),
    .tl_o (tl_reg_d2h),

    .en_ifetch_i(prim_mubi_pkg::MuBi4False),
    .intg_error_o(),

    .we_o    (reg_we),
    .re_o    (reg_re),
    .addr_o  (reg_addr),
    .wdata_o (reg_wdata),
    .be_o    (reg_be),
    .busy_i  (reg_busy),
    .rdata_i (reg_rdata),
    .error_i (reg_error)
  );

  // cdc oversampling signals

  assign reg_rdata = reg_rdata_next ;
  assign reg_error = (devmode_i & addrmiss) | wr_err | intg_err;

  // Define SW related signals
  // Format: <reg>_<field>_{wd|we|qs}
  //        or <reg>_{wd|we|qs} if field == 1 or 0
  logic intr_state_we;
  logic intr_state_classa_qs;
  logic intr_state_classa_wd;
  logic intr_state_classb_qs;
  logic intr_state_classb_wd;
  logic intr_state_classc_qs;
  logic intr_state_classc_wd;
  logic intr_state_classd_qs;
  logic intr_state_classd_wd;
  logic intr_enable_we;
  logic intr_enable_classa_qs;
  logic intr_enable_classa_wd;
  logic intr_enable_classb_qs;
  logic intr_enable_classb_wd;
  logic intr_enable_classc_qs;
  logic intr_enable_classc_wd;
  logic intr_enable_classd_qs;
  logic intr_enable_classd_wd;
  logic intr_test_we;
  logic intr_test_classa_wd;
  logic intr_test_classb_wd;
  logic intr_test_classc_wd;
  logic intr_test_classd_wd;
  logic ping_timer_regwen_we;
  logic ping_timer_regwen_qs;
  logic ping_timer_regwen_wd;
  logic ping_timeout_cyc_shadowed_re;
  logic ping_timeout_cyc_shadowed_we;
  logic [15:0] ping_timeout_cyc_shadowed_qs;
  logic [15:0] ping_timeout_cyc_shadowed_wd;
  logic ping_timeout_cyc_shadowed_storage_err;
  logic ping_timeout_cyc_shadowed_update_err;
  logic ping_timer_en_shadowed_re;
  logic ping_timer_en_shadowed_we;
  logic ping_timer_en_shadowed_qs;
  logic ping_timer_en_shadowed_wd;
  logic ping_timer_en_shadowed_storage_err;
  logic ping_timer_en_shadowed_update_err;
  logic alert_regwen_0_we;
  logic alert_regwen_0_qs;
  logic alert_regwen_0_wd;
  logic alert_regwen_1_we;
  logic alert_regwen_1_qs;
  logic alert_regwen_1_wd;
  logic alert_regwen_2_we;
  logic alert_regwen_2_qs;
  logic alert_regwen_2_wd;
  logic alert_regwen_3_we;
  logic alert_regwen_3_qs;
  logic alert_regwen_3_wd;
  logic alert_regwen_4_we;
  logic alert_regwen_4_qs;
  logic alert_regwen_4_wd;
  logic alert_regwen_5_we;
  logic alert_regwen_5_qs;
  logic alert_regwen_5_wd;
  logic alert_regwen_6_we;
  logic alert_regwen_6_qs;
  logic alert_regwen_6_wd;
  logic alert_regwen_7_we;
  logic alert_regwen_7_qs;
  logic alert_regwen_7_wd;
  logic alert_regwen_8_we;
  logic alert_regwen_8_qs;
  logic alert_regwen_8_wd;
  logic alert_regwen_9_we;
  logic alert_regwen_9_qs;
  logic alert_regwen_9_wd;
  logic alert_regwen_10_we;
  logic alert_regwen_10_qs;
  logic alert_regwen_10_wd;
  logic alert_regwen_11_we;
  logic alert_regwen_11_qs;
  logic alert_regwen_11_wd;
  logic alert_regwen_12_we;
  logic alert_regwen_12_qs;
  logic alert_regwen_12_wd;
  logic alert_regwen_13_we;
  logic alert_regwen_13_qs;
  logic alert_regwen_13_wd;
  logic alert_regwen_14_we;
  logic alert_regwen_14_qs;
  logic alert_regwen_14_wd;
  logic alert_regwen_15_we;
  logic alert_regwen_15_qs;
  logic alert_regwen_15_wd;
  logic alert_regwen_16_we;
  logic alert_regwen_16_qs;
  logic alert_regwen_16_wd;
  logic alert_regwen_17_we;
  logic alert_regwen_17_qs;
  logic alert_regwen_17_wd;
  logic alert_regwen_18_we;
  logic alert_regwen_18_qs;
  logic alert_regwen_18_wd;
  logic alert_regwen_19_we;
  logic alert_regwen_19_qs;
  logic alert_regwen_19_wd;
  logic alert_regwen_20_we;
  logic alert_regwen_20_qs;
  logic alert_regwen_20_wd;
  logic alert_regwen_21_we;
  logic alert_regwen_21_qs;
  logic alert_regwen_21_wd;
  logic alert_regwen_22_we;
  logic alert_regwen_22_qs;
  logic alert_regwen_22_wd;
  logic alert_regwen_23_we;
  logic alert_regwen_23_qs;
  logic alert_regwen_23_wd;
  logic alert_regwen_24_we;
  logic alert_regwen_24_qs;
  logic alert_regwen_24_wd;
  logic alert_regwen_25_we;
  logic alert_regwen_25_qs;
  logic alert_regwen_25_wd;
  logic alert_regwen_26_we;
  logic alert_regwen_26_qs;
  logic alert_regwen_26_wd;
  logic alert_regwen_27_we;
  logic alert_regwen_27_qs;
  logic alert_regwen_27_wd;
  logic alert_regwen_28_we;
  logic alert_regwen_28_qs;
  logic alert_regwen_28_wd;
  logic alert_regwen_29_we;
  logic alert_regwen_29_qs;
  logic alert_regwen_29_wd;
  logic alert_regwen_30_we;
  logic alert_regwen_30_qs;
  logic alert_regwen_30_wd;
  logic alert_regwen_31_we;
  logic alert_regwen_31_qs;
  logic alert_regwen_31_wd;
  logic alert_regwen_32_we;
  logic alert_regwen_32_qs;
  logic alert_regwen_32_wd;
  logic alert_regwen_33_we;
  logic alert_regwen_33_qs;
  logic alert_regwen_33_wd;
  logic alert_regwen_34_we;
  logic alert_regwen_34_qs;
  logic alert_regwen_34_wd;
  logic alert_regwen_35_we;
  logic alert_regwen_35_qs;
  logic alert_regwen_35_wd;
  logic alert_regwen_36_we;
  logic alert_regwen_36_qs;
  logic alert_regwen_36_wd;
  logic alert_regwen_37_we;
  logic alert_regwen_37_qs;
  logic alert_regwen_37_wd;
  logic alert_regwen_38_we;
  logic alert_regwen_38_qs;
  logic alert_regwen_38_wd;
  logic alert_regwen_39_we;
  logic alert_regwen_39_qs;
  logic alert_regwen_39_wd;
  logic alert_regwen_40_we;
  logic alert_regwen_40_qs;
  logic alert_regwen_40_wd;
  logic alert_regwen_41_we;
  logic alert_regwen_41_qs;
  logic alert_regwen_41_wd;
  logic alert_regwen_42_we;
  logic alert_regwen_42_qs;
  logic alert_regwen_42_wd;
  logic alert_regwen_43_we;
  logic alert_regwen_43_qs;
  logic alert_regwen_43_wd;
  logic alert_regwen_44_we;
  logic alert_regwen_44_qs;
  logic alert_regwen_44_wd;
  logic alert_regwen_45_we;
  logic alert_regwen_45_qs;
  logic alert_regwen_45_wd;
  logic alert_regwen_46_we;
  logic alert_regwen_46_qs;
  logic alert_regwen_46_wd;
  logic alert_regwen_47_we;
  logic alert_regwen_47_qs;
  logic alert_regwen_47_wd;
  logic alert_regwen_48_we;
  logic alert_regwen_48_qs;
  logic alert_regwen_48_wd;
  logic alert_regwen_49_we;
  logic alert_regwen_49_qs;
  logic alert_regwen_49_wd;
  logic alert_regwen_50_we;
  logic alert_regwen_50_qs;
  logic alert_regwen_50_wd;
  logic alert_regwen_51_we;
  logic alert_regwen_51_qs;
  logic alert_regwen_51_wd;
  logic alert_regwen_52_we;
  logic alert_regwen_52_qs;
  logic alert_regwen_52_wd;
  logic alert_regwen_53_we;
  logic alert_regwen_53_qs;
  logic alert_regwen_53_wd;
  logic alert_regwen_54_we;
  logic alert_regwen_54_qs;
  logic alert_regwen_54_wd;
  logic alert_regwen_55_we;
  logic alert_regwen_55_qs;
  logic alert_regwen_55_wd;
  logic alert_regwen_56_we;
  logic alert_regwen_56_qs;
  logic alert_regwen_56_wd;
  logic alert_regwen_57_we;
  logic alert_regwen_57_qs;
  logic alert_regwen_57_wd;
  logic alert_regwen_58_we;
  logic alert_regwen_58_qs;
  logic alert_regwen_58_wd;
  logic alert_regwen_59_we;
  logic alert_regwen_59_qs;
  logic alert_regwen_59_wd;
  logic alert_regwen_60_we;
  logic alert_regwen_60_qs;
  logic alert_regwen_60_wd;
  logic alert_regwen_61_we;
  logic alert_regwen_61_qs;
  logic alert_regwen_61_wd;
  logic alert_regwen_62_we;
  logic alert_regwen_62_qs;
  logic alert_regwen_62_wd;
  logic alert_regwen_63_we;
  logic alert_regwen_63_qs;
  logic alert_regwen_63_wd;
  logic alert_regwen_64_we;
  logic alert_regwen_64_qs;
  logic alert_regwen_64_wd;
  logic alert_en_shadowed_0_re;
  logic alert_en_shadowed_0_we;
  logic alert_en_shadowed_0_qs;
  logic alert_en_shadowed_0_wd;
  logic alert_en_shadowed_0_storage_err;
  logic alert_en_shadowed_0_update_err;
  logic alert_en_shadowed_1_re;
  logic alert_en_shadowed_1_we;
  logic alert_en_shadowed_1_qs;
  logic alert_en_shadowed_1_wd;
  logic alert_en_shadowed_1_storage_err;
  logic alert_en_shadowed_1_update_err;
  logic alert_en_shadowed_2_re;
  logic alert_en_shadowed_2_we;
  logic alert_en_shadowed_2_qs;
  logic alert_en_shadowed_2_wd;
  logic alert_en_shadowed_2_storage_err;
  logic alert_en_shadowed_2_update_err;
  logic alert_en_shadowed_3_re;
  logic alert_en_shadowed_3_we;
  logic alert_en_shadowed_3_qs;
  logic alert_en_shadowed_3_wd;
  logic alert_en_shadowed_3_storage_err;
  logic alert_en_shadowed_3_update_err;
  logic alert_en_shadowed_4_re;
  logic alert_en_shadowed_4_we;
  logic alert_en_shadowed_4_qs;
  logic alert_en_shadowed_4_wd;
  logic alert_en_shadowed_4_storage_err;
  logic alert_en_shadowed_4_update_err;
  logic alert_en_shadowed_5_re;
  logic alert_en_shadowed_5_we;
  logic alert_en_shadowed_5_qs;
  logic alert_en_shadowed_5_wd;
  logic alert_en_shadowed_5_storage_err;
  logic alert_en_shadowed_5_update_err;
  logic alert_en_shadowed_6_re;
  logic alert_en_shadowed_6_we;
  logic alert_en_shadowed_6_qs;
  logic alert_en_shadowed_6_wd;
  logic alert_en_shadowed_6_storage_err;
  logic alert_en_shadowed_6_update_err;
  logic alert_en_shadowed_7_re;
  logic alert_en_shadowed_7_we;
  logic alert_en_shadowed_7_qs;
  logic alert_en_shadowed_7_wd;
  logic alert_en_shadowed_7_storage_err;
  logic alert_en_shadowed_7_update_err;
  logic alert_en_shadowed_8_re;
  logic alert_en_shadowed_8_we;
  logic alert_en_shadowed_8_qs;
  logic alert_en_shadowed_8_wd;
  logic alert_en_shadowed_8_storage_err;
  logic alert_en_shadowed_8_update_err;
  logic alert_en_shadowed_9_re;
  logic alert_en_shadowed_9_we;
  logic alert_en_shadowed_9_qs;
  logic alert_en_shadowed_9_wd;
  logic alert_en_shadowed_9_storage_err;
  logic alert_en_shadowed_9_update_err;
  logic alert_en_shadowed_10_re;
  logic alert_en_shadowed_10_we;
  logic alert_en_shadowed_10_qs;
  logic alert_en_shadowed_10_wd;
  logic alert_en_shadowed_10_storage_err;
  logic alert_en_shadowed_10_update_err;
  logic alert_en_shadowed_11_re;
  logic alert_en_shadowed_11_we;
  logic alert_en_shadowed_11_qs;
  logic alert_en_shadowed_11_wd;
  logic alert_en_shadowed_11_storage_err;
  logic alert_en_shadowed_11_update_err;
  logic alert_en_shadowed_12_re;
  logic alert_en_shadowed_12_we;
  logic alert_en_shadowed_12_qs;
  logic alert_en_shadowed_12_wd;
  logic alert_en_shadowed_12_storage_err;
  logic alert_en_shadowed_12_update_err;
  logic alert_en_shadowed_13_re;
  logic alert_en_shadowed_13_we;
  logic alert_en_shadowed_13_qs;
  logic alert_en_shadowed_13_wd;
  logic alert_en_shadowed_13_storage_err;
  logic alert_en_shadowed_13_update_err;
  logic alert_en_shadowed_14_re;
  logic alert_en_shadowed_14_we;
  logic alert_en_shadowed_14_qs;
  logic alert_en_shadowed_14_wd;
  logic alert_en_shadowed_14_storage_err;
  logic alert_en_shadowed_14_update_err;
  logic alert_en_shadowed_15_re;
  logic alert_en_shadowed_15_we;
  logic alert_en_shadowed_15_qs;
  logic alert_en_shadowed_15_wd;
  logic alert_en_shadowed_15_storage_err;
  logic alert_en_shadowed_15_update_err;
  logic alert_en_shadowed_16_re;
  logic alert_en_shadowed_16_we;
  logic alert_en_shadowed_16_qs;
  logic alert_en_shadowed_16_wd;
  logic alert_en_shadowed_16_storage_err;
  logic alert_en_shadowed_16_update_err;
  logic alert_en_shadowed_17_re;
  logic alert_en_shadowed_17_we;
  logic alert_en_shadowed_17_qs;
  logic alert_en_shadowed_17_wd;
  logic alert_en_shadowed_17_storage_err;
  logic alert_en_shadowed_17_update_err;
  logic alert_en_shadowed_18_re;
  logic alert_en_shadowed_18_we;
  logic alert_en_shadowed_18_qs;
  logic alert_en_shadowed_18_wd;
  logic alert_en_shadowed_18_storage_err;
  logic alert_en_shadowed_18_update_err;
  logic alert_en_shadowed_19_re;
  logic alert_en_shadowed_19_we;
  logic alert_en_shadowed_19_qs;
  logic alert_en_shadowed_19_wd;
  logic alert_en_shadowed_19_storage_err;
  logic alert_en_shadowed_19_update_err;
  logic alert_en_shadowed_20_re;
  logic alert_en_shadowed_20_we;
  logic alert_en_shadowed_20_qs;
  logic alert_en_shadowed_20_wd;
  logic alert_en_shadowed_20_storage_err;
  logic alert_en_shadowed_20_update_err;
  logic alert_en_shadowed_21_re;
  logic alert_en_shadowed_21_we;
  logic alert_en_shadowed_21_qs;
  logic alert_en_shadowed_21_wd;
  logic alert_en_shadowed_21_storage_err;
  logic alert_en_shadowed_21_update_err;
  logic alert_en_shadowed_22_re;
  logic alert_en_shadowed_22_we;
  logic alert_en_shadowed_22_qs;
  logic alert_en_shadowed_22_wd;
  logic alert_en_shadowed_22_storage_err;
  logic alert_en_shadowed_22_update_err;
  logic alert_en_shadowed_23_re;
  logic alert_en_shadowed_23_we;
  logic alert_en_shadowed_23_qs;
  logic alert_en_shadowed_23_wd;
  logic alert_en_shadowed_23_storage_err;
  logic alert_en_shadowed_23_update_err;
  logic alert_en_shadowed_24_re;
  logic alert_en_shadowed_24_we;
  logic alert_en_shadowed_24_qs;
  logic alert_en_shadowed_24_wd;
  logic alert_en_shadowed_24_storage_err;
  logic alert_en_shadowed_24_update_err;
  logic alert_en_shadowed_25_re;
  logic alert_en_shadowed_25_we;
  logic alert_en_shadowed_25_qs;
  logic alert_en_shadowed_25_wd;
  logic alert_en_shadowed_25_storage_err;
  logic alert_en_shadowed_25_update_err;
  logic alert_en_shadowed_26_re;
  logic alert_en_shadowed_26_we;
  logic alert_en_shadowed_26_qs;
  logic alert_en_shadowed_26_wd;
  logic alert_en_shadowed_26_storage_err;
  logic alert_en_shadowed_26_update_err;
  logic alert_en_shadowed_27_re;
  logic alert_en_shadowed_27_we;
  logic alert_en_shadowed_27_qs;
  logic alert_en_shadowed_27_wd;
  logic alert_en_shadowed_27_storage_err;
  logic alert_en_shadowed_27_update_err;
  logic alert_en_shadowed_28_re;
  logic alert_en_shadowed_28_we;
  logic alert_en_shadowed_28_qs;
  logic alert_en_shadowed_28_wd;
  logic alert_en_shadowed_28_storage_err;
  logic alert_en_shadowed_28_update_err;
  logic alert_en_shadowed_29_re;
  logic alert_en_shadowed_29_we;
  logic alert_en_shadowed_29_qs;
  logic alert_en_shadowed_29_wd;
  logic alert_en_shadowed_29_storage_err;
  logic alert_en_shadowed_29_update_err;
  logic alert_en_shadowed_30_re;
  logic alert_en_shadowed_30_we;
  logic alert_en_shadowed_30_qs;
  logic alert_en_shadowed_30_wd;
  logic alert_en_shadowed_30_storage_err;
  logic alert_en_shadowed_30_update_err;
  logic alert_en_shadowed_31_re;
  logic alert_en_shadowed_31_we;
  logic alert_en_shadowed_31_qs;
  logic alert_en_shadowed_31_wd;
  logic alert_en_shadowed_31_storage_err;
  logic alert_en_shadowed_31_update_err;
  logic alert_en_shadowed_32_re;
  logic alert_en_shadowed_32_we;
  logic alert_en_shadowed_32_qs;
  logic alert_en_shadowed_32_wd;
  logic alert_en_shadowed_32_storage_err;
  logic alert_en_shadowed_32_update_err;
  logic alert_en_shadowed_33_re;
  logic alert_en_shadowed_33_we;
  logic alert_en_shadowed_33_qs;
  logic alert_en_shadowed_33_wd;
  logic alert_en_shadowed_33_storage_err;
  logic alert_en_shadowed_33_update_err;
  logic alert_en_shadowed_34_re;
  logic alert_en_shadowed_34_we;
  logic alert_en_shadowed_34_qs;
  logic alert_en_shadowed_34_wd;
  logic alert_en_shadowed_34_storage_err;
  logic alert_en_shadowed_34_update_err;
  logic alert_en_shadowed_35_re;
  logic alert_en_shadowed_35_we;
  logic alert_en_shadowed_35_qs;
  logic alert_en_shadowed_35_wd;
  logic alert_en_shadowed_35_storage_err;
  logic alert_en_shadowed_35_update_err;
  logic alert_en_shadowed_36_re;
  logic alert_en_shadowed_36_we;
  logic alert_en_shadowed_36_qs;
  logic alert_en_shadowed_36_wd;
  logic alert_en_shadowed_36_storage_err;
  logic alert_en_shadowed_36_update_err;
  logic alert_en_shadowed_37_re;
  logic alert_en_shadowed_37_we;
  logic alert_en_shadowed_37_qs;
  logic alert_en_shadowed_37_wd;
  logic alert_en_shadowed_37_storage_err;
  logic alert_en_shadowed_37_update_err;
  logic alert_en_shadowed_38_re;
  logic alert_en_shadowed_38_we;
  logic alert_en_shadowed_38_qs;
  logic alert_en_shadowed_38_wd;
  logic alert_en_shadowed_38_storage_err;
  logic alert_en_shadowed_38_update_err;
  logic alert_en_shadowed_39_re;
  logic alert_en_shadowed_39_we;
  logic alert_en_shadowed_39_qs;
  logic alert_en_shadowed_39_wd;
  logic alert_en_shadowed_39_storage_err;
  logic alert_en_shadowed_39_update_err;
  logic alert_en_shadowed_40_re;
  logic alert_en_shadowed_40_we;
  logic alert_en_shadowed_40_qs;
  logic alert_en_shadowed_40_wd;
  logic alert_en_shadowed_40_storage_err;
  logic alert_en_shadowed_40_update_err;
  logic alert_en_shadowed_41_re;
  logic alert_en_shadowed_41_we;
  logic alert_en_shadowed_41_qs;
  logic alert_en_shadowed_41_wd;
  logic alert_en_shadowed_41_storage_err;
  logic alert_en_shadowed_41_update_err;
  logic alert_en_shadowed_42_re;
  logic alert_en_shadowed_42_we;
  logic alert_en_shadowed_42_qs;
  logic alert_en_shadowed_42_wd;
  logic alert_en_shadowed_42_storage_err;
  logic alert_en_shadowed_42_update_err;
  logic alert_en_shadowed_43_re;
  logic alert_en_shadowed_43_we;
  logic alert_en_shadowed_43_qs;
  logic alert_en_shadowed_43_wd;
  logic alert_en_shadowed_43_storage_err;
  logic alert_en_shadowed_43_update_err;
  logic alert_en_shadowed_44_re;
  logic alert_en_shadowed_44_we;
  logic alert_en_shadowed_44_qs;
  logic alert_en_shadowed_44_wd;
  logic alert_en_shadowed_44_storage_err;
  logic alert_en_shadowed_44_update_err;
  logic alert_en_shadowed_45_re;
  logic alert_en_shadowed_45_we;
  logic alert_en_shadowed_45_qs;
  logic alert_en_shadowed_45_wd;
  logic alert_en_shadowed_45_storage_err;
  logic alert_en_shadowed_45_update_err;
  logic alert_en_shadowed_46_re;
  logic alert_en_shadowed_46_we;
  logic alert_en_shadowed_46_qs;
  logic alert_en_shadowed_46_wd;
  logic alert_en_shadowed_46_storage_err;
  logic alert_en_shadowed_46_update_err;
  logic alert_en_shadowed_47_re;
  logic alert_en_shadowed_47_we;
  logic alert_en_shadowed_47_qs;
  logic alert_en_shadowed_47_wd;
  logic alert_en_shadowed_47_storage_err;
  logic alert_en_shadowed_47_update_err;
  logic alert_en_shadowed_48_re;
  logic alert_en_shadowed_48_we;
  logic alert_en_shadowed_48_qs;
  logic alert_en_shadowed_48_wd;
  logic alert_en_shadowed_48_storage_err;
  logic alert_en_shadowed_48_update_err;
  logic alert_en_shadowed_49_re;
  logic alert_en_shadowed_49_we;
  logic alert_en_shadowed_49_qs;
  logic alert_en_shadowed_49_wd;
  logic alert_en_shadowed_49_storage_err;
  logic alert_en_shadowed_49_update_err;
  logic alert_en_shadowed_50_re;
  logic alert_en_shadowed_50_we;
  logic alert_en_shadowed_50_qs;
  logic alert_en_shadowed_50_wd;
  logic alert_en_shadowed_50_storage_err;
  logic alert_en_shadowed_50_update_err;
  logic alert_en_shadowed_51_re;
  logic alert_en_shadowed_51_we;
  logic alert_en_shadowed_51_qs;
  logic alert_en_shadowed_51_wd;
  logic alert_en_shadowed_51_storage_err;
  logic alert_en_shadowed_51_update_err;
  logic alert_en_shadowed_52_re;
  logic alert_en_shadowed_52_we;
  logic alert_en_shadowed_52_qs;
  logic alert_en_shadowed_52_wd;
  logic alert_en_shadowed_52_storage_err;
  logic alert_en_shadowed_52_update_err;
  logic alert_en_shadowed_53_re;
  logic alert_en_shadowed_53_we;
  logic alert_en_shadowed_53_qs;
  logic alert_en_shadowed_53_wd;
  logic alert_en_shadowed_53_storage_err;
  logic alert_en_shadowed_53_update_err;
  logic alert_en_shadowed_54_re;
  logic alert_en_shadowed_54_we;
  logic alert_en_shadowed_54_qs;
  logic alert_en_shadowed_54_wd;
  logic alert_en_shadowed_54_storage_err;
  logic alert_en_shadowed_54_update_err;
  logic alert_en_shadowed_55_re;
  logic alert_en_shadowed_55_we;
  logic alert_en_shadowed_55_qs;
  logic alert_en_shadowed_55_wd;
  logic alert_en_shadowed_55_storage_err;
  logic alert_en_shadowed_55_update_err;
  logic alert_en_shadowed_56_re;
  logic alert_en_shadowed_56_we;
  logic alert_en_shadowed_56_qs;
  logic alert_en_shadowed_56_wd;
  logic alert_en_shadowed_56_storage_err;
  logic alert_en_shadowed_56_update_err;
  logic alert_en_shadowed_57_re;
  logic alert_en_shadowed_57_we;
  logic alert_en_shadowed_57_qs;
  logic alert_en_shadowed_57_wd;
  logic alert_en_shadowed_57_storage_err;
  logic alert_en_shadowed_57_update_err;
  logic alert_en_shadowed_58_re;
  logic alert_en_shadowed_58_we;
  logic alert_en_shadowed_58_qs;
  logic alert_en_shadowed_58_wd;
  logic alert_en_shadowed_58_storage_err;
  logic alert_en_shadowed_58_update_err;
  logic alert_en_shadowed_59_re;
  logic alert_en_shadowed_59_we;
  logic alert_en_shadowed_59_qs;
  logic alert_en_shadowed_59_wd;
  logic alert_en_shadowed_59_storage_err;
  logic alert_en_shadowed_59_update_err;
  logic alert_en_shadowed_60_re;
  logic alert_en_shadowed_60_we;
  logic alert_en_shadowed_60_qs;
  logic alert_en_shadowed_60_wd;
  logic alert_en_shadowed_60_storage_err;
  logic alert_en_shadowed_60_update_err;
  logic alert_en_shadowed_61_re;
  logic alert_en_shadowed_61_we;
  logic alert_en_shadowed_61_qs;
  logic alert_en_shadowed_61_wd;
  logic alert_en_shadowed_61_storage_err;
  logic alert_en_shadowed_61_update_err;
  logic alert_en_shadowed_62_re;
  logic alert_en_shadowed_62_we;
  logic alert_en_shadowed_62_qs;
  logic alert_en_shadowed_62_wd;
  logic alert_en_shadowed_62_storage_err;
  logic alert_en_shadowed_62_update_err;
  logic alert_en_shadowed_63_re;
  logic alert_en_shadowed_63_we;
  logic alert_en_shadowed_63_qs;
  logic alert_en_shadowed_63_wd;
  logic alert_en_shadowed_63_storage_err;
  logic alert_en_shadowed_63_update_err;
  logic alert_en_shadowed_64_re;
  logic alert_en_shadowed_64_we;
  logic alert_en_shadowed_64_qs;
  logic alert_en_shadowed_64_wd;
  logic alert_en_shadowed_64_storage_err;
  logic alert_en_shadowed_64_update_err;
  logic alert_class_shadowed_0_re;
  logic alert_class_shadowed_0_we;
  logic [1:0] alert_class_shadowed_0_qs;
  logic [1:0] alert_class_shadowed_0_wd;
  logic alert_class_shadowed_0_storage_err;
  logic alert_class_shadowed_0_update_err;
  logic alert_class_shadowed_1_re;
  logic alert_class_shadowed_1_we;
  logic [1:0] alert_class_shadowed_1_qs;
  logic [1:0] alert_class_shadowed_1_wd;
  logic alert_class_shadowed_1_storage_err;
  logic alert_class_shadowed_1_update_err;
  logic alert_class_shadowed_2_re;
  logic alert_class_shadowed_2_we;
  logic [1:0] alert_class_shadowed_2_qs;
  logic [1:0] alert_class_shadowed_2_wd;
  logic alert_class_shadowed_2_storage_err;
  logic alert_class_shadowed_2_update_err;
  logic alert_class_shadowed_3_re;
  logic alert_class_shadowed_3_we;
  logic [1:0] alert_class_shadowed_3_qs;
  logic [1:0] alert_class_shadowed_3_wd;
  logic alert_class_shadowed_3_storage_err;
  logic alert_class_shadowed_3_update_err;
  logic alert_class_shadowed_4_re;
  logic alert_class_shadowed_4_we;
  logic [1:0] alert_class_shadowed_4_qs;
  logic [1:0] alert_class_shadowed_4_wd;
  logic alert_class_shadowed_4_storage_err;
  logic alert_class_shadowed_4_update_err;
  logic alert_class_shadowed_5_re;
  logic alert_class_shadowed_5_we;
  logic [1:0] alert_class_shadowed_5_qs;
  logic [1:0] alert_class_shadowed_5_wd;
  logic alert_class_shadowed_5_storage_err;
  logic alert_class_shadowed_5_update_err;
  logic alert_class_shadowed_6_re;
  logic alert_class_shadowed_6_we;
  logic [1:0] alert_class_shadowed_6_qs;
  logic [1:0] alert_class_shadowed_6_wd;
  logic alert_class_shadowed_6_storage_err;
  logic alert_class_shadowed_6_update_err;
  logic alert_class_shadowed_7_re;
  logic alert_class_shadowed_7_we;
  logic [1:0] alert_class_shadowed_7_qs;
  logic [1:0] alert_class_shadowed_7_wd;
  logic alert_class_shadowed_7_storage_err;
  logic alert_class_shadowed_7_update_err;
  logic alert_class_shadowed_8_re;
  logic alert_class_shadowed_8_we;
  logic [1:0] alert_class_shadowed_8_qs;
  logic [1:0] alert_class_shadowed_8_wd;
  logic alert_class_shadowed_8_storage_err;
  logic alert_class_shadowed_8_update_err;
  logic alert_class_shadowed_9_re;
  logic alert_class_shadowed_9_we;
  logic [1:0] alert_class_shadowed_9_qs;
  logic [1:0] alert_class_shadowed_9_wd;
  logic alert_class_shadowed_9_storage_err;
  logic alert_class_shadowed_9_update_err;
  logic alert_class_shadowed_10_re;
  logic alert_class_shadowed_10_we;
  logic [1:0] alert_class_shadowed_10_qs;
  logic [1:0] alert_class_shadowed_10_wd;
  logic alert_class_shadowed_10_storage_err;
  logic alert_class_shadowed_10_update_err;
  logic alert_class_shadowed_11_re;
  logic alert_class_shadowed_11_we;
  logic [1:0] alert_class_shadowed_11_qs;
  logic [1:0] alert_class_shadowed_11_wd;
  logic alert_class_shadowed_11_storage_err;
  logic alert_class_shadowed_11_update_err;
  logic alert_class_shadowed_12_re;
  logic alert_class_shadowed_12_we;
  logic [1:0] alert_class_shadowed_12_qs;
  logic [1:0] alert_class_shadowed_12_wd;
  logic alert_class_shadowed_12_storage_err;
  logic alert_class_shadowed_12_update_err;
  logic alert_class_shadowed_13_re;
  logic alert_class_shadowed_13_we;
  logic [1:0] alert_class_shadowed_13_qs;
  logic [1:0] alert_class_shadowed_13_wd;
  logic alert_class_shadowed_13_storage_err;
  logic alert_class_shadowed_13_update_err;
  logic alert_class_shadowed_14_re;
  logic alert_class_shadowed_14_we;
  logic [1:0] alert_class_shadowed_14_qs;
  logic [1:0] alert_class_shadowed_14_wd;
  logic alert_class_shadowed_14_storage_err;
  logic alert_class_shadowed_14_update_err;
  logic alert_class_shadowed_15_re;
  logic alert_class_shadowed_15_we;
  logic [1:0] alert_class_shadowed_15_qs;
  logic [1:0] alert_class_shadowed_15_wd;
  logic alert_class_shadowed_15_storage_err;
  logic alert_class_shadowed_15_update_err;
  logic alert_class_shadowed_16_re;
  logic alert_class_shadowed_16_we;
  logic [1:0] alert_class_shadowed_16_qs;
  logic [1:0] alert_class_shadowed_16_wd;
  logic alert_class_shadowed_16_storage_err;
  logic alert_class_shadowed_16_update_err;
  logic alert_class_shadowed_17_re;
  logic alert_class_shadowed_17_we;
  logic [1:0] alert_class_shadowed_17_qs;
  logic [1:0] alert_class_shadowed_17_wd;
  logic alert_class_shadowed_17_storage_err;
  logic alert_class_shadowed_17_update_err;
  logic alert_class_shadowed_18_re;
  logic alert_class_shadowed_18_we;
  logic [1:0] alert_class_shadowed_18_qs;
  logic [1:0] alert_class_shadowed_18_wd;
  logic alert_class_shadowed_18_storage_err;
  logic alert_class_shadowed_18_update_err;
  logic alert_class_shadowed_19_re;
  logic alert_class_shadowed_19_we;
  logic [1:0] alert_class_shadowed_19_qs;
  logic [1:0] alert_class_shadowed_19_wd;
  logic alert_class_shadowed_19_storage_err;
  logic alert_class_shadowed_19_update_err;
  logic alert_class_shadowed_20_re;
  logic alert_class_shadowed_20_we;
  logic [1:0] alert_class_shadowed_20_qs;
  logic [1:0] alert_class_shadowed_20_wd;
  logic alert_class_shadowed_20_storage_err;
  logic alert_class_shadowed_20_update_err;
  logic alert_class_shadowed_21_re;
  logic alert_class_shadowed_21_we;
  logic [1:0] alert_class_shadowed_21_qs;
  logic [1:0] alert_class_shadowed_21_wd;
  logic alert_class_shadowed_21_storage_err;
  logic alert_class_shadowed_21_update_err;
  logic alert_class_shadowed_22_re;
  logic alert_class_shadowed_22_we;
  logic [1:0] alert_class_shadowed_22_qs;
  logic [1:0] alert_class_shadowed_22_wd;
  logic alert_class_shadowed_22_storage_err;
  logic alert_class_shadowed_22_update_err;
  logic alert_class_shadowed_23_re;
  logic alert_class_shadowed_23_we;
  logic [1:0] alert_class_shadowed_23_qs;
  logic [1:0] alert_class_shadowed_23_wd;
  logic alert_class_shadowed_23_storage_err;
  logic alert_class_shadowed_23_update_err;
  logic alert_class_shadowed_24_re;
  logic alert_class_shadowed_24_we;
  logic [1:0] alert_class_shadowed_24_qs;
  logic [1:0] alert_class_shadowed_24_wd;
  logic alert_class_shadowed_24_storage_err;
  logic alert_class_shadowed_24_update_err;
  logic alert_class_shadowed_25_re;
  logic alert_class_shadowed_25_we;
  logic [1:0] alert_class_shadowed_25_qs;
  logic [1:0] alert_class_shadowed_25_wd;
  logic alert_class_shadowed_25_storage_err;
  logic alert_class_shadowed_25_update_err;
  logic alert_class_shadowed_26_re;
  logic alert_class_shadowed_26_we;
  logic [1:0] alert_class_shadowed_26_qs;
  logic [1:0] alert_class_shadowed_26_wd;
  logic alert_class_shadowed_26_storage_err;
  logic alert_class_shadowed_26_update_err;
  logic alert_class_shadowed_27_re;
  logic alert_class_shadowed_27_we;
  logic [1:0] alert_class_shadowed_27_qs;
  logic [1:0] alert_class_shadowed_27_wd;
  logic alert_class_shadowed_27_storage_err;
  logic alert_class_shadowed_27_update_err;
  logic alert_class_shadowed_28_re;
  logic alert_class_shadowed_28_we;
  logic [1:0] alert_class_shadowed_28_qs;
  logic [1:0] alert_class_shadowed_28_wd;
  logic alert_class_shadowed_28_storage_err;
  logic alert_class_shadowed_28_update_err;
  logic alert_class_shadowed_29_re;
  logic alert_class_shadowed_29_we;
  logic [1:0] alert_class_shadowed_29_qs;
  logic [1:0] alert_class_shadowed_29_wd;
  logic alert_class_shadowed_29_storage_err;
  logic alert_class_shadowed_29_update_err;
  logic alert_class_shadowed_30_re;
  logic alert_class_shadowed_30_we;
  logic [1:0] alert_class_shadowed_30_qs;
  logic [1:0] alert_class_shadowed_30_wd;
  logic alert_class_shadowed_30_storage_err;
  logic alert_class_shadowed_30_update_err;
  logic alert_class_shadowed_31_re;
  logic alert_class_shadowed_31_we;
  logic [1:0] alert_class_shadowed_31_qs;
  logic [1:0] alert_class_shadowed_31_wd;
  logic alert_class_shadowed_31_storage_err;
  logic alert_class_shadowed_31_update_err;
  logic alert_class_shadowed_32_re;
  logic alert_class_shadowed_32_we;
  logic [1:0] alert_class_shadowed_32_qs;
  logic [1:0] alert_class_shadowed_32_wd;
  logic alert_class_shadowed_32_storage_err;
  logic alert_class_shadowed_32_update_err;
  logic alert_class_shadowed_33_re;
  logic alert_class_shadowed_33_we;
  logic [1:0] alert_class_shadowed_33_qs;
  logic [1:0] alert_class_shadowed_33_wd;
  logic alert_class_shadowed_33_storage_err;
  logic alert_class_shadowed_33_update_err;
  logic alert_class_shadowed_34_re;
  logic alert_class_shadowed_34_we;
  logic [1:0] alert_class_shadowed_34_qs;
  logic [1:0] alert_class_shadowed_34_wd;
  logic alert_class_shadowed_34_storage_err;
  logic alert_class_shadowed_34_update_err;
  logic alert_class_shadowed_35_re;
  logic alert_class_shadowed_35_we;
  logic [1:0] alert_class_shadowed_35_qs;
  logic [1:0] alert_class_shadowed_35_wd;
  logic alert_class_shadowed_35_storage_err;
  logic alert_class_shadowed_35_update_err;
  logic alert_class_shadowed_36_re;
  logic alert_class_shadowed_36_we;
  logic [1:0] alert_class_shadowed_36_qs;
  logic [1:0] alert_class_shadowed_36_wd;
  logic alert_class_shadowed_36_storage_err;
  logic alert_class_shadowed_36_update_err;
  logic alert_class_shadowed_37_re;
  logic alert_class_shadowed_37_we;
  logic [1:0] alert_class_shadowed_37_qs;
  logic [1:0] alert_class_shadowed_37_wd;
  logic alert_class_shadowed_37_storage_err;
  logic alert_class_shadowed_37_update_err;
  logic alert_class_shadowed_38_re;
  logic alert_class_shadowed_38_we;
  logic [1:0] alert_class_shadowed_38_qs;
  logic [1:0] alert_class_shadowed_38_wd;
  logic alert_class_shadowed_38_storage_err;
  logic alert_class_shadowed_38_update_err;
  logic alert_class_shadowed_39_re;
  logic alert_class_shadowed_39_we;
  logic [1:0] alert_class_shadowed_39_qs;
  logic [1:0] alert_class_shadowed_39_wd;
  logic alert_class_shadowed_39_storage_err;
  logic alert_class_shadowed_39_update_err;
  logic alert_class_shadowed_40_re;
  logic alert_class_shadowed_40_we;
  logic [1:0] alert_class_shadowed_40_qs;
  logic [1:0] alert_class_shadowed_40_wd;
  logic alert_class_shadowed_40_storage_err;
  logic alert_class_shadowed_40_update_err;
  logic alert_class_shadowed_41_re;
  logic alert_class_shadowed_41_we;
  logic [1:0] alert_class_shadowed_41_qs;
  logic [1:0] alert_class_shadowed_41_wd;
  logic alert_class_shadowed_41_storage_err;
  logic alert_class_shadowed_41_update_err;
  logic alert_class_shadowed_42_re;
  logic alert_class_shadowed_42_we;
  logic [1:0] alert_class_shadowed_42_qs;
  logic [1:0] alert_class_shadowed_42_wd;
  logic alert_class_shadowed_42_storage_err;
  logic alert_class_shadowed_42_update_err;
  logic alert_class_shadowed_43_re;
  logic alert_class_shadowed_43_we;
  logic [1:0] alert_class_shadowed_43_qs;
  logic [1:0] alert_class_shadowed_43_wd;
  logic alert_class_shadowed_43_storage_err;
  logic alert_class_shadowed_43_update_err;
  logic alert_class_shadowed_44_re;
  logic alert_class_shadowed_44_we;
  logic [1:0] alert_class_shadowed_44_qs;
  logic [1:0] alert_class_shadowed_44_wd;
  logic alert_class_shadowed_44_storage_err;
  logic alert_class_shadowed_44_update_err;
  logic alert_class_shadowed_45_re;
  logic alert_class_shadowed_45_we;
  logic [1:0] alert_class_shadowed_45_qs;
  logic [1:0] alert_class_shadowed_45_wd;
  logic alert_class_shadowed_45_storage_err;
  logic alert_class_shadowed_45_update_err;
  logic alert_class_shadowed_46_re;
  logic alert_class_shadowed_46_we;
  logic [1:0] alert_class_shadowed_46_qs;
  logic [1:0] alert_class_shadowed_46_wd;
  logic alert_class_shadowed_46_storage_err;
  logic alert_class_shadowed_46_update_err;
  logic alert_class_shadowed_47_re;
  logic alert_class_shadowed_47_we;
  logic [1:0] alert_class_shadowed_47_qs;
  logic [1:0] alert_class_shadowed_47_wd;
  logic alert_class_shadowed_47_storage_err;
  logic alert_class_shadowed_47_update_err;
  logic alert_class_shadowed_48_re;
  logic alert_class_shadowed_48_we;
  logic [1:0] alert_class_shadowed_48_qs;
  logic [1:0] alert_class_shadowed_48_wd;
  logic alert_class_shadowed_48_storage_err;
  logic alert_class_shadowed_48_update_err;
  logic alert_class_shadowed_49_re;
  logic alert_class_shadowed_49_we;
  logic [1:0] alert_class_shadowed_49_qs;
  logic [1:0] alert_class_shadowed_49_wd;
  logic alert_class_shadowed_49_storage_err;
  logic alert_class_shadowed_49_update_err;
  logic alert_class_shadowed_50_re;
  logic alert_class_shadowed_50_we;
  logic [1:0] alert_class_shadowed_50_qs;
  logic [1:0] alert_class_shadowed_50_wd;
  logic alert_class_shadowed_50_storage_err;
  logic alert_class_shadowed_50_update_err;
  logic alert_class_shadowed_51_re;
  logic alert_class_shadowed_51_we;
  logic [1:0] alert_class_shadowed_51_qs;
  logic [1:0] alert_class_shadowed_51_wd;
  logic alert_class_shadowed_51_storage_err;
  logic alert_class_shadowed_51_update_err;
  logic alert_class_shadowed_52_re;
  logic alert_class_shadowed_52_we;
  logic [1:0] alert_class_shadowed_52_qs;
  logic [1:0] alert_class_shadowed_52_wd;
  logic alert_class_shadowed_52_storage_err;
  logic alert_class_shadowed_52_update_err;
  logic alert_class_shadowed_53_re;
  logic alert_class_shadowed_53_we;
  logic [1:0] alert_class_shadowed_53_qs;
  logic [1:0] alert_class_shadowed_53_wd;
  logic alert_class_shadowed_53_storage_err;
  logic alert_class_shadowed_53_update_err;
  logic alert_class_shadowed_54_re;
  logic alert_class_shadowed_54_we;
  logic [1:0] alert_class_shadowed_54_qs;
  logic [1:0] alert_class_shadowed_54_wd;
  logic alert_class_shadowed_54_storage_err;
  logic alert_class_shadowed_54_update_err;
  logic alert_class_shadowed_55_re;
  logic alert_class_shadowed_55_we;
  logic [1:0] alert_class_shadowed_55_qs;
  logic [1:0] alert_class_shadowed_55_wd;
  logic alert_class_shadowed_55_storage_err;
  logic alert_class_shadowed_55_update_err;
  logic alert_class_shadowed_56_re;
  logic alert_class_shadowed_56_we;
  logic [1:0] alert_class_shadowed_56_qs;
  logic [1:0] alert_class_shadowed_56_wd;
  logic alert_class_shadowed_56_storage_err;
  logic alert_class_shadowed_56_update_err;
  logic alert_class_shadowed_57_re;
  logic alert_class_shadowed_57_we;
  logic [1:0] alert_class_shadowed_57_qs;
  logic [1:0] alert_class_shadowed_57_wd;
  logic alert_class_shadowed_57_storage_err;
  logic alert_class_shadowed_57_update_err;
  logic alert_class_shadowed_58_re;
  logic alert_class_shadowed_58_we;
  logic [1:0] alert_class_shadowed_58_qs;
  logic [1:0] alert_class_shadowed_58_wd;
  logic alert_class_shadowed_58_storage_err;
  logic alert_class_shadowed_58_update_err;
  logic alert_class_shadowed_59_re;
  logic alert_class_shadowed_59_we;
  logic [1:0] alert_class_shadowed_59_qs;
  logic [1:0] alert_class_shadowed_59_wd;
  logic alert_class_shadowed_59_storage_err;
  logic alert_class_shadowed_59_update_err;
  logic alert_class_shadowed_60_re;
  logic alert_class_shadowed_60_we;
  logic [1:0] alert_class_shadowed_60_qs;
  logic [1:0] alert_class_shadowed_60_wd;
  logic alert_class_shadowed_60_storage_err;
  logic alert_class_shadowed_60_update_err;
  logic alert_class_shadowed_61_re;
  logic alert_class_shadowed_61_we;
  logic [1:0] alert_class_shadowed_61_qs;
  logic [1:0] alert_class_shadowed_61_wd;
  logic alert_class_shadowed_61_storage_err;
  logic alert_class_shadowed_61_update_err;
  logic alert_class_shadowed_62_re;
  logic alert_class_shadowed_62_we;
  logic [1:0] alert_class_shadowed_62_qs;
  logic [1:0] alert_class_shadowed_62_wd;
  logic alert_class_shadowed_62_storage_err;
  logic alert_class_shadowed_62_update_err;
  logic alert_class_shadowed_63_re;
  logic alert_class_shadowed_63_we;
  logic [1:0] alert_class_shadowed_63_qs;
  logic [1:0] alert_class_shadowed_63_wd;
  logic alert_class_shadowed_63_storage_err;
  logic alert_class_shadowed_63_update_err;
  logic alert_class_shadowed_64_re;
  logic alert_class_shadowed_64_we;
  logic [1:0] alert_class_shadowed_64_qs;
  logic [1:0] alert_class_shadowed_64_wd;
  logic alert_class_shadowed_64_storage_err;
  logic alert_class_shadowed_64_update_err;
  logic alert_cause_0_we;
  logic alert_cause_0_qs;
  logic alert_cause_0_wd;
  logic alert_cause_1_we;
  logic alert_cause_1_qs;
  logic alert_cause_1_wd;
  logic alert_cause_2_we;
  logic alert_cause_2_qs;
  logic alert_cause_2_wd;
  logic alert_cause_3_we;
  logic alert_cause_3_qs;
  logic alert_cause_3_wd;
  logic alert_cause_4_we;
  logic alert_cause_4_qs;
  logic alert_cause_4_wd;
  logic alert_cause_5_we;
  logic alert_cause_5_qs;
  logic alert_cause_5_wd;
  logic alert_cause_6_we;
  logic alert_cause_6_qs;
  logic alert_cause_6_wd;
  logic alert_cause_7_we;
  logic alert_cause_7_qs;
  logic alert_cause_7_wd;
  logic alert_cause_8_we;
  logic alert_cause_8_qs;
  logic alert_cause_8_wd;
  logic alert_cause_9_we;
  logic alert_cause_9_qs;
  logic alert_cause_9_wd;
  logic alert_cause_10_we;
  logic alert_cause_10_qs;
  logic alert_cause_10_wd;
  logic alert_cause_11_we;
  logic alert_cause_11_qs;
  logic alert_cause_11_wd;
  logic alert_cause_12_we;
  logic alert_cause_12_qs;
  logic alert_cause_12_wd;
  logic alert_cause_13_we;
  logic alert_cause_13_qs;
  logic alert_cause_13_wd;
  logic alert_cause_14_we;
  logic alert_cause_14_qs;
  logic alert_cause_14_wd;
  logic alert_cause_15_we;
  logic alert_cause_15_qs;
  logic alert_cause_15_wd;
  logic alert_cause_16_we;
  logic alert_cause_16_qs;
  logic alert_cause_16_wd;
  logic alert_cause_17_we;
  logic alert_cause_17_qs;
  logic alert_cause_17_wd;
  logic alert_cause_18_we;
  logic alert_cause_18_qs;
  logic alert_cause_18_wd;
  logic alert_cause_19_we;
  logic alert_cause_19_qs;
  logic alert_cause_19_wd;
  logic alert_cause_20_we;
  logic alert_cause_20_qs;
  logic alert_cause_20_wd;
  logic alert_cause_21_we;
  logic alert_cause_21_qs;
  logic alert_cause_21_wd;
  logic alert_cause_22_we;
  logic alert_cause_22_qs;
  logic alert_cause_22_wd;
  logic alert_cause_23_we;
  logic alert_cause_23_qs;
  logic alert_cause_23_wd;
  logic alert_cause_24_we;
  logic alert_cause_24_qs;
  logic alert_cause_24_wd;
  logic alert_cause_25_we;
  logic alert_cause_25_qs;
  logic alert_cause_25_wd;
  logic alert_cause_26_we;
  logic alert_cause_26_qs;
  logic alert_cause_26_wd;
  logic alert_cause_27_we;
  logic alert_cause_27_qs;
  logic alert_cause_27_wd;
  logic alert_cause_28_we;
  logic alert_cause_28_qs;
  logic alert_cause_28_wd;
  logic alert_cause_29_we;
  logic alert_cause_29_qs;
  logic alert_cause_29_wd;
  logic alert_cause_30_we;
  logic alert_cause_30_qs;
  logic alert_cause_30_wd;
  logic alert_cause_31_we;
  logic alert_cause_31_qs;
  logic alert_cause_31_wd;
  logic alert_cause_32_we;
  logic alert_cause_32_qs;
  logic alert_cause_32_wd;
  logic alert_cause_33_we;
  logic alert_cause_33_qs;
  logic alert_cause_33_wd;
  logic alert_cause_34_we;
  logic alert_cause_34_qs;
  logic alert_cause_34_wd;
  logic alert_cause_35_we;
  logic alert_cause_35_qs;
  logic alert_cause_35_wd;
  logic alert_cause_36_we;
  logic alert_cause_36_qs;
  logic alert_cause_36_wd;
  logic alert_cause_37_we;
  logic alert_cause_37_qs;
  logic alert_cause_37_wd;
  logic alert_cause_38_we;
  logic alert_cause_38_qs;
  logic alert_cause_38_wd;
  logic alert_cause_39_we;
  logic alert_cause_39_qs;
  logic alert_cause_39_wd;
  logic alert_cause_40_we;
  logic alert_cause_40_qs;
  logic alert_cause_40_wd;
  logic alert_cause_41_we;
  logic alert_cause_41_qs;
  logic alert_cause_41_wd;
  logic alert_cause_42_we;
  logic alert_cause_42_qs;
  logic alert_cause_42_wd;
  logic alert_cause_43_we;
  logic alert_cause_43_qs;
  logic alert_cause_43_wd;
  logic alert_cause_44_we;
  logic alert_cause_44_qs;
  logic alert_cause_44_wd;
  logic alert_cause_45_we;
  logic alert_cause_45_qs;
  logic alert_cause_45_wd;
  logic alert_cause_46_we;
  logic alert_cause_46_qs;
  logic alert_cause_46_wd;
  logic alert_cause_47_we;
  logic alert_cause_47_qs;
  logic alert_cause_47_wd;
  logic alert_cause_48_we;
  logic alert_cause_48_qs;
  logic alert_cause_48_wd;
  logic alert_cause_49_we;
  logic alert_cause_49_qs;
  logic alert_cause_49_wd;
  logic alert_cause_50_we;
  logic alert_cause_50_qs;
  logic alert_cause_50_wd;
  logic alert_cause_51_we;
  logic alert_cause_51_qs;
  logic alert_cause_51_wd;
  logic alert_cause_52_we;
  logic alert_cause_52_qs;
  logic alert_cause_52_wd;
  logic alert_cause_53_we;
  logic alert_cause_53_qs;
  logic alert_cause_53_wd;
  logic alert_cause_54_we;
  logic alert_cause_54_qs;
  logic alert_cause_54_wd;
  logic alert_cause_55_we;
  logic alert_cause_55_qs;
  logic alert_cause_55_wd;
  logic alert_cause_56_we;
  logic alert_cause_56_qs;
  logic alert_cause_56_wd;
  logic alert_cause_57_we;
  logic alert_cause_57_qs;
  logic alert_cause_57_wd;
  logic alert_cause_58_we;
  logic alert_cause_58_qs;
  logic alert_cause_58_wd;
  logic alert_cause_59_we;
  logic alert_cause_59_qs;
  logic alert_cause_59_wd;
  logic alert_cause_60_we;
  logic alert_cause_60_qs;
  logic alert_cause_60_wd;
  logic alert_cause_61_we;
  logic alert_cause_61_qs;
  logic alert_cause_61_wd;
  logic alert_cause_62_we;
  logic alert_cause_62_qs;
  logic alert_cause_62_wd;
  logic alert_cause_63_we;
  logic alert_cause_63_qs;
  logic alert_cause_63_wd;
  logic alert_cause_64_we;
  logic alert_cause_64_qs;
  logic alert_cause_64_wd;
  logic loc_alert_regwen_0_we;
  logic loc_alert_regwen_0_qs;
  logic loc_alert_regwen_0_wd;
  logic loc_alert_regwen_1_we;
  logic loc_alert_regwen_1_qs;
  logic loc_alert_regwen_1_wd;
  logic loc_alert_regwen_2_we;
  logic loc_alert_regwen_2_qs;
  logic loc_alert_regwen_2_wd;
  logic loc_alert_regwen_3_we;
  logic loc_alert_regwen_3_qs;
  logic loc_alert_regwen_3_wd;
  logic loc_alert_regwen_4_we;
  logic loc_alert_regwen_4_qs;
  logic loc_alert_regwen_4_wd;
  logic loc_alert_regwen_5_we;
  logic loc_alert_regwen_5_qs;
  logic loc_alert_regwen_5_wd;
  logic loc_alert_regwen_6_we;
  logic loc_alert_regwen_6_qs;
  logic loc_alert_regwen_6_wd;
  logic loc_alert_en_shadowed_0_re;
  logic loc_alert_en_shadowed_0_we;
  logic loc_alert_en_shadowed_0_qs;
  logic loc_alert_en_shadowed_0_wd;
  logic loc_alert_en_shadowed_0_storage_err;
  logic loc_alert_en_shadowed_0_update_err;
  logic loc_alert_en_shadowed_1_re;
  logic loc_alert_en_shadowed_1_we;
  logic loc_alert_en_shadowed_1_qs;
  logic loc_alert_en_shadowed_1_wd;
  logic loc_alert_en_shadowed_1_storage_err;
  logic loc_alert_en_shadowed_1_update_err;
  logic loc_alert_en_shadowed_2_re;
  logic loc_alert_en_shadowed_2_we;
  logic loc_alert_en_shadowed_2_qs;
  logic loc_alert_en_shadowed_2_wd;
  logic loc_alert_en_shadowed_2_storage_err;
  logic loc_alert_en_shadowed_2_update_err;
  logic loc_alert_en_shadowed_3_re;
  logic loc_alert_en_shadowed_3_we;
  logic loc_alert_en_shadowed_3_qs;
  logic loc_alert_en_shadowed_3_wd;
  logic loc_alert_en_shadowed_3_storage_err;
  logic loc_alert_en_shadowed_3_update_err;
  logic loc_alert_en_shadowed_4_re;
  logic loc_alert_en_shadowed_4_we;
  logic loc_alert_en_shadowed_4_qs;
  logic loc_alert_en_shadowed_4_wd;
  logic loc_alert_en_shadowed_4_storage_err;
  logic loc_alert_en_shadowed_4_update_err;
  logic loc_alert_en_shadowed_5_re;
  logic loc_alert_en_shadowed_5_we;
  logic loc_alert_en_shadowed_5_qs;
  logic loc_alert_en_shadowed_5_wd;
  logic loc_alert_en_shadowed_5_storage_err;
  logic loc_alert_en_shadowed_5_update_err;
  logic loc_alert_en_shadowed_6_re;
  logic loc_alert_en_shadowed_6_we;
  logic loc_alert_en_shadowed_6_qs;
  logic loc_alert_en_shadowed_6_wd;
  logic loc_alert_en_shadowed_6_storage_err;
  logic loc_alert_en_shadowed_6_update_err;
  logic loc_alert_class_shadowed_0_re;
  logic loc_alert_class_shadowed_0_we;
  logic [1:0] loc_alert_class_shadowed_0_qs;
  logic [1:0] loc_alert_class_shadowed_0_wd;
  logic loc_alert_class_shadowed_0_storage_err;
  logic loc_alert_class_shadowed_0_update_err;
  logic loc_alert_class_shadowed_1_re;
  logic loc_alert_class_shadowed_1_we;
  logic [1:0] loc_alert_class_shadowed_1_qs;
  logic [1:0] loc_alert_class_shadowed_1_wd;
  logic loc_alert_class_shadowed_1_storage_err;
  logic loc_alert_class_shadowed_1_update_err;
  logic loc_alert_class_shadowed_2_re;
  logic loc_alert_class_shadowed_2_we;
  logic [1:0] loc_alert_class_shadowed_2_qs;
  logic [1:0] loc_alert_class_shadowed_2_wd;
  logic loc_alert_class_shadowed_2_storage_err;
  logic loc_alert_class_shadowed_2_update_err;
  logic loc_alert_class_shadowed_3_re;
  logic loc_alert_class_shadowed_3_we;
  logic [1:0] loc_alert_class_shadowed_3_qs;
  logic [1:0] loc_alert_class_shadowed_3_wd;
  logic loc_alert_class_shadowed_3_storage_err;
  logic loc_alert_class_shadowed_3_update_err;
  logic loc_alert_class_shadowed_4_re;
  logic loc_alert_class_shadowed_4_we;
  logic [1:0] loc_alert_class_shadowed_4_qs;
  logic [1:0] loc_alert_class_shadowed_4_wd;
  logic loc_alert_class_shadowed_4_storage_err;
  logic loc_alert_class_shadowed_4_update_err;
  logic loc_alert_class_shadowed_5_re;
  logic loc_alert_class_shadowed_5_we;
  logic [1:0] loc_alert_class_shadowed_5_qs;
  logic [1:0] loc_alert_class_shadowed_5_wd;
  logic loc_alert_class_shadowed_5_storage_err;
  logic loc_alert_class_shadowed_5_update_err;
  logic loc_alert_class_shadowed_6_re;
  logic loc_alert_class_shadowed_6_we;
  logic [1:0] loc_alert_class_shadowed_6_qs;
  logic [1:0] loc_alert_class_shadowed_6_wd;
  logic loc_alert_class_shadowed_6_storage_err;
  logic loc_alert_class_shadowed_6_update_err;
  logic loc_alert_cause_0_we;
  logic loc_alert_cause_0_qs;
  logic loc_alert_cause_0_wd;
  logic loc_alert_cause_1_we;
  logic loc_alert_cause_1_qs;
  logic loc_alert_cause_1_wd;
  logic loc_alert_cause_2_we;
  logic loc_alert_cause_2_qs;
  logic loc_alert_cause_2_wd;
  logic loc_alert_cause_3_we;
  logic loc_alert_cause_3_qs;
  logic loc_alert_cause_3_wd;
  logic loc_alert_cause_4_we;
  logic loc_alert_cause_4_qs;
  logic loc_alert_cause_4_wd;
  logic loc_alert_cause_5_we;
  logic loc_alert_cause_5_qs;
  logic loc_alert_cause_5_wd;
  logic loc_alert_cause_6_we;
  logic loc_alert_cause_6_qs;
  logic loc_alert_cause_6_wd;
  logic classa_regwen_we;
  logic classa_regwen_qs;
  logic classa_regwen_wd;
  logic classa_ctrl_shadowed_re;
  logic classa_ctrl_shadowed_we;
  logic classa_ctrl_shadowed_en_qs;
  logic classa_ctrl_shadowed_en_wd;
  logic classa_ctrl_shadowed_en_storage_err;
  logic classa_ctrl_shadowed_en_update_err;
  logic classa_ctrl_shadowed_lock_qs;
  logic classa_ctrl_shadowed_lock_wd;
  logic classa_ctrl_shadowed_lock_storage_err;
  logic classa_ctrl_shadowed_lock_update_err;
  logic classa_ctrl_shadowed_en_e0_qs;
  logic classa_ctrl_shadowed_en_e0_wd;
  logic classa_ctrl_shadowed_en_e0_storage_err;
  logic classa_ctrl_shadowed_en_e0_update_err;
  logic classa_ctrl_shadowed_en_e1_qs;
  logic classa_ctrl_shadowed_en_e1_wd;
  logic classa_ctrl_shadowed_en_e1_storage_err;
  logic classa_ctrl_shadowed_en_e1_update_err;
  logic classa_ctrl_shadowed_en_e2_qs;
  logic classa_ctrl_shadowed_en_e2_wd;
  logic classa_ctrl_shadowed_en_e2_storage_err;
  logic classa_ctrl_shadowed_en_e2_update_err;
  logic classa_ctrl_shadowed_en_e3_qs;
  logic classa_ctrl_shadowed_en_e3_wd;
  logic classa_ctrl_shadowed_en_e3_storage_err;
  logic classa_ctrl_shadowed_en_e3_update_err;
  logic [1:0] classa_ctrl_shadowed_map_e0_qs;
  logic [1:0] classa_ctrl_shadowed_map_e0_wd;
  logic classa_ctrl_shadowed_map_e0_storage_err;
  logic classa_ctrl_shadowed_map_e0_update_err;
  logic [1:0] classa_ctrl_shadowed_map_e1_qs;
  logic [1:0] classa_ctrl_shadowed_map_e1_wd;
  logic classa_ctrl_shadowed_map_e1_storage_err;
  logic classa_ctrl_shadowed_map_e1_update_err;
  logic [1:0] classa_ctrl_shadowed_map_e2_qs;
  logic [1:0] classa_ctrl_shadowed_map_e2_wd;
  logic classa_ctrl_shadowed_map_e2_storage_err;
  logic classa_ctrl_shadowed_map_e2_update_err;
  logic [1:0] classa_ctrl_shadowed_map_e3_qs;
  logic [1:0] classa_ctrl_shadowed_map_e3_wd;
  logic classa_ctrl_shadowed_map_e3_storage_err;
  logic classa_ctrl_shadowed_map_e3_update_err;
  logic classa_clr_regwen_we;
  logic classa_clr_regwen_qs;
  logic classa_clr_regwen_wd;
  logic classa_clr_shadowed_re;
  logic classa_clr_shadowed_we;
  logic classa_clr_shadowed_qs;
  logic classa_clr_shadowed_wd;
  logic classa_clr_shadowed_storage_err;
  logic classa_clr_shadowed_update_err;
  logic classa_accum_cnt_re;
  logic [15:0] classa_accum_cnt_qs;
  logic classa_accum_thresh_shadowed_re;
  logic classa_accum_thresh_shadowed_we;
  logic [15:0] classa_accum_thresh_shadowed_qs;
  logic [15:0] classa_accum_thresh_shadowed_wd;
  logic classa_accum_thresh_shadowed_storage_err;
  logic classa_accum_thresh_shadowed_update_err;
  logic classa_timeout_cyc_shadowed_re;
  logic classa_timeout_cyc_shadowed_we;
  logic [31:0] classa_timeout_cyc_shadowed_qs;
  logic [31:0] classa_timeout_cyc_shadowed_wd;
  logic classa_timeout_cyc_shadowed_storage_err;
  logic classa_timeout_cyc_shadowed_update_err;
  logic classa_crashdump_trigger_shadowed_re;
  logic classa_crashdump_trigger_shadowed_we;
  logic [1:0] classa_crashdump_trigger_shadowed_qs;
  logic [1:0] classa_crashdump_trigger_shadowed_wd;
  logic classa_crashdump_trigger_shadowed_storage_err;
  logic classa_crashdump_trigger_shadowed_update_err;
  logic classa_phase0_cyc_shadowed_re;
  logic classa_phase0_cyc_shadowed_we;
  logic [31:0] classa_phase0_cyc_shadowed_qs;
  logic [31:0] classa_phase0_cyc_shadowed_wd;
  logic classa_phase0_cyc_shadowed_storage_err;
  logic classa_phase0_cyc_shadowed_update_err;
  logic classa_phase1_cyc_shadowed_re;
  logic classa_phase1_cyc_shadowed_we;
  logic [31:0] classa_phase1_cyc_shadowed_qs;
  logic [31:0] classa_phase1_cyc_shadowed_wd;
  logic classa_phase1_cyc_shadowed_storage_err;
  logic classa_phase1_cyc_shadowed_update_err;
  logic classa_phase2_cyc_shadowed_re;
  logic classa_phase2_cyc_shadowed_we;
  logic [31:0] classa_phase2_cyc_shadowed_qs;
  logic [31:0] classa_phase2_cyc_shadowed_wd;
  logic classa_phase2_cyc_shadowed_storage_err;
  logic classa_phase2_cyc_shadowed_update_err;
  logic classa_phase3_cyc_shadowed_re;
  logic classa_phase3_cyc_shadowed_we;
  logic [31:0] classa_phase3_cyc_shadowed_qs;
  logic [31:0] classa_phase3_cyc_shadowed_wd;
  logic classa_phase3_cyc_shadowed_storage_err;
  logic classa_phase3_cyc_shadowed_update_err;
  logic classa_esc_cnt_re;
  logic [31:0] classa_esc_cnt_qs;
  logic classa_state_re;
  logic [2:0] classa_state_qs;
  logic classb_regwen_we;
  logic classb_regwen_qs;
  logic classb_regwen_wd;
  logic classb_ctrl_shadowed_re;
  logic classb_ctrl_shadowed_we;
  logic classb_ctrl_shadowed_en_qs;
  logic classb_ctrl_shadowed_en_wd;
  logic classb_ctrl_shadowed_en_storage_err;
  logic classb_ctrl_shadowed_en_update_err;
  logic classb_ctrl_shadowed_lock_qs;
  logic classb_ctrl_shadowed_lock_wd;
  logic classb_ctrl_shadowed_lock_storage_err;
  logic classb_ctrl_shadowed_lock_update_err;
  logic classb_ctrl_shadowed_en_e0_qs;
  logic classb_ctrl_shadowed_en_e0_wd;
  logic classb_ctrl_shadowed_en_e0_storage_err;
  logic classb_ctrl_shadowed_en_e0_update_err;
  logic classb_ctrl_shadowed_en_e1_qs;
  logic classb_ctrl_shadowed_en_e1_wd;
  logic classb_ctrl_shadowed_en_e1_storage_err;
  logic classb_ctrl_shadowed_en_e1_update_err;
  logic classb_ctrl_shadowed_en_e2_qs;
  logic classb_ctrl_shadowed_en_e2_wd;
  logic classb_ctrl_shadowed_en_e2_storage_err;
  logic classb_ctrl_shadowed_en_e2_update_err;
  logic classb_ctrl_shadowed_en_e3_qs;
  logic classb_ctrl_shadowed_en_e3_wd;
  logic classb_ctrl_shadowed_en_e3_storage_err;
  logic classb_ctrl_shadowed_en_e3_update_err;
  logic [1:0] classb_ctrl_shadowed_map_e0_qs;
  logic [1:0] classb_ctrl_shadowed_map_e0_wd;
  logic classb_ctrl_shadowed_map_e0_storage_err;
  logic classb_ctrl_shadowed_map_e0_update_err;
  logic [1:0] classb_ctrl_shadowed_map_e1_qs;
  logic [1:0] classb_ctrl_shadowed_map_e1_wd;
  logic classb_ctrl_shadowed_map_e1_storage_err;
  logic classb_ctrl_shadowed_map_e1_update_err;
  logic [1:0] classb_ctrl_shadowed_map_e2_qs;
  logic [1:0] classb_ctrl_shadowed_map_e2_wd;
  logic classb_ctrl_shadowed_map_e2_storage_err;
  logic classb_ctrl_shadowed_map_e2_update_err;
  logic [1:0] classb_ctrl_shadowed_map_e3_qs;
  logic [1:0] classb_ctrl_shadowed_map_e3_wd;
  logic classb_ctrl_shadowed_map_e3_storage_err;
  logic classb_ctrl_shadowed_map_e3_update_err;
  logic classb_clr_regwen_we;
  logic classb_clr_regwen_qs;
  logic classb_clr_regwen_wd;
  logic classb_clr_shadowed_re;
  logic classb_clr_shadowed_we;
  logic classb_clr_shadowed_qs;
  logic classb_clr_shadowed_wd;
  logic classb_clr_shadowed_storage_err;
  logic classb_clr_shadowed_update_err;
  logic classb_accum_cnt_re;
  logic [15:0] classb_accum_cnt_qs;
  logic classb_accum_thresh_shadowed_re;
  logic classb_accum_thresh_shadowed_we;
  logic [15:0] classb_accum_thresh_shadowed_qs;
  logic [15:0] classb_accum_thresh_shadowed_wd;
  logic classb_accum_thresh_shadowed_storage_err;
  logic classb_accum_thresh_shadowed_update_err;
  logic classb_timeout_cyc_shadowed_re;
  logic classb_timeout_cyc_shadowed_we;
  logic [31:0] classb_timeout_cyc_shadowed_qs;
  logic [31:0] classb_timeout_cyc_shadowed_wd;
  logic classb_timeout_cyc_shadowed_storage_err;
  logic classb_timeout_cyc_shadowed_update_err;
  logic classb_crashdump_trigger_shadowed_re;
  logic classb_crashdump_trigger_shadowed_we;
  logic [1:0] classb_crashdump_trigger_shadowed_qs;
  logic [1:0] classb_crashdump_trigger_shadowed_wd;
  logic classb_crashdump_trigger_shadowed_storage_err;
  logic classb_crashdump_trigger_shadowed_update_err;
  logic classb_phase0_cyc_shadowed_re;
  logic classb_phase0_cyc_shadowed_we;
  logic [31:0] classb_phase0_cyc_shadowed_qs;
  logic [31:0] classb_phase0_cyc_shadowed_wd;
  logic classb_phase0_cyc_shadowed_storage_err;
  logic classb_phase0_cyc_shadowed_update_err;
  logic classb_phase1_cyc_shadowed_re;
  logic classb_phase1_cyc_shadowed_we;
  logic [31:0] classb_phase1_cyc_shadowed_qs;
  logic [31:0] classb_phase1_cyc_shadowed_wd;
  logic classb_phase1_cyc_shadowed_storage_err;
  logic classb_phase1_cyc_shadowed_update_err;
  logic classb_phase2_cyc_shadowed_re;
  logic classb_phase2_cyc_shadowed_we;
  logic [31:0] classb_phase2_cyc_shadowed_qs;
  logic [31:0] classb_phase2_cyc_shadowed_wd;
  logic classb_phase2_cyc_shadowed_storage_err;
  logic classb_phase2_cyc_shadowed_update_err;
  logic classb_phase3_cyc_shadowed_re;
  logic classb_phase3_cyc_shadowed_we;
  logic [31:0] classb_phase3_cyc_shadowed_qs;
  logic [31:0] classb_phase3_cyc_shadowed_wd;
  logic classb_phase3_cyc_shadowed_storage_err;
  logic classb_phase3_cyc_shadowed_update_err;
  logic classb_esc_cnt_re;
  logic [31:0] classb_esc_cnt_qs;
  logic classb_state_re;
  logic [2:0] classb_state_qs;
  logic classc_regwen_we;
  logic classc_regwen_qs;
  logic classc_regwen_wd;
  logic classc_ctrl_shadowed_re;
  logic classc_ctrl_shadowed_we;
  logic classc_ctrl_shadowed_en_qs;
  logic classc_ctrl_shadowed_en_wd;
  logic classc_ctrl_shadowed_en_storage_err;
  logic classc_ctrl_shadowed_en_update_err;
  logic classc_ctrl_shadowed_lock_qs;
  logic classc_ctrl_shadowed_lock_wd;
  logic classc_ctrl_shadowed_lock_storage_err;
  logic classc_ctrl_shadowed_lock_update_err;
  logic classc_ctrl_shadowed_en_e0_qs;
  logic classc_ctrl_shadowed_en_e0_wd;
  logic classc_ctrl_shadowed_en_e0_storage_err;
  logic classc_ctrl_shadowed_en_e0_update_err;
  logic classc_ctrl_shadowed_en_e1_qs;
  logic classc_ctrl_shadowed_en_e1_wd;
  logic classc_ctrl_shadowed_en_e1_storage_err;
  logic classc_ctrl_shadowed_en_e1_update_err;
  logic classc_ctrl_shadowed_en_e2_qs;
  logic classc_ctrl_shadowed_en_e2_wd;
  logic classc_ctrl_shadowed_en_e2_storage_err;
  logic classc_ctrl_shadowed_en_e2_update_err;
  logic classc_ctrl_shadowed_en_e3_qs;
  logic classc_ctrl_shadowed_en_e3_wd;
  logic classc_ctrl_shadowed_en_e3_storage_err;
  logic classc_ctrl_shadowed_en_e3_update_err;
  logic [1:0] classc_ctrl_shadowed_map_e0_qs;
  logic [1:0] classc_ctrl_shadowed_map_e0_wd;
  logic classc_ctrl_shadowed_map_e0_storage_err;
  logic classc_ctrl_shadowed_map_e0_update_err;
  logic [1:0] classc_ctrl_shadowed_map_e1_qs;
  logic [1:0] classc_ctrl_shadowed_map_e1_wd;
  logic classc_ctrl_shadowed_map_e1_storage_err;
  logic classc_ctrl_shadowed_map_e1_update_err;
  logic [1:0] classc_ctrl_shadowed_map_e2_qs;
  logic [1:0] classc_ctrl_shadowed_map_e2_wd;
  logic classc_ctrl_shadowed_map_e2_storage_err;
  logic classc_ctrl_shadowed_map_e2_update_err;
  logic [1:0] classc_ctrl_shadowed_map_e3_qs;
  logic [1:0] classc_ctrl_shadowed_map_e3_wd;
  logic classc_ctrl_shadowed_map_e3_storage_err;
  logic classc_ctrl_shadowed_map_e3_update_err;
  logic classc_clr_regwen_we;
  logic classc_clr_regwen_qs;
  logic classc_clr_regwen_wd;
  logic classc_clr_shadowed_re;
  logic classc_clr_shadowed_we;
  logic classc_clr_shadowed_qs;
  logic classc_clr_shadowed_wd;
  logic classc_clr_shadowed_storage_err;
  logic classc_clr_shadowed_update_err;
  logic classc_accum_cnt_re;
  logic [15:0] classc_accum_cnt_qs;
  logic classc_accum_thresh_shadowed_re;
  logic classc_accum_thresh_shadowed_we;
  logic [15:0] classc_accum_thresh_shadowed_qs;
  logic [15:0] classc_accum_thresh_shadowed_wd;
  logic classc_accum_thresh_shadowed_storage_err;
  logic classc_accum_thresh_shadowed_update_err;
  logic classc_timeout_cyc_shadowed_re;
  logic classc_timeout_cyc_shadowed_we;
  logic [31:0] classc_timeout_cyc_shadowed_qs;
  logic [31:0] classc_timeout_cyc_shadowed_wd;
  logic classc_timeout_cyc_shadowed_storage_err;
  logic classc_timeout_cyc_shadowed_update_err;
  logic classc_crashdump_trigger_shadowed_re;
  logic classc_crashdump_trigger_shadowed_we;
  logic [1:0] classc_crashdump_trigger_shadowed_qs;
  logic [1:0] classc_crashdump_trigger_shadowed_wd;
  logic classc_crashdump_trigger_shadowed_storage_err;
  logic classc_crashdump_trigger_shadowed_update_err;
  logic classc_phase0_cyc_shadowed_re;
  logic classc_phase0_cyc_shadowed_we;
  logic [31:0] classc_phase0_cyc_shadowed_qs;
  logic [31:0] classc_phase0_cyc_shadowed_wd;
  logic classc_phase0_cyc_shadowed_storage_err;
  logic classc_phase0_cyc_shadowed_update_err;
  logic classc_phase1_cyc_shadowed_re;
  logic classc_phase1_cyc_shadowed_we;
  logic [31:0] classc_phase1_cyc_shadowed_qs;
  logic [31:0] classc_phase1_cyc_shadowed_wd;
  logic classc_phase1_cyc_shadowed_storage_err;
  logic classc_phase1_cyc_shadowed_update_err;
  logic classc_phase2_cyc_shadowed_re;
  logic classc_phase2_cyc_shadowed_we;
  logic [31:0] classc_phase2_cyc_shadowed_qs;
  logic [31:0] classc_phase2_cyc_shadowed_wd;
  logic classc_phase2_cyc_shadowed_storage_err;
  logic classc_phase2_cyc_shadowed_update_err;
  logic classc_phase3_cyc_shadowed_re;
  logic classc_phase3_cyc_shadowed_we;
  logic [31:0] classc_phase3_cyc_shadowed_qs;
  logic [31:0] classc_phase3_cyc_shadowed_wd;
  logic classc_phase3_cyc_shadowed_storage_err;
  logic classc_phase3_cyc_shadowed_update_err;
  logic classc_esc_cnt_re;
  logic [31:0] classc_esc_cnt_qs;
  logic classc_state_re;
  logic [2:0] classc_state_qs;
  logic classd_regwen_we;
  logic classd_regwen_qs;
  logic classd_regwen_wd;
  logic classd_ctrl_shadowed_re;
  logic classd_ctrl_shadowed_we;
  logic classd_ctrl_shadowed_en_qs;
  logic classd_ctrl_shadowed_en_wd;
  logic classd_ctrl_shadowed_en_storage_err;
  logic classd_ctrl_shadowed_en_update_err;
  logic classd_ctrl_shadowed_lock_qs;
  logic classd_ctrl_shadowed_lock_wd;
  logic classd_ctrl_shadowed_lock_storage_err;
  logic classd_ctrl_shadowed_lock_update_err;
  logic classd_ctrl_shadowed_en_e0_qs;
  logic classd_ctrl_shadowed_en_e0_wd;
  logic classd_ctrl_shadowed_en_e0_storage_err;
  logic classd_ctrl_shadowed_en_e0_update_err;
  logic classd_ctrl_shadowed_en_e1_qs;
  logic classd_ctrl_shadowed_en_e1_wd;
  logic classd_ctrl_shadowed_en_e1_storage_err;
  logic classd_ctrl_shadowed_en_e1_update_err;
  logic classd_ctrl_shadowed_en_e2_qs;
  logic classd_ctrl_shadowed_en_e2_wd;
  logic classd_ctrl_shadowed_en_e2_storage_err;
  logic classd_ctrl_shadowed_en_e2_update_err;
  logic classd_ctrl_shadowed_en_e3_qs;
  logic classd_ctrl_shadowed_en_e3_wd;
  logic classd_ctrl_shadowed_en_e3_storage_err;
  logic classd_ctrl_shadowed_en_e3_update_err;
  logic [1:0] classd_ctrl_shadowed_map_e0_qs;
  logic [1:0] classd_ctrl_shadowed_map_e0_wd;
  logic classd_ctrl_shadowed_map_e0_storage_err;
  logic classd_ctrl_shadowed_map_e0_update_err;
  logic [1:0] classd_ctrl_shadowed_map_e1_qs;
  logic [1:0] classd_ctrl_shadowed_map_e1_wd;
  logic classd_ctrl_shadowed_map_e1_storage_err;
  logic classd_ctrl_shadowed_map_e1_update_err;
  logic [1:0] classd_ctrl_shadowed_map_e2_qs;
  logic [1:0] classd_ctrl_shadowed_map_e2_wd;
  logic classd_ctrl_shadowed_map_e2_storage_err;
  logic classd_ctrl_shadowed_map_e2_update_err;
  logic [1:0] classd_ctrl_shadowed_map_e3_qs;
  logic [1:0] classd_ctrl_shadowed_map_e3_wd;
  logic classd_ctrl_shadowed_map_e3_storage_err;
  logic classd_ctrl_shadowed_map_e3_update_err;
  logic classd_clr_regwen_we;
  logic classd_clr_regwen_qs;
  logic classd_clr_regwen_wd;
  logic classd_clr_shadowed_re;
  logic classd_clr_shadowed_we;
  logic classd_clr_shadowed_qs;
  logic classd_clr_shadowed_wd;
  logic classd_clr_shadowed_storage_err;
  logic classd_clr_shadowed_update_err;
  logic classd_accum_cnt_re;
  logic [15:0] classd_accum_cnt_qs;
  logic classd_accum_thresh_shadowed_re;
  logic classd_accum_thresh_shadowed_we;
  logic [15:0] classd_accum_thresh_shadowed_qs;
  logic [15:0] classd_accum_thresh_shadowed_wd;
  logic classd_accum_thresh_shadowed_storage_err;
  logic classd_accum_thresh_shadowed_update_err;
  logic classd_timeout_cyc_shadowed_re;
  logic classd_timeout_cyc_shadowed_we;
  logic [31:0] classd_timeout_cyc_shadowed_qs;
  logic [31:0] classd_timeout_cyc_shadowed_wd;
  logic classd_timeout_cyc_shadowed_storage_err;
  logic classd_timeout_cyc_shadowed_update_err;
  logic classd_crashdump_trigger_shadowed_re;
  logic classd_crashdump_trigger_shadowed_we;
  logic [1:0] classd_crashdump_trigger_shadowed_qs;
  logic [1:0] classd_crashdump_trigger_shadowed_wd;
  logic classd_crashdump_trigger_shadowed_storage_err;
  logic classd_crashdump_trigger_shadowed_update_err;
  logic classd_phase0_cyc_shadowed_re;
  logic classd_phase0_cyc_shadowed_we;
  logic [31:0] classd_phase0_cyc_shadowed_qs;
  logic [31:0] classd_phase0_cyc_shadowed_wd;
  logic classd_phase0_cyc_shadowed_storage_err;
  logic classd_phase0_cyc_shadowed_update_err;
  logic classd_phase1_cyc_shadowed_re;
  logic classd_phase1_cyc_shadowed_we;
  logic [31:0] classd_phase1_cyc_shadowed_qs;
  logic [31:0] classd_phase1_cyc_shadowed_wd;
  logic classd_phase1_cyc_shadowed_storage_err;
  logic classd_phase1_cyc_shadowed_update_err;
  logic classd_phase2_cyc_shadowed_re;
  logic classd_phase2_cyc_shadowed_we;
  logic [31:0] classd_phase2_cyc_shadowed_qs;
  logic [31:0] classd_phase2_cyc_shadowed_wd;
  logic classd_phase2_cyc_shadowed_storage_err;
  logic classd_phase2_cyc_shadowed_update_err;
  logic classd_phase3_cyc_shadowed_re;
  logic classd_phase3_cyc_shadowed_we;
  logic [31:0] classd_phase3_cyc_shadowed_qs;
  logic [31:0] classd_phase3_cyc_shadowed_wd;
  logic classd_phase3_cyc_shadowed_storage_err;
  logic classd_phase3_cyc_shadowed_update_err;
  logic classd_esc_cnt_re;
  logic [31:0] classd_esc_cnt_qs;
  logic classd_state_re;
  logic [2:0] classd_state_qs;

  // Register instances
  // R[intr_state]: V(False)
  //   F[classa]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_classa (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_classa_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.classa.de),
    .d      (hw2reg.intr_state.classa.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.classa.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_classa_qs)
  );

  //   F[classb]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_classb (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_classb_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.classb.de),
    .d      (hw2reg.intr_state.classb.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.classb.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_classb_qs)
  );

  //   F[classc]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_classc (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_classc_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.classc.de),
    .d      (hw2reg.intr_state.classc.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.classc.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_classc_qs)
  );

  //   F[classd]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_intr_state_classd (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_state_we),
    .wd     (intr_state_classd_wd),

    // from internal hardware
    .de     (hw2reg.intr_state.classd.de),
    .d      (hw2reg.intr_state.classd.d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_state.classd.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_state_classd_qs)
  );


  // R[intr_enable]: V(False)
  //   F[classa]: 0:0
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_classa (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_classa_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.classa.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_classa_qs)
  );

  //   F[classb]: 1:1
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_classb (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_classb_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.classb.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_classb_qs)
  );

  //   F[classc]: 2:2
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_classc (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_classc_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.classc.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_classc_qs)
  );

  //   F[classd]: 3:3
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_intr_enable_classd (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (intr_enable_we),
    .wd     (intr_enable_classd_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.intr_enable.classd.q),
    .ds     (),

    // to register interface (read)
    .qs     (intr_enable_classd_qs)
  );


  // R[intr_test]: V(True)
  logic intr_test_qe;
  logic [3:0] intr_test_flds_we;
  assign intr_test_qe = &intr_test_flds_we;
  //   F[classa]: 0:0
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_classa (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_classa_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[0]),
    .q      (reg2hw.intr_test.classa.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.classa.qe = intr_test_qe;

  //   F[classb]: 1:1
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_classb (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_classb_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[1]),
    .q      (reg2hw.intr_test.classb.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.classb.qe = intr_test_qe;

  //   F[classc]: 2:2
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_classc (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_classc_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[2]),
    .q      (reg2hw.intr_test.classc.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.classc.qe = intr_test_qe;

  //   F[classd]: 3:3
  prim_subreg_ext #(
    .DW    (1)
  ) u_intr_test_classd (
    .re     (1'b0),
    .we     (intr_test_we),
    .wd     (intr_test_classd_wd),
    .d      ('0),
    .qre    (),
    .qe     (intr_test_flds_we[3]),
    .q      (reg2hw.intr_test.classd.q),
    .ds     (),
    .qs     ()
  );
  assign reg2hw.intr_test.classd.qe = intr_test_qe;


  // R[ping_timer_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_ping_timer_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (ping_timer_regwen_we),
    .wd     (ping_timer_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (ping_timer_regwen_qs)
  );


  // R[ping_timeout_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic ping_timeout_cyc_shadowed_gated_we;
  assign ping_timeout_cyc_shadowed_gated_we = ping_timeout_cyc_shadowed_we & ping_timer_regwen_qs;
  prim_subreg_shadow #(
    .DW      (16),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (16'h100)
  ) u_ping_timeout_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (ping_timeout_cyc_shadowed_re),
    .we     (ping_timeout_cyc_shadowed_gated_we),
    .wd     (ping_timeout_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.ping_timeout_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (ping_timeout_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (ping_timeout_cyc_shadowed_update_err),
    .err_storage (ping_timeout_cyc_shadowed_storage_err)
  );


  // R[ping_timer_en_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic ping_timer_en_shadowed_gated_we;
  assign ping_timer_en_shadowed_gated_we = ping_timer_en_shadowed_we & ping_timer_regwen_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1S),
    .RESVAL  (1'h0)
  ) u_ping_timer_en_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (ping_timer_en_shadowed_re),
    .we     (ping_timer_en_shadowed_gated_we),
    .wd     (ping_timer_en_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.ping_timer_en_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (ping_timer_en_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (ping_timer_en_shadowed_update_err),
    .err_storage (ping_timer_en_shadowed_storage_err)
  );


  // Subregister 0 of Multireg alert_regwen
  // R[alert_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_0_we),
    .wd     (alert_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_0_qs)
  );


  // Subregister 1 of Multireg alert_regwen
  // R[alert_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_1_we),
    .wd     (alert_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_1_qs)
  );


  // Subregister 2 of Multireg alert_regwen
  // R[alert_regwen_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_2_we),
    .wd     (alert_regwen_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_2_qs)
  );


  // Subregister 3 of Multireg alert_regwen
  // R[alert_regwen_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_3_we),
    .wd     (alert_regwen_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_3_qs)
  );


  // Subregister 4 of Multireg alert_regwen
  // R[alert_regwen_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_4_we),
    .wd     (alert_regwen_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_4_qs)
  );


  // Subregister 5 of Multireg alert_regwen
  // R[alert_regwen_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_5_we),
    .wd     (alert_regwen_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_5_qs)
  );


  // Subregister 6 of Multireg alert_regwen
  // R[alert_regwen_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_6_we),
    .wd     (alert_regwen_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_6_qs)
  );


  // Subregister 7 of Multireg alert_regwen
  // R[alert_regwen_7]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_7_we),
    .wd     (alert_regwen_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[7].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_7_qs)
  );


  // Subregister 8 of Multireg alert_regwen
  // R[alert_regwen_8]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_8_we),
    .wd     (alert_regwen_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[8].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_8_qs)
  );


  // Subregister 9 of Multireg alert_regwen
  // R[alert_regwen_9]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_9_we),
    .wd     (alert_regwen_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[9].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_9_qs)
  );


  // Subregister 10 of Multireg alert_regwen
  // R[alert_regwen_10]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_10 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_10_we),
    .wd     (alert_regwen_10_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[10].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_10_qs)
  );


  // Subregister 11 of Multireg alert_regwen
  // R[alert_regwen_11]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_11 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_11_we),
    .wd     (alert_regwen_11_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[11].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_11_qs)
  );


  // Subregister 12 of Multireg alert_regwen
  // R[alert_regwen_12]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_12 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_12_we),
    .wd     (alert_regwen_12_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[12].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_12_qs)
  );


  // Subregister 13 of Multireg alert_regwen
  // R[alert_regwen_13]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_13 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_13_we),
    .wd     (alert_regwen_13_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[13].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_13_qs)
  );


  // Subregister 14 of Multireg alert_regwen
  // R[alert_regwen_14]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_14 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_14_we),
    .wd     (alert_regwen_14_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[14].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_14_qs)
  );


  // Subregister 15 of Multireg alert_regwen
  // R[alert_regwen_15]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_15 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_15_we),
    .wd     (alert_regwen_15_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[15].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_15_qs)
  );


  // Subregister 16 of Multireg alert_regwen
  // R[alert_regwen_16]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_16 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_16_we),
    .wd     (alert_regwen_16_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[16].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_16_qs)
  );


  // Subregister 17 of Multireg alert_regwen
  // R[alert_regwen_17]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_17 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_17_we),
    .wd     (alert_regwen_17_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[17].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_17_qs)
  );


  // Subregister 18 of Multireg alert_regwen
  // R[alert_regwen_18]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_18 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_18_we),
    .wd     (alert_regwen_18_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[18].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_18_qs)
  );


  // Subregister 19 of Multireg alert_regwen
  // R[alert_regwen_19]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_19 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_19_we),
    .wd     (alert_regwen_19_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[19].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_19_qs)
  );


  // Subregister 20 of Multireg alert_regwen
  // R[alert_regwen_20]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_20 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_20_we),
    .wd     (alert_regwen_20_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[20].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_20_qs)
  );


  // Subregister 21 of Multireg alert_regwen
  // R[alert_regwen_21]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_21 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_21_we),
    .wd     (alert_regwen_21_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[21].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_21_qs)
  );


  // Subregister 22 of Multireg alert_regwen
  // R[alert_regwen_22]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_22 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_22_we),
    .wd     (alert_regwen_22_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[22].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_22_qs)
  );


  // Subregister 23 of Multireg alert_regwen
  // R[alert_regwen_23]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_23 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_23_we),
    .wd     (alert_regwen_23_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[23].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_23_qs)
  );


  // Subregister 24 of Multireg alert_regwen
  // R[alert_regwen_24]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_24 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_24_we),
    .wd     (alert_regwen_24_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[24].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_24_qs)
  );


  // Subregister 25 of Multireg alert_regwen
  // R[alert_regwen_25]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_25 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_25_we),
    .wd     (alert_regwen_25_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[25].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_25_qs)
  );


  // Subregister 26 of Multireg alert_regwen
  // R[alert_regwen_26]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_26 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_26_we),
    .wd     (alert_regwen_26_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[26].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_26_qs)
  );


  // Subregister 27 of Multireg alert_regwen
  // R[alert_regwen_27]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_27 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_27_we),
    .wd     (alert_regwen_27_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[27].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_27_qs)
  );


  // Subregister 28 of Multireg alert_regwen
  // R[alert_regwen_28]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_28 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_28_we),
    .wd     (alert_regwen_28_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[28].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_28_qs)
  );


  // Subregister 29 of Multireg alert_regwen
  // R[alert_regwen_29]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_29 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_29_we),
    .wd     (alert_regwen_29_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[29].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_29_qs)
  );


  // Subregister 30 of Multireg alert_regwen
  // R[alert_regwen_30]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_30 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_30_we),
    .wd     (alert_regwen_30_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[30].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_30_qs)
  );


  // Subregister 31 of Multireg alert_regwen
  // R[alert_regwen_31]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_31 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_31_we),
    .wd     (alert_regwen_31_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[31].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_31_qs)
  );


  // Subregister 32 of Multireg alert_regwen
  // R[alert_regwen_32]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_32 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_32_we),
    .wd     (alert_regwen_32_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[32].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_32_qs)
  );


  // Subregister 33 of Multireg alert_regwen
  // R[alert_regwen_33]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_33 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_33_we),
    .wd     (alert_regwen_33_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[33].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_33_qs)
  );


  // Subregister 34 of Multireg alert_regwen
  // R[alert_regwen_34]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_34 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_34_we),
    .wd     (alert_regwen_34_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[34].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_34_qs)
  );


  // Subregister 35 of Multireg alert_regwen
  // R[alert_regwen_35]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_35 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_35_we),
    .wd     (alert_regwen_35_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[35].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_35_qs)
  );


  // Subregister 36 of Multireg alert_regwen
  // R[alert_regwen_36]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_36 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_36_we),
    .wd     (alert_regwen_36_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[36].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_36_qs)
  );


  // Subregister 37 of Multireg alert_regwen
  // R[alert_regwen_37]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_37 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_37_we),
    .wd     (alert_regwen_37_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[37].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_37_qs)
  );


  // Subregister 38 of Multireg alert_regwen
  // R[alert_regwen_38]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_38 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_38_we),
    .wd     (alert_regwen_38_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[38].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_38_qs)
  );


  // Subregister 39 of Multireg alert_regwen
  // R[alert_regwen_39]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_39 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_39_we),
    .wd     (alert_regwen_39_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[39].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_39_qs)
  );


  // Subregister 40 of Multireg alert_regwen
  // R[alert_regwen_40]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_40 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_40_we),
    .wd     (alert_regwen_40_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[40].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_40_qs)
  );


  // Subregister 41 of Multireg alert_regwen
  // R[alert_regwen_41]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_41 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_41_we),
    .wd     (alert_regwen_41_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[41].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_41_qs)
  );


  // Subregister 42 of Multireg alert_regwen
  // R[alert_regwen_42]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_42 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_42_we),
    .wd     (alert_regwen_42_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[42].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_42_qs)
  );


  // Subregister 43 of Multireg alert_regwen
  // R[alert_regwen_43]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_43 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_43_we),
    .wd     (alert_regwen_43_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[43].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_43_qs)
  );


  // Subregister 44 of Multireg alert_regwen
  // R[alert_regwen_44]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_44 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_44_we),
    .wd     (alert_regwen_44_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[44].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_44_qs)
  );


  // Subregister 45 of Multireg alert_regwen
  // R[alert_regwen_45]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_45 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_45_we),
    .wd     (alert_regwen_45_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[45].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_45_qs)
  );


  // Subregister 46 of Multireg alert_regwen
  // R[alert_regwen_46]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_46 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_46_we),
    .wd     (alert_regwen_46_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[46].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_46_qs)
  );


  // Subregister 47 of Multireg alert_regwen
  // R[alert_regwen_47]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_47 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_47_we),
    .wd     (alert_regwen_47_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[47].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_47_qs)
  );


  // Subregister 48 of Multireg alert_regwen
  // R[alert_regwen_48]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_48 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_48_we),
    .wd     (alert_regwen_48_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[48].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_48_qs)
  );


  // Subregister 49 of Multireg alert_regwen
  // R[alert_regwen_49]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_49 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_49_we),
    .wd     (alert_regwen_49_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[49].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_49_qs)
  );


  // Subregister 50 of Multireg alert_regwen
  // R[alert_regwen_50]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_50 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_50_we),
    .wd     (alert_regwen_50_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[50].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_50_qs)
  );


  // Subregister 51 of Multireg alert_regwen
  // R[alert_regwen_51]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_51 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_51_we),
    .wd     (alert_regwen_51_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[51].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_51_qs)
  );


  // Subregister 52 of Multireg alert_regwen
  // R[alert_regwen_52]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_52 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_52_we),
    .wd     (alert_regwen_52_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[52].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_52_qs)
  );


  // Subregister 53 of Multireg alert_regwen
  // R[alert_regwen_53]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_53 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_53_we),
    .wd     (alert_regwen_53_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[53].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_53_qs)
  );


  // Subregister 54 of Multireg alert_regwen
  // R[alert_regwen_54]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_54 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_54_we),
    .wd     (alert_regwen_54_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[54].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_54_qs)
  );


  // Subregister 55 of Multireg alert_regwen
  // R[alert_regwen_55]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_55 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_55_we),
    .wd     (alert_regwen_55_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[55].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_55_qs)
  );


  // Subregister 56 of Multireg alert_regwen
  // R[alert_regwen_56]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_56 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_56_we),
    .wd     (alert_regwen_56_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[56].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_56_qs)
  );


  // Subregister 57 of Multireg alert_regwen
  // R[alert_regwen_57]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_57 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_57_we),
    .wd     (alert_regwen_57_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[57].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_57_qs)
  );


  // Subregister 58 of Multireg alert_regwen
  // R[alert_regwen_58]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_58 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_58_we),
    .wd     (alert_regwen_58_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[58].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_58_qs)
  );


  // Subregister 59 of Multireg alert_regwen
  // R[alert_regwen_59]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_59 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_59_we),
    .wd     (alert_regwen_59_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[59].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_59_qs)
  );


  // Subregister 60 of Multireg alert_regwen
  // R[alert_regwen_60]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_60 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_60_we),
    .wd     (alert_regwen_60_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[60].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_60_qs)
  );


  // Subregister 61 of Multireg alert_regwen
  // R[alert_regwen_61]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_61 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_61_we),
    .wd     (alert_regwen_61_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[61].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_61_qs)
  );


  // Subregister 62 of Multireg alert_regwen
  // R[alert_regwen_62]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_62 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_62_we),
    .wd     (alert_regwen_62_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[62].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_62_qs)
  );


  // Subregister 63 of Multireg alert_regwen
  // R[alert_regwen_63]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_63 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_63_we),
    .wd     (alert_regwen_63_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[63].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_63_qs)
  );


  // Subregister 64 of Multireg alert_regwen
  // R[alert_regwen_64]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_alert_regwen_64 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_regwen_64_we),
    .wd     (alert_regwen_64_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_regwen[64].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_regwen_64_qs)
  );


  // Subregister 0 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_0]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_0_gated_we;
  assign alert_en_shadowed_0_gated_we = alert_en_shadowed_0_we & alert_regwen_0_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_0_re),
    .we     (alert_en_shadowed_0_gated_we),
    .wd     (alert_en_shadowed_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_0_update_err),
    .err_storage (alert_en_shadowed_0_storage_err)
  );


  // Subregister 1 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_1]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_1_gated_we;
  assign alert_en_shadowed_1_gated_we = alert_en_shadowed_1_we & alert_regwen_1_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_1_re),
    .we     (alert_en_shadowed_1_gated_we),
    .wd     (alert_en_shadowed_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_1_update_err),
    .err_storage (alert_en_shadowed_1_storage_err)
  );


  // Subregister 2 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_2]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_2_gated_we;
  assign alert_en_shadowed_2_gated_we = alert_en_shadowed_2_we & alert_regwen_2_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_2_re),
    .we     (alert_en_shadowed_2_gated_we),
    .wd     (alert_en_shadowed_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_2_update_err),
    .err_storage (alert_en_shadowed_2_storage_err)
  );


  // Subregister 3 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_3]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_3_gated_we;
  assign alert_en_shadowed_3_gated_we = alert_en_shadowed_3_we & alert_regwen_3_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_3_re),
    .we     (alert_en_shadowed_3_gated_we),
    .wd     (alert_en_shadowed_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_3_update_err),
    .err_storage (alert_en_shadowed_3_storage_err)
  );


  // Subregister 4 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_4]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_4_gated_we;
  assign alert_en_shadowed_4_gated_we = alert_en_shadowed_4_we & alert_regwen_4_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_4_re),
    .we     (alert_en_shadowed_4_gated_we),
    .wd     (alert_en_shadowed_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_4_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_4_update_err),
    .err_storage (alert_en_shadowed_4_storage_err)
  );


  // Subregister 5 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_5]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_5_gated_we;
  assign alert_en_shadowed_5_gated_we = alert_en_shadowed_5_we & alert_regwen_5_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_5_re),
    .we     (alert_en_shadowed_5_gated_we),
    .wd     (alert_en_shadowed_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_5_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_5_update_err),
    .err_storage (alert_en_shadowed_5_storage_err)
  );


  // Subregister 6 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_6]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_6_gated_we;
  assign alert_en_shadowed_6_gated_we = alert_en_shadowed_6_we & alert_regwen_6_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_6_re),
    .we     (alert_en_shadowed_6_gated_we),
    .wd     (alert_en_shadowed_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_6_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_6_update_err),
    .err_storage (alert_en_shadowed_6_storage_err)
  );


  // Subregister 7 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_7]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_7_gated_we;
  assign alert_en_shadowed_7_gated_we = alert_en_shadowed_7_we & alert_regwen_7_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_7_re),
    .we     (alert_en_shadowed_7_gated_we),
    .wd     (alert_en_shadowed_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[7].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_7_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_7_update_err),
    .err_storage (alert_en_shadowed_7_storage_err)
  );


  // Subregister 8 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_8]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_8_gated_we;
  assign alert_en_shadowed_8_gated_we = alert_en_shadowed_8_we & alert_regwen_8_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_8_re),
    .we     (alert_en_shadowed_8_gated_we),
    .wd     (alert_en_shadowed_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[8].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_8_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_8_update_err),
    .err_storage (alert_en_shadowed_8_storage_err)
  );


  // Subregister 9 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_9]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_9_gated_we;
  assign alert_en_shadowed_9_gated_we = alert_en_shadowed_9_we & alert_regwen_9_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_9_re),
    .we     (alert_en_shadowed_9_gated_we),
    .wd     (alert_en_shadowed_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[9].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_9_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_9_update_err),
    .err_storage (alert_en_shadowed_9_storage_err)
  );


  // Subregister 10 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_10]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_10_gated_we;
  assign alert_en_shadowed_10_gated_we = alert_en_shadowed_10_we & alert_regwen_10_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_10 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_10_re),
    .we     (alert_en_shadowed_10_gated_we),
    .wd     (alert_en_shadowed_10_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[10].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_10_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_10_update_err),
    .err_storage (alert_en_shadowed_10_storage_err)
  );


  // Subregister 11 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_11]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_11_gated_we;
  assign alert_en_shadowed_11_gated_we = alert_en_shadowed_11_we & alert_regwen_11_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_11 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_11_re),
    .we     (alert_en_shadowed_11_gated_we),
    .wd     (alert_en_shadowed_11_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[11].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_11_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_11_update_err),
    .err_storage (alert_en_shadowed_11_storage_err)
  );


  // Subregister 12 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_12]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_12_gated_we;
  assign alert_en_shadowed_12_gated_we = alert_en_shadowed_12_we & alert_regwen_12_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_12 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_12_re),
    .we     (alert_en_shadowed_12_gated_we),
    .wd     (alert_en_shadowed_12_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[12].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_12_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_12_update_err),
    .err_storage (alert_en_shadowed_12_storage_err)
  );


  // Subregister 13 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_13]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_13_gated_we;
  assign alert_en_shadowed_13_gated_we = alert_en_shadowed_13_we & alert_regwen_13_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_13 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_13_re),
    .we     (alert_en_shadowed_13_gated_we),
    .wd     (alert_en_shadowed_13_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[13].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_13_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_13_update_err),
    .err_storage (alert_en_shadowed_13_storage_err)
  );


  // Subregister 14 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_14]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_14_gated_we;
  assign alert_en_shadowed_14_gated_we = alert_en_shadowed_14_we & alert_regwen_14_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_14 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_14_re),
    .we     (alert_en_shadowed_14_gated_we),
    .wd     (alert_en_shadowed_14_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[14].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_14_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_14_update_err),
    .err_storage (alert_en_shadowed_14_storage_err)
  );


  // Subregister 15 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_15]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_15_gated_we;
  assign alert_en_shadowed_15_gated_we = alert_en_shadowed_15_we & alert_regwen_15_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_15 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_15_re),
    .we     (alert_en_shadowed_15_gated_we),
    .wd     (alert_en_shadowed_15_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[15].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_15_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_15_update_err),
    .err_storage (alert_en_shadowed_15_storage_err)
  );


  // Subregister 16 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_16]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_16_gated_we;
  assign alert_en_shadowed_16_gated_we = alert_en_shadowed_16_we & alert_regwen_16_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_16 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_16_re),
    .we     (alert_en_shadowed_16_gated_we),
    .wd     (alert_en_shadowed_16_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[16].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_16_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_16_update_err),
    .err_storage (alert_en_shadowed_16_storage_err)
  );


  // Subregister 17 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_17]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_17_gated_we;
  assign alert_en_shadowed_17_gated_we = alert_en_shadowed_17_we & alert_regwen_17_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_17 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_17_re),
    .we     (alert_en_shadowed_17_gated_we),
    .wd     (alert_en_shadowed_17_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[17].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_17_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_17_update_err),
    .err_storage (alert_en_shadowed_17_storage_err)
  );


  // Subregister 18 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_18]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_18_gated_we;
  assign alert_en_shadowed_18_gated_we = alert_en_shadowed_18_we & alert_regwen_18_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_18 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_18_re),
    .we     (alert_en_shadowed_18_gated_we),
    .wd     (alert_en_shadowed_18_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[18].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_18_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_18_update_err),
    .err_storage (alert_en_shadowed_18_storage_err)
  );


  // Subregister 19 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_19]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_19_gated_we;
  assign alert_en_shadowed_19_gated_we = alert_en_shadowed_19_we & alert_regwen_19_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_19 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_19_re),
    .we     (alert_en_shadowed_19_gated_we),
    .wd     (alert_en_shadowed_19_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[19].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_19_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_19_update_err),
    .err_storage (alert_en_shadowed_19_storage_err)
  );


  // Subregister 20 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_20]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_20_gated_we;
  assign alert_en_shadowed_20_gated_we = alert_en_shadowed_20_we & alert_regwen_20_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_20 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_20_re),
    .we     (alert_en_shadowed_20_gated_we),
    .wd     (alert_en_shadowed_20_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[20].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_20_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_20_update_err),
    .err_storage (alert_en_shadowed_20_storage_err)
  );


  // Subregister 21 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_21]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_21_gated_we;
  assign alert_en_shadowed_21_gated_we = alert_en_shadowed_21_we & alert_regwen_21_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_21 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_21_re),
    .we     (alert_en_shadowed_21_gated_we),
    .wd     (alert_en_shadowed_21_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[21].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_21_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_21_update_err),
    .err_storage (alert_en_shadowed_21_storage_err)
  );


  // Subregister 22 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_22]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_22_gated_we;
  assign alert_en_shadowed_22_gated_we = alert_en_shadowed_22_we & alert_regwen_22_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_22 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_22_re),
    .we     (alert_en_shadowed_22_gated_we),
    .wd     (alert_en_shadowed_22_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[22].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_22_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_22_update_err),
    .err_storage (alert_en_shadowed_22_storage_err)
  );


  // Subregister 23 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_23]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_23_gated_we;
  assign alert_en_shadowed_23_gated_we = alert_en_shadowed_23_we & alert_regwen_23_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_23 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_23_re),
    .we     (alert_en_shadowed_23_gated_we),
    .wd     (alert_en_shadowed_23_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[23].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_23_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_23_update_err),
    .err_storage (alert_en_shadowed_23_storage_err)
  );


  // Subregister 24 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_24]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_24_gated_we;
  assign alert_en_shadowed_24_gated_we = alert_en_shadowed_24_we & alert_regwen_24_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_24 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_24_re),
    .we     (alert_en_shadowed_24_gated_we),
    .wd     (alert_en_shadowed_24_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[24].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_24_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_24_update_err),
    .err_storage (alert_en_shadowed_24_storage_err)
  );


  // Subregister 25 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_25]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_25_gated_we;
  assign alert_en_shadowed_25_gated_we = alert_en_shadowed_25_we & alert_regwen_25_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_25 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_25_re),
    .we     (alert_en_shadowed_25_gated_we),
    .wd     (alert_en_shadowed_25_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[25].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_25_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_25_update_err),
    .err_storage (alert_en_shadowed_25_storage_err)
  );


  // Subregister 26 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_26]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_26_gated_we;
  assign alert_en_shadowed_26_gated_we = alert_en_shadowed_26_we & alert_regwen_26_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_26 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_26_re),
    .we     (alert_en_shadowed_26_gated_we),
    .wd     (alert_en_shadowed_26_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[26].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_26_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_26_update_err),
    .err_storage (alert_en_shadowed_26_storage_err)
  );


  // Subregister 27 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_27]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_27_gated_we;
  assign alert_en_shadowed_27_gated_we = alert_en_shadowed_27_we & alert_regwen_27_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_27 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_27_re),
    .we     (alert_en_shadowed_27_gated_we),
    .wd     (alert_en_shadowed_27_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[27].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_27_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_27_update_err),
    .err_storage (alert_en_shadowed_27_storage_err)
  );


  // Subregister 28 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_28]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_28_gated_we;
  assign alert_en_shadowed_28_gated_we = alert_en_shadowed_28_we & alert_regwen_28_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_28 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_28_re),
    .we     (alert_en_shadowed_28_gated_we),
    .wd     (alert_en_shadowed_28_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[28].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_28_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_28_update_err),
    .err_storage (alert_en_shadowed_28_storage_err)
  );


  // Subregister 29 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_29]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_29_gated_we;
  assign alert_en_shadowed_29_gated_we = alert_en_shadowed_29_we & alert_regwen_29_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_29 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_29_re),
    .we     (alert_en_shadowed_29_gated_we),
    .wd     (alert_en_shadowed_29_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[29].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_29_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_29_update_err),
    .err_storage (alert_en_shadowed_29_storage_err)
  );


  // Subregister 30 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_30]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_30_gated_we;
  assign alert_en_shadowed_30_gated_we = alert_en_shadowed_30_we & alert_regwen_30_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_30 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_30_re),
    .we     (alert_en_shadowed_30_gated_we),
    .wd     (alert_en_shadowed_30_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[30].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_30_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_30_update_err),
    .err_storage (alert_en_shadowed_30_storage_err)
  );


  // Subregister 31 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_31]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_31_gated_we;
  assign alert_en_shadowed_31_gated_we = alert_en_shadowed_31_we & alert_regwen_31_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_31 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_31_re),
    .we     (alert_en_shadowed_31_gated_we),
    .wd     (alert_en_shadowed_31_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[31].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_31_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_31_update_err),
    .err_storage (alert_en_shadowed_31_storage_err)
  );


  // Subregister 32 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_32]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_32_gated_we;
  assign alert_en_shadowed_32_gated_we = alert_en_shadowed_32_we & alert_regwen_32_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_32 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_32_re),
    .we     (alert_en_shadowed_32_gated_we),
    .wd     (alert_en_shadowed_32_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[32].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_32_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_32_update_err),
    .err_storage (alert_en_shadowed_32_storage_err)
  );


  // Subregister 33 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_33]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_33_gated_we;
  assign alert_en_shadowed_33_gated_we = alert_en_shadowed_33_we & alert_regwen_33_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_33 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_33_re),
    .we     (alert_en_shadowed_33_gated_we),
    .wd     (alert_en_shadowed_33_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[33].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_33_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_33_update_err),
    .err_storage (alert_en_shadowed_33_storage_err)
  );


  // Subregister 34 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_34]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_34_gated_we;
  assign alert_en_shadowed_34_gated_we = alert_en_shadowed_34_we & alert_regwen_34_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_34 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_34_re),
    .we     (alert_en_shadowed_34_gated_we),
    .wd     (alert_en_shadowed_34_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[34].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_34_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_34_update_err),
    .err_storage (alert_en_shadowed_34_storage_err)
  );


  // Subregister 35 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_35]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_35_gated_we;
  assign alert_en_shadowed_35_gated_we = alert_en_shadowed_35_we & alert_regwen_35_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_35 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_35_re),
    .we     (alert_en_shadowed_35_gated_we),
    .wd     (alert_en_shadowed_35_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[35].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_35_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_35_update_err),
    .err_storage (alert_en_shadowed_35_storage_err)
  );


  // Subregister 36 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_36]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_36_gated_we;
  assign alert_en_shadowed_36_gated_we = alert_en_shadowed_36_we & alert_regwen_36_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_36 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_36_re),
    .we     (alert_en_shadowed_36_gated_we),
    .wd     (alert_en_shadowed_36_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[36].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_36_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_36_update_err),
    .err_storage (alert_en_shadowed_36_storage_err)
  );


  // Subregister 37 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_37]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_37_gated_we;
  assign alert_en_shadowed_37_gated_we = alert_en_shadowed_37_we & alert_regwen_37_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_37 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_37_re),
    .we     (alert_en_shadowed_37_gated_we),
    .wd     (alert_en_shadowed_37_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[37].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_37_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_37_update_err),
    .err_storage (alert_en_shadowed_37_storage_err)
  );


  // Subregister 38 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_38]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_38_gated_we;
  assign alert_en_shadowed_38_gated_we = alert_en_shadowed_38_we & alert_regwen_38_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_38 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_38_re),
    .we     (alert_en_shadowed_38_gated_we),
    .wd     (alert_en_shadowed_38_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[38].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_38_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_38_update_err),
    .err_storage (alert_en_shadowed_38_storage_err)
  );


  // Subregister 39 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_39]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_39_gated_we;
  assign alert_en_shadowed_39_gated_we = alert_en_shadowed_39_we & alert_regwen_39_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_39 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_39_re),
    .we     (alert_en_shadowed_39_gated_we),
    .wd     (alert_en_shadowed_39_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[39].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_39_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_39_update_err),
    .err_storage (alert_en_shadowed_39_storage_err)
  );


  // Subregister 40 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_40]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_40_gated_we;
  assign alert_en_shadowed_40_gated_we = alert_en_shadowed_40_we & alert_regwen_40_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_40 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_40_re),
    .we     (alert_en_shadowed_40_gated_we),
    .wd     (alert_en_shadowed_40_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[40].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_40_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_40_update_err),
    .err_storage (alert_en_shadowed_40_storage_err)
  );


  // Subregister 41 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_41]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_41_gated_we;
  assign alert_en_shadowed_41_gated_we = alert_en_shadowed_41_we & alert_regwen_41_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_41 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_41_re),
    .we     (alert_en_shadowed_41_gated_we),
    .wd     (alert_en_shadowed_41_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[41].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_41_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_41_update_err),
    .err_storage (alert_en_shadowed_41_storage_err)
  );


  // Subregister 42 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_42]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_42_gated_we;
  assign alert_en_shadowed_42_gated_we = alert_en_shadowed_42_we & alert_regwen_42_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_42 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_42_re),
    .we     (alert_en_shadowed_42_gated_we),
    .wd     (alert_en_shadowed_42_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[42].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_42_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_42_update_err),
    .err_storage (alert_en_shadowed_42_storage_err)
  );


  // Subregister 43 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_43]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_43_gated_we;
  assign alert_en_shadowed_43_gated_we = alert_en_shadowed_43_we & alert_regwen_43_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_43 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_43_re),
    .we     (alert_en_shadowed_43_gated_we),
    .wd     (alert_en_shadowed_43_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[43].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_43_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_43_update_err),
    .err_storage (alert_en_shadowed_43_storage_err)
  );


  // Subregister 44 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_44]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_44_gated_we;
  assign alert_en_shadowed_44_gated_we = alert_en_shadowed_44_we & alert_regwen_44_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_44 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_44_re),
    .we     (alert_en_shadowed_44_gated_we),
    .wd     (alert_en_shadowed_44_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[44].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_44_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_44_update_err),
    .err_storage (alert_en_shadowed_44_storage_err)
  );


  // Subregister 45 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_45]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_45_gated_we;
  assign alert_en_shadowed_45_gated_we = alert_en_shadowed_45_we & alert_regwen_45_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_45 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_45_re),
    .we     (alert_en_shadowed_45_gated_we),
    .wd     (alert_en_shadowed_45_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[45].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_45_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_45_update_err),
    .err_storage (alert_en_shadowed_45_storage_err)
  );


  // Subregister 46 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_46]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_46_gated_we;
  assign alert_en_shadowed_46_gated_we = alert_en_shadowed_46_we & alert_regwen_46_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_46 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_46_re),
    .we     (alert_en_shadowed_46_gated_we),
    .wd     (alert_en_shadowed_46_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[46].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_46_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_46_update_err),
    .err_storage (alert_en_shadowed_46_storage_err)
  );


  // Subregister 47 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_47]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_47_gated_we;
  assign alert_en_shadowed_47_gated_we = alert_en_shadowed_47_we & alert_regwen_47_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_47 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_47_re),
    .we     (alert_en_shadowed_47_gated_we),
    .wd     (alert_en_shadowed_47_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[47].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_47_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_47_update_err),
    .err_storage (alert_en_shadowed_47_storage_err)
  );


  // Subregister 48 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_48]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_48_gated_we;
  assign alert_en_shadowed_48_gated_we = alert_en_shadowed_48_we & alert_regwen_48_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_48 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_48_re),
    .we     (alert_en_shadowed_48_gated_we),
    .wd     (alert_en_shadowed_48_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[48].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_48_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_48_update_err),
    .err_storage (alert_en_shadowed_48_storage_err)
  );


  // Subregister 49 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_49]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_49_gated_we;
  assign alert_en_shadowed_49_gated_we = alert_en_shadowed_49_we & alert_regwen_49_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_49 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_49_re),
    .we     (alert_en_shadowed_49_gated_we),
    .wd     (alert_en_shadowed_49_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[49].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_49_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_49_update_err),
    .err_storage (alert_en_shadowed_49_storage_err)
  );


  // Subregister 50 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_50]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_50_gated_we;
  assign alert_en_shadowed_50_gated_we = alert_en_shadowed_50_we & alert_regwen_50_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_50 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_50_re),
    .we     (alert_en_shadowed_50_gated_we),
    .wd     (alert_en_shadowed_50_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[50].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_50_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_50_update_err),
    .err_storage (alert_en_shadowed_50_storage_err)
  );


  // Subregister 51 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_51]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_51_gated_we;
  assign alert_en_shadowed_51_gated_we = alert_en_shadowed_51_we & alert_regwen_51_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_51 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_51_re),
    .we     (alert_en_shadowed_51_gated_we),
    .wd     (alert_en_shadowed_51_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[51].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_51_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_51_update_err),
    .err_storage (alert_en_shadowed_51_storage_err)
  );


  // Subregister 52 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_52]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_52_gated_we;
  assign alert_en_shadowed_52_gated_we = alert_en_shadowed_52_we & alert_regwen_52_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_52 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_52_re),
    .we     (alert_en_shadowed_52_gated_we),
    .wd     (alert_en_shadowed_52_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[52].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_52_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_52_update_err),
    .err_storage (alert_en_shadowed_52_storage_err)
  );


  // Subregister 53 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_53]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_53_gated_we;
  assign alert_en_shadowed_53_gated_we = alert_en_shadowed_53_we & alert_regwen_53_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_53 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_53_re),
    .we     (alert_en_shadowed_53_gated_we),
    .wd     (alert_en_shadowed_53_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[53].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_53_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_53_update_err),
    .err_storage (alert_en_shadowed_53_storage_err)
  );


  // Subregister 54 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_54]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_54_gated_we;
  assign alert_en_shadowed_54_gated_we = alert_en_shadowed_54_we & alert_regwen_54_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_54 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_54_re),
    .we     (alert_en_shadowed_54_gated_we),
    .wd     (alert_en_shadowed_54_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[54].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_54_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_54_update_err),
    .err_storage (alert_en_shadowed_54_storage_err)
  );


  // Subregister 55 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_55]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_55_gated_we;
  assign alert_en_shadowed_55_gated_we = alert_en_shadowed_55_we & alert_regwen_55_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_55 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_55_re),
    .we     (alert_en_shadowed_55_gated_we),
    .wd     (alert_en_shadowed_55_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[55].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_55_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_55_update_err),
    .err_storage (alert_en_shadowed_55_storage_err)
  );


  // Subregister 56 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_56]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_56_gated_we;
  assign alert_en_shadowed_56_gated_we = alert_en_shadowed_56_we & alert_regwen_56_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_56 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_56_re),
    .we     (alert_en_shadowed_56_gated_we),
    .wd     (alert_en_shadowed_56_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[56].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_56_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_56_update_err),
    .err_storage (alert_en_shadowed_56_storage_err)
  );


  // Subregister 57 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_57]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_57_gated_we;
  assign alert_en_shadowed_57_gated_we = alert_en_shadowed_57_we & alert_regwen_57_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_57 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_57_re),
    .we     (alert_en_shadowed_57_gated_we),
    .wd     (alert_en_shadowed_57_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[57].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_57_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_57_update_err),
    .err_storage (alert_en_shadowed_57_storage_err)
  );


  // Subregister 58 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_58]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_58_gated_we;
  assign alert_en_shadowed_58_gated_we = alert_en_shadowed_58_we & alert_regwen_58_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_58 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_58_re),
    .we     (alert_en_shadowed_58_gated_we),
    .wd     (alert_en_shadowed_58_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[58].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_58_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_58_update_err),
    .err_storage (alert_en_shadowed_58_storage_err)
  );


  // Subregister 59 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_59]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_59_gated_we;
  assign alert_en_shadowed_59_gated_we = alert_en_shadowed_59_we & alert_regwen_59_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_59 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_59_re),
    .we     (alert_en_shadowed_59_gated_we),
    .wd     (alert_en_shadowed_59_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[59].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_59_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_59_update_err),
    .err_storage (alert_en_shadowed_59_storage_err)
  );


  // Subregister 60 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_60]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_60_gated_we;
  assign alert_en_shadowed_60_gated_we = alert_en_shadowed_60_we & alert_regwen_60_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_60 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_60_re),
    .we     (alert_en_shadowed_60_gated_we),
    .wd     (alert_en_shadowed_60_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[60].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_60_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_60_update_err),
    .err_storage (alert_en_shadowed_60_storage_err)
  );


  // Subregister 61 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_61]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_61_gated_we;
  assign alert_en_shadowed_61_gated_we = alert_en_shadowed_61_we & alert_regwen_61_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_61 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_61_re),
    .we     (alert_en_shadowed_61_gated_we),
    .wd     (alert_en_shadowed_61_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[61].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_61_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_61_update_err),
    .err_storage (alert_en_shadowed_61_storage_err)
  );


  // Subregister 62 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_62]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_62_gated_we;
  assign alert_en_shadowed_62_gated_we = alert_en_shadowed_62_we & alert_regwen_62_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_62 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_62_re),
    .we     (alert_en_shadowed_62_gated_we),
    .wd     (alert_en_shadowed_62_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[62].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_62_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_62_update_err),
    .err_storage (alert_en_shadowed_62_storage_err)
  );


  // Subregister 63 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_63]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_63_gated_we;
  assign alert_en_shadowed_63_gated_we = alert_en_shadowed_63_we & alert_regwen_63_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_63 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_63_re),
    .we     (alert_en_shadowed_63_gated_we),
    .wd     (alert_en_shadowed_63_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[63].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_63_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_63_update_err),
    .err_storage (alert_en_shadowed_63_storage_err)
  );


  // Subregister 64 of Multireg alert_en_shadowed
  // R[alert_en_shadowed_64]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_en_shadowed_64_gated_we;
  assign alert_en_shadowed_64_gated_we = alert_en_shadowed_64_we & alert_regwen_64_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_alert_en_shadowed_64 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_en_shadowed_64_re),
    .we     (alert_en_shadowed_64_gated_we),
    .wd     (alert_en_shadowed_64_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_en_shadowed[64].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_en_shadowed_64_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_en_shadowed_64_update_err),
    .err_storage (alert_en_shadowed_64_storage_err)
  );


  // Subregister 0 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_0]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_0_gated_we;
  assign alert_class_shadowed_0_gated_we = alert_class_shadowed_0_we & alert_regwen_0_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_0_re),
    .we     (alert_class_shadowed_0_gated_we),
    .wd     (alert_class_shadowed_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_0_update_err),
    .err_storage (alert_class_shadowed_0_storage_err)
  );


  // Subregister 1 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_1]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_1_gated_we;
  assign alert_class_shadowed_1_gated_we = alert_class_shadowed_1_we & alert_regwen_1_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_1_re),
    .we     (alert_class_shadowed_1_gated_we),
    .wd     (alert_class_shadowed_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_1_update_err),
    .err_storage (alert_class_shadowed_1_storage_err)
  );


  // Subregister 2 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_2]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_2_gated_we;
  assign alert_class_shadowed_2_gated_we = alert_class_shadowed_2_we & alert_regwen_2_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_2_re),
    .we     (alert_class_shadowed_2_gated_we),
    .wd     (alert_class_shadowed_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_2_update_err),
    .err_storage (alert_class_shadowed_2_storage_err)
  );


  // Subregister 3 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_3]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_3_gated_we;
  assign alert_class_shadowed_3_gated_we = alert_class_shadowed_3_we & alert_regwen_3_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_3_re),
    .we     (alert_class_shadowed_3_gated_we),
    .wd     (alert_class_shadowed_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_3_update_err),
    .err_storage (alert_class_shadowed_3_storage_err)
  );


  // Subregister 4 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_4]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_4_gated_we;
  assign alert_class_shadowed_4_gated_we = alert_class_shadowed_4_we & alert_regwen_4_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_4_re),
    .we     (alert_class_shadowed_4_gated_we),
    .wd     (alert_class_shadowed_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_4_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_4_update_err),
    .err_storage (alert_class_shadowed_4_storage_err)
  );


  // Subregister 5 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_5]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_5_gated_we;
  assign alert_class_shadowed_5_gated_we = alert_class_shadowed_5_we & alert_regwen_5_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_5_re),
    .we     (alert_class_shadowed_5_gated_we),
    .wd     (alert_class_shadowed_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_5_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_5_update_err),
    .err_storage (alert_class_shadowed_5_storage_err)
  );


  // Subregister 6 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_6]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_6_gated_we;
  assign alert_class_shadowed_6_gated_we = alert_class_shadowed_6_we & alert_regwen_6_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_6_re),
    .we     (alert_class_shadowed_6_gated_we),
    .wd     (alert_class_shadowed_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_6_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_6_update_err),
    .err_storage (alert_class_shadowed_6_storage_err)
  );


  // Subregister 7 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_7]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_7_gated_we;
  assign alert_class_shadowed_7_gated_we = alert_class_shadowed_7_we & alert_regwen_7_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_7_re),
    .we     (alert_class_shadowed_7_gated_we),
    .wd     (alert_class_shadowed_7_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[7].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_7_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_7_update_err),
    .err_storage (alert_class_shadowed_7_storage_err)
  );


  // Subregister 8 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_8]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_8_gated_we;
  assign alert_class_shadowed_8_gated_we = alert_class_shadowed_8_we & alert_regwen_8_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_8_re),
    .we     (alert_class_shadowed_8_gated_we),
    .wd     (alert_class_shadowed_8_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[8].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_8_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_8_update_err),
    .err_storage (alert_class_shadowed_8_storage_err)
  );


  // Subregister 9 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_9]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_9_gated_we;
  assign alert_class_shadowed_9_gated_we = alert_class_shadowed_9_we & alert_regwen_9_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_9_re),
    .we     (alert_class_shadowed_9_gated_we),
    .wd     (alert_class_shadowed_9_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[9].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_9_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_9_update_err),
    .err_storage (alert_class_shadowed_9_storage_err)
  );


  // Subregister 10 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_10]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_10_gated_we;
  assign alert_class_shadowed_10_gated_we = alert_class_shadowed_10_we & alert_regwen_10_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_10 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_10_re),
    .we     (alert_class_shadowed_10_gated_we),
    .wd     (alert_class_shadowed_10_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[10].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_10_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_10_update_err),
    .err_storage (alert_class_shadowed_10_storage_err)
  );


  // Subregister 11 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_11]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_11_gated_we;
  assign alert_class_shadowed_11_gated_we = alert_class_shadowed_11_we & alert_regwen_11_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_11 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_11_re),
    .we     (alert_class_shadowed_11_gated_we),
    .wd     (alert_class_shadowed_11_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[11].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_11_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_11_update_err),
    .err_storage (alert_class_shadowed_11_storage_err)
  );


  // Subregister 12 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_12]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_12_gated_we;
  assign alert_class_shadowed_12_gated_we = alert_class_shadowed_12_we & alert_regwen_12_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_12 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_12_re),
    .we     (alert_class_shadowed_12_gated_we),
    .wd     (alert_class_shadowed_12_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[12].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_12_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_12_update_err),
    .err_storage (alert_class_shadowed_12_storage_err)
  );


  // Subregister 13 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_13]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_13_gated_we;
  assign alert_class_shadowed_13_gated_we = alert_class_shadowed_13_we & alert_regwen_13_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_13 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_13_re),
    .we     (alert_class_shadowed_13_gated_we),
    .wd     (alert_class_shadowed_13_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[13].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_13_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_13_update_err),
    .err_storage (alert_class_shadowed_13_storage_err)
  );


  // Subregister 14 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_14]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_14_gated_we;
  assign alert_class_shadowed_14_gated_we = alert_class_shadowed_14_we & alert_regwen_14_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_14 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_14_re),
    .we     (alert_class_shadowed_14_gated_we),
    .wd     (alert_class_shadowed_14_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[14].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_14_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_14_update_err),
    .err_storage (alert_class_shadowed_14_storage_err)
  );


  // Subregister 15 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_15]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_15_gated_we;
  assign alert_class_shadowed_15_gated_we = alert_class_shadowed_15_we & alert_regwen_15_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_15 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_15_re),
    .we     (alert_class_shadowed_15_gated_we),
    .wd     (alert_class_shadowed_15_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[15].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_15_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_15_update_err),
    .err_storage (alert_class_shadowed_15_storage_err)
  );


  // Subregister 16 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_16]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_16_gated_we;
  assign alert_class_shadowed_16_gated_we = alert_class_shadowed_16_we & alert_regwen_16_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_16 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_16_re),
    .we     (alert_class_shadowed_16_gated_we),
    .wd     (alert_class_shadowed_16_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[16].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_16_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_16_update_err),
    .err_storage (alert_class_shadowed_16_storage_err)
  );


  // Subregister 17 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_17]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_17_gated_we;
  assign alert_class_shadowed_17_gated_we = alert_class_shadowed_17_we & alert_regwen_17_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_17 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_17_re),
    .we     (alert_class_shadowed_17_gated_we),
    .wd     (alert_class_shadowed_17_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[17].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_17_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_17_update_err),
    .err_storage (alert_class_shadowed_17_storage_err)
  );


  // Subregister 18 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_18]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_18_gated_we;
  assign alert_class_shadowed_18_gated_we = alert_class_shadowed_18_we & alert_regwen_18_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_18 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_18_re),
    .we     (alert_class_shadowed_18_gated_we),
    .wd     (alert_class_shadowed_18_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[18].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_18_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_18_update_err),
    .err_storage (alert_class_shadowed_18_storage_err)
  );


  // Subregister 19 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_19]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_19_gated_we;
  assign alert_class_shadowed_19_gated_we = alert_class_shadowed_19_we & alert_regwen_19_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_19 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_19_re),
    .we     (alert_class_shadowed_19_gated_we),
    .wd     (alert_class_shadowed_19_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[19].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_19_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_19_update_err),
    .err_storage (alert_class_shadowed_19_storage_err)
  );


  // Subregister 20 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_20]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_20_gated_we;
  assign alert_class_shadowed_20_gated_we = alert_class_shadowed_20_we & alert_regwen_20_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_20 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_20_re),
    .we     (alert_class_shadowed_20_gated_we),
    .wd     (alert_class_shadowed_20_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[20].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_20_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_20_update_err),
    .err_storage (alert_class_shadowed_20_storage_err)
  );


  // Subregister 21 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_21]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_21_gated_we;
  assign alert_class_shadowed_21_gated_we = alert_class_shadowed_21_we & alert_regwen_21_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_21 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_21_re),
    .we     (alert_class_shadowed_21_gated_we),
    .wd     (alert_class_shadowed_21_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[21].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_21_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_21_update_err),
    .err_storage (alert_class_shadowed_21_storage_err)
  );


  // Subregister 22 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_22]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_22_gated_we;
  assign alert_class_shadowed_22_gated_we = alert_class_shadowed_22_we & alert_regwen_22_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_22 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_22_re),
    .we     (alert_class_shadowed_22_gated_we),
    .wd     (alert_class_shadowed_22_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[22].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_22_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_22_update_err),
    .err_storage (alert_class_shadowed_22_storage_err)
  );


  // Subregister 23 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_23]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_23_gated_we;
  assign alert_class_shadowed_23_gated_we = alert_class_shadowed_23_we & alert_regwen_23_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_23 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_23_re),
    .we     (alert_class_shadowed_23_gated_we),
    .wd     (alert_class_shadowed_23_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[23].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_23_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_23_update_err),
    .err_storage (alert_class_shadowed_23_storage_err)
  );


  // Subregister 24 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_24]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_24_gated_we;
  assign alert_class_shadowed_24_gated_we = alert_class_shadowed_24_we & alert_regwen_24_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_24 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_24_re),
    .we     (alert_class_shadowed_24_gated_we),
    .wd     (alert_class_shadowed_24_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[24].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_24_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_24_update_err),
    .err_storage (alert_class_shadowed_24_storage_err)
  );


  // Subregister 25 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_25]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_25_gated_we;
  assign alert_class_shadowed_25_gated_we = alert_class_shadowed_25_we & alert_regwen_25_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_25 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_25_re),
    .we     (alert_class_shadowed_25_gated_we),
    .wd     (alert_class_shadowed_25_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[25].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_25_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_25_update_err),
    .err_storage (alert_class_shadowed_25_storage_err)
  );


  // Subregister 26 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_26]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_26_gated_we;
  assign alert_class_shadowed_26_gated_we = alert_class_shadowed_26_we & alert_regwen_26_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_26 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_26_re),
    .we     (alert_class_shadowed_26_gated_we),
    .wd     (alert_class_shadowed_26_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[26].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_26_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_26_update_err),
    .err_storage (alert_class_shadowed_26_storage_err)
  );


  // Subregister 27 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_27]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_27_gated_we;
  assign alert_class_shadowed_27_gated_we = alert_class_shadowed_27_we & alert_regwen_27_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_27 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_27_re),
    .we     (alert_class_shadowed_27_gated_we),
    .wd     (alert_class_shadowed_27_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[27].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_27_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_27_update_err),
    .err_storage (alert_class_shadowed_27_storage_err)
  );


  // Subregister 28 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_28]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_28_gated_we;
  assign alert_class_shadowed_28_gated_we = alert_class_shadowed_28_we & alert_regwen_28_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_28 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_28_re),
    .we     (alert_class_shadowed_28_gated_we),
    .wd     (alert_class_shadowed_28_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[28].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_28_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_28_update_err),
    .err_storage (alert_class_shadowed_28_storage_err)
  );


  // Subregister 29 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_29]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_29_gated_we;
  assign alert_class_shadowed_29_gated_we = alert_class_shadowed_29_we & alert_regwen_29_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_29 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_29_re),
    .we     (alert_class_shadowed_29_gated_we),
    .wd     (alert_class_shadowed_29_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[29].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_29_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_29_update_err),
    .err_storage (alert_class_shadowed_29_storage_err)
  );


  // Subregister 30 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_30]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_30_gated_we;
  assign alert_class_shadowed_30_gated_we = alert_class_shadowed_30_we & alert_regwen_30_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_30 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_30_re),
    .we     (alert_class_shadowed_30_gated_we),
    .wd     (alert_class_shadowed_30_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[30].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_30_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_30_update_err),
    .err_storage (alert_class_shadowed_30_storage_err)
  );


  // Subregister 31 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_31]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_31_gated_we;
  assign alert_class_shadowed_31_gated_we = alert_class_shadowed_31_we & alert_regwen_31_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_31 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_31_re),
    .we     (alert_class_shadowed_31_gated_we),
    .wd     (alert_class_shadowed_31_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[31].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_31_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_31_update_err),
    .err_storage (alert_class_shadowed_31_storage_err)
  );


  // Subregister 32 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_32]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_32_gated_we;
  assign alert_class_shadowed_32_gated_we = alert_class_shadowed_32_we & alert_regwen_32_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_32 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_32_re),
    .we     (alert_class_shadowed_32_gated_we),
    .wd     (alert_class_shadowed_32_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[32].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_32_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_32_update_err),
    .err_storage (alert_class_shadowed_32_storage_err)
  );


  // Subregister 33 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_33]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_33_gated_we;
  assign alert_class_shadowed_33_gated_we = alert_class_shadowed_33_we & alert_regwen_33_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_33 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_33_re),
    .we     (alert_class_shadowed_33_gated_we),
    .wd     (alert_class_shadowed_33_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[33].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_33_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_33_update_err),
    .err_storage (alert_class_shadowed_33_storage_err)
  );


  // Subregister 34 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_34]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_34_gated_we;
  assign alert_class_shadowed_34_gated_we = alert_class_shadowed_34_we & alert_regwen_34_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_34 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_34_re),
    .we     (alert_class_shadowed_34_gated_we),
    .wd     (alert_class_shadowed_34_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[34].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_34_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_34_update_err),
    .err_storage (alert_class_shadowed_34_storage_err)
  );


  // Subregister 35 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_35]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_35_gated_we;
  assign alert_class_shadowed_35_gated_we = alert_class_shadowed_35_we & alert_regwen_35_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_35 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_35_re),
    .we     (alert_class_shadowed_35_gated_we),
    .wd     (alert_class_shadowed_35_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[35].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_35_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_35_update_err),
    .err_storage (alert_class_shadowed_35_storage_err)
  );


  // Subregister 36 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_36]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_36_gated_we;
  assign alert_class_shadowed_36_gated_we = alert_class_shadowed_36_we & alert_regwen_36_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_36 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_36_re),
    .we     (alert_class_shadowed_36_gated_we),
    .wd     (alert_class_shadowed_36_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[36].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_36_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_36_update_err),
    .err_storage (alert_class_shadowed_36_storage_err)
  );


  // Subregister 37 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_37]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_37_gated_we;
  assign alert_class_shadowed_37_gated_we = alert_class_shadowed_37_we & alert_regwen_37_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_37 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_37_re),
    .we     (alert_class_shadowed_37_gated_we),
    .wd     (alert_class_shadowed_37_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[37].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_37_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_37_update_err),
    .err_storage (alert_class_shadowed_37_storage_err)
  );


  // Subregister 38 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_38]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_38_gated_we;
  assign alert_class_shadowed_38_gated_we = alert_class_shadowed_38_we & alert_regwen_38_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_38 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_38_re),
    .we     (alert_class_shadowed_38_gated_we),
    .wd     (alert_class_shadowed_38_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[38].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_38_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_38_update_err),
    .err_storage (alert_class_shadowed_38_storage_err)
  );


  // Subregister 39 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_39]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_39_gated_we;
  assign alert_class_shadowed_39_gated_we = alert_class_shadowed_39_we & alert_regwen_39_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_39 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_39_re),
    .we     (alert_class_shadowed_39_gated_we),
    .wd     (alert_class_shadowed_39_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[39].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_39_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_39_update_err),
    .err_storage (alert_class_shadowed_39_storage_err)
  );


  // Subregister 40 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_40]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_40_gated_we;
  assign alert_class_shadowed_40_gated_we = alert_class_shadowed_40_we & alert_regwen_40_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_40 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_40_re),
    .we     (alert_class_shadowed_40_gated_we),
    .wd     (alert_class_shadowed_40_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[40].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_40_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_40_update_err),
    .err_storage (alert_class_shadowed_40_storage_err)
  );


  // Subregister 41 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_41]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_41_gated_we;
  assign alert_class_shadowed_41_gated_we = alert_class_shadowed_41_we & alert_regwen_41_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_41 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_41_re),
    .we     (alert_class_shadowed_41_gated_we),
    .wd     (alert_class_shadowed_41_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[41].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_41_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_41_update_err),
    .err_storage (alert_class_shadowed_41_storage_err)
  );


  // Subregister 42 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_42]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_42_gated_we;
  assign alert_class_shadowed_42_gated_we = alert_class_shadowed_42_we & alert_regwen_42_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_42 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_42_re),
    .we     (alert_class_shadowed_42_gated_we),
    .wd     (alert_class_shadowed_42_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[42].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_42_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_42_update_err),
    .err_storage (alert_class_shadowed_42_storage_err)
  );


  // Subregister 43 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_43]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_43_gated_we;
  assign alert_class_shadowed_43_gated_we = alert_class_shadowed_43_we & alert_regwen_43_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_43 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_43_re),
    .we     (alert_class_shadowed_43_gated_we),
    .wd     (alert_class_shadowed_43_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[43].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_43_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_43_update_err),
    .err_storage (alert_class_shadowed_43_storage_err)
  );


  // Subregister 44 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_44]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_44_gated_we;
  assign alert_class_shadowed_44_gated_we = alert_class_shadowed_44_we & alert_regwen_44_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_44 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_44_re),
    .we     (alert_class_shadowed_44_gated_we),
    .wd     (alert_class_shadowed_44_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[44].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_44_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_44_update_err),
    .err_storage (alert_class_shadowed_44_storage_err)
  );


  // Subregister 45 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_45]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_45_gated_we;
  assign alert_class_shadowed_45_gated_we = alert_class_shadowed_45_we & alert_regwen_45_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_45 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_45_re),
    .we     (alert_class_shadowed_45_gated_we),
    .wd     (alert_class_shadowed_45_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[45].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_45_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_45_update_err),
    .err_storage (alert_class_shadowed_45_storage_err)
  );


  // Subregister 46 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_46]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_46_gated_we;
  assign alert_class_shadowed_46_gated_we = alert_class_shadowed_46_we & alert_regwen_46_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_46 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_46_re),
    .we     (alert_class_shadowed_46_gated_we),
    .wd     (alert_class_shadowed_46_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[46].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_46_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_46_update_err),
    .err_storage (alert_class_shadowed_46_storage_err)
  );


  // Subregister 47 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_47]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_47_gated_we;
  assign alert_class_shadowed_47_gated_we = alert_class_shadowed_47_we & alert_regwen_47_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_47 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_47_re),
    .we     (alert_class_shadowed_47_gated_we),
    .wd     (alert_class_shadowed_47_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[47].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_47_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_47_update_err),
    .err_storage (alert_class_shadowed_47_storage_err)
  );


  // Subregister 48 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_48]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_48_gated_we;
  assign alert_class_shadowed_48_gated_we = alert_class_shadowed_48_we & alert_regwen_48_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_48 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_48_re),
    .we     (alert_class_shadowed_48_gated_we),
    .wd     (alert_class_shadowed_48_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[48].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_48_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_48_update_err),
    .err_storage (alert_class_shadowed_48_storage_err)
  );


  // Subregister 49 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_49]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_49_gated_we;
  assign alert_class_shadowed_49_gated_we = alert_class_shadowed_49_we & alert_regwen_49_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_49 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_49_re),
    .we     (alert_class_shadowed_49_gated_we),
    .wd     (alert_class_shadowed_49_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[49].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_49_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_49_update_err),
    .err_storage (alert_class_shadowed_49_storage_err)
  );


  // Subregister 50 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_50]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_50_gated_we;
  assign alert_class_shadowed_50_gated_we = alert_class_shadowed_50_we & alert_regwen_50_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_50 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_50_re),
    .we     (alert_class_shadowed_50_gated_we),
    .wd     (alert_class_shadowed_50_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[50].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_50_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_50_update_err),
    .err_storage (alert_class_shadowed_50_storage_err)
  );


  // Subregister 51 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_51]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_51_gated_we;
  assign alert_class_shadowed_51_gated_we = alert_class_shadowed_51_we & alert_regwen_51_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_51 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_51_re),
    .we     (alert_class_shadowed_51_gated_we),
    .wd     (alert_class_shadowed_51_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[51].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_51_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_51_update_err),
    .err_storage (alert_class_shadowed_51_storage_err)
  );


  // Subregister 52 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_52]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_52_gated_we;
  assign alert_class_shadowed_52_gated_we = alert_class_shadowed_52_we & alert_regwen_52_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_52 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_52_re),
    .we     (alert_class_shadowed_52_gated_we),
    .wd     (alert_class_shadowed_52_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[52].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_52_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_52_update_err),
    .err_storage (alert_class_shadowed_52_storage_err)
  );


  // Subregister 53 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_53]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_53_gated_we;
  assign alert_class_shadowed_53_gated_we = alert_class_shadowed_53_we & alert_regwen_53_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_53 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_53_re),
    .we     (alert_class_shadowed_53_gated_we),
    .wd     (alert_class_shadowed_53_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[53].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_53_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_53_update_err),
    .err_storage (alert_class_shadowed_53_storage_err)
  );


  // Subregister 54 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_54]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_54_gated_we;
  assign alert_class_shadowed_54_gated_we = alert_class_shadowed_54_we & alert_regwen_54_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_54 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_54_re),
    .we     (alert_class_shadowed_54_gated_we),
    .wd     (alert_class_shadowed_54_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[54].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_54_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_54_update_err),
    .err_storage (alert_class_shadowed_54_storage_err)
  );


  // Subregister 55 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_55]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_55_gated_we;
  assign alert_class_shadowed_55_gated_we = alert_class_shadowed_55_we & alert_regwen_55_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_55 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_55_re),
    .we     (alert_class_shadowed_55_gated_we),
    .wd     (alert_class_shadowed_55_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[55].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_55_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_55_update_err),
    .err_storage (alert_class_shadowed_55_storage_err)
  );


  // Subregister 56 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_56]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_56_gated_we;
  assign alert_class_shadowed_56_gated_we = alert_class_shadowed_56_we & alert_regwen_56_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_56 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_56_re),
    .we     (alert_class_shadowed_56_gated_we),
    .wd     (alert_class_shadowed_56_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[56].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_56_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_56_update_err),
    .err_storage (alert_class_shadowed_56_storage_err)
  );


  // Subregister 57 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_57]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_57_gated_we;
  assign alert_class_shadowed_57_gated_we = alert_class_shadowed_57_we & alert_regwen_57_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_57 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_57_re),
    .we     (alert_class_shadowed_57_gated_we),
    .wd     (alert_class_shadowed_57_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[57].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_57_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_57_update_err),
    .err_storage (alert_class_shadowed_57_storage_err)
  );


  // Subregister 58 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_58]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_58_gated_we;
  assign alert_class_shadowed_58_gated_we = alert_class_shadowed_58_we & alert_regwen_58_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_58 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_58_re),
    .we     (alert_class_shadowed_58_gated_we),
    .wd     (alert_class_shadowed_58_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[58].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_58_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_58_update_err),
    .err_storage (alert_class_shadowed_58_storage_err)
  );


  // Subregister 59 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_59]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_59_gated_we;
  assign alert_class_shadowed_59_gated_we = alert_class_shadowed_59_we & alert_regwen_59_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_59 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_59_re),
    .we     (alert_class_shadowed_59_gated_we),
    .wd     (alert_class_shadowed_59_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[59].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_59_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_59_update_err),
    .err_storage (alert_class_shadowed_59_storage_err)
  );


  // Subregister 60 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_60]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_60_gated_we;
  assign alert_class_shadowed_60_gated_we = alert_class_shadowed_60_we & alert_regwen_60_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_60 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_60_re),
    .we     (alert_class_shadowed_60_gated_we),
    .wd     (alert_class_shadowed_60_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[60].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_60_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_60_update_err),
    .err_storage (alert_class_shadowed_60_storage_err)
  );


  // Subregister 61 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_61]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_61_gated_we;
  assign alert_class_shadowed_61_gated_we = alert_class_shadowed_61_we & alert_regwen_61_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_61 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_61_re),
    .we     (alert_class_shadowed_61_gated_we),
    .wd     (alert_class_shadowed_61_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[61].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_61_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_61_update_err),
    .err_storage (alert_class_shadowed_61_storage_err)
  );


  // Subregister 62 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_62]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_62_gated_we;
  assign alert_class_shadowed_62_gated_we = alert_class_shadowed_62_we & alert_regwen_62_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_62 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_62_re),
    .we     (alert_class_shadowed_62_gated_we),
    .wd     (alert_class_shadowed_62_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[62].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_62_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_62_update_err),
    .err_storage (alert_class_shadowed_62_storage_err)
  );


  // Subregister 63 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_63]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_63_gated_we;
  assign alert_class_shadowed_63_gated_we = alert_class_shadowed_63_we & alert_regwen_63_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_63 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_63_re),
    .we     (alert_class_shadowed_63_gated_we),
    .wd     (alert_class_shadowed_63_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[63].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_63_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_63_update_err),
    .err_storage (alert_class_shadowed_63_storage_err)
  );


  // Subregister 64 of Multireg alert_class_shadowed
  // R[alert_class_shadowed_64]: V(False)
  // Create REGWEN-gated WE signal
  logic alert_class_shadowed_64_gated_we;
  assign alert_class_shadowed_64_gated_we = alert_class_shadowed_64_we & alert_regwen_64_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_alert_class_shadowed_64 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (alert_class_shadowed_64_re),
    .we     (alert_class_shadowed_64_gated_we),
    .wd     (alert_class_shadowed_64_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_class_shadowed[64].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_class_shadowed_64_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (alert_class_shadowed_64_update_err),
    .err_storage (alert_class_shadowed_64_storage_err)
  );


  // Subregister 0 of Multireg alert_cause
  // R[alert_cause_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_0_we),
    .wd     (alert_cause_0_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[0].de),
    .d      (hw2reg.alert_cause[0].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_0_qs)
  );


  // Subregister 1 of Multireg alert_cause
  // R[alert_cause_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_1_we),
    .wd     (alert_cause_1_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[1].de),
    .d      (hw2reg.alert_cause[1].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_1_qs)
  );


  // Subregister 2 of Multireg alert_cause
  // R[alert_cause_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_2_we),
    .wd     (alert_cause_2_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[2].de),
    .d      (hw2reg.alert_cause[2].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_2_qs)
  );


  // Subregister 3 of Multireg alert_cause
  // R[alert_cause_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_3_we),
    .wd     (alert_cause_3_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[3].de),
    .d      (hw2reg.alert_cause[3].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_3_qs)
  );


  // Subregister 4 of Multireg alert_cause
  // R[alert_cause_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_4_we),
    .wd     (alert_cause_4_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[4].de),
    .d      (hw2reg.alert_cause[4].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_4_qs)
  );


  // Subregister 5 of Multireg alert_cause
  // R[alert_cause_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_5_we),
    .wd     (alert_cause_5_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[5].de),
    .d      (hw2reg.alert_cause[5].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_5_qs)
  );


  // Subregister 6 of Multireg alert_cause
  // R[alert_cause_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_6_we),
    .wd     (alert_cause_6_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[6].de),
    .d      (hw2reg.alert_cause[6].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_6_qs)
  );


  // Subregister 7 of Multireg alert_cause
  // R[alert_cause_7]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_7 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_7_we),
    .wd     (alert_cause_7_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[7].de),
    .d      (hw2reg.alert_cause[7].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[7].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_7_qs)
  );


  // Subregister 8 of Multireg alert_cause
  // R[alert_cause_8]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_8 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_8_we),
    .wd     (alert_cause_8_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[8].de),
    .d      (hw2reg.alert_cause[8].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[8].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_8_qs)
  );


  // Subregister 9 of Multireg alert_cause
  // R[alert_cause_9]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_9 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_9_we),
    .wd     (alert_cause_9_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[9].de),
    .d      (hw2reg.alert_cause[9].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[9].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_9_qs)
  );


  // Subregister 10 of Multireg alert_cause
  // R[alert_cause_10]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_10 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_10_we),
    .wd     (alert_cause_10_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[10].de),
    .d      (hw2reg.alert_cause[10].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[10].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_10_qs)
  );


  // Subregister 11 of Multireg alert_cause
  // R[alert_cause_11]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_11 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_11_we),
    .wd     (alert_cause_11_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[11].de),
    .d      (hw2reg.alert_cause[11].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[11].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_11_qs)
  );


  // Subregister 12 of Multireg alert_cause
  // R[alert_cause_12]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_12 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_12_we),
    .wd     (alert_cause_12_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[12].de),
    .d      (hw2reg.alert_cause[12].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[12].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_12_qs)
  );


  // Subregister 13 of Multireg alert_cause
  // R[alert_cause_13]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_13 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_13_we),
    .wd     (alert_cause_13_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[13].de),
    .d      (hw2reg.alert_cause[13].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[13].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_13_qs)
  );


  // Subregister 14 of Multireg alert_cause
  // R[alert_cause_14]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_14 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_14_we),
    .wd     (alert_cause_14_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[14].de),
    .d      (hw2reg.alert_cause[14].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[14].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_14_qs)
  );


  // Subregister 15 of Multireg alert_cause
  // R[alert_cause_15]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_15 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_15_we),
    .wd     (alert_cause_15_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[15].de),
    .d      (hw2reg.alert_cause[15].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[15].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_15_qs)
  );


  // Subregister 16 of Multireg alert_cause
  // R[alert_cause_16]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_16 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_16_we),
    .wd     (alert_cause_16_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[16].de),
    .d      (hw2reg.alert_cause[16].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[16].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_16_qs)
  );


  // Subregister 17 of Multireg alert_cause
  // R[alert_cause_17]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_17 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_17_we),
    .wd     (alert_cause_17_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[17].de),
    .d      (hw2reg.alert_cause[17].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[17].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_17_qs)
  );


  // Subregister 18 of Multireg alert_cause
  // R[alert_cause_18]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_18 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_18_we),
    .wd     (alert_cause_18_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[18].de),
    .d      (hw2reg.alert_cause[18].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[18].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_18_qs)
  );


  // Subregister 19 of Multireg alert_cause
  // R[alert_cause_19]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_19 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_19_we),
    .wd     (alert_cause_19_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[19].de),
    .d      (hw2reg.alert_cause[19].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[19].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_19_qs)
  );


  // Subregister 20 of Multireg alert_cause
  // R[alert_cause_20]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_20 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_20_we),
    .wd     (alert_cause_20_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[20].de),
    .d      (hw2reg.alert_cause[20].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[20].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_20_qs)
  );


  // Subregister 21 of Multireg alert_cause
  // R[alert_cause_21]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_21 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_21_we),
    .wd     (alert_cause_21_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[21].de),
    .d      (hw2reg.alert_cause[21].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[21].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_21_qs)
  );


  // Subregister 22 of Multireg alert_cause
  // R[alert_cause_22]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_22 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_22_we),
    .wd     (alert_cause_22_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[22].de),
    .d      (hw2reg.alert_cause[22].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[22].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_22_qs)
  );


  // Subregister 23 of Multireg alert_cause
  // R[alert_cause_23]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_23 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_23_we),
    .wd     (alert_cause_23_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[23].de),
    .d      (hw2reg.alert_cause[23].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[23].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_23_qs)
  );


  // Subregister 24 of Multireg alert_cause
  // R[alert_cause_24]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_24 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_24_we),
    .wd     (alert_cause_24_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[24].de),
    .d      (hw2reg.alert_cause[24].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[24].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_24_qs)
  );


  // Subregister 25 of Multireg alert_cause
  // R[alert_cause_25]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_25 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_25_we),
    .wd     (alert_cause_25_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[25].de),
    .d      (hw2reg.alert_cause[25].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[25].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_25_qs)
  );


  // Subregister 26 of Multireg alert_cause
  // R[alert_cause_26]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_26 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_26_we),
    .wd     (alert_cause_26_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[26].de),
    .d      (hw2reg.alert_cause[26].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[26].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_26_qs)
  );


  // Subregister 27 of Multireg alert_cause
  // R[alert_cause_27]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_27 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_27_we),
    .wd     (alert_cause_27_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[27].de),
    .d      (hw2reg.alert_cause[27].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[27].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_27_qs)
  );


  // Subregister 28 of Multireg alert_cause
  // R[alert_cause_28]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_28 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_28_we),
    .wd     (alert_cause_28_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[28].de),
    .d      (hw2reg.alert_cause[28].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[28].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_28_qs)
  );


  // Subregister 29 of Multireg alert_cause
  // R[alert_cause_29]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_29 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_29_we),
    .wd     (alert_cause_29_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[29].de),
    .d      (hw2reg.alert_cause[29].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[29].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_29_qs)
  );


  // Subregister 30 of Multireg alert_cause
  // R[alert_cause_30]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_30 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_30_we),
    .wd     (alert_cause_30_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[30].de),
    .d      (hw2reg.alert_cause[30].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[30].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_30_qs)
  );


  // Subregister 31 of Multireg alert_cause
  // R[alert_cause_31]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_31 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_31_we),
    .wd     (alert_cause_31_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[31].de),
    .d      (hw2reg.alert_cause[31].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[31].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_31_qs)
  );


  // Subregister 32 of Multireg alert_cause
  // R[alert_cause_32]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_32 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_32_we),
    .wd     (alert_cause_32_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[32].de),
    .d      (hw2reg.alert_cause[32].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[32].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_32_qs)
  );


  // Subregister 33 of Multireg alert_cause
  // R[alert_cause_33]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_33 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_33_we),
    .wd     (alert_cause_33_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[33].de),
    .d      (hw2reg.alert_cause[33].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[33].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_33_qs)
  );


  // Subregister 34 of Multireg alert_cause
  // R[alert_cause_34]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_34 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_34_we),
    .wd     (alert_cause_34_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[34].de),
    .d      (hw2reg.alert_cause[34].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[34].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_34_qs)
  );


  // Subregister 35 of Multireg alert_cause
  // R[alert_cause_35]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_35 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_35_we),
    .wd     (alert_cause_35_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[35].de),
    .d      (hw2reg.alert_cause[35].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[35].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_35_qs)
  );


  // Subregister 36 of Multireg alert_cause
  // R[alert_cause_36]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_36 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_36_we),
    .wd     (alert_cause_36_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[36].de),
    .d      (hw2reg.alert_cause[36].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[36].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_36_qs)
  );


  // Subregister 37 of Multireg alert_cause
  // R[alert_cause_37]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_37 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_37_we),
    .wd     (alert_cause_37_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[37].de),
    .d      (hw2reg.alert_cause[37].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[37].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_37_qs)
  );


  // Subregister 38 of Multireg alert_cause
  // R[alert_cause_38]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_38 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_38_we),
    .wd     (alert_cause_38_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[38].de),
    .d      (hw2reg.alert_cause[38].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[38].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_38_qs)
  );


  // Subregister 39 of Multireg alert_cause
  // R[alert_cause_39]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_39 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_39_we),
    .wd     (alert_cause_39_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[39].de),
    .d      (hw2reg.alert_cause[39].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[39].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_39_qs)
  );


  // Subregister 40 of Multireg alert_cause
  // R[alert_cause_40]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_40 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_40_we),
    .wd     (alert_cause_40_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[40].de),
    .d      (hw2reg.alert_cause[40].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[40].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_40_qs)
  );


  // Subregister 41 of Multireg alert_cause
  // R[alert_cause_41]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_41 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_41_we),
    .wd     (alert_cause_41_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[41].de),
    .d      (hw2reg.alert_cause[41].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[41].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_41_qs)
  );


  // Subregister 42 of Multireg alert_cause
  // R[alert_cause_42]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_42 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_42_we),
    .wd     (alert_cause_42_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[42].de),
    .d      (hw2reg.alert_cause[42].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[42].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_42_qs)
  );


  // Subregister 43 of Multireg alert_cause
  // R[alert_cause_43]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_43 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_43_we),
    .wd     (alert_cause_43_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[43].de),
    .d      (hw2reg.alert_cause[43].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[43].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_43_qs)
  );


  // Subregister 44 of Multireg alert_cause
  // R[alert_cause_44]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_44 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_44_we),
    .wd     (alert_cause_44_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[44].de),
    .d      (hw2reg.alert_cause[44].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[44].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_44_qs)
  );


  // Subregister 45 of Multireg alert_cause
  // R[alert_cause_45]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_45 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_45_we),
    .wd     (alert_cause_45_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[45].de),
    .d      (hw2reg.alert_cause[45].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[45].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_45_qs)
  );


  // Subregister 46 of Multireg alert_cause
  // R[alert_cause_46]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_46 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_46_we),
    .wd     (alert_cause_46_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[46].de),
    .d      (hw2reg.alert_cause[46].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[46].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_46_qs)
  );


  // Subregister 47 of Multireg alert_cause
  // R[alert_cause_47]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_47 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_47_we),
    .wd     (alert_cause_47_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[47].de),
    .d      (hw2reg.alert_cause[47].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[47].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_47_qs)
  );


  // Subregister 48 of Multireg alert_cause
  // R[alert_cause_48]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_48 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_48_we),
    .wd     (alert_cause_48_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[48].de),
    .d      (hw2reg.alert_cause[48].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[48].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_48_qs)
  );


  // Subregister 49 of Multireg alert_cause
  // R[alert_cause_49]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_49 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_49_we),
    .wd     (alert_cause_49_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[49].de),
    .d      (hw2reg.alert_cause[49].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[49].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_49_qs)
  );


  // Subregister 50 of Multireg alert_cause
  // R[alert_cause_50]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_50 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_50_we),
    .wd     (alert_cause_50_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[50].de),
    .d      (hw2reg.alert_cause[50].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[50].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_50_qs)
  );


  // Subregister 51 of Multireg alert_cause
  // R[alert_cause_51]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_51 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_51_we),
    .wd     (alert_cause_51_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[51].de),
    .d      (hw2reg.alert_cause[51].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[51].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_51_qs)
  );


  // Subregister 52 of Multireg alert_cause
  // R[alert_cause_52]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_52 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_52_we),
    .wd     (alert_cause_52_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[52].de),
    .d      (hw2reg.alert_cause[52].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[52].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_52_qs)
  );


  // Subregister 53 of Multireg alert_cause
  // R[alert_cause_53]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_53 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_53_we),
    .wd     (alert_cause_53_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[53].de),
    .d      (hw2reg.alert_cause[53].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[53].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_53_qs)
  );


  // Subregister 54 of Multireg alert_cause
  // R[alert_cause_54]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_54 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_54_we),
    .wd     (alert_cause_54_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[54].de),
    .d      (hw2reg.alert_cause[54].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[54].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_54_qs)
  );


  // Subregister 55 of Multireg alert_cause
  // R[alert_cause_55]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_55 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_55_we),
    .wd     (alert_cause_55_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[55].de),
    .d      (hw2reg.alert_cause[55].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[55].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_55_qs)
  );


  // Subregister 56 of Multireg alert_cause
  // R[alert_cause_56]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_56 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_56_we),
    .wd     (alert_cause_56_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[56].de),
    .d      (hw2reg.alert_cause[56].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[56].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_56_qs)
  );


  // Subregister 57 of Multireg alert_cause
  // R[alert_cause_57]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_57 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_57_we),
    .wd     (alert_cause_57_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[57].de),
    .d      (hw2reg.alert_cause[57].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[57].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_57_qs)
  );


  // Subregister 58 of Multireg alert_cause
  // R[alert_cause_58]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_58 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_58_we),
    .wd     (alert_cause_58_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[58].de),
    .d      (hw2reg.alert_cause[58].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[58].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_58_qs)
  );


  // Subregister 59 of Multireg alert_cause
  // R[alert_cause_59]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_59 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_59_we),
    .wd     (alert_cause_59_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[59].de),
    .d      (hw2reg.alert_cause[59].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[59].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_59_qs)
  );


  // Subregister 60 of Multireg alert_cause
  // R[alert_cause_60]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_60 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_60_we),
    .wd     (alert_cause_60_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[60].de),
    .d      (hw2reg.alert_cause[60].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[60].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_60_qs)
  );


  // Subregister 61 of Multireg alert_cause
  // R[alert_cause_61]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_61 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_61_we),
    .wd     (alert_cause_61_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[61].de),
    .d      (hw2reg.alert_cause[61].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[61].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_61_qs)
  );


  // Subregister 62 of Multireg alert_cause
  // R[alert_cause_62]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_62 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_62_we),
    .wd     (alert_cause_62_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[62].de),
    .d      (hw2reg.alert_cause[62].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[62].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_62_qs)
  );


  // Subregister 63 of Multireg alert_cause
  // R[alert_cause_63]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_63 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_63_we),
    .wd     (alert_cause_63_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[63].de),
    .d      (hw2reg.alert_cause[63].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[63].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_63_qs)
  );


  // Subregister 64 of Multireg alert_cause
  // R[alert_cause_64]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_alert_cause_64 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (alert_cause_64_we),
    .wd     (alert_cause_64_wd),

    // from internal hardware
    .de     (hw2reg.alert_cause[64].de),
    .d      (hw2reg.alert_cause[64].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.alert_cause[64].q),
    .ds     (),

    // to register interface (read)
    .qs     (alert_cause_64_qs)
  );


  // Subregister 0 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_0_we),
    .wd     (loc_alert_regwen_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_0_qs)
  );


  // Subregister 1 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_1_we),
    .wd     (loc_alert_regwen_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_1_qs)
  );


  // Subregister 2 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_2_we),
    .wd     (loc_alert_regwen_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_2_qs)
  );


  // Subregister 3 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_3_we),
    .wd     (loc_alert_regwen_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_3_qs)
  );


  // Subregister 4 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_4_we),
    .wd     (loc_alert_regwen_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_4_qs)
  );


  // Subregister 5 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_5_we),
    .wd     (loc_alert_regwen_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_5_qs)
  );


  // Subregister 6 of Multireg loc_alert_regwen
  // R[loc_alert_regwen_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_loc_alert_regwen_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_regwen_6_we),
    .wd     (loc_alert_regwen_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_regwen_6_qs)
  );


  // Subregister 0 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_0]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_0_gated_we;
  assign loc_alert_en_shadowed_0_gated_we = loc_alert_en_shadowed_0_we & loc_alert_regwen_0_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_0_re),
    .we     (loc_alert_en_shadowed_0_gated_we),
    .wd     (loc_alert_en_shadowed_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_0_update_err),
    .err_storage (loc_alert_en_shadowed_0_storage_err)
  );


  // Subregister 1 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_1]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_1_gated_we;
  assign loc_alert_en_shadowed_1_gated_we = loc_alert_en_shadowed_1_we & loc_alert_regwen_1_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_1_re),
    .we     (loc_alert_en_shadowed_1_gated_we),
    .wd     (loc_alert_en_shadowed_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_1_update_err),
    .err_storage (loc_alert_en_shadowed_1_storage_err)
  );


  // Subregister 2 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_2]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_2_gated_we;
  assign loc_alert_en_shadowed_2_gated_we = loc_alert_en_shadowed_2_we & loc_alert_regwen_2_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_2_re),
    .we     (loc_alert_en_shadowed_2_gated_we),
    .wd     (loc_alert_en_shadowed_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_2_update_err),
    .err_storage (loc_alert_en_shadowed_2_storage_err)
  );


  // Subregister 3 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_3]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_3_gated_we;
  assign loc_alert_en_shadowed_3_gated_we = loc_alert_en_shadowed_3_we & loc_alert_regwen_3_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_3_re),
    .we     (loc_alert_en_shadowed_3_gated_we),
    .wd     (loc_alert_en_shadowed_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_3_update_err),
    .err_storage (loc_alert_en_shadowed_3_storage_err)
  );


  // Subregister 4 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_4]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_4_gated_we;
  assign loc_alert_en_shadowed_4_gated_we = loc_alert_en_shadowed_4_we & loc_alert_regwen_4_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_4_re),
    .we     (loc_alert_en_shadowed_4_gated_we),
    .wd     (loc_alert_en_shadowed_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_4_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_4_update_err),
    .err_storage (loc_alert_en_shadowed_4_storage_err)
  );


  // Subregister 5 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_5]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_5_gated_we;
  assign loc_alert_en_shadowed_5_gated_we = loc_alert_en_shadowed_5_we & loc_alert_regwen_5_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_5_re),
    .we     (loc_alert_en_shadowed_5_gated_we),
    .wd     (loc_alert_en_shadowed_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_5_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_5_update_err),
    .err_storage (loc_alert_en_shadowed_5_storage_err)
  );


  // Subregister 6 of Multireg loc_alert_en_shadowed
  // R[loc_alert_en_shadowed_6]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_en_shadowed_6_gated_we;
  assign loc_alert_en_shadowed_6_gated_we = loc_alert_en_shadowed_6_we & loc_alert_regwen_6_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_loc_alert_en_shadowed_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_en_shadowed_6_re),
    .we     (loc_alert_en_shadowed_6_gated_we),
    .wd     (loc_alert_en_shadowed_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_en_shadowed[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_en_shadowed_6_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_en_shadowed_6_update_err),
    .err_storage (loc_alert_en_shadowed_6_storage_err)
  );


  // Subregister 0 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_0]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_0_gated_we;
  assign loc_alert_class_shadowed_0_gated_we =
    loc_alert_class_shadowed_0_we & loc_alert_regwen_0_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_0_re),
    .we     (loc_alert_class_shadowed_0_gated_we),
    .wd     (loc_alert_class_shadowed_0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_0_update_err),
    .err_storage (loc_alert_class_shadowed_0_storage_err)
  );


  // Subregister 1 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_1]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_1_gated_we;
  assign loc_alert_class_shadowed_1_gated_we =
    loc_alert_class_shadowed_1_we & loc_alert_regwen_1_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_1_re),
    .we     (loc_alert_class_shadowed_1_gated_we),
    .wd     (loc_alert_class_shadowed_1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_1_update_err),
    .err_storage (loc_alert_class_shadowed_1_storage_err)
  );


  // Subregister 2 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_2]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_2_gated_we;
  assign loc_alert_class_shadowed_2_gated_we =
    loc_alert_class_shadowed_2_we & loc_alert_regwen_2_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_2_re),
    .we     (loc_alert_class_shadowed_2_gated_we),
    .wd     (loc_alert_class_shadowed_2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_2_update_err),
    .err_storage (loc_alert_class_shadowed_2_storage_err)
  );


  // Subregister 3 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_3]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_3_gated_we;
  assign loc_alert_class_shadowed_3_gated_we =
    loc_alert_class_shadowed_3_we & loc_alert_regwen_3_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_3_re),
    .we     (loc_alert_class_shadowed_3_gated_we),
    .wd     (loc_alert_class_shadowed_3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_3_update_err),
    .err_storage (loc_alert_class_shadowed_3_storage_err)
  );


  // Subregister 4 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_4]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_4_gated_we;
  assign loc_alert_class_shadowed_4_gated_we =
    loc_alert_class_shadowed_4_we & loc_alert_regwen_4_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_4_re),
    .we     (loc_alert_class_shadowed_4_gated_we),
    .wd     (loc_alert_class_shadowed_4_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_4_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_4_update_err),
    .err_storage (loc_alert_class_shadowed_4_storage_err)
  );


  // Subregister 5 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_5]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_5_gated_we;
  assign loc_alert_class_shadowed_5_gated_we =
    loc_alert_class_shadowed_5_we & loc_alert_regwen_5_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_5_re),
    .we     (loc_alert_class_shadowed_5_gated_we),
    .wd     (loc_alert_class_shadowed_5_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_5_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_5_update_err),
    .err_storage (loc_alert_class_shadowed_5_storage_err)
  );


  // Subregister 6 of Multireg loc_alert_class_shadowed
  // R[loc_alert_class_shadowed_6]: V(False)
  // Create REGWEN-gated WE signal
  logic loc_alert_class_shadowed_6_gated_we;
  assign loc_alert_class_shadowed_6_gated_we =
    loc_alert_class_shadowed_6_we & loc_alert_regwen_6_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_loc_alert_class_shadowed_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (loc_alert_class_shadowed_6_re),
    .we     (loc_alert_class_shadowed_6_gated_we),
    .wd     (loc_alert_class_shadowed_6_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_class_shadowed[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_class_shadowed_6_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (loc_alert_class_shadowed_6_update_err),
    .err_storage (loc_alert_class_shadowed_6_storage_err)
  );


  // Subregister 0 of Multireg loc_alert_cause
  // R[loc_alert_cause_0]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_0_we),
    .wd     (loc_alert_cause_0_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[0].de),
    .d      (hw2reg.loc_alert_cause[0].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[0].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_0_qs)
  );


  // Subregister 1 of Multireg loc_alert_cause
  // R[loc_alert_cause_1]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_1_we),
    .wd     (loc_alert_cause_1_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[1].de),
    .d      (hw2reg.loc_alert_cause[1].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[1].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_1_qs)
  );


  // Subregister 2 of Multireg loc_alert_cause
  // R[loc_alert_cause_2]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_2_we),
    .wd     (loc_alert_cause_2_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[2].de),
    .d      (hw2reg.loc_alert_cause[2].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[2].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_2_qs)
  );


  // Subregister 3 of Multireg loc_alert_cause
  // R[loc_alert_cause_3]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_3_we),
    .wd     (loc_alert_cause_3_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[3].de),
    .d      (hw2reg.loc_alert_cause[3].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[3].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_3_qs)
  );


  // Subregister 4 of Multireg loc_alert_cause
  // R[loc_alert_cause_4]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_4 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_4_we),
    .wd     (loc_alert_cause_4_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[4].de),
    .d      (hw2reg.loc_alert_cause[4].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[4].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_4_qs)
  );


  // Subregister 5 of Multireg loc_alert_cause
  // R[loc_alert_cause_5]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_5 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_5_we),
    .wd     (loc_alert_cause_5_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[5].de),
    .d      (hw2reg.loc_alert_cause[5].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[5].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_5_qs)
  );


  // Subregister 6 of Multireg loc_alert_cause
  // R[loc_alert_cause_6]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW1C),
    .RESVAL  (1'h0)
  ) u_loc_alert_cause_6 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (loc_alert_cause_6_we),
    .wd     (loc_alert_cause_6_wd),

    // from internal hardware
    .de     (hw2reg.loc_alert_cause[6].de),
    .d      (hw2reg.loc_alert_cause[6].d),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.loc_alert_cause[6].q),
    .ds     (),

    // to register interface (read)
    .qs     (loc_alert_cause_6_qs)
  );


  // R[classa_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classa_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classa_regwen_we),
    .wd     (classa_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classa_regwen_qs)
  );


  // R[classa_ctrl_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_ctrl_shadowed_gated_we;
  assign classa_ctrl_shadowed_gated_we = classa_ctrl_shadowed_we & classa_regwen_qs;
  //   F[en]: 0:0
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classa_ctrl_shadowed_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.en.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_en_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_en_update_err),
    .err_storage (classa_ctrl_shadowed_en_storage_err)
  );

  //   F[lock]: 1:1
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classa_ctrl_shadowed_lock (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_lock_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.lock.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_lock_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_lock_update_err),
    .err_storage (classa_ctrl_shadowed_lock_storage_err)
  );

  //   F[en_e0]: 2:2
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classa_ctrl_shadowed_en_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_en_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.en_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_en_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_en_e0_update_err),
    .err_storage (classa_ctrl_shadowed_en_e0_storage_err)
  );

  //   F[en_e1]: 3:3
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classa_ctrl_shadowed_en_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_en_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.en_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_en_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_en_e1_update_err),
    .err_storage (classa_ctrl_shadowed_en_e1_storage_err)
  );

  //   F[en_e2]: 4:4
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classa_ctrl_shadowed_en_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_en_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.en_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_en_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_en_e2_update_err),
    .err_storage (classa_ctrl_shadowed_en_e2_storage_err)
  );

  //   F[en_e3]: 5:5
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classa_ctrl_shadowed_en_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_en_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.en_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_en_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_en_e3_update_err),
    .err_storage (classa_ctrl_shadowed_en_e3_storage_err)
  );

  //   F[map_e0]: 7:6
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classa_ctrl_shadowed_map_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_map_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.map_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_map_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_map_e0_update_err),
    .err_storage (classa_ctrl_shadowed_map_e0_storage_err)
  );

  //   F[map_e1]: 9:8
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h1)
  ) u_classa_ctrl_shadowed_map_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_map_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.map_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_map_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_map_e1_update_err),
    .err_storage (classa_ctrl_shadowed_map_e1_storage_err)
  );

  //   F[map_e2]: 11:10
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h2)
  ) u_classa_ctrl_shadowed_map_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_map_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.map_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_map_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_map_e2_update_err),
    .err_storage (classa_ctrl_shadowed_map_e2_storage_err)
  );

  //   F[map_e3]: 13:12
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h3)
  ) u_classa_ctrl_shadowed_map_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_ctrl_shadowed_re),
    .we     (classa_ctrl_shadowed_gated_we),
    .wd     (classa_ctrl_shadowed_map_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_ctrl_shadowed.map_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_ctrl_shadowed_map_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_ctrl_shadowed_map_e3_update_err),
    .err_storage (classa_ctrl_shadowed_map_e3_storage_err)
  );


  // R[classa_clr_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classa_clr_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classa_clr_regwen_we),
    .wd     (classa_clr_regwen_wd),

    // from internal hardware
    .de     (hw2reg.classa_clr_regwen.de),
    .d      (hw2reg.classa_clr_regwen.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classa_clr_regwen_qs)
  );


  // R[classa_clr_shadowed]: V(False)
  logic classa_clr_shadowed_qe;
  logic [0:0] classa_clr_shadowed_flds_we;
  prim_flop #(
    .Width(1),
    .ResetValue(0)
  ) u_classa_clr_shadowed0_qe (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .d_i(&classa_clr_shadowed_flds_we),
    .q_o(classa_clr_shadowed_qe)
  );
  // Create REGWEN-gated WE signal
  logic classa_clr_shadowed_gated_we;
  assign classa_clr_shadowed_gated_we = classa_clr_shadowed_we & classa_clr_regwen_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classa_clr_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_clr_shadowed_re),
    .we     (classa_clr_shadowed_gated_we),
    .wd     (classa_clr_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (classa_clr_shadowed_flds_we[0]),
    .q      (reg2hw.classa_clr_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_clr_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_clr_shadowed_update_err),
    .err_storage (classa_clr_shadowed_storage_err)
  );
  assign reg2hw.classa_clr_shadowed.qe = classa_clr_shadowed_qe;


  // R[classa_accum_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (16)
  ) u_classa_accum_cnt (
    .re     (classa_accum_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classa_accum_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classa_accum_cnt_qs)
  );


  // R[classa_accum_thresh_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_accum_thresh_shadowed_gated_we;
  assign classa_accum_thresh_shadowed_gated_we = classa_accum_thresh_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (16),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (16'h0)
  ) u_classa_accum_thresh_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_accum_thresh_shadowed_re),
    .we     (classa_accum_thresh_shadowed_gated_we),
    .wd     (classa_accum_thresh_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_accum_thresh_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_accum_thresh_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_accum_thresh_shadowed_update_err),
    .err_storage (classa_accum_thresh_shadowed_storage_err)
  );


  // R[classa_timeout_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_timeout_cyc_shadowed_gated_we;
  assign classa_timeout_cyc_shadowed_gated_we = classa_timeout_cyc_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classa_timeout_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_timeout_cyc_shadowed_re),
    .we     (classa_timeout_cyc_shadowed_gated_we),
    .wd     (classa_timeout_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_timeout_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_timeout_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_timeout_cyc_shadowed_update_err),
    .err_storage (classa_timeout_cyc_shadowed_storage_err)
  );


  // R[classa_crashdump_trigger_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_crashdump_trigger_shadowed_gated_we;
  assign classa_crashdump_trigger_shadowed_gated_we =
    classa_crashdump_trigger_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classa_crashdump_trigger_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_crashdump_trigger_shadowed_re),
    .we     (classa_crashdump_trigger_shadowed_gated_we),
    .wd     (classa_crashdump_trigger_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_crashdump_trigger_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_crashdump_trigger_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_crashdump_trigger_shadowed_update_err),
    .err_storage (classa_crashdump_trigger_shadowed_storage_err)
  );


  // R[classa_phase0_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_phase0_cyc_shadowed_gated_we;
  assign classa_phase0_cyc_shadowed_gated_we = classa_phase0_cyc_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classa_phase0_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_phase0_cyc_shadowed_re),
    .we     (classa_phase0_cyc_shadowed_gated_we),
    .wd     (classa_phase0_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_phase0_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_phase0_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_phase0_cyc_shadowed_update_err),
    .err_storage (classa_phase0_cyc_shadowed_storage_err)
  );


  // R[classa_phase1_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_phase1_cyc_shadowed_gated_we;
  assign classa_phase1_cyc_shadowed_gated_we = classa_phase1_cyc_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classa_phase1_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_phase1_cyc_shadowed_re),
    .we     (classa_phase1_cyc_shadowed_gated_we),
    .wd     (classa_phase1_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_phase1_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_phase1_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_phase1_cyc_shadowed_update_err),
    .err_storage (classa_phase1_cyc_shadowed_storage_err)
  );


  // R[classa_phase2_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_phase2_cyc_shadowed_gated_we;
  assign classa_phase2_cyc_shadowed_gated_we = classa_phase2_cyc_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classa_phase2_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_phase2_cyc_shadowed_re),
    .we     (classa_phase2_cyc_shadowed_gated_we),
    .wd     (classa_phase2_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_phase2_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_phase2_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_phase2_cyc_shadowed_update_err),
    .err_storage (classa_phase2_cyc_shadowed_storage_err)
  );


  // R[classa_phase3_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classa_phase3_cyc_shadowed_gated_we;
  assign classa_phase3_cyc_shadowed_gated_we = classa_phase3_cyc_shadowed_we & classa_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classa_phase3_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classa_phase3_cyc_shadowed_re),
    .we     (classa_phase3_cyc_shadowed_gated_we),
    .wd     (classa_phase3_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classa_phase3_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classa_phase3_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classa_phase3_cyc_shadowed_update_err),
    .err_storage (classa_phase3_cyc_shadowed_storage_err)
  );


  // R[classa_esc_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (32)
  ) u_classa_esc_cnt (
    .re     (classa_esc_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classa_esc_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classa_esc_cnt_qs)
  );


  // R[classa_state]: V(True)
  prim_subreg_ext #(
    .DW    (3)
  ) u_classa_state (
    .re     (classa_state_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classa_state.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classa_state_qs)
  );


  // R[classb_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classb_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classb_regwen_we),
    .wd     (classb_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classb_regwen_qs)
  );


  // R[classb_ctrl_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_ctrl_shadowed_gated_we;
  assign classb_ctrl_shadowed_gated_we = classb_ctrl_shadowed_we & classb_regwen_qs;
  //   F[en]: 0:0
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classb_ctrl_shadowed_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.en.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_en_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_en_update_err),
    .err_storage (classb_ctrl_shadowed_en_storage_err)
  );

  //   F[lock]: 1:1
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classb_ctrl_shadowed_lock (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_lock_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.lock.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_lock_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_lock_update_err),
    .err_storage (classb_ctrl_shadowed_lock_storage_err)
  );

  //   F[en_e0]: 2:2
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classb_ctrl_shadowed_en_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_en_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.en_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_en_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_en_e0_update_err),
    .err_storage (classb_ctrl_shadowed_en_e0_storage_err)
  );

  //   F[en_e1]: 3:3
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classb_ctrl_shadowed_en_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_en_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.en_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_en_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_en_e1_update_err),
    .err_storage (classb_ctrl_shadowed_en_e1_storage_err)
  );

  //   F[en_e2]: 4:4
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classb_ctrl_shadowed_en_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_en_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.en_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_en_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_en_e2_update_err),
    .err_storage (classb_ctrl_shadowed_en_e2_storage_err)
  );

  //   F[en_e3]: 5:5
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classb_ctrl_shadowed_en_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_en_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.en_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_en_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_en_e3_update_err),
    .err_storage (classb_ctrl_shadowed_en_e3_storage_err)
  );

  //   F[map_e0]: 7:6
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classb_ctrl_shadowed_map_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_map_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.map_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_map_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_map_e0_update_err),
    .err_storage (classb_ctrl_shadowed_map_e0_storage_err)
  );

  //   F[map_e1]: 9:8
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h1)
  ) u_classb_ctrl_shadowed_map_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_map_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.map_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_map_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_map_e1_update_err),
    .err_storage (classb_ctrl_shadowed_map_e1_storage_err)
  );

  //   F[map_e2]: 11:10
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h2)
  ) u_classb_ctrl_shadowed_map_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_map_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.map_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_map_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_map_e2_update_err),
    .err_storage (classb_ctrl_shadowed_map_e2_storage_err)
  );

  //   F[map_e3]: 13:12
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h3)
  ) u_classb_ctrl_shadowed_map_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_ctrl_shadowed_re),
    .we     (classb_ctrl_shadowed_gated_we),
    .wd     (classb_ctrl_shadowed_map_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_ctrl_shadowed.map_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_ctrl_shadowed_map_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_ctrl_shadowed_map_e3_update_err),
    .err_storage (classb_ctrl_shadowed_map_e3_storage_err)
  );


  // R[classb_clr_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classb_clr_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classb_clr_regwen_we),
    .wd     (classb_clr_regwen_wd),

    // from internal hardware
    .de     (hw2reg.classb_clr_regwen.de),
    .d      (hw2reg.classb_clr_regwen.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classb_clr_regwen_qs)
  );


  // R[classb_clr_shadowed]: V(False)
  logic classb_clr_shadowed_qe;
  logic [0:0] classb_clr_shadowed_flds_we;
  prim_flop #(
    .Width(1),
    .ResetValue(0)
  ) u_classb_clr_shadowed0_qe (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .d_i(&classb_clr_shadowed_flds_we),
    .q_o(classb_clr_shadowed_qe)
  );
  // Create REGWEN-gated WE signal
  logic classb_clr_shadowed_gated_we;
  assign classb_clr_shadowed_gated_we = classb_clr_shadowed_we & classb_clr_regwen_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classb_clr_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_clr_shadowed_re),
    .we     (classb_clr_shadowed_gated_we),
    .wd     (classb_clr_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (classb_clr_shadowed_flds_we[0]),
    .q      (reg2hw.classb_clr_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_clr_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_clr_shadowed_update_err),
    .err_storage (classb_clr_shadowed_storage_err)
  );
  assign reg2hw.classb_clr_shadowed.qe = classb_clr_shadowed_qe;


  // R[classb_accum_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (16)
  ) u_classb_accum_cnt (
    .re     (classb_accum_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classb_accum_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classb_accum_cnt_qs)
  );


  // R[classb_accum_thresh_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_accum_thresh_shadowed_gated_we;
  assign classb_accum_thresh_shadowed_gated_we = classb_accum_thresh_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (16),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (16'h0)
  ) u_classb_accum_thresh_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_accum_thresh_shadowed_re),
    .we     (classb_accum_thresh_shadowed_gated_we),
    .wd     (classb_accum_thresh_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_accum_thresh_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_accum_thresh_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_accum_thresh_shadowed_update_err),
    .err_storage (classb_accum_thresh_shadowed_storage_err)
  );


  // R[classb_timeout_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_timeout_cyc_shadowed_gated_we;
  assign classb_timeout_cyc_shadowed_gated_we = classb_timeout_cyc_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classb_timeout_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_timeout_cyc_shadowed_re),
    .we     (classb_timeout_cyc_shadowed_gated_we),
    .wd     (classb_timeout_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_timeout_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_timeout_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_timeout_cyc_shadowed_update_err),
    .err_storage (classb_timeout_cyc_shadowed_storage_err)
  );


  // R[classb_crashdump_trigger_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_crashdump_trigger_shadowed_gated_we;
  assign classb_crashdump_trigger_shadowed_gated_we =
    classb_crashdump_trigger_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classb_crashdump_trigger_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_crashdump_trigger_shadowed_re),
    .we     (classb_crashdump_trigger_shadowed_gated_we),
    .wd     (classb_crashdump_trigger_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_crashdump_trigger_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_crashdump_trigger_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_crashdump_trigger_shadowed_update_err),
    .err_storage (classb_crashdump_trigger_shadowed_storage_err)
  );


  // R[classb_phase0_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_phase0_cyc_shadowed_gated_we;
  assign classb_phase0_cyc_shadowed_gated_we = classb_phase0_cyc_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classb_phase0_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_phase0_cyc_shadowed_re),
    .we     (classb_phase0_cyc_shadowed_gated_we),
    .wd     (classb_phase0_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_phase0_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_phase0_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_phase0_cyc_shadowed_update_err),
    .err_storage (classb_phase0_cyc_shadowed_storage_err)
  );


  // R[classb_phase1_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_phase1_cyc_shadowed_gated_we;
  assign classb_phase1_cyc_shadowed_gated_we = classb_phase1_cyc_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classb_phase1_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_phase1_cyc_shadowed_re),
    .we     (classb_phase1_cyc_shadowed_gated_we),
    .wd     (classb_phase1_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_phase1_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_phase1_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_phase1_cyc_shadowed_update_err),
    .err_storage (classb_phase1_cyc_shadowed_storage_err)
  );


  // R[classb_phase2_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_phase2_cyc_shadowed_gated_we;
  assign classb_phase2_cyc_shadowed_gated_we = classb_phase2_cyc_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classb_phase2_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_phase2_cyc_shadowed_re),
    .we     (classb_phase2_cyc_shadowed_gated_we),
    .wd     (classb_phase2_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_phase2_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_phase2_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_phase2_cyc_shadowed_update_err),
    .err_storage (classb_phase2_cyc_shadowed_storage_err)
  );


  // R[classb_phase3_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classb_phase3_cyc_shadowed_gated_we;
  assign classb_phase3_cyc_shadowed_gated_we = classb_phase3_cyc_shadowed_we & classb_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classb_phase3_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classb_phase3_cyc_shadowed_re),
    .we     (classb_phase3_cyc_shadowed_gated_we),
    .wd     (classb_phase3_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classb_phase3_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classb_phase3_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classb_phase3_cyc_shadowed_update_err),
    .err_storage (classb_phase3_cyc_shadowed_storage_err)
  );


  // R[classb_esc_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (32)
  ) u_classb_esc_cnt (
    .re     (classb_esc_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classb_esc_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classb_esc_cnt_qs)
  );


  // R[classb_state]: V(True)
  prim_subreg_ext #(
    .DW    (3)
  ) u_classb_state (
    .re     (classb_state_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classb_state.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classb_state_qs)
  );


  // R[classc_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classc_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classc_regwen_we),
    .wd     (classc_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classc_regwen_qs)
  );


  // R[classc_ctrl_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_ctrl_shadowed_gated_we;
  assign classc_ctrl_shadowed_gated_we = classc_ctrl_shadowed_we & classc_regwen_qs;
  //   F[en]: 0:0
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classc_ctrl_shadowed_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.en.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_en_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_en_update_err),
    .err_storage (classc_ctrl_shadowed_en_storage_err)
  );

  //   F[lock]: 1:1
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classc_ctrl_shadowed_lock (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_lock_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.lock.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_lock_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_lock_update_err),
    .err_storage (classc_ctrl_shadowed_lock_storage_err)
  );

  //   F[en_e0]: 2:2
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classc_ctrl_shadowed_en_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_en_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.en_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_en_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_en_e0_update_err),
    .err_storage (classc_ctrl_shadowed_en_e0_storage_err)
  );

  //   F[en_e1]: 3:3
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classc_ctrl_shadowed_en_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_en_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.en_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_en_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_en_e1_update_err),
    .err_storage (classc_ctrl_shadowed_en_e1_storage_err)
  );

  //   F[en_e2]: 4:4
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classc_ctrl_shadowed_en_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_en_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.en_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_en_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_en_e2_update_err),
    .err_storage (classc_ctrl_shadowed_en_e2_storage_err)
  );

  //   F[en_e3]: 5:5
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classc_ctrl_shadowed_en_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_en_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.en_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_en_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_en_e3_update_err),
    .err_storage (classc_ctrl_shadowed_en_e3_storage_err)
  );

  //   F[map_e0]: 7:6
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classc_ctrl_shadowed_map_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_map_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.map_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_map_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_map_e0_update_err),
    .err_storage (classc_ctrl_shadowed_map_e0_storage_err)
  );

  //   F[map_e1]: 9:8
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h1)
  ) u_classc_ctrl_shadowed_map_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_map_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.map_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_map_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_map_e1_update_err),
    .err_storage (classc_ctrl_shadowed_map_e1_storage_err)
  );

  //   F[map_e2]: 11:10
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h2)
  ) u_classc_ctrl_shadowed_map_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_map_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.map_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_map_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_map_e2_update_err),
    .err_storage (classc_ctrl_shadowed_map_e2_storage_err)
  );

  //   F[map_e3]: 13:12
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h3)
  ) u_classc_ctrl_shadowed_map_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_ctrl_shadowed_re),
    .we     (classc_ctrl_shadowed_gated_we),
    .wd     (classc_ctrl_shadowed_map_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_ctrl_shadowed.map_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_ctrl_shadowed_map_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_ctrl_shadowed_map_e3_update_err),
    .err_storage (classc_ctrl_shadowed_map_e3_storage_err)
  );


  // R[classc_clr_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classc_clr_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classc_clr_regwen_we),
    .wd     (classc_clr_regwen_wd),

    // from internal hardware
    .de     (hw2reg.classc_clr_regwen.de),
    .d      (hw2reg.classc_clr_regwen.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classc_clr_regwen_qs)
  );


  // R[classc_clr_shadowed]: V(False)
  logic classc_clr_shadowed_qe;
  logic [0:0] classc_clr_shadowed_flds_we;
  prim_flop #(
    .Width(1),
    .ResetValue(0)
  ) u_classc_clr_shadowed0_qe (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .d_i(&classc_clr_shadowed_flds_we),
    .q_o(classc_clr_shadowed_qe)
  );
  // Create REGWEN-gated WE signal
  logic classc_clr_shadowed_gated_we;
  assign classc_clr_shadowed_gated_we = classc_clr_shadowed_we & classc_clr_regwen_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classc_clr_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_clr_shadowed_re),
    .we     (classc_clr_shadowed_gated_we),
    .wd     (classc_clr_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (classc_clr_shadowed_flds_we[0]),
    .q      (reg2hw.classc_clr_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_clr_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_clr_shadowed_update_err),
    .err_storage (classc_clr_shadowed_storage_err)
  );
  assign reg2hw.classc_clr_shadowed.qe = classc_clr_shadowed_qe;


  // R[classc_accum_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (16)
  ) u_classc_accum_cnt (
    .re     (classc_accum_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classc_accum_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classc_accum_cnt_qs)
  );


  // R[classc_accum_thresh_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_accum_thresh_shadowed_gated_we;
  assign classc_accum_thresh_shadowed_gated_we = classc_accum_thresh_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (16),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (16'h0)
  ) u_classc_accum_thresh_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_accum_thresh_shadowed_re),
    .we     (classc_accum_thresh_shadowed_gated_we),
    .wd     (classc_accum_thresh_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_accum_thresh_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_accum_thresh_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_accum_thresh_shadowed_update_err),
    .err_storage (classc_accum_thresh_shadowed_storage_err)
  );


  // R[classc_timeout_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_timeout_cyc_shadowed_gated_we;
  assign classc_timeout_cyc_shadowed_gated_we = classc_timeout_cyc_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classc_timeout_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_timeout_cyc_shadowed_re),
    .we     (classc_timeout_cyc_shadowed_gated_we),
    .wd     (classc_timeout_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_timeout_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_timeout_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_timeout_cyc_shadowed_update_err),
    .err_storage (classc_timeout_cyc_shadowed_storage_err)
  );


  // R[classc_crashdump_trigger_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_crashdump_trigger_shadowed_gated_we;
  assign classc_crashdump_trigger_shadowed_gated_we =
    classc_crashdump_trigger_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classc_crashdump_trigger_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_crashdump_trigger_shadowed_re),
    .we     (classc_crashdump_trigger_shadowed_gated_we),
    .wd     (classc_crashdump_trigger_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_crashdump_trigger_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_crashdump_trigger_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_crashdump_trigger_shadowed_update_err),
    .err_storage (classc_crashdump_trigger_shadowed_storage_err)
  );


  // R[classc_phase0_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_phase0_cyc_shadowed_gated_we;
  assign classc_phase0_cyc_shadowed_gated_we = classc_phase0_cyc_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classc_phase0_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_phase0_cyc_shadowed_re),
    .we     (classc_phase0_cyc_shadowed_gated_we),
    .wd     (classc_phase0_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_phase0_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_phase0_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_phase0_cyc_shadowed_update_err),
    .err_storage (classc_phase0_cyc_shadowed_storage_err)
  );


  // R[classc_phase1_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_phase1_cyc_shadowed_gated_we;
  assign classc_phase1_cyc_shadowed_gated_we = classc_phase1_cyc_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classc_phase1_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_phase1_cyc_shadowed_re),
    .we     (classc_phase1_cyc_shadowed_gated_we),
    .wd     (classc_phase1_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_phase1_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_phase1_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_phase1_cyc_shadowed_update_err),
    .err_storage (classc_phase1_cyc_shadowed_storage_err)
  );


  // R[classc_phase2_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_phase2_cyc_shadowed_gated_we;
  assign classc_phase2_cyc_shadowed_gated_we = classc_phase2_cyc_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classc_phase2_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_phase2_cyc_shadowed_re),
    .we     (classc_phase2_cyc_shadowed_gated_we),
    .wd     (classc_phase2_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_phase2_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_phase2_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_phase2_cyc_shadowed_update_err),
    .err_storage (classc_phase2_cyc_shadowed_storage_err)
  );


  // R[classc_phase3_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classc_phase3_cyc_shadowed_gated_we;
  assign classc_phase3_cyc_shadowed_gated_we = classc_phase3_cyc_shadowed_we & classc_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classc_phase3_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classc_phase3_cyc_shadowed_re),
    .we     (classc_phase3_cyc_shadowed_gated_we),
    .wd     (classc_phase3_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classc_phase3_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classc_phase3_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classc_phase3_cyc_shadowed_update_err),
    .err_storage (classc_phase3_cyc_shadowed_storage_err)
  );


  // R[classc_esc_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (32)
  ) u_classc_esc_cnt (
    .re     (classc_esc_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classc_esc_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classc_esc_cnt_qs)
  );


  // R[classc_state]: V(True)
  prim_subreg_ext #(
    .DW    (3)
  ) u_classc_state (
    .re     (classc_state_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classc_state.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classc_state_qs)
  );


  // R[classd_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classd_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classd_regwen_we),
    .wd     (classd_regwen_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classd_regwen_qs)
  );


  // R[classd_ctrl_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_ctrl_shadowed_gated_we;
  assign classd_ctrl_shadowed_gated_we = classd_ctrl_shadowed_we & classd_regwen_qs;
  //   F[en]: 0:0
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classd_ctrl_shadowed_en (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_en_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.en.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_en_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_en_update_err),
    .err_storage (classd_ctrl_shadowed_en_storage_err)
  );

  //   F[lock]: 1:1
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classd_ctrl_shadowed_lock (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_lock_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.lock.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_lock_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_lock_update_err),
    .err_storage (classd_ctrl_shadowed_lock_storage_err)
  );

  //   F[en_e0]: 2:2
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classd_ctrl_shadowed_en_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_en_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.en_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_en_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_en_e0_update_err),
    .err_storage (classd_ctrl_shadowed_en_e0_storage_err)
  );

  //   F[en_e1]: 3:3
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classd_ctrl_shadowed_en_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_en_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.en_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_en_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_en_e1_update_err),
    .err_storage (classd_ctrl_shadowed_en_e1_storage_err)
  );

  //   F[en_e2]: 4:4
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classd_ctrl_shadowed_en_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_en_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.en_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_en_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_en_e2_update_err),
    .err_storage (classd_ctrl_shadowed_en_e2_storage_err)
  );

  //   F[en_e3]: 5:5
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h1)
  ) u_classd_ctrl_shadowed_en_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_en_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.en_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_en_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_en_e3_update_err),
    .err_storage (classd_ctrl_shadowed_en_e3_storage_err)
  );

  //   F[map_e0]: 7:6
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classd_ctrl_shadowed_map_e0 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_map_e0_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.map_e0.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_map_e0_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_map_e0_update_err),
    .err_storage (classd_ctrl_shadowed_map_e0_storage_err)
  );

  //   F[map_e1]: 9:8
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h1)
  ) u_classd_ctrl_shadowed_map_e1 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_map_e1_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.map_e1.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_map_e1_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_map_e1_update_err),
    .err_storage (classd_ctrl_shadowed_map_e1_storage_err)
  );

  //   F[map_e2]: 11:10
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h2)
  ) u_classd_ctrl_shadowed_map_e2 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_map_e2_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.map_e2.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_map_e2_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_map_e2_update_err),
    .err_storage (classd_ctrl_shadowed_map_e2_storage_err)
  );

  //   F[map_e3]: 13:12
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h3)
  ) u_classd_ctrl_shadowed_map_e3 (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_ctrl_shadowed_re),
    .we     (classd_ctrl_shadowed_gated_we),
    .wd     (classd_ctrl_shadowed_map_e3_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_ctrl_shadowed.map_e3.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_ctrl_shadowed_map_e3_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_ctrl_shadowed_map_e3_update_err),
    .err_storage (classd_ctrl_shadowed_map_e3_storage_err)
  );


  // R[classd_clr_regwen]: V(False)
  prim_subreg #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessW0C),
    .RESVAL  (1'h1)
  ) u_classd_clr_regwen (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),

    // from register interface
    .we     (classd_clr_regwen_we),
    .wd     (classd_clr_regwen_wd),

    // from internal hardware
    .de     (hw2reg.classd_clr_regwen.de),
    .d      (hw2reg.classd_clr_regwen.d),

    // to internal hardware
    .qe     (),
    .q      (),
    .ds     (),

    // to register interface (read)
    .qs     (classd_clr_regwen_qs)
  );


  // R[classd_clr_shadowed]: V(False)
  logic classd_clr_shadowed_qe;
  logic [0:0] classd_clr_shadowed_flds_we;
  prim_flop #(
    .Width(1),
    .ResetValue(0)
  ) u_classd_clr_shadowed0_qe (
    .clk_i(clk_i),
    .rst_ni(rst_ni),
    .d_i(&classd_clr_shadowed_flds_we),
    .q_o(classd_clr_shadowed_qe)
  );
  // Create REGWEN-gated WE signal
  logic classd_clr_shadowed_gated_we;
  assign classd_clr_shadowed_gated_we = classd_clr_shadowed_we & classd_clr_regwen_qs;
  prim_subreg_shadow #(
    .DW      (1),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (1'h0)
  ) u_classd_clr_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_clr_shadowed_re),
    .we     (classd_clr_shadowed_gated_we),
    .wd     (classd_clr_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (classd_clr_shadowed_flds_we[0]),
    .q      (reg2hw.classd_clr_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_clr_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_clr_shadowed_update_err),
    .err_storage (classd_clr_shadowed_storage_err)
  );
  assign reg2hw.classd_clr_shadowed.qe = classd_clr_shadowed_qe;


  // R[classd_accum_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (16)
  ) u_classd_accum_cnt (
    .re     (classd_accum_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classd_accum_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classd_accum_cnt_qs)
  );


  // R[classd_accum_thresh_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_accum_thresh_shadowed_gated_we;
  assign classd_accum_thresh_shadowed_gated_we = classd_accum_thresh_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (16),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (16'h0)
  ) u_classd_accum_thresh_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_accum_thresh_shadowed_re),
    .we     (classd_accum_thresh_shadowed_gated_we),
    .wd     (classd_accum_thresh_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_accum_thresh_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_accum_thresh_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_accum_thresh_shadowed_update_err),
    .err_storage (classd_accum_thresh_shadowed_storage_err)
  );


  // R[classd_timeout_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_timeout_cyc_shadowed_gated_we;
  assign classd_timeout_cyc_shadowed_gated_we = classd_timeout_cyc_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classd_timeout_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_timeout_cyc_shadowed_re),
    .we     (classd_timeout_cyc_shadowed_gated_we),
    .wd     (classd_timeout_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_timeout_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_timeout_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_timeout_cyc_shadowed_update_err),
    .err_storage (classd_timeout_cyc_shadowed_storage_err)
  );


  // R[classd_crashdump_trigger_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_crashdump_trigger_shadowed_gated_we;
  assign classd_crashdump_trigger_shadowed_gated_we =
    classd_crashdump_trigger_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (2),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (2'h0)
  ) u_classd_crashdump_trigger_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_crashdump_trigger_shadowed_re),
    .we     (classd_crashdump_trigger_shadowed_gated_we),
    .wd     (classd_crashdump_trigger_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_crashdump_trigger_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_crashdump_trigger_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_crashdump_trigger_shadowed_update_err),
    .err_storage (classd_crashdump_trigger_shadowed_storage_err)
  );


  // R[classd_phase0_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_phase0_cyc_shadowed_gated_we;
  assign classd_phase0_cyc_shadowed_gated_we = classd_phase0_cyc_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classd_phase0_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_phase0_cyc_shadowed_re),
    .we     (classd_phase0_cyc_shadowed_gated_we),
    .wd     (classd_phase0_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_phase0_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_phase0_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_phase0_cyc_shadowed_update_err),
    .err_storage (classd_phase0_cyc_shadowed_storage_err)
  );


  // R[classd_phase1_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_phase1_cyc_shadowed_gated_we;
  assign classd_phase1_cyc_shadowed_gated_we = classd_phase1_cyc_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classd_phase1_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_phase1_cyc_shadowed_re),
    .we     (classd_phase1_cyc_shadowed_gated_we),
    .wd     (classd_phase1_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_phase1_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_phase1_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_phase1_cyc_shadowed_update_err),
    .err_storage (classd_phase1_cyc_shadowed_storage_err)
  );


  // R[classd_phase2_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_phase2_cyc_shadowed_gated_we;
  assign classd_phase2_cyc_shadowed_gated_we = classd_phase2_cyc_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classd_phase2_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_phase2_cyc_shadowed_re),
    .we     (classd_phase2_cyc_shadowed_gated_we),
    .wd     (classd_phase2_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_phase2_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_phase2_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_phase2_cyc_shadowed_update_err),
    .err_storage (classd_phase2_cyc_shadowed_storage_err)
  );


  // R[classd_phase3_cyc_shadowed]: V(False)
  // Create REGWEN-gated WE signal
  logic classd_phase3_cyc_shadowed_gated_we;
  assign classd_phase3_cyc_shadowed_gated_we = classd_phase3_cyc_shadowed_we & classd_regwen_qs;
  prim_subreg_shadow #(
    .DW      (32),
    .SwAccess(prim_subreg_pkg::SwAccessRW),
    .RESVAL  (32'h0)
  ) u_classd_phase3_cyc_shadowed (
    .clk_i   (clk_i),
    .rst_ni  (rst_ni),
    .rst_shadowed_ni (rst_shadowed_ni),

    // from register interface
    .re     (classd_phase3_cyc_shadowed_re),
    .we     (classd_phase3_cyc_shadowed_gated_we),
    .wd     (classd_phase3_cyc_shadowed_wd),

    // from internal hardware
    .de     (1'b0),
    .d      ('0),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.classd_phase3_cyc_shadowed.q),
    .ds     (),

    // to register interface (read)
    .qs     (classd_phase3_cyc_shadowed_qs),

    // Shadow register phase. Relevant for hwext only.
    .phase  (),

    // Shadow register error conditions
    .err_update  (classd_phase3_cyc_shadowed_update_err),
    .err_storage (classd_phase3_cyc_shadowed_storage_err)
  );


  // R[classd_esc_cnt]: V(True)
  prim_subreg_ext #(
    .DW    (32)
  ) u_classd_esc_cnt (
    .re     (classd_esc_cnt_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classd_esc_cnt.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classd_esc_cnt_qs)
  );


  // R[classd_state]: V(True)
  prim_subreg_ext #(
    .DW    (3)
  ) u_classd_state (
    .re     (classd_state_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.classd_state.d),
    .qre    (),
    .qe     (),
    .q      (),
    .ds     (),
    .qs     (classd_state_qs)
  );



  logic [349:0] addr_hit;
  always_comb begin
    addr_hit = '0;
    addr_hit[  0] = (reg_addr == ALERT_HANDLER_INTR_STATE_OFFSET);
    addr_hit[  1] = (reg_addr == ALERT_HANDLER_INTR_ENABLE_OFFSET);
    addr_hit[  2] = (reg_addr == ALERT_HANDLER_INTR_TEST_OFFSET);
    addr_hit[  3] = (reg_addr == ALERT_HANDLER_PING_TIMER_REGWEN_OFFSET);
    addr_hit[  4] = (reg_addr == ALERT_HANDLER_PING_TIMEOUT_CYC_SHADOWED_OFFSET);
    addr_hit[  5] = (reg_addr == ALERT_HANDLER_PING_TIMER_EN_SHADOWED_OFFSET);
    addr_hit[  6] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_0_OFFSET);
    addr_hit[  7] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_1_OFFSET);
    addr_hit[  8] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_2_OFFSET);
    addr_hit[  9] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_3_OFFSET);
    addr_hit[ 10] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_4_OFFSET);
    addr_hit[ 11] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_5_OFFSET);
    addr_hit[ 12] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_6_OFFSET);
    addr_hit[ 13] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_7_OFFSET);
    addr_hit[ 14] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_8_OFFSET);
    addr_hit[ 15] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_9_OFFSET);
    addr_hit[ 16] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_10_OFFSET);
    addr_hit[ 17] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_11_OFFSET);
    addr_hit[ 18] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_12_OFFSET);
    addr_hit[ 19] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_13_OFFSET);
    addr_hit[ 20] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_14_OFFSET);
    addr_hit[ 21] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_15_OFFSET);
    addr_hit[ 22] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_16_OFFSET);
    addr_hit[ 23] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_17_OFFSET);
    addr_hit[ 24] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_18_OFFSET);
    addr_hit[ 25] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_19_OFFSET);
    addr_hit[ 26] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_20_OFFSET);
    addr_hit[ 27] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_21_OFFSET);
    addr_hit[ 28] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_22_OFFSET);
    addr_hit[ 29] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_23_OFFSET);
    addr_hit[ 30] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_24_OFFSET);
    addr_hit[ 31] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_25_OFFSET);
    addr_hit[ 32] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_26_OFFSET);
    addr_hit[ 33] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_27_OFFSET);
    addr_hit[ 34] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_28_OFFSET);
    addr_hit[ 35] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_29_OFFSET);
    addr_hit[ 36] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_30_OFFSET);
    addr_hit[ 37] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_31_OFFSET);
    addr_hit[ 38] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_32_OFFSET);
    addr_hit[ 39] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_33_OFFSET);
    addr_hit[ 40] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_34_OFFSET);
    addr_hit[ 41] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_35_OFFSET);
    addr_hit[ 42] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_36_OFFSET);
    addr_hit[ 43] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_37_OFFSET);
    addr_hit[ 44] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_38_OFFSET);
    addr_hit[ 45] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_39_OFFSET);
    addr_hit[ 46] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_40_OFFSET);
    addr_hit[ 47] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_41_OFFSET);
    addr_hit[ 48] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_42_OFFSET);
    addr_hit[ 49] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_43_OFFSET);
    addr_hit[ 50] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_44_OFFSET);
    addr_hit[ 51] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_45_OFFSET);
    addr_hit[ 52] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_46_OFFSET);
    addr_hit[ 53] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_47_OFFSET);
    addr_hit[ 54] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_48_OFFSET);
    addr_hit[ 55] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_49_OFFSET);
    addr_hit[ 56] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_50_OFFSET);
    addr_hit[ 57] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_51_OFFSET);
    addr_hit[ 58] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_52_OFFSET);
    addr_hit[ 59] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_53_OFFSET);
    addr_hit[ 60] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_54_OFFSET);
    addr_hit[ 61] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_55_OFFSET);
    addr_hit[ 62] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_56_OFFSET);
    addr_hit[ 63] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_57_OFFSET);
    addr_hit[ 64] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_58_OFFSET);
    addr_hit[ 65] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_59_OFFSET);
    addr_hit[ 66] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_60_OFFSET);
    addr_hit[ 67] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_61_OFFSET);
    addr_hit[ 68] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_62_OFFSET);
    addr_hit[ 69] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_63_OFFSET);
    addr_hit[ 70] = (reg_addr == ALERT_HANDLER_ALERT_REGWEN_64_OFFSET);
    addr_hit[ 71] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_0_OFFSET);
    addr_hit[ 72] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_1_OFFSET);
    addr_hit[ 73] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_2_OFFSET);
    addr_hit[ 74] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_3_OFFSET);
    addr_hit[ 75] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_4_OFFSET);
    addr_hit[ 76] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_5_OFFSET);
    addr_hit[ 77] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_6_OFFSET);
    addr_hit[ 78] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_7_OFFSET);
    addr_hit[ 79] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_8_OFFSET);
    addr_hit[ 80] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_9_OFFSET);
    addr_hit[ 81] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_10_OFFSET);
    addr_hit[ 82] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_11_OFFSET);
    addr_hit[ 83] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_12_OFFSET);
    addr_hit[ 84] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_13_OFFSET);
    addr_hit[ 85] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_14_OFFSET);
    addr_hit[ 86] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_15_OFFSET);
    addr_hit[ 87] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_16_OFFSET);
    addr_hit[ 88] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_17_OFFSET);
    addr_hit[ 89] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_18_OFFSET);
    addr_hit[ 90] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_19_OFFSET);
    addr_hit[ 91] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_20_OFFSET);
    addr_hit[ 92] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_21_OFFSET);
    addr_hit[ 93] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_22_OFFSET);
    addr_hit[ 94] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_23_OFFSET);
    addr_hit[ 95] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_24_OFFSET);
    addr_hit[ 96] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_25_OFFSET);
    addr_hit[ 97] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_26_OFFSET);
    addr_hit[ 98] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_27_OFFSET);
    addr_hit[ 99] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_28_OFFSET);
    addr_hit[100] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_29_OFFSET);
    addr_hit[101] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_30_OFFSET);
    addr_hit[102] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_31_OFFSET);
    addr_hit[103] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_32_OFFSET);
    addr_hit[104] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_33_OFFSET);
    addr_hit[105] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_34_OFFSET);
    addr_hit[106] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_35_OFFSET);
    addr_hit[107] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_36_OFFSET);
    addr_hit[108] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_37_OFFSET);
    addr_hit[109] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_38_OFFSET);
    addr_hit[110] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_39_OFFSET);
    addr_hit[111] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_40_OFFSET);
    addr_hit[112] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_41_OFFSET);
    addr_hit[113] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_42_OFFSET);
    addr_hit[114] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_43_OFFSET);
    addr_hit[115] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_44_OFFSET);
    addr_hit[116] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_45_OFFSET);
    addr_hit[117] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_46_OFFSET);
    addr_hit[118] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_47_OFFSET);
    addr_hit[119] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_48_OFFSET);
    addr_hit[120] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_49_OFFSET);
    addr_hit[121] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_50_OFFSET);
    addr_hit[122] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_51_OFFSET);
    addr_hit[123] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_52_OFFSET);
    addr_hit[124] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_53_OFFSET);
    addr_hit[125] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_54_OFFSET);
    addr_hit[126] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_55_OFFSET);
    addr_hit[127] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_56_OFFSET);
    addr_hit[128] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_57_OFFSET);
    addr_hit[129] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_58_OFFSET);
    addr_hit[130] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_59_OFFSET);
    addr_hit[131] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_60_OFFSET);
    addr_hit[132] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_61_OFFSET);
    addr_hit[133] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_62_OFFSET);
    addr_hit[134] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_63_OFFSET);
    addr_hit[135] = (reg_addr == ALERT_HANDLER_ALERT_EN_SHADOWED_64_OFFSET);
    addr_hit[136] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_0_OFFSET);
    addr_hit[137] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_1_OFFSET);
    addr_hit[138] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_2_OFFSET);
    addr_hit[139] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_3_OFFSET);
    addr_hit[140] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_4_OFFSET);
    addr_hit[141] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_5_OFFSET);
    addr_hit[142] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_6_OFFSET);
    addr_hit[143] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_7_OFFSET);
    addr_hit[144] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_8_OFFSET);
    addr_hit[145] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_9_OFFSET);
    addr_hit[146] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_10_OFFSET);
    addr_hit[147] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_11_OFFSET);
    addr_hit[148] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_12_OFFSET);
    addr_hit[149] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_13_OFFSET);
    addr_hit[150] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_14_OFFSET);
    addr_hit[151] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_15_OFFSET);
    addr_hit[152] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_16_OFFSET);
    addr_hit[153] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_17_OFFSET);
    addr_hit[154] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_18_OFFSET);
    addr_hit[155] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_19_OFFSET);
    addr_hit[156] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_20_OFFSET);
    addr_hit[157] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_21_OFFSET);
    addr_hit[158] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_22_OFFSET);
    addr_hit[159] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_23_OFFSET);
    addr_hit[160] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_24_OFFSET);
    addr_hit[161] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_25_OFFSET);
    addr_hit[162] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_26_OFFSET);
    addr_hit[163] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_27_OFFSET);
    addr_hit[164] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_28_OFFSET);
    addr_hit[165] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_29_OFFSET);
    addr_hit[166] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_30_OFFSET);
    addr_hit[167] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_31_OFFSET);
    addr_hit[168] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_32_OFFSET);
    addr_hit[169] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_33_OFFSET);
    addr_hit[170] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_34_OFFSET);
    addr_hit[171] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_35_OFFSET);
    addr_hit[172] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_36_OFFSET);
    addr_hit[173] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_37_OFFSET);
    addr_hit[174] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_38_OFFSET);
    addr_hit[175] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_39_OFFSET);
    addr_hit[176] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_40_OFFSET);
    addr_hit[177] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_41_OFFSET);
    addr_hit[178] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_42_OFFSET);
    addr_hit[179] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_43_OFFSET);
    addr_hit[180] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_44_OFFSET);
    addr_hit[181] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_45_OFFSET);
    addr_hit[182] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_46_OFFSET);
    addr_hit[183] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_47_OFFSET);
    addr_hit[184] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_48_OFFSET);
    addr_hit[185] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_49_OFFSET);
    addr_hit[186] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_50_OFFSET);
    addr_hit[187] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_51_OFFSET);
    addr_hit[188] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_52_OFFSET);
    addr_hit[189] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_53_OFFSET);
    addr_hit[190] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_54_OFFSET);
    addr_hit[191] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_55_OFFSET);
    addr_hit[192] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_56_OFFSET);
    addr_hit[193] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_57_OFFSET);
    addr_hit[194] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_58_OFFSET);
    addr_hit[195] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_59_OFFSET);
    addr_hit[196] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_60_OFFSET);
    addr_hit[197] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_61_OFFSET);
    addr_hit[198] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_62_OFFSET);
    addr_hit[199] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_63_OFFSET);
    addr_hit[200] = (reg_addr == ALERT_HANDLER_ALERT_CLASS_SHADOWED_64_OFFSET);
    addr_hit[201] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_0_OFFSET);
    addr_hit[202] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_1_OFFSET);
    addr_hit[203] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_2_OFFSET);
    addr_hit[204] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_3_OFFSET);
    addr_hit[205] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_4_OFFSET);
    addr_hit[206] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_5_OFFSET);
    addr_hit[207] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_6_OFFSET);
    addr_hit[208] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_7_OFFSET);
    addr_hit[209] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_8_OFFSET);
    addr_hit[210] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_9_OFFSET);
    addr_hit[211] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_10_OFFSET);
    addr_hit[212] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_11_OFFSET);
    addr_hit[213] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_12_OFFSET);
    addr_hit[214] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_13_OFFSET);
    addr_hit[215] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_14_OFFSET);
    addr_hit[216] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_15_OFFSET);
    addr_hit[217] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_16_OFFSET);
    addr_hit[218] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_17_OFFSET);
    addr_hit[219] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_18_OFFSET);
    addr_hit[220] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_19_OFFSET);
    addr_hit[221] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_20_OFFSET);
    addr_hit[222] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_21_OFFSET);
    addr_hit[223] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_22_OFFSET);
    addr_hit[224] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_23_OFFSET);
    addr_hit[225] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_24_OFFSET);
    addr_hit[226] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_25_OFFSET);
    addr_hit[227] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_26_OFFSET);
    addr_hit[228] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_27_OFFSET);
    addr_hit[229] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_28_OFFSET);
    addr_hit[230] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_29_OFFSET);
    addr_hit[231] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_30_OFFSET);
    addr_hit[232] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_31_OFFSET);
    addr_hit[233] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_32_OFFSET);
    addr_hit[234] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_33_OFFSET);
    addr_hit[235] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_34_OFFSET);
    addr_hit[236] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_35_OFFSET);
    addr_hit[237] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_36_OFFSET);
    addr_hit[238] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_37_OFFSET);
    addr_hit[239] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_38_OFFSET);
    addr_hit[240] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_39_OFFSET);
    addr_hit[241] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_40_OFFSET);
    addr_hit[242] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_41_OFFSET);
    addr_hit[243] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_42_OFFSET);
    addr_hit[244] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_43_OFFSET);
    addr_hit[245] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_44_OFFSET);
    addr_hit[246] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_45_OFFSET);
    addr_hit[247] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_46_OFFSET);
    addr_hit[248] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_47_OFFSET);
    addr_hit[249] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_48_OFFSET);
    addr_hit[250] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_49_OFFSET);
    addr_hit[251] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_50_OFFSET);
    addr_hit[252] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_51_OFFSET);
    addr_hit[253] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_52_OFFSET);
    addr_hit[254] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_53_OFFSET);
    addr_hit[255] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_54_OFFSET);
    addr_hit[256] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_55_OFFSET);
    addr_hit[257] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_56_OFFSET);
    addr_hit[258] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_57_OFFSET);
    addr_hit[259] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_58_OFFSET);
    addr_hit[260] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_59_OFFSET);
    addr_hit[261] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_60_OFFSET);
    addr_hit[262] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_61_OFFSET);
    addr_hit[263] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_62_OFFSET);
    addr_hit[264] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_63_OFFSET);
    addr_hit[265] = (reg_addr == ALERT_HANDLER_ALERT_CAUSE_64_OFFSET);
    addr_hit[266] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_0_OFFSET);
    addr_hit[267] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_1_OFFSET);
    addr_hit[268] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_2_OFFSET);
    addr_hit[269] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_3_OFFSET);
    addr_hit[270] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_4_OFFSET);
    addr_hit[271] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_5_OFFSET);
    addr_hit[272] = (reg_addr == ALERT_HANDLER_LOC_ALERT_REGWEN_6_OFFSET);
    addr_hit[273] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_0_OFFSET);
    addr_hit[274] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_1_OFFSET);
    addr_hit[275] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_2_OFFSET);
    addr_hit[276] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_3_OFFSET);
    addr_hit[277] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_4_OFFSET);
    addr_hit[278] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_5_OFFSET);
    addr_hit[279] = (reg_addr == ALERT_HANDLER_LOC_ALERT_EN_SHADOWED_6_OFFSET);
    addr_hit[280] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_0_OFFSET);
    addr_hit[281] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_1_OFFSET);
    addr_hit[282] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_2_OFFSET);
    addr_hit[283] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_3_OFFSET);
    addr_hit[284] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_4_OFFSET);
    addr_hit[285] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_5_OFFSET);
    addr_hit[286] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CLASS_SHADOWED_6_OFFSET);
    addr_hit[287] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_0_OFFSET);
    addr_hit[288] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_1_OFFSET);
    addr_hit[289] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_2_OFFSET);
    addr_hit[290] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_3_OFFSET);
    addr_hit[291] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_4_OFFSET);
    addr_hit[292] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_5_OFFSET);
    addr_hit[293] = (reg_addr == ALERT_HANDLER_LOC_ALERT_CAUSE_6_OFFSET);
    addr_hit[294] = (reg_addr == ALERT_HANDLER_CLASSA_REGWEN_OFFSET);
    addr_hit[295] = (reg_addr == ALERT_HANDLER_CLASSA_CTRL_SHADOWED_OFFSET);
    addr_hit[296] = (reg_addr == ALERT_HANDLER_CLASSA_CLR_REGWEN_OFFSET);
    addr_hit[297] = (reg_addr == ALERT_HANDLER_CLASSA_CLR_SHADOWED_OFFSET);
    addr_hit[298] = (reg_addr == ALERT_HANDLER_CLASSA_ACCUM_CNT_OFFSET);
    addr_hit[299] = (reg_addr == ALERT_HANDLER_CLASSA_ACCUM_THRESH_SHADOWED_OFFSET);
    addr_hit[300] = (reg_addr == ALERT_HANDLER_CLASSA_TIMEOUT_CYC_SHADOWED_OFFSET);
    addr_hit[301] = (reg_addr == ALERT_HANDLER_CLASSA_CRASHDUMP_TRIGGER_SHADOWED_OFFSET);
    addr_hit[302] = (reg_addr == ALERT_HANDLER_CLASSA_PHASE0_CYC_SHADOWED_OFFSET);
    addr_hit[303] = (reg_addr == ALERT_HANDLER_CLASSA_PHASE1_CYC_SHADOWED_OFFSET);
    addr_hit[304] = (reg_addr == ALERT_HANDLER_CLASSA_PHASE2_CYC_SHADOWED_OFFSET);
    addr_hit[305] = (reg_addr == ALERT_HANDLER_CLASSA_PHASE3_CYC_SHADOWED_OFFSET);
    addr_hit[306] = (reg_addr == ALERT_HANDLER_CLASSA_ESC_CNT_OFFSET);
    addr_hit[307] = (reg_addr == ALERT_HANDLER_CLASSA_STATE_OFFSET);
    addr_hit[308] = (reg_addr == ALERT_HANDLER_CLASSB_REGWEN_OFFSET);
    addr_hit[309] = (reg_addr == ALERT_HANDLER_CLASSB_CTRL_SHADOWED_OFFSET);
    addr_hit[310] = (reg_addr == ALERT_HANDLER_CLASSB_CLR_REGWEN_OFFSET);
    addr_hit[311] = (reg_addr == ALERT_HANDLER_CLASSB_CLR_SHADOWED_OFFSET);
    addr_hit[312] = (reg_addr == ALERT_HANDLER_CLASSB_ACCUM_CNT_OFFSET);
    addr_hit[313] = (reg_addr == ALERT_HANDLER_CLASSB_ACCUM_THRESH_SHADOWED_OFFSET);
    addr_hit[314] = (reg_addr == ALERT_HANDLER_CLASSB_TIMEOUT_CYC_SHADOWED_OFFSET);
    addr_hit[315] = (reg_addr == ALERT_HANDLER_CLASSB_CRASHDUMP_TRIGGER_SHADOWED_OFFSET);
    addr_hit[316] = (reg_addr == ALERT_HANDLER_CLASSB_PHASE0_CYC_SHADOWED_OFFSET);
    addr_hit[317] = (reg_addr == ALERT_HANDLER_CLASSB_PHASE1_CYC_SHADOWED_OFFSET);
    addr_hit[318] = (reg_addr == ALERT_HANDLER_CLASSB_PHASE2_CYC_SHADOWED_OFFSET);
    addr_hit[319] = (reg_addr == ALERT_HANDLER_CLASSB_PHASE3_CYC_SHADOWED_OFFSET);
    addr_hit[320] = (reg_addr == ALERT_HANDLER_CLASSB_ESC_CNT_OFFSET);
    addr_hit[321] = (reg_addr == ALERT_HANDLER_CLASSB_STATE_OFFSET);
    addr_hit[322] = (reg_addr == ALERT_HANDLER_CLASSC_REGWEN_OFFSET);
    addr_hit[323] = (reg_addr == ALERT_HANDLER_CLASSC_CTRL_SHADOWED_OFFSET);
    addr_hit[324] = (reg_addr == ALERT_HANDLER_CLASSC_CLR_REGWEN_OFFSET);
    addr_hit[325] = (reg_addr == ALERT_HANDLER_CLASSC_CLR_SHADOWED_OFFSET);
    addr_hit[326] = (reg_addr == ALERT_HANDLER_CLASSC_ACCUM_CNT_OFFSET);
    addr_hit[327] = (reg_addr == ALERT_HANDLER_CLASSC_ACCUM_THRESH_SHADOWED_OFFSET);
    addr_hit[328] = (reg_addr == ALERT_HANDLER_CLASSC_TIMEOUT_CYC_SHADOWED_OFFSET);
    addr_hit[329] = (reg_addr == ALERT_HANDLER_CLASSC_CRASHDUMP_TRIGGER_SHADOWED_OFFSET);
    addr_hit[330] = (reg_addr == ALERT_HANDLER_CLASSC_PHASE0_CYC_SHADOWED_OFFSET);
    addr_hit[331] = (reg_addr == ALERT_HANDLER_CLASSC_PHASE1_CYC_SHADOWED_OFFSET);
    addr_hit[332] = (reg_addr == ALERT_HANDLER_CLASSC_PHASE2_CYC_SHADOWED_OFFSET);
    addr_hit[333] = (reg_addr == ALERT_HANDLER_CLASSC_PHASE3_CYC_SHADOWED_OFFSET);
    addr_hit[334] = (reg_addr == ALERT_HANDLER_CLASSC_ESC_CNT_OFFSET);
    addr_hit[335] = (reg_addr == ALERT_HANDLER_CLASSC_STATE_OFFSET);
    addr_hit[336] = (reg_addr == ALERT_HANDLER_CLASSD_REGWEN_OFFSET);
    addr_hit[337] = (reg_addr == ALERT_HANDLER_CLASSD_CTRL_SHADOWED_OFFSET);
    addr_hit[338] = (reg_addr == ALERT_HANDLER_CLASSD_CLR_REGWEN_OFFSET);
    addr_hit[339] = (reg_addr == ALERT_HANDLER_CLASSD_CLR_SHADOWED_OFFSET);
    addr_hit[340] = (reg_addr == ALERT_HANDLER_CLASSD_ACCUM_CNT_OFFSET);
    addr_hit[341] = (reg_addr == ALERT_HANDLER_CLASSD_ACCUM_THRESH_SHADOWED_OFFSET);
    addr_hit[342] = (reg_addr == ALERT_HANDLER_CLASSD_TIMEOUT_CYC_SHADOWED_OFFSET);
    addr_hit[343] = (reg_addr == ALERT_HANDLER_CLASSD_CRASHDUMP_TRIGGER_SHADOWED_OFFSET);
    addr_hit[344] = (reg_addr == ALERT_HANDLER_CLASSD_PHASE0_CYC_SHADOWED_OFFSET);
    addr_hit[345] = (reg_addr == ALERT_HANDLER_CLASSD_PHASE1_CYC_SHADOWED_OFFSET);
    addr_hit[346] = (reg_addr == ALERT_HANDLER_CLASSD_PHASE2_CYC_SHADOWED_OFFSET);
    addr_hit[347] = (reg_addr == ALERT_HANDLER_CLASSD_PHASE3_CYC_SHADOWED_OFFSET);
    addr_hit[348] = (reg_addr == ALERT_HANDLER_CLASSD_ESC_CNT_OFFSET);
    addr_hit[349] = (reg_addr == ALERT_HANDLER_CLASSD_STATE_OFFSET);
  end

  assign addrmiss = (reg_re || reg_we) ? ~|addr_hit : 1'b0 ;

  // Check sub-word write is permitted
  always_comb begin
    wr_err = (reg_we &
              ((addr_hit[  0] & (|(ALERT_HANDLER_PERMIT[  0] & ~reg_be))) |
               (addr_hit[  1] & (|(ALERT_HANDLER_PERMIT[  1] & ~reg_be))) |
               (addr_hit[  2] & (|(ALERT_HANDLER_PERMIT[  2] & ~reg_be))) |
               (addr_hit[  3] & (|(ALERT_HANDLER_PERMIT[  3] & ~reg_be))) |
               (addr_hit[  4] & (|(ALERT_HANDLER_PERMIT[  4] & ~reg_be))) |
               (addr_hit[  5] & (|(ALERT_HANDLER_PERMIT[  5] & ~reg_be))) |
               (addr_hit[  6] & (|(ALERT_HANDLER_PERMIT[  6] & ~reg_be))) |
               (addr_hit[  7] & (|(ALERT_HANDLER_PERMIT[  7] & ~reg_be))) |
               (addr_hit[  8] & (|(ALERT_HANDLER_PERMIT[  8] & ~reg_be))) |
               (addr_hit[  9] & (|(ALERT_HANDLER_PERMIT[  9] & ~reg_be))) |
               (addr_hit[ 10] & (|(ALERT_HANDLER_PERMIT[ 10] & ~reg_be))) |
               (addr_hit[ 11] & (|(ALERT_HANDLER_PERMIT[ 11] & ~reg_be))) |
               (addr_hit[ 12] & (|(ALERT_HANDLER_PERMIT[ 12] & ~reg_be))) |
               (addr_hit[ 13] & (|(ALERT_HANDLER_PERMIT[ 13] & ~reg_be))) |
               (addr_hit[ 14] & (|(ALERT_HANDLER_PERMIT[ 14] & ~reg_be))) |
               (addr_hit[ 15] & (|(ALERT_HANDLER_PERMIT[ 15] & ~reg_be))) |
               (addr_hit[ 16] & (|(ALERT_HANDLER_PERMIT[ 16] & ~reg_be))) |
               (addr_hit[ 17] & (|(ALERT_HANDLER_PERMIT[ 17] & ~reg_be))) |
               (addr_hit[ 18] & (|(ALERT_HANDLER_PERMIT[ 18] & ~reg_be))) |
               (addr_hit[ 19] & (|(ALERT_HANDLER_PERMIT[ 19] & ~reg_be))) |
               (addr_hit[ 20] & (|(ALERT_HANDLER_PERMIT[ 20] & ~reg_be))) |
               (addr_hit[ 21] & (|(ALERT_HANDLER_PERMIT[ 21] & ~reg_be))) |
               (addr_hit[ 22] & (|(ALERT_HANDLER_PERMIT[ 22] & ~reg_be))) |
               (addr_hit[ 23] & (|(ALERT_HANDLER_PERMIT[ 23] & ~reg_be))) |
               (addr_hit[ 24] & (|(ALERT_HANDLER_PERMIT[ 24] & ~reg_be))) |
               (addr_hit[ 25] & (|(ALERT_HANDLER_PERMIT[ 25] & ~reg_be))) |
               (addr_hit[ 26] & (|(ALERT_HANDLER_PERMIT[ 26] & ~reg_be))) |
               (addr_hit[ 27] & (|(ALERT_HANDLER_PERMIT[ 27] & ~reg_be))) |
               (addr_hit[ 28] & (|(ALERT_HANDLER_PERMIT[ 28] & ~reg_be))) |
               (addr_hit[ 29] & (|(ALERT_HANDLER_PERMIT[ 29] & ~reg_be))) |
               (addr_hit[ 30] & (|(ALERT_HANDLER_PERMIT[ 30] & ~reg_be))) |
               (addr_hit[ 31] & (|(ALERT_HANDLER_PERMIT[ 31] & ~reg_be))) |
               (addr_hit[ 32] & (|(ALERT_HANDLER_PERMIT[ 32] & ~reg_be))) |
               (addr_hit[ 33] & (|(ALERT_HANDLER_PERMIT[ 33] & ~reg_be))) |
               (addr_hit[ 34] & (|(ALERT_HANDLER_PERMIT[ 34] & ~reg_be))) |
               (addr_hit[ 35] & (|(ALERT_HANDLER_PERMIT[ 35] & ~reg_be))) |
               (addr_hit[ 36] & (|(ALERT_HANDLER_PERMIT[ 36] & ~reg_be))) |
               (addr_hit[ 37] & (|(ALERT_HANDLER_PERMIT[ 37] & ~reg_be))) |
               (addr_hit[ 38] & (|(ALERT_HANDLER_PERMIT[ 38] & ~reg_be))) |
               (addr_hit[ 39] & (|(ALERT_HANDLER_PERMIT[ 39] & ~reg_be))) |
               (addr_hit[ 40] & (|(ALERT_HANDLER_PERMIT[ 40] & ~reg_be))) |
               (addr_hit[ 41] & (|(ALERT_HANDLER_PERMIT[ 41] & ~reg_be))) |
               (addr_hit[ 42] & (|(ALERT_HANDLER_PERMIT[ 42] & ~reg_be))) |
               (addr_hit[ 43] & (|(ALERT_HANDLER_PERMIT[ 43] & ~reg_be))) |
               (addr_hit[ 44] & (|(ALERT_HANDLER_PERMIT[ 44] & ~reg_be))) |
               (addr_hit[ 45] & (|(ALERT_HANDLER_PERMIT[ 45] & ~reg_be))) |
               (addr_hit[ 46] & (|(ALERT_HANDLER_PERMIT[ 46] & ~reg_be))) |
               (addr_hit[ 47] & (|(ALERT_HANDLER_PERMIT[ 47] & ~reg_be))) |
               (addr_hit[ 48] & (|(ALERT_HANDLER_PERMIT[ 48] & ~reg_be))) |
               (addr_hit[ 49] & (|(ALERT_HANDLER_PERMIT[ 49] & ~reg_be))) |
               (addr_hit[ 50] & (|(ALERT_HANDLER_PERMIT[ 50] & ~reg_be))) |
               (addr_hit[ 51] & (|(ALERT_HANDLER_PERMIT[ 51] & ~reg_be))) |
               (addr_hit[ 52] & (|(ALERT_HANDLER_PERMIT[ 52] & ~reg_be))) |
               (addr_hit[ 53] & (|(ALERT_HANDLER_PERMIT[ 53] & ~reg_be))) |
               (addr_hit[ 54] & (|(ALERT_HANDLER_PERMIT[ 54] & ~reg_be))) |
               (addr_hit[ 55] & (|(ALERT_HANDLER_PERMIT[ 55] & ~reg_be))) |
               (addr_hit[ 56] & (|(ALERT_HANDLER_PERMIT[ 56] & ~reg_be))) |
               (addr_hit[ 57] & (|(ALERT_HANDLER_PERMIT[ 57] & ~reg_be))) |
               (addr_hit[ 58] & (|(ALERT_HANDLER_PERMIT[ 58] & ~reg_be))) |
               (addr_hit[ 59] & (|(ALERT_HANDLER_PERMIT[ 59] & ~reg_be))) |
               (addr_hit[ 60] & (|(ALERT_HANDLER_PERMIT[ 60] & ~reg_be))) |
               (addr_hit[ 61] & (|(ALERT_HANDLER_PERMIT[ 61] & ~reg_be))) |
               (addr_hit[ 62] & (|(ALERT_HANDLER_PERMIT[ 62] & ~reg_be))) |
               (addr_hit[ 63] & (|(ALERT_HANDLER_PERMIT[ 63] & ~reg_be))) |
               (addr_hit[ 64] & (|(ALERT_HANDLER_PERMIT[ 64] & ~reg_be))) |
               (addr_hit[ 65] & (|(ALERT_HANDLER_PERMIT[ 65] & ~reg_be))) |
               (addr_hit[ 66] & (|(ALERT_HANDLER_PERMIT[ 66] & ~reg_be))) |
               (addr_hit[ 67] & (|(ALERT_HANDLER_PERMIT[ 67] & ~reg_be))) |
               (addr_hit[ 68] & (|(ALERT_HANDLER_PERMIT[ 68] & ~reg_be))) |
               (addr_hit[ 69] & (|(ALERT_HANDLER_PERMIT[ 69] & ~reg_be))) |
               (addr_hit[ 70] & (|(ALERT_HANDLER_PERMIT[ 70] & ~reg_be))) |
               (addr_hit[ 71] & (|(ALERT_HANDLER_PERMIT[ 71] & ~reg_be))) |
               (addr_hit[ 72] & (|(ALERT_HANDLER_PERMIT[ 72] & ~reg_be))) |
               (addr_hit[ 73] & (|(ALERT_HANDLER_PERMIT[ 73] & ~reg_be))) |
               (addr_hit[ 74] & (|(ALERT_HANDLER_PERMIT[ 74] & ~reg_be))) |
               (addr_hit[ 75] & (|(ALERT_HANDLER_PERMIT[ 75] & ~reg_be))) |
               (addr_hit[ 76] & (|(ALERT_HANDLER_PERMIT[ 76] & ~reg_be))) |
               (addr_hit[ 77] & (|(ALERT_HANDLER_PERMIT[ 77] & ~reg_be))) |
               (addr_hit[ 78] & (|(ALERT_HANDLER_PERMIT[ 78] & ~reg_be))) |
               (addr_hit[ 79] & (|(ALERT_HANDLER_PERMIT[ 79] & ~reg_be))) |
               (addr_hit[ 80] & (|(ALERT_HANDLER_PERMIT[ 80] & ~reg_be))) |
               (addr_hit[ 81] & (|(ALERT_HANDLER_PERMIT[ 81] & ~reg_be))) |
               (addr_hit[ 82] & (|(ALERT_HANDLER_PERMIT[ 82] & ~reg_be))) |
               (addr_hit[ 83] & (|(ALERT_HANDLER_PERMIT[ 83] & ~reg_be))) |
               (addr_hit[ 84] & (|(ALERT_HANDLER_PERMIT[ 84] & ~reg_be))) |
               (addr_hit[ 85] & (|(ALERT_HANDLER_PERMIT[ 85] & ~reg_be))) |
               (addr_hit[ 86] & (|(ALERT_HANDLER_PERMIT[ 86] & ~reg_be))) |
               (addr_hit[ 87] & (|(ALERT_HANDLER_PERMIT[ 87] & ~reg_be))) |
               (addr_hit[ 88] & (|(ALERT_HANDLER_PERMIT[ 88] & ~reg_be))) |
               (addr_hit[ 89] & (|(ALERT_HANDLER_PERMIT[ 89] & ~reg_be))) |
               (addr_hit[ 90] & (|(ALERT_HANDLER_PERMIT[ 90] & ~reg_be))) |
               (addr_hit[ 91] & (|(ALERT_HANDLER_PERMIT[ 91] & ~reg_be))) |
               (addr_hit[ 92] & (|(ALERT_HANDLER_PERMIT[ 92] & ~reg_be))) |
               (addr_hit[ 93] & (|(ALERT_HANDLER_PERMIT[ 93] & ~reg_be))) |
               (addr_hit[ 94] & (|(ALERT_HANDLER_PERMIT[ 94] & ~reg_be))) |
               (addr_hit[ 95] & (|(ALERT_HANDLER_PERMIT[ 95] & ~reg_be))) |
               (addr_hit[ 96] & (|(ALERT_HANDLER_PERMIT[ 96] & ~reg_be))) |
               (addr_hit[ 97] & (|(ALERT_HANDLER_PERMIT[ 97] & ~reg_be))) |
               (addr_hit[ 98] & (|(ALERT_HANDLER_PERMIT[ 98] & ~reg_be))) |
               (addr_hit[ 99] & (|(ALERT_HANDLER_PERMIT[ 99] & ~reg_be))) |
               (addr_hit[100] & (|(ALERT_HANDLER_PERMIT[100] & ~reg_be))) |
               (addr_hit[101] & (|(ALERT_HANDLER_PERMIT[101] & ~reg_be))) |
               (addr_hit[102] & (|(ALERT_HANDLER_PERMIT[102] & ~reg_be))) |
               (addr_hit[103] & (|(ALERT_HANDLER_PERMIT[103] & ~reg_be))) |
               (addr_hit[104] & (|(ALERT_HANDLER_PERMIT[104] & ~reg_be))) |
               (addr_hit[105] & (|(ALERT_HANDLER_PERMIT[105] & ~reg_be))) |
               (addr_hit[106] & (|(ALERT_HANDLER_PERMIT[106] & ~reg_be))) |
               (addr_hit[107] & (|(ALERT_HANDLER_PERMIT[107] & ~reg_be))) |
               (addr_hit[108] & (|(ALERT_HANDLER_PERMIT[108] & ~reg_be))) |
               (addr_hit[109] & (|(ALERT_HANDLER_PERMIT[109] & ~reg_be))) |
               (addr_hit[110] & (|(ALERT_HANDLER_PERMIT[110] & ~reg_be))) |
               (addr_hit[111] & (|(ALERT_HANDLER_PERMIT[111] & ~reg_be))) |
               (addr_hit[112] & (|(ALERT_HANDLER_PERMIT[112] & ~reg_be))) |
               (addr_hit[113] & (|(ALERT_HANDLER_PERMIT[113] & ~reg_be))) |
               (addr_hit[114] & (|(ALERT_HANDLER_PERMIT[114] & ~reg_be))) |
               (addr_hit[115] & (|(ALERT_HANDLER_PERMIT[115] & ~reg_be))) |
               (addr_hit[116] & (|(ALERT_HANDLER_PERMIT[116] & ~reg_be))) |
               (addr_hit[117] & (|(ALERT_HANDLER_PERMIT[117] & ~reg_be))) |
               (addr_hit[118] & (|(ALERT_HANDLER_PERMIT[118] & ~reg_be))) |
               (addr_hit[119] & (|(ALERT_HANDLER_PERMIT[119] & ~reg_be))) |
               (addr_hit[120] & (|(ALERT_HANDLER_PERMIT[120] & ~reg_be))) |
               (addr_hit[121] & (|(ALERT_HANDLER_PERMIT[121] & ~reg_be))) |
               (addr_hit[122] & (|(ALERT_HANDLER_PERMIT[122] & ~reg_be))) |
               (addr_hit[123] & (|(ALERT_HANDLER_PERMIT[123] & ~reg_be))) |
               (addr_hit[124] & (|(ALERT_HANDLER_PERMIT[124] & ~reg_be))) |
               (addr_hit[125] & (|(ALERT_HANDLER_PERMIT[125] & ~reg_be))) |
               (addr_hit[126] & (|(ALERT_HANDLER_PERMIT[126] & ~reg_be))) |
               (addr_hit[127] & (|(ALERT_HANDLER_PERMIT[127] & ~reg_be))) |
               (addr_hit[128] & (|(ALERT_HANDLER_PERMIT[128] & ~reg_be))) |
               (addr_hit[129] & (|(ALERT_HANDLER_PERMIT[129] & ~reg_be))) |
               (addr_hit[130] & (|(ALERT_HANDLER_PERMIT[130] & ~reg_be))) |
               (addr_hit[131] & (|(ALERT_HANDLER_PERMIT[131] & ~reg_be))) |
               (addr_hit[132] & (|(ALERT_HANDLER_PERMIT[132] & ~reg_be))) |
               (addr_hit[133] & (|(ALERT_HANDLER_PERMIT[133] & ~reg_be))) |
               (addr_hit[134] & (|(ALERT_HANDLER_PERMIT[134] & ~reg_be))) |
               (addr_hit[135] & (|(ALERT_HANDLER_PERMIT[135] & ~reg_be))) |
               (addr_hit[136] & (|(ALERT_HANDLER_PERMIT[136] & ~reg_be))) |
               (addr_hit[137] & (|(ALERT_HANDLER_PERMIT[137] & ~reg_be))) |
               (addr_hit[138] & (|(ALERT_HANDLER_PERMIT[138] & ~reg_be))) |
               (addr_hit[139] & (|(ALERT_HANDLER_PERMIT[139] & ~reg_be))) |
               (addr_hit[140] & (|(ALERT_HANDLER_PERMIT[140] & ~reg_be))) |
               (addr_hit[141] & (|(ALERT_HANDLER_PERMIT[141] & ~reg_be))) |
               (addr_hit[142] & (|(ALERT_HANDLER_PERMIT[142] & ~reg_be))) |
               (addr_hit[143] & (|(ALERT_HANDLER_PERMIT[143] & ~reg_be))) |
               (addr_hit[144] & (|(ALERT_HANDLER_PERMIT[144] & ~reg_be))) |
               (addr_hit[145] & (|(ALERT_HANDLER_PERMIT[145] & ~reg_be))) |
               (addr_hit[146] & (|(ALERT_HANDLER_PERMIT[146] & ~reg_be))) |
               (addr_hit[147] & (|(ALERT_HANDLER_PERMIT[147] & ~reg_be))) |
               (addr_hit[148] & (|(ALERT_HANDLER_PERMIT[148] & ~reg_be))) |
               (addr_hit[149] & (|(ALERT_HANDLER_PERMIT[149] & ~reg_be))) |
               (addr_hit[150] & (|(ALERT_HANDLER_PERMIT[150] & ~reg_be))) |
               (addr_hit[151] & (|(ALERT_HANDLER_PERMIT[151] & ~reg_be))) |
               (addr_hit[152] & (|(ALERT_HANDLER_PERMIT[152] & ~reg_be))) |
               (addr_hit[153] & (|(ALERT_HANDLER_PERMIT[153] & ~reg_be))) |
               (addr_hit[154] & (|(ALERT_HANDLER_PERMIT[154] & ~reg_be))) |
               (addr_hit[155] & (|(ALERT_HANDLER_PERMIT[155] & ~reg_be))) |
               (addr_hit[156] & (|(ALERT_HANDLER_PERMIT[156] & ~reg_be))) |
               (addr_hit[157] & (|(ALERT_HANDLER_PERMIT[157] & ~reg_be))) |
               (addr_hit[158] & (|(ALERT_HANDLER_PERMIT[158] & ~reg_be))) |
               (addr_hit[159] & (|(ALERT_HANDLER_PERMIT[159] & ~reg_be))) |
               (addr_hit[160] & (|(ALERT_HANDLER_PERMIT[160] & ~reg_be))) |
               (addr_hit[161] & (|(ALERT_HANDLER_PERMIT[161] & ~reg_be))) |
               (addr_hit[162] & (|(ALERT_HANDLER_PERMIT[162] & ~reg_be))) |
               (addr_hit[163] & (|(ALERT_HANDLER_PERMIT[163] & ~reg_be))) |
               (addr_hit[164] & (|(ALERT_HANDLER_PERMIT[164] & ~reg_be))) |
               (addr_hit[165] & (|(ALERT_HANDLER_PERMIT[165] & ~reg_be))) |
               (addr_hit[166] & (|(ALERT_HANDLER_PERMIT[166] & ~reg_be))) |
               (addr_hit[167] & (|(ALERT_HANDLER_PERMIT[167] & ~reg_be))) |
               (addr_hit[168] & (|(ALERT_HANDLER_PERMIT[168] & ~reg_be))) |
               (addr_hit[169] & (|(ALERT_HANDLER_PERMIT[169] & ~reg_be))) |
               (addr_hit[170] & (|(ALERT_HANDLER_PERMIT[170] & ~reg_be))) |
               (addr_hit[171] & (|(ALERT_HANDLER_PERMIT[171] & ~reg_be))) |
               (addr_hit[172] & (|(ALERT_HANDLER_PERMIT[172] & ~reg_be))) |
               (addr_hit[173] & (|(ALERT_HANDLER_PERMIT[173] & ~reg_be))) |
               (addr_hit[174] & (|(ALERT_HANDLER_PERMIT[174] & ~reg_be))) |
               (addr_hit[175] & (|(ALERT_HANDLER_PERMIT[175] & ~reg_be))) |
               (addr_hit[176] & (|(ALERT_HANDLER_PERMIT[176] & ~reg_be))) |
               (addr_hit[177] & (|(ALERT_HANDLER_PERMIT[177] & ~reg_be))) |
               (addr_hit[178] & (|(ALERT_HANDLER_PERMIT[178] & ~reg_be))) |
               (addr_hit[179] & (|(ALERT_HANDLER_PERMIT[179] & ~reg_be))) |
               (addr_hit[180] & (|(ALERT_HANDLER_PERMIT[180] & ~reg_be))) |
               (addr_hit[181] & (|(ALERT_HANDLER_PERMIT[181] & ~reg_be))) |
               (addr_hit[182] & (|(ALERT_HANDLER_PERMIT[182] & ~reg_be))) |
               (addr_hit[183] & (|(ALERT_HANDLER_PERMIT[183] & ~reg_be))) |
               (addr_hit[184] & (|(ALERT_HANDLER_PERMIT[184] & ~reg_be))) |
               (addr_hit[185] & (|(ALERT_HANDLER_PERMIT[185] & ~reg_be))) |
               (addr_hit[186] & (|(ALERT_HANDLER_PERMIT[186] & ~reg_be))) |
               (addr_hit[187] & (|(ALERT_HANDLER_PERMIT[187] & ~reg_be))) |
               (addr_hit[188] & (|(ALERT_HANDLER_PERMIT[188] & ~reg_be))) |
               (addr_hit[189] & (|(ALERT_HANDLER_PERMIT[189] & ~reg_be))) |
               (addr_hit[190] & (|(ALERT_HANDLER_PERMIT[190] & ~reg_be))) |
               (addr_hit[191] & (|(ALERT_HANDLER_PERMIT[191] & ~reg_be))) |
               (addr_hit[192] & (|(ALERT_HANDLER_PERMIT[192] & ~reg_be))) |
               (addr_hit[193] & (|(ALERT_HANDLER_PERMIT[193] & ~reg_be))) |
               (addr_hit[194] & (|(ALERT_HANDLER_PERMIT[194] & ~reg_be))) |
               (addr_hit[195] & (|(ALERT_HANDLER_PERMIT[195] & ~reg_be))) |
               (addr_hit[196] & (|(ALERT_HANDLER_PERMIT[196] & ~reg_be))) |
               (addr_hit[197] & (|(ALERT_HANDLER_PERMIT[197] & ~reg_be))) |
               (addr_hit[198] & (|(ALERT_HANDLER_PERMIT[198] & ~reg_be))) |
               (addr_hit[199] & (|(ALERT_HANDLER_PERMIT[199] & ~reg_be))) |
               (addr_hit[200] & (|(ALERT_HANDLER_PERMIT[200] & ~reg_be))) |
               (addr_hit[201] & (|(ALERT_HANDLER_PERMIT[201] & ~reg_be))) |
               (addr_hit[202] & (|(ALERT_HANDLER_PERMIT[202] & ~reg_be))) |
               (addr_hit[203] & (|(ALERT_HANDLER_PERMIT[203] & ~reg_be))) |
               (addr_hit[204] & (|(ALERT_HANDLER_PERMIT[204] & ~reg_be))) |
               (addr_hit[205] & (|(ALERT_HANDLER_PERMIT[205] & ~reg_be))) |
               (addr_hit[206] & (|(ALERT_HANDLER_PERMIT[206] & ~reg_be))) |
               (addr_hit[207] & (|(ALERT_HANDLER_PERMIT[207] & ~reg_be))) |
               (addr_hit[208] & (|(ALERT_HANDLER_PERMIT[208] & ~reg_be))) |
               (addr_hit[209] & (|(ALERT_HANDLER_PERMIT[209] & ~reg_be))) |
               (addr_hit[210] & (|(ALERT_HANDLER_PERMIT[210] & ~reg_be))) |
               (addr_hit[211] & (|(ALERT_HANDLER_PERMIT[211] & ~reg_be))) |
               (addr_hit[212] & (|(ALERT_HANDLER_PERMIT[212] & ~reg_be))) |
               (addr_hit[213] & (|(ALERT_HANDLER_PERMIT[213] & ~reg_be))) |
               (addr_hit[214] & (|(ALERT_HANDLER_PERMIT[214] & ~reg_be))) |
               (addr_hit[215] & (|(ALERT_HANDLER_PERMIT[215] & ~reg_be))) |
               (addr_hit[216] & (|(ALERT_HANDLER_PERMIT[216] & ~reg_be))) |
               (addr_hit[217] & (|(ALERT_HANDLER_PERMIT[217] & ~reg_be))) |
               (addr_hit[218] & (|(ALERT_HANDLER_PERMIT[218] & ~reg_be))) |
               (addr_hit[219] & (|(ALERT_HANDLER_PERMIT[219] & ~reg_be))) |
               (addr_hit[220] & (|(ALERT_HANDLER_PERMIT[220] & ~reg_be))) |
               (addr_hit[221] & (|(ALERT_HANDLER_PERMIT[221] & ~reg_be))) |
               (addr_hit[222] & (|(ALERT_HANDLER_PERMIT[222] & ~reg_be))) |
               (addr_hit[223] & (|(ALERT_HANDLER_PERMIT[223] & ~reg_be))) |
               (addr_hit[224] & (|(ALERT_HANDLER_PERMIT[224] & ~reg_be))) |
               (addr_hit[225] & (|(ALERT_HANDLER_PERMIT[225] & ~reg_be))) |
               (addr_hit[226] & (|(ALERT_HANDLER_PERMIT[226] & ~reg_be))) |
               (addr_hit[227] & (|(ALERT_HANDLER_PERMIT[227] & ~reg_be))) |
               (addr_hit[228] & (|(ALERT_HANDLER_PERMIT[228] & ~reg_be))) |
               (addr_hit[229] & (|(ALERT_HANDLER_PERMIT[229] & ~reg_be))) |
               (addr_hit[230] & (|(ALERT_HANDLER_PERMIT[230] & ~reg_be))) |
               (addr_hit[231] & (|(ALERT_HANDLER_PERMIT[231] & ~reg_be))) |
               (addr_hit[232] & (|(ALERT_HANDLER_PERMIT[232] & ~reg_be))) |
               (addr_hit[233] & (|(ALERT_HANDLER_PERMIT[233] & ~reg_be))) |
               (addr_hit[234] & (|(ALERT_HANDLER_PERMIT[234] & ~reg_be))) |
               (addr_hit[235] & (|(ALERT_HANDLER_PERMIT[235] & ~reg_be))) |
               (addr_hit[236] & (|(ALERT_HANDLER_PERMIT[236] & ~reg_be))) |
               (addr_hit[237] & (|(ALERT_HANDLER_PERMIT[237] & ~reg_be))) |
               (addr_hit[238] & (|(ALERT_HANDLER_PERMIT[238] & ~reg_be))) |
               (addr_hit[239] & (|(ALERT_HANDLER_PERMIT[239] & ~reg_be))) |
               (addr_hit[240] & (|(ALERT_HANDLER_PERMIT[240] & ~reg_be))) |
               (addr_hit[241] & (|(ALERT_HANDLER_PERMIT[241] & ~reg_be))) |
               (addr_hit[242] & (|(ALERT_HANDLER_PERMIT[242] & ~reg_be))) |
               (addr_hit[243] & (|(ALERT_HANDLER_PERMIT[243] & ~reg_be))) |
               (addr_hit[244] & (|(ALERT_HANDLER_PERMIT[244] & ~reg_be))) |
               (addr_hit[245] & (|(ALERT_HANDLER_PERMIT[245] & ~reg_be))) |
               (addr_hit[246] & (|(ALERT_HANDLER_PERMIT[246] & ~reg_be))) |
               (addr_hit[247] & (|(ALERT_HANDLER_PERMIT[247] & ~reg_be))) |
               (addr_hit[248] & (|(ALERT_HANDLER_PERMIT[248] & ~reg_be))) |
               (addr_hit[249] & (|(ALERT_HANDLER_PERMIT[249] & ~reg_be))) |
               (addr_hit[250] & (|(ALERT_HANDLER_PERMIT[250] & ~reg_be))) |
               (addr_hit[251] & (|(ALERT_HANDLER_PERMIT[251] & ~reg_be))) |
               (addr_hit[252] & (|(ALERT_HANDLER_PERMIT[252] & ~reg_be))) |
               (addr_hit[253] & (|(ALERT_HANDLER_PERMIT[253] & ~reg_be))) |
               (addr_hit[254] & (|(ALERT_HANDLER_PERMIT[254] & ~reg_be))) |
               (addr_hit[255] & (|(ALERT_HANDLER_PERMIT[255] & ~reg_be))) |
               (addr_hit[256] & (|(ALERT_HANDLER_PERMIT[256] & ~reg_be))) |
               (addr_hit[257] & (|(ALERT_HANDLER_PERMIT[257] & ~reg_be))) |
               (addr_hit[258] & (|(ALERT_HANDLER_PERMIT[258] & ~reg_be))) |
               (addr_hit[259] & (|(ALERT_HANDLER_PERMIT[259] & ~reg_be))) |
               (addr_hit[260] & (|(ALERT_HANDLER_PERMIT[260] & ~reg_be))) |
               (addr_hit[261] & (|(ALERT_HANDLER_PERMIT[261] & ~reg_be))) |
               (addr_hit[262] & (|(ALERT_HANDLER_PERMIT[262] & ~reg_be))) |
               (addr_hit[263] & (|(ALERT_HANDLER_PERMIT[263] & ~reg_be))) |
               (addr_hit[264] & (|(ALERT_HANDLER_PERMIT[264] & ~reg_be))) |
               (addr_hit[265] & (|(ALERT_HANDLER_PERMIT[265] & ~reg_be))) |
               (addr_hit[266] & (|(ALERT_HANDLER_PERMIT[266] & ~reg_be))) |
               (addr_hit[267] & (|(ALERT_HANDLER_PERMIT[267] & ~reg_be))) |
               (addr_hit[268] & (|(ALERT_HANDLER_PERMIT[268] & ~reg_be))) |
               (addr_hit[269] & (|(ALERT_HANDLER_PERMIT[269] & ~reg_be))) |
               (addr_hit[270] & (|(ALERT_HANDLER_PERMIT[270] & ~reg_be))) |
               (addr_hit[271] & (|(ALERT_HANDLER_PERMIT[271] & ~reg_be))) |
               (addr_hit[272] & (|(ALERT_HANDLER_PERMIT[272] & ~reg_be))) |
               (addr_hit[273] & (|(ALERT_HANDLER_PERMIT[273] & ~reg_be))) |
               (addr_hit[274] & (|(ALERT_HANDLER_PERMIT[274] & ~reg_be))) |
               (addr_hit[275] & (|(ALERT_HANDLER_PERMIT[275] & ~reg_be))) |
               (addr_hit[276] & (|(ALERT_HANDLER_PERMIT[276] & ~reg_be))) |
               (addr_hit[277] & (|(ALERT_HANDLER_PERMIT[277] & ~reg_be))) |
               (addr_hit[278] & (|(ALERT_HANDLER_PERMIT[278] & ~reg_be))) |
               (addr_hit[279] & (|(ALERT_HANDLER_PERMIT[279] & ~reg_be))) |
               (addr_hit[280] & (|(ALERT_HANDLER_PERMIT[280] & ~reg_be))) |
               (addr_hit[281] & (|(ALERT_HANDLER_PERMIT[281] & ~reg_be))) |
               (addr_hit[282] & (|(ALERT_HANDLER_PERMIT[282] & ~reg_be))) |
               (addr_hit[283] & (|(ALERT_HANDLER_PERMIT[283] & ~reg_be))) |
               (addr_hit[284] & (|(ALERT_HANDLER_PERMIT[284] & ~reg_be))) |
               (addr_hit[285] & (|(ALERT_HANDLER_PERMIT[285] & ~reg_be))) |
               (addr_hit[286] & (|(ALERT_HANDLER_PERMIT[286] & ~reg_be))) |
               (addr_hit[287] & (|(ALERT_HANDLER_PERMIT[287] & ~reg_be))) |
               (addr_hit[288] & (|(ALERT_HANDLER_PERMIT[288] & ~reg_be))) |
               (addr_hit[289] & (|(ALERT_HANDLER_PERMIT[289] & ~reg_be))) |
               (addr_hit[290] & (|(ALERT_HANDLER_PERMIT[290] & ~reg_be))) |
               (addr_hit[291] & (|(ALERT_HANDLER_PERMIT[291] & ~reg_be))) |
               (addr_hit[292] & (|(ALERT_HANDLER_PERMIT[292] & ~reg_be))) |
               (addr_hit[293] & (|(ALERT_HANDLER_PERMIT[293] & ~reg_be))) |
               (addr_hit[294] & (|(ALERT_HANDLER_PERMIT[294] & ~reg_be))) |
               (addr_hit[295] & (|(ALERT_HANDLER_PERMIT[295] & ~reg_be))) |
               (addr_hit[296] & (|(ALERT_HANDLER_PERMIT[296] & ~reg_be))) |
               (addr_hit[297] & (|(ALERT_HANDLER_PERMIT[297] & ~reg_be))) |
               (addr_hit[298] & (|(ALERT_HANDLER_PERMIT[298] & ~reg_be))) |
               (addr_hit[299] & (|(ALERT_HANDLER_PERMIT[299] & ~reg_be))) |
               (addr_hit[300] & (|(ALERT_HANDLER_PERMIT[300] & ~reg_be))) |
               (addr_hit[301] & (|(ALERT_HANDLER_PERMIT[301] & ~reg_be))) |
               (addr_hit[302] & (|(ALERT_HANDLER_PERMIT[302] & ~reg_be))) |
               (addr_hit[303] & (|(ALERT_HANDLER_PERMIT[303] & ~reg_be))) |
               (addr_hit[304] & (|(ALERT_HANDLER_PERMIT[304] & ~reg_be))) |
               (addr_hit[305] & (|(ALERT_HANDLER_PERMIT[305] & ~reg_be))) |
               (addr_hit[306] & (|(ALERT_HANDLER_PERMIT[306] & ~reg_be))) |
               (addr_hit[307] & (|(ALERT_HANDLER_PERMIT[307] & ~reg_be))) |
               (addr_hit[308] & (|(ALERT_HANDLER_PERMIT[308] & ~reg_be))) |
               (addr_hit[309] & (|(ALERT_HANDLER_PERMIT[309] & ~reg_be))) |
               (addr_hit[310] & (|(ALERT_HANDLER_PERMIT[310] & ~reg_be))) |
               (addr_hit[311] & (|(ALERT_HANDLER_PERMIT[311] & ~reg_be))) |
               (addr_hit[312] & (|(ALERT_HANDLER_PERMIT[312] & ~reg_be))) |
               (addr_hit[313] & (|(ALERT_HANDLER_PERMIT[313] & ~reg_be))) |
               (addr_hit[314] & (|(ALERT_HANDLER_PERMIT[314] & ~reg_be))) |
               (addr_hit[315] & (|(ALERT_HANDLER_PERMIT[315] & ~reg_be))) |
               (addr_hit[316] & (|(ALERT_HANDLER_PERMIT[316] & ~reg_be))) |
               (addr_hit[317] & (|(ALERT_HANDLER_PERMIT[317] & ~reg_be))) |
               (addr_hit[318] & (|(ALERT_HANDLER_PERMIT[318] & ~reg_be))) |
               (addr_hit[319] & (|(ALERT_HANDLER_PERMIT[319] & ~reg_be))) |
               (addr_hit[320] & (|(ALERT_HANDLER_PERMIT[320] & ~reg_be))) |
               (addr_hit[321] & (|(ALERT_HANDLER_PERMIT[321] & ~reg_be))) |
               (addr_hit[322] & (|(ALERT_HANDLER_PERMIT[322] & ~reg_be))) |
               (addr_hit[323] & (|(ALERT_HANDLER_PERMIT[323] & ~reg_be))) |
               (addr_hit[324] & (|(ALERT_HANDLER_PERMIT[324] & ~reg_be))) |
               (addr_hit[325] & (|(ALERT_HANDLER_PERMIT[325] & ~reg_be))) |
               (addr_hit[326] & (|(ALERT_HANDLER_PERMIT[326] & ~reg_be))) |
               (addr_hit[327] & (|(ALERT_HANDLER_PERMIT[327] & ~reg_be))) |
               (addr_hit[328] & (|(ALERT_HANDLER_PERMIT[328] & ~reg_be))) |
               (addr_hit[329] & (|(ALERT_HANDLER_PERMIT[329] & ~reg_be))) |
               (addr_hit[330] & (|(ALERT_HANDLER_PERMIT[330] & ~reg_be))) |
               (addr_hit[331] & (|(ALERT_HANDLER_PERMIT[331] & ~reg_be))) |
               (addr_hit[332] & (|(ALERT_HANDLER_PERMIT[332] & ~reg_be))) |
               (addr_hit[333] & (|(ALERT_HANDLER_PERMIT[333] & ~reg_be))) |
               (addr_hit[334] & (|(ALERT_HANDLER_PERMIT[334] & ~reg_be))) |
               (addr_hit[335] & (|(ALERT_HANDLER_PERMIT[335] & ~reg_be))) |
               (addr_hit[336] & (|(ALERT_HANDLER_PERMIT[336] & ~reg_be))) |
               (addr_hit[337] & (|(ALERT_HANDLER_PERMIT[337] & ~reg_be))) |
               (addr_hit[338] & (|(ALERT_HANDLER_PERMIT[338] & ~reg_be))) |
               (addr_hit[339] & (|(ALERT_HANDLER_PERMIT[339] & ~reg_be))) |
               (addr_hit[340] & (|(ALERT_HANDLER_PERMIT[340] & ~reg_be))) |
               (addr_hit[341] & (|(ALERT_HANDLER_PERMIT[341] & ~reg_be))) |
               (addr_hit[342] & (|(ALERT_HANDLER_PERMIT[342] & ~reg_be))) |
               (addr_hit[343] & (|(ALERT_HANDLER_PERMIT[343] & ~reg_be))) |
               (addr_hit[344] & (|(ALERT_HANDLER_PERMIT[344] & ~reg_be))) |
               (addr_hit[345] & (|(ALERT_HANDLER_PERMIT[345] & ~reg_be))) |
               (addr_hit[346] & (|(ALERT_HANDLER_PERMIT[346] & ~reg_be))) |
               (addr_hit[347] & (|(ALERT_HANDLER_PERMIT[347] & ~reg_be))) |
               (addr_hit[348] & (|(ALERT_HANDLER_PERMIT[348] & ~reg_be))) |
               (addr_hit[349] & (|(ALERT_HANDLER_PERMIT[349] & ~reg_be)))));
  end

  // Generate write-enables
  assign intr_state_we = addr_hit[0] & reg_we & !reg_error;

  assign intr_state_classa_wd = reg_wdata[0];

  assign intr_state_classb_wd = reg_wdata[1];

  assign intr_state_classc_wd = reg_wdata[2];

  assign intr_state_classd_wd = reg_wdata[3];
  assign intr_enable_we = addr_hit[1] & reg_we & !reg_error;

  assign intr_enable_classa_wd = reg_wdata[0];

  assign intr_enable_classb_wd = reg_wdata[1];

  assign intr_enable_classc_wd = reg_wdata[2];

  assign intr_enable_classd_wd = reg_wdata[3];
  assign intr_test_we = addr_hit[2] & reg_we & !reg_error;

  assign intr_test_classa_wd = reg_wdata[0];

  assign intr_test_classb_wd = reg_wdata[1];

  assign intr_test_classc_wd = reg_wdata[2];

  assign intr_test_classd_wd = reg_wdata[3];
  assign ping_timer_regwen_we = addr_hit[3] & reg_we & !reg_error;

  assign ping_timer_regwen_wd = reg_wdata[0];
  assign ping_timeout_cyc_shadowed_re = addr_hit[4] & reg_re & !reg_error;
  assign ping_timeout_cyc_shadowed_we = addr_hit[4] & reg_we & !reg_error;

  assign ping_timeout_cyc_shadowed_wd = reg_wdata[15:0];
  assign ping_timer_en_shadowed_re = addr_hit[5] & reg_re & !reg_error;
  assign ping_timer_en_shadowed_we = addr_hit[5] & reg_we & !reg_error;

  assign ping_timer_en_shadowed_wd = reg_wdata[0];
  assign alert_regwen_0_we = addr_hit[6] & reg_we & !reg_error;

  assign alert_regwen_0_wd = reg_wdata[0];
  assign alert_regwen_1_we = addr_hit[7] & reg_we & !reg_error;

  assign alert_regwen_1_wd = reg_wdata[0];
  assign alert_regwen_2_we = addr_hit[8] & reg_we & !reg_error;

  assign alert_regwen_2_wd = reg_wdata[0];
  assign alert_regwen_3_we = addr_hit[9] & reg_we & !reg_error;

  assign alert_regwen_3_wd = reg_wdata[0];
  assign alert_regwen_4_we = addr_hit[10] & reg_we & !reg_error;

  assign alert_regwen_4_wd = reg_wdata[0];
  assign alert_regwen_5_we = addr_hit[11] & reg_we & !reg_error;

  assign alert_regwen_5_wd = reg_wdata[0];
  assign alert_regwen_6_we = addr_hit[12] & reg_we & !reg_error;

  assign alert_regwen_6_wd = reg_wdata[0];
  assign alert_regwen_7_we = addr_hit[13] & reg_we & !reg_error;

  assign alert_regwen_7_wd = reg_wdata[0];
  assign alert_regwen_8_we = addr_hit[14] & reg_we & !reg_error;

  assign alert_regwen_8_wd = reg_wdata[0];
  assign alert_regwen_9_we = addr_hit[15] & reg_we & !reg_error;

  assign alert_regwen_9_wd = reg_wdata[0];
  assign alert_regwen_10_we = addr_hit[16] & reg_we & !reg_error;

  assign alert_regwen_10_wd = reg_wdata[0];
  assign alert_regwen_11_we = addr_hit[17] & reg_we & !reg_error;

  assign alert_regwen_11_wd = reg_wdata[0];
  assign alert_regwen_12_we = addr_hit[18] & reg_we & !reg_error;

  assign alert_regwen_12_wd = reg_wdata[0];
  assign alert_regwen_13_we = addr_hit[19] & reg_we & !reg_error;

  assign alert_regwen_13_wd = reg_wdata[0];
  assign alert_regwen_14_we = addr_hit[20] & reg_we & !reg_error;

  assign alert_regwen_14_wd = reg_wdata[0];
  assign alert_regwen_15_we = addr_hit[21] & reg_we & !reg_error;

  assign alert_regwen_15_wd = reg_wdata[0];
  assign alert_regwen_16_we = addr_hit[22] & reg_we & !reg_error;

  assign alert_regwen_16_wd = reg_wdata[0];
  assign alert_regwen_17_we = addr_hit[23] & reg_we & !reg_error;

  assign alert_regwen_17_wd = reg_wdata[0];
  assign alert_regwen_18_we = addr_hit[24] & reg_we & !reg_error;

  assign alert_regwen_18_wd = reg_wdata[0];
  assign alert_regwen_19_we = addr_hit[25] & reg_we & !reg_error;

  assign alert_regwen_19_wd = reg_wdata[0];
  assign alert_regwen_20_we = addr_hit[26] & reg_we & !reg_error;

  assign alert_regwen_20_wd = reg_wdata[0];
  assign alert_regwen_21_we = addr_hit[27] & reg_we & !reg_error;

  assign alert_regwen_21_wd = reg_wdata[0];
  assign alert_regwen_22_we = addr_hit[28] & reg_we & !reg_error;

  assign alert_regwen_22_wd = reg_wdata[0];
  assign alert_regwen_23_we = addr_hit[29] & reg_we & !reg_error;

  assign alert_regwen_23_wd = reg_wdata[0];
  assign alert_regwen_24_we = addr_hit[30] & reg_we & !reg_error;

  assign alert_regwen_24_wd = reg_wdata[0];
  assign alert_regwen_25_we = addr_hit[31] & reg_we & !reg_error;

  assign alert_regwen_25_wd = reg_wdata[0];
  assign alert_regwen_26_we = addr_hit[32] & reg_we & !reg_error;

  assign alert_regwen_26_wd = reg_wdata[0];
  assign alert_regwen_27_we = addr_hit[33] & reg_we & !reg_error;

  assign alert_regwen_27_wd = reg_wdata[0];
  assign alert_regwen_28_we = addr_hit[34] & reg_we & !reg_error;

  assign alert_regwen_28_wd = reg_wdata[0];
  assign alert_regwen_29_we = addr_hit[35] & reg_we & !reg_error;

  assign alert_regwen_29_wd = reg_wdata[0];
  assign alert_regwen_30_we = addr_hit[36] & reg_we & !reg_error;

  assign alert_regwen_30_wd = reg_wdata[0];
  assign alert_regwen_31_we = addr_hit[37] & reg_we & !reg_error;

  assign alert_regwen_31_wd = reg_wdata[0];
  assign alert_regwen_32_we = addr_hit[38] & reg_we & !reg_error;

  assign alert_regwen_32_wd = reg_wdata[0];
  assign alert_regwen_33_we = addr_hit[39] & reg_we & !reg_error;

  assign alert_regwen_33_wd = reg_wdata[0];
  assign alert_regwen_34_we = addr_hit[40] & reg_we & !reg_error;

  assign alert_regwen_34_wd = reg_wdata[0];
  assign alert_regwen_35_we = addr_hit[41] & reg_we & !reg_error;

  assign alert_regwen_35_wd = reg_wdata[0];
  assign alert_regwen_36_we = addr_hit[42] & reg_we & !reg_error;

  assign alert_regwen_36_wd = reg_wdata[0];
  assign alert_regwen_37_we = addr_hit[43] & reg_we & !reg_error;

  assign alert_regwen_37_wd = reg_wdata[0];
  assign alert_regwen_38_we = addr_hit[44] & reg_we & !reg_error;

  assign alert_regwen_38_wd = reg_wdata[0];
  assign alert_regwen_39_we = addr_hit[45] & reg_we & !reg_error;

  assign alert_regwen_39_wd = reg_wdata[0];
  assign alert_regwen_40_we = addr_hit[46] & reg_we & !reg_error;

  assign alert_regwen_40_wd = reg_wdata[0];
  assign alert_regwen_41_we = addr_hit[47] & reg_we & !reg_error;

  assign alert_regwen_41_wd = reg_wdata[0];
  assign alert_regwen_42_we = addr_hit[48] & reg_we & !reg_error;

  assign alert_regwen_42_wd = reg_wdata[0];
  assign alert_regwen_43_we = addr_hit[49] & reg_we & !reg_error;

  assign alert_regwen_43_wd = reg_wdata[0];
  assign alert_regwen_44_we = addr_hit[50] & reg_we & !reg_error;

  assign alert_regwen_44_wd = reg_wdata[0];
  assign alert_regwen_45_we = addr_hit[51] & reg_we & !reg_error;

  assign alert_regwen_45_wd = reg_wdata[0];
  assign alert_regwen_46_we = addr_hit[52] & reg_we & !reg_error;

  assign alert_regwen_46_wd = reg_wdata[0];
  assign alert_regwen_47_we = addr_hit[53] & reg_we & !reg_error;

  assign alert_regwen_47_wd = reg_wdata[0];
  assign alert_regwen_48_we = addr_hit[54] & reg_we & !reg_error;

  assign alert_regwen_48_wd = reg_wdata[0];
  assign alert_regwen_49_we = addr_hit[55] & reg_we & !reg_error;

  assign alert_regwen_49_wd = reg_wdata[0];
  assign alert_regwen_50_we = addr_hit[56] & reg_we & !reg_error;

  assign alert_regwen_50_wd = reg_wdata[0];
  assign alert_regwen_51_we = addr_hit[57] & reg_we & !reg_error;

  assign alert_regwen_51_wd = reg_wdata[0];
  assign alert_regwen_52_we = addr_hit[58] & reg_we & !reg_error;

  assign alert_regwen_52_wd = reg_wdata[0];
  assign alert_regwen_53_we = addr_hit[59] & reg_we & !reg_error;

  assign alert_regwen_53_wd = reg_wdata[0];
  assign alert_regwen_54_we = addr_hit[60] & reg_we & !reg_error;

  assign alert_regwen_54_wd = reg_wdata[0];
  assign alert_regwen_55_we = addr_hit[61] & reg_we & !reg_error;

  assign alert_regwen_55_wd = reg_wdata[0];
  assign alert_regwen_56_we = addr_hit[62] & reg_we & !reg_error;

  assign alert_regwen_56_wd = reg_wdata[0];
  assign alert_regwen_57_we = addr_hit[63] & reg_we & !reg_error;

  assign alert_regwen_57_wd = reg_wdata[0];
  assign alert_regwen_58_we = addr_hit[64] & reg_we & !reg_error;

  assign alert_regwen_58_wd = reg_wdata[0];
  assign alert_regwen_59_we = addr_hit[65] & reg_we & !reg_error;

  assign alert_regwen_59_wd = reg_wdata[0];
  assign alert_regwen_60_we = addr_hit[66] & reg_we & !reg_error;

  assign alert_regwen_60_wd = reg_wdata[0];
  assign alert_regwen_61_we = addr_hit[67] & reg_we & !reg_error;

  assign alert_regwen_61_wd = reg_wdata[0];
  assign alert_regwen_62_we = addr_hit[68] & reg_we & !reg_error;

  assign alert_regwen_62_wd = reg_wdata[0];
  assign alert_regwen_63_we = addr_hit[69] & reg_we & !reg_error;

  assign alert_regwen_63_wd = reg_wdata[0];
  assign alert_regwen_64_we = addr_hit[70] & reg_we & !reg_error;

  assign alert_regwen_64_wd = reg_wdata[0];
  assign alert_en_shadowed_0_re = addr_hit[71] & reg_re & !reg_error;
  assign alert_en_shadowed_0_we = addr_hit[71] & reg_we & !reg_error;

  assign alert_en_shadowed_0_wd = reg_wdata[0];
  assign alert_en_shadowed_1_re = addr_hit[72] & reg_re & !reg_error;
  assign alert_en_shadowed_1_we = addr_hit[72] & reg_we & !reg_error;

  assign alert_en_shadowed_1_wd = reg_wdata[0];
  assign alert_en_shadowed_2_re = addr_hit[73] & reg_re & !reg_error;
  assign alert_en_shadowed_2_we = addr_hit[73] & reg_we & !reg_error;

  assign alert_en_shadowed_2_wd = reg_wdata[0];
  assign alert_en_shadowed_3_re = addr_hit[74] & reg_re & !reg_error;
  assign alert_en_shadowed_3_we = addr_hit[74] & reg_we & !reg_error;

  assign alert_en_shadowed_3_wd = reg_wdata[0];
  assign alert_en_shadowed_4_re = addr_hit[75] & reg_re & !reg_error;
  assign alert_en_shadowed_4_we = addr_hit[75] & reg_we & !reg_error;

  assign alert_en_shadowed_4_wd = reg_wdata[0];
  assign alert_en_shadowed_5_re = addr_hit[76] & reg_re & !reg_error;
  assign alert_en_shadowed_5_we = addr_hit[76] & reg_we & !reg_error;

  assign alert_en_shadowed_5_wd = reg_wdata[0];
  assign alert_en_shadowed_6_re = addr_hit[77] & reg_re & !reg_error;
  assign alert_en_shadowed_6_we = addr_hit[77] & reg_we & !reg_error;

  assign alert_en_shadowed_6_wd = reg_wdata[0];
  assign alert_en_shadowed_7_re = addr_hit[78] & reg_re & !reg_error;
  assign alert_en_shadowed_7_we = addr_hit[78] & reg_we & !reg_error;

  assign alert_en_shadowed_7_wd = reg_wdata[0];
  assign alert_en_shadowed_8_re = addr_hit[79] & reg_re & !reg_error;
  assign alert_en_shadowed_8_we = addr_hit[79] & reg_we & !reg_error;

  assign alert_en_shadowed_8_wd = reg_wdata[0];
  assign alert_en_shadowed_9_re = addr_hit[80] & reg_re & !reg_error;
  assign alert_en_shadowed_9_we = addr_hit[80] & reg_we & !reg_error;

  assign alert_en_shadowed_9_wd = reg_wdata[0];
  assign alert_en_shadowed_10_re = addr_hit[81] & reg_re & !reg_error;
  assign alert_en_shadowed_10_we = addr_hit[81] & reg_we & !reg_error;

  assign alert_en_shadowed_10_wd = reg_wdata[0];
  assign alert_en_shadowed_11_re = addr_hit[82] & reg_re & !reg_error;
  assign alert_en_shadowed_11_we = addr_hit[82] & reg_we & !reg_error;

  assign alert_en_shadowed_11_wd = reg_wdata[0];
  assign alert_en_shadowed_12_re = addr_hit[83] & reg_re & !reg_error;
  assign alert_en_shadowed_12_we = addr_hit[83] & reg_we & !reg_error;

  assign alert_en_shadowed_12_wd = reg_wdata[0];
  assign alert_en_shadowed_13_re = addr_hit[84] & reg_re & !reg_error;
  assign alert_en_shadowed_13_we = addr_hit[84] & reg_we & !reg_error;

  assign alert_en_shadowed_13_wd = reg_wdata[0];
  assign alert_en_shadowed_14_re = addr_hit[85] & reg_re & !reg_error;
  assign alert_en_shadowed_14_we = addr_hit[85] & reg_we & !reg_error;

  assign alert_en_shadowed_14_wd = reg_wdata[0];
  assign alert_en_shadowed_15_re = addr_hit[86] & reg_re & !reg_error;
  assign alert_en_shadowed_15_we = addr_hit[86] & reg_we & !reg_error;

  assign alert_en_shadowed_15_wd = reg_wdata[0];
  assign alert_en_shadowed_16_re = addr_hit[87] & reg_re & !reg_error;
  assign alert_en_shadowed_16_we = addr_hit[87] & reg_we & !reg_error;

  assign alert_en_shadowed_16_wd = reg_wdata[0];
  assign alert_en_shadowed_17_re = addr_hit[88] & reg_re & !reg_error;
  assign alert_en_shadowed_17_we = addr_hit[88] & reg_we & !reg_error;

  assign alert_en_shadowed_17_wd = reg_wdata[0];
  assign alert_en_shadowed_18_re = addr_hit[89] & reg_re & !reg_error;
  assign alert_en_shadowed_18_we = addr_hit[89] & reg_we & !reg_error;

  assign alert_en_shadowed_18_wd = reg_wdata[0];
  assign alert_en_shadowed_19_re = addr_hit[90] & reg_re & !reg_error;
  assign alert_en_shadowed_19_we = addr_hit[90] & reg_we & !reg_error;

  assign alert_en_shadowed_19_wd = reg_wdata[0];
  assign alert_en_shadowed_20_re = addr_hit[91] & reg_re & !reg_error;
  assign alert_en_shadowed_20_we = addr_hit[91] & reg_we & !reg_error;

  assign alert_en_shadowed_20_wd = reg_wdata[0];
  assign alert_en_shadowed_21_re = addr_hit[92] & reg_re & !reg_error;
  assign alert_en_shadowed_21_we = addr_hit[92] & reg_we & !reg_error;

  assign alert_en_shadowed_21_wd = reg_wdata[0];
  assign alert_en_shadowed_22_re = addr_hit[93] & reg_re & !reg_error;
  assign alert_en_shadowed_22_we = addr_hit[93] & reg_we & !reg_error;

  assign alert_en_shadowed_22_wd = reg_wdata[0];
  assign alert_en_shadowed_23_re = addr_hit[94] & reg_re & !reg_error;
  assign alert_en_shadowed_23_we = addr_hit[94] & reg_we & !reg_error;

  assign alert_en_shadowed_23_wd = reg_wdata[0];
  assign alert_en_shadowed_24_re = addr_hit[95] & reg_re & !reg_error;
  assign alert_en_shadowed_24_we = addr_hit[95] & reg_we & !reg_error;

  assign alert_en_shadowed_24_wd = reg_wdata[0];
  assign alert_en_shadowed_25_re = addr_hit[96] & reg_re & !reg_error;
  assign alert_en_shadowed_25_we = addr_hit[96] & reg_we & !reg_error;

  assign alert_en_shadowed_25_wd = reg_wdata[0];
  assign alert_en_shadowed_26_re = addr_hit[97] & reg_re & !reg_error;
  assign alert_en_shadowed_26_we = addr_hit[97] & reg_we & !reg_error;

  assign alert_en_shadowed_26_wd = reg_wdata[0];
  assign alert_en_shadowed_27_re = addr_hit[98] & reg_re & !reg_error;
  assign alert_en_shadowed_27_we = addr_hit[98] & reg_we & !reg_error;

  assign alert_en_shadowed_27_wd = reg_wdata[0];
  assign alert_en_shadowed_28_re = addr_hit[99] & reg_re & !reg_error;
  assign alert_en_shadowed_28_we = addr_hit[99] & reg_we & !reg_error;

  assign alert_en_shadowed_28_wd = reg_wdata[0];
  assign alert_en_shadowed_29_re = addr_hit[100] & reg_re & !reg_error;
  assign alert_en_shadowed_29_we = addr_hit[100] & reg_we & !reg_error;

  assign alert_en_shadowed_29_wd = reg_wdata[0];
  assign alert_en_shadowed_30_re = addr_hit[101] & reg_re & !reg_error;
  assign alert_en_shadowed_30_we = addr_hit[101] & reg_we & !reg_error;

  assign alert_en_shadowed_30_wd = reg_wdata[0];
  assign alert_en_shadowed_31_re = addr_hit[102] & reg_re & !reg_error;
  assign alert_en_shadowed_31_we = addr_hit[102] & reg_we & !reg_error;

  assign alert_en_shadowed_31_wd = reg_wdata[0];
  assign alert_en_shadowed_32_re = addr_hit[103] & reg_re & !reg_error;
  assign alert_en_shadowed_32_we = addr_hit[103] & reg_we & !reg_error;

  assign alert_en_shadowed_32_wd = reg_wdata[0];
  assign alert_en_shadowed_33_re = addr_hit[104] & reg_re & !reg_error;
  assign alert_en_shadowed_33_we = addr_hit[104] & reg_we & !reg_error;

  assign alert_en_shadowed_33_wd = reg_wdata[0];
  assign alert_en_shadowed_34_re = addr_hit[105] & reg_re & !reg_error;
  assign alert_en_shadowed_34_we = addr_hit[105] & reg_we & !reg_error;

  assign alert_en_shadowed_34_wd = reg_wdata[0];
  assign alert_en_shadowed_35_re = addr_hit[106] & reg_re & !reg_error;
  assign alert_en_shadowed_35_we = addr_hit[106] & reg_we & !reg_error;

  assign alert_en_shadowed_35_wd = reg_wdata[0];
  assign alert_en_shadowed_36_re = addr_hit[107] & reg_re & !reg_error;
  assign alert_en_shadowed_36_we = addr_hit[107] & reg_we & !reg_error;

  assign alert_en_shadowed_36_wd = reg_wdata[0];
  assign alert_en_shadowed_37_re = addr_hit[108] & reg_re & !reg_error;
  assign alert_en_shadowed_37_we = addr_hit[108] & reg_we & !reg_error;

  assign alert_en_shadowed_37_wd = reg_wdata[0];
  assign alert_en_shadowed_38_re = addr_hit[109] & reg_re & !reg_error;
  assign alert_en_shadowed_38_we = addr_hit[109] & reg_we & !reg_error;

  assign alert_en_shadowed_38_wd = reg_wdata[0];
  assign alert_en_shadowed_39_re = addr_hit[110] & reg_re & !reg_error;
  assign alert_en_shadowed_39_we = addr_hit[110] & reg_we & !reg_error;

  assign alert_en_shadowed_39_wd = reg_wdata[0];
  assign alert_en_shadowed_40_re = addr_hit[111] & reg_re & !reg_error;
  assign alert_en_shadowed_40_we = addr_hit[111] & reg_we & !reg_error;

  assign alert_en_shadowed_40_wd = reg_wdata[0];
  assign alert_en_shadowed_41_re = addr_hit[112] & reg_re & !reg_error;
  assign alert_en_shadowed_41_we = addr_hit[112] & reg_we & !reg_error;

  assign alert_en_shadowed_41_wd = reg_wdata[0];
  assign alert_en_shadowed_42_re = addr_hit[113] & reg_re & !reg_error;
  assign alert_en_shadowed_42_we = addr_hit[113] & reg_we & !reg_error;

  assign alert_en_shadowed_42_wd = reg_wdata[0];
  assign alert_en_shadowed_43_re = addr_hit[114] & reg_re & !reg_error;
  assign alert_en_shadowed_43_we = addr_hit[114] & reg_we & !reg_error;

  assign alert_en_shadowed_43_wd = reg_wdata[0];
  assign alert_en_shadowed_44_re = addr_hit[115] & reg_re & !reg_error;
  assign alert_en_shadowed_44_we = addr_hit[115] & reg_we & !reg_error;

  assign alert_en_shadowed_44_wd = reg_wdata[0];
  assign alert_en_shadowed_45_re = addr_hit[116] & reg_re & !reg_error;
  assign alert_en_shadowed_45_we = addr_hit[116] & reg_we & !reg_error;

  assign alert_en_shadowed_45_wd = reg_wdata[0];
  assign alert_en_shadowed_46_re = addr_hit[117] & reg_re & !reg_error;
  assign alert_en_shadowed_46_we = addr_hit[117] & reg_we & !reg_error;

  assign alert_en_shadowed_46_wd = reg_wdata[0];
  assign alert_en_shadowed_47_re = addr_hit[118] & reg_re & !reg_error;
  assign alert_en_shadowed_47_we = addr_hit[118] & reg_we & !reg_error;

  assign alert_en_shadowed_47_wd = reg_wdata[0];
  assign alert_en_shadowed_48_re = addr_hit[119] & reg_re & !reg_error;
  assign alert_en_shadowed_48_we = addr_hit[119] & reg_we & !reg_error;

  assign alert_en_shadowed_48_wd = reg_wdata[0];
  assign alert_en_shadowed_49_re = addr_hit[120] & reg_re & !reg_error;
  assign alert_en_shadowed_49_we = addr_hit[120] & reg_we & !reg_error;

  assign alert_en_shadowed_49_wd = reg_wdata[0];
  assign alert_en_shadowed_50_re = addr_hit[121] & reg_re & !reg_error;
  assign alert_en_shadowed_50_we = addr_hit[121] & reg_we & !reg_error;

  assign alert_en_shadowed_50_wd = reg_wdata[0];
  assign alert_en_shadowed_51_re = addr_hit[122] & reg_re & !reg_error;
  assign alert_en_shadowed_51_we = addr_hit[122] & reg_we & !reg_error;

  assign alert_en_shadowed_51_wd = reg_wdata[0];
  assign alert_en_shadowed_52_re = addr_hit[123] & reg_re & !reg_error;
  assign alert_en_shadowed_52_we = addr_hit[123] & reg_we & !reg_error;

  assign alert_en_shadowed_52_wd = reg_wdata[0];
  assign alert_en_shadowed_53_re = addr_hit[124] & reg_re & !reg_error;
  assign alert_en_shadowed_53_we = addr_hit[124] & reg_we & !reg_error;

  assign alert_en_shadowed_53_wd = reg_wdata[0];
  assign alert_en_shadowed_54_re = addr_hit[125] & reg_re & !reg_error;
  assign alert_en_shadowed_54_we = addr_hit[125] & reg_we & !reg_error;

  assign alert_en_shadowed_54_wd = reg_wdata[0];
  assign alert_en_shadowed_55_re = addr_hit[126] & reg_re & !reg_error;
  assign alert_en_shadowed_55_we = addr_hit[126] & reg_we & !reg_error;

  assign alert_en_shadowed_55_wd = reg_wdata[0];
  assign alert_en_shadowed_56_re = addr_hit[127] & reg_re & !reg_error;
  assign alert_en_shadowed_56_we = addr_hit[127] & reg_we & !reg_error;

  assign alert_en_shadowed_56_wd = reg_wdata[0];
  assign alert_en_shadowed_57_re = addr_hit[128] & reg_re & !reg_error;
  assign alert_en_shadowed_57_we = addr_hit[128] & reg_we & !reg_error;

  assign alert_en_shadowed_57_wd = reg_wdata[0];
  assign alert_en_shadowed_58_re = addr_hit[129] & reg_re & !reg_error;
  assign alert_en_shadowed_58_we = addr_hit[129] & reg_we & !reg_error;

  assign alert_en_shadowed_58_wd = reg_wdata[0];
  assign alert_en_shadowed_59_re = addr_hit[130] & reg_re & !reg_error;
  assign alert_en_shadowed_59_we = addr_hit[130] & reg_we & !reg_error;

  assign alert_en_shadowed_59_wd = reg_wdata[0];
  assign alert_en_shadowed_60_re = addr_hit[131] & reg_re & !reg_error;
  assign alert_en_shadowed_60_we = addr_hit[131] & reg_we & !reg_error;

  assign alert_en_shadowed_60_wd = reg_wdata[0];
  assign alert_en_shadowed_61_re = addr_hit[132] & reg_re & !reg_error;
  assign alert_en_shadowed_61_we = addr_hit[132] & reg_we & !reg_error;

  assign alert_en_shadowed_61_wd = reg_wdata[0];
  assign alert_en_shadowed_62_re = addr_hit[133] & reg_re & !reg_error;
  assign alert_en_shadowed_62_we = addr_hit[133] & reg_we & !reg_error;

  assign alert_en_shadowed_62_wd = reg_wdata[0];
  assign alert_en_shadowed_63_re = addr_hit[134] & reg_re & !reg_error;
  assign alert_en_shadowed_63_we = addr_hit[134] & reg_we & !reg_error;

  assign alert_en_shadowed_63_wd = reg_wdata[0];
  assign alert_en_shadowed_64_re = addr_hit[135] & reg_re & !reg_error;
  assign alert_en_shadowed_64_we = addr_hit[135] & reg_we & !reg_error;

  assign alert_en_shadowed_64_wd = reg_wdata[0];
  assign alert_class_shadowed_0_re = addr_hit[136] & reg_re & !reg_error;
  assign alert_class_shadowed_0_we = addr_hit[136] & reg_we & !reg_error;

  assign alert_class_shadowed_0_wd = reg_wdata[1:0];
  assign alert_class_shadowed_1_re = addr_hit[137] & reg_re & !reg_error;
  assign alert_class_shadowed_1_we = addr_hit[137] & reg_we & !reg_error;

  assign alert_class_shadowed_1_wd = reg_wdata[1:0];
  assign alert_class_shadowed_2_re = addr_hit[138] & reg_re & !reg_error;
  assign alert_class_shadowed_2_we = addr_hit[138] & reg_we & !reg_error;

  assign alert_class_shadowed_2_wd = reg_wdata[1:0];
  assign alert_class_shadowed_3_re = addr_hit[139] & reg_re & !reg_error;
  assign alert_class_shadowed_3_we = addr_hit[139] & reg_we & !reg_error;

  assign alert_class_shadowed_3_wd = reg_wdata[1:0];
  assign alert_class_shadowed_4_re = addr_hit[140] & reg_re & !reg_error;
  assign alert_class_shadowed_4_we = addr_hit[140] & reg_we & !reg_error;

  assign alert_class_shadowed_4_wd = reg_wdata[1:0];
  assign alert_class_shadowed_5_re = addr_hit[141] & reg_re & !reg_error;
  assign alert_class_shadowed_5_we = addr_hit[141] & reg_we & !reg_error;

  assign alert_class_shadowed_5_wd = reg_wdata[1:0];
  assign alert_class_shadowed_6_re = addr_hit[142] & reg_re & !reg_error;
  assign alert_class_shadowed_6_we = addr_hit[142] & reg_we & !reg_error;

  assign alert_class_shadowed_6_wd = reg_wdata[1:0];
  assign alert_class_shadowed_7_re = addr_hit[143] & reg_re & !reg_error;
  assign alert_class_shadowed_7_we = addr_hit[143] & reg_we & !reg_error;

  assign alert_class_shadowed_7_wd = reg_wdata[1:0];
  assign alert_class_shadowed_8_re = addr_hit[144] & reg_re & !reg_error;
  assign alert_class_shadowed_8_we = addr_hit[144] & reg_we & !reg_error;

  assign alert_class_shadowed_8_wd = reg_wdata[1:0];
  assign alert_class_shadowed_9_re = addr_hit[145] & reg_re & !reg_error;
  assign alert_class_shadowed_9_we = addr_hit[145] & reg_we & !reg_error;

  assign alert_class_shadowed_9_wd = reg_wdata[1:0];
  assign alert_class_shadowed_10_re = addr_hit[146] & reg_re & !reg_error;
  assign alert_class_shadowed_10_we = addr_hit[146] & reg_we & !reg_error;

  assign alert_class_shadowed_10_wd = reg_wdata[1:0];
  assign alert_class_shadowed_11_re = addr_hit[147] & reg_re & !reg_error;
  assign alert_class_shadowed_11_we = addr_hit[147] & reg_we & !reg_error;

  assign alert_class_shadowed_11_wd = reg_wdata[1:0];
  assign alert_class_shadowed_12_re = addr_hit[148] & reg_re & !reg_error;
  assign alert_class_shadowed_12_we = addr_hit[148] & reg_we & !reg_error;

  assign alert_class_shadowed_12_wd = reg_wdata[1:0];
  assign alert_class_shadowed_13_re = addr_hit[149] & reg_re & !reg_error;
  assign alert_class_shadowed_13_we = addr_hit[149] & reg_we & !reg_error;

  assign alert_class_shadowed_13_wd = reg_wdata[1:0];
  assign alert_class_shadowed_14_re = addr_hit[150] & reg_re & !reg_error;
  assign alert_class_shadowed_14_we = addr_hit[150] & reg_we & !reg_error;

  assign alert_class_shadowed_14_wd = reg_wdata[1:0];
  assign alert_class_shadowed_15_re = addr_hit[151] & reg_re & !reg_error;
  assign alert_class_shadowed_15_we = addr_hit[151] & reg_we & !reg_error;

  assign alert_class_shadowed_15_wd = reg_wdata[1:0];
  assign alert_class_shadowed_16_re = addr_hit[152] & reg_re & !reg_error;
  assign alert_class_shadowed_16_we = addr_hit[152] & reg_we & !reg_error;

  assign alert_class_shadowed_16_wd = reg_wdata[1:0];
  assign alert_class_shadowed_17_re = addr_hit[153] & reg_re & !reg_error;
  assign alert_class_shadowed_17_we = addr_hit[153] & reg_we & !reg_error;

  assign alert_class_shadowed_17_wd = reg_wdata[1:0];
  assign alert_class_shadowed_18_re = addr_hit[154] & reg_re & !reg_error;
  assign alert_class_shadowed_18_we = addr_hit[154] & reg_we & !reg_error;

  assign alert_class_shadowed_18_wd = reg_wdata[1:0];
  assign alert_class_shadowed_19_re = addr_hit[155] & reg_re & !reg_error;
  assign alert_class_shadowed_19_we = addr_hit[155] & reg_we & !reg_error;

  assign alert_class_shadowed_19_wd = reg_wdata[1:0];
  assign alert_class_shadowed_20_re = addr_hit[156] & reg_re & !reg_error;
  assign alert_class_shadowed_20_we = addr_hit[156] & reg_we & !reg_error;

  assign alert_class_shadowed_20_wd = reg_wdata[1:0];
  assign alert_class_shadowed_21_re = addr_hit[157] & reg_re & !reg_error;
  assign alert_class_shadowed_21_we = addr_hit[157] & reg_we & !reg_error;

  assign alert_class_shadowed_21_wd = reg_wdata[1:0];
  assign alert_class_shadowed_22_re = addr_hit[158] & reg_re & !reg_error;
  assign alert_class_shadowed_22_we = addr_hit[158] & reg_we & !reg_error;

  assign alert_class_shadowed_22_wd = reg_wdata[1:0];
  assign alert_class_shadowed_23_re = addr_hit[159] & reg_re & !reg_error;
  assign alert_class_shadowed_23_we = addr_hit[159] & reg_we & !reg_error;

  assign alert_class_shadowed_23_wd = reg_wdata[1:0];
  assign alert_class_shadowed_24_re = addr_hit[160] & reg_re & !reg_error;
  assign alert_class_shadowed_24_we = addr_hit[160] & reg_we & !reg_error;

  assign alert_class_shadowed_24_wd = reg_wdata[1:0];
  assign alert_class_shadowed_25_re = addr_hit[161] & reg_re & !reg_error;
  assign alert_class_shadowed_25_we = addr_hit[161] & reg_we & !reg_error;

  assign alert_class_shadowed_25_wd = reg_wdata[1:0];
  assign alert_class_shadowed_26_re = addr_hit[162] & reg_re & !reg_error;
  assign alert_class_shadowed_26_we = addr_hit[162] & reg_we & !reg_error;

  assign alert_class_shadowed_26_wd = reg_wdata[1:0];
  assign alert_class_shadowed_27_re = addr_hit[163] & reg_re & !reg_error;
  assign alert_class_shadowed_27_we = addr_hit[163] & reg_we & !reg_error;

  assign alert_class_shadowed_27_wd = reg_wdata[1:0];
  assign alert_class_shadowed_28_re = addr_hit[164] & reg_re & !reg_error;
  assign alert_class_shadowed_28_we = addr_hit[164] & reg_we & !reg_error;

  assign alert_class_shadowed_28_wd = reg_wdata[1:0];
  assign alert_class_shadowed_29_re = addr_hit[165] & reg_re & !reg_error;
  assign alert_class_shadowed_29_we = addr_hit[165] & reg_we & !reg_error;

  assign alert_class_shadowed_29_wd = reg_wdata[1:0];
  assign alert_class_shadowed_30_re = addr_hit[166] & reg_re & !reg_error;
  assign alert_class_shadowed_30_we = addr_hit[166] & reg_we & !reg_error;

  assign alert_class_shadowed_30_wd = reg_wdata[1:0];
  assign alert_class_shadowed_31_re = addr_hit[167] & reg_re & !reg_error;
  assign alert_class_shadowed_31_we = addr_hit[167] & reg_we & !reg_error;

  assign alert_class_shadowed_31_wd = reg_wdata[1:0];
  assign alert_class_shadowed_32_re = addr_hit[168] & reg_re & !reg_error;
  assign alert_class_shadowed_32_we = addr_hit[168] & reg_we & !reg_error;

  assign alert_class_shadowed_32_wd = reg_wdata[1:0];
  assign alert_class_shadowed_33_re = addr_hit[169] & reg_re & !reg_error;
  assign alert_class_shadowed_33_we = addr_hit[169] & reg_we & !reg_error;

  assign alert_class_shadowed_33_wd = reg_wdata[1:0];
  assign alert_class_shadowed_34_re = addr_hit[170] & reg_re & !reg_error;
  assign alert_class_shadowed_34_we = addr_hit[170] & reg_we & !reg_error;

  assign alert_class_shadowed_34_wd = reg_wdata[1:0];
  assign alert_class_shadowed_35_re = addr_hit[171] & reg_re & !reg_error;
  assign alert_class_shadowed_35_we = addr_hit[171] & reg_we & !reg_error;

  assign alert_class_shadowed_35_wd = reg_wdata[1:0];
  assign alert_class_shadowed_36_re = addr_hit[172] & reg_re & !reg_error;
  assign alert_class_shadowed_36_we = addr_hit[172] & reg_we & !reg_error;

  assign alert_class_shadowed_36_wd = reg_wdata[1:0];
  assign alert_class_shadowed_37_re = addr_hit[173] & reg_re & !reg_error;
  assign alert_class_shadowed_37_we = addr_hit[173] & reg_we & !reg_error;

  assign alert_class_shadowed_37_wd = reg_wdata[1:0];
  assign alert_class_shadowed_38_re = addr_hit[174] & reg_re & !reg_error;
  assign alert_class_shadowed_38_we = addr_hit[174] & reg_we & !reg_error;

  assign alert_class_shadowed_38_wd = reg_wdata[1:0];
  assign alert_class_shadowed_39_re = addr_hit[175] & reg_re & !reg_error;
  assign alert_class_shadowed_39_we = addr_hit[175] & reg_we & !reg_error;

  assign alert_class_shadowed_39_wd = reg_wdata[1:0];
  assign alert_class_shadowed_40_re = addr_hit[176] & reg_re & !reg_error;
  assign alert_class_shadowed_40_we = addr_hit[176] & reg_we & !reg_error;

  assign alert_class_shadowed_40_wd = reg_wdata[1:0];
  assign alert_class_shadowed_41_re = addr_hit[177] & reg_re & !reg_error;
  assign alert_class_shadowed_41_we = addr_hit[177] & reg_we & !reg_error;

  assign alert_class_shadowed_41_wd = reg_wdata[1:0];
  assign alert_class_shadowed_42_re = addr_hit[178] & reg_re & !reg_error;
  assign alert_class_shadowed_42_we = addr_hit[178] & reg_we & !reg_error;

  assign alert_class_shadowed_42_wd = reg_wdata[1:0];
  assign alert_class_shadowed_43_re = addr_hit[179] & reg_re & !reg_error;
  assign alert_class_shadowed_43_we = addr_hit[179] & reg_we & !reg_error;

  assign alert_class_shadowed_43_wd = reg_wdata[1:0];
  assign alert_class_shadowed_44_re = addr_hit[180] & reg_re & !reg_error;
  assign alert_class_shadowed_44_we = addr_hit[180] & reg_we & !reg_error;

  assign alert_class_shadowed_44_wd = reg_wdata[1:0];
  assign alert_class_shadowed_45_re = addr_hit[181] & reg_re & !reg_error;
  assign alert_class_shadowed_45_we = addr_hit[181] & reg_we & !reg_error;

  assign alert_class_shadowed_45_wd = reg_wdata[1:0];
  assign alert_class_shadowed_46_re = addr_hit[182] & reg_re & !reg_error;
  assign alert_class_shadowed_46_we = addr_hit[182] & reg_we & !reg_error;

  assign alert_class_shadowed_46_wd = reg_wdata[1:0];
  assign alert_class_shadowed_47_re = addr_hit[183] & reg_re & !reg_error;
  assign alert_class_shadowed_47_we = addr_hit[183] & reg_we & !reg_error;

  assign alert_class_shadowed_47_wd = reg_wdata[1:0];
  assign alert_class_shadowed_48_re = addr_hit[184] & reg_re & !reg_error;
  assign alert_class_shadowed_48_we = addr_hit[184] & reg_we & !reg_error;

  assign alert_class_shadowed_48_wd = reg_wdata[1:0];
  assign alert_class_shadowed_49_re = addr_hit[185] & reg_re & !reg_error;
  assign alert_class_shadowed_49_we = addr_hit[185] & reg_we & !reg_error;

  assign alert_class_shadowed_49_wd = reg_wdata[1:0];
  assign alert_class_shadowed_50_re = addr_hit[186] & reg_re & !reg_error;
  assign alert_class_shadowed_50_we = addr_hit[186] & reg_we & !reg_error;

  assign alert_class_shadowed_50_wd = reg_wdata[1:0];
  assign alert_class_shadowed_51_re = addr_hit[187] & reg_re & !reg_error;
  assign alert_class_shadowed_51_we = addr_hit[187] & reg_we & !reg_error;

  assign alert_class_shadowed_51_wd = reg_wdata[1:0];
  assign alert_class_shadowed_52_re = addr_hit[188] & reg_re & !reg_error;
  assign alert_class_shadowed_52_we = addr_hit[188] & reg_we & !reg_error;

  assign alert_class_shadowed_52_wd = reg_wdata[1:0];
  assign alert_class_shadowed_53_re = addr_hit[189] & reg_re & !reg_error;
  assign alert_class_shadowed_53_we = addr_hit[189] & reg_we & !reg_error;

  assign alert_class_shadowed_53_wd = reg_wdata[1:0];
  assign alert_class_shadowed_54_re = addr_hit[190] & reg_re & !reg_error;
  assign alert_class_shadowed_54_we = addr_hit[190] & reg_we & !reg_error;

  assign alert_class_shadowed_54_wd = reg_wdata[1:0];
  assign alert_class_shadowed_55_re = addr_hit[191] & reg_re & !reg_error;
  assign alert_class_shadowed_55_we = addr_hit[191] & reg_we & !reg_error;

  assign alert_class_shadowed_55_wd = reg_wdata[1:0];
  assign alert_class_shadowed_56_re = addr_hit[192] & reg_re & !reg_error;
  assign alert_class_shadowed_56_we = addr_hit[192] & reg_we & !reg_error;

  assign alert_class_shadowed_56_wd = reg_wdata[1:0];
  assign alert_class_shadowed_57_re = addr_hit[193] & reg_re & !reg_error;
  assign alert_class_shadowed_57_we = addr_hit[193] & reg_we & !reg_error;

  assign alert_class_shadowed_57_wd = reg_wdata[1:0];
  assign alert_class_shadowed_58_re = addr_hit[194] & reg_re & !reg_error;
  assign alert_class_shadowed_58_we = addr_hit[194] & reg_we & !reg_error;

  assign alert_class_shadowed_58_wd = reg_wdata[1:0];
  assign alert_class_shadowed_59_re = addr_hit[195] & reg_re & !reg_error;
  assign alert_class_shadowed_59_we = addr_hit[195] & reg_we & !reg_error;

  assign alert_class_shadowed_59_wd = reg_wdata[1:0];
  assign alert_class_shadowed_60_re = addr_hit[196] & reg_re & !reg_error;
  assign alert_class_shadowed_60_we = addr_hit[196] & reg_we & !reg_error;

  assign alert_class_shadowed_60_wd = reg_wdata[1:0];
  assign alert_class_shadowed_61_re = addr_hit[197] & reg_re & !reg_error;
  assign alert_class_shadowed_61_we = addr_hit[197] & reg_we & !reg_error;

  assign alert_class_shadowed_61_wd = reg_wdata[1:0];
  assign alert_class_shadowed_62_re = addr_hit[198] & reg_re & !reg_error;
  assign alert_class_shadowed_62_we = addr_hit[198] & reg_we & !reg_error;

  assign alert_class_shadowed_62_wd = reg_wdata[1:0];
  assign alert_class_shadowed_63_re = addr_hit[199] & reg_re & !reg_error;
  assign alert_class_shadowed_63_we = addr_hit[199] & reg_we & !reg_error;

  assign alert_class_shadowed_63_wd = reg_wdata[1:0];
  assign alert_class_shadowed_64_re = addr_hit[200] & reg_re & !reg_error;
  assign alert_class_shadowed_64_we = addr_hit[200] & reg_we & !reg_error;

  assign alert_class_shadowed_64_wd = reg_wdata[1:0];
  assign alert_cause_0_we = addr_hit[201] & reg_we & !reg_error;

  assign alert_cause_0_wd = reg_wdata[0];
  assign alert_cause_1_we = addr_hit[202] & reg_we & !reg_error;

  assign alert_cause_1_wd = reg_wdata[0];
  assign alert_cause_2_we = addr_hit[203] & reg_we & !reg_error;

  assign alert_cause_2_wd = reg_wdata[0];
  assign alert_cause_3_we = addr_hit[204] & reg_we & !reg_error;

  assign alert_cause_3_wd = reg_wdata[0];
  assign alert_cause_4_we = addr_hit[205] & reg_we & !reg_error;

  assign alert_cause_4_wd = reg_wdata[0];
  assign alert_cause_5_we = addr_hit[206] & reg_we & !reg_error;

  assign alert_cause_5_wd = reg_wdata[0];
  assign alert_cause_6_we = addr_hit[207] & reg_we & !reg_error;

  assign alert_cause_6_wd = reg_wdata[0];
  assign alert_cause_7_we = addr_hit[208] & reg_we & !reg_error;

  assign alert_cause_7_wd = reg_wdata[0];
  assign alert_cause_8_we = addr_hit[209] & reg_we & !reg_error;

  assign alert_cause_8_wd = reg_wdata[0];
  assign alert_cause_9_we = addr_hit[210] & reg_we & !reg_error;

  assign alert_cause_9_wd = reg_wdata[0];
  assign alert_cause_10_we = addr_hit[211] & reg_we & !reg_error;

  assign alert_cause_10_wd = reg_wdata[0];
  assign alert_cause_11_we = addr_hit[212] & reg_we & !reg_error;

  assign alert_cause_11_wd = reg_wdata[0];
  assign alert_cause_12_we = addr_hit[213] & reg_we & !reg_error;

  assign alert_cause_12_wd = reg_wdata[0];
  assign alert_cause_13_we = addr_hit[214] & reg_we & !reg_error;

  assign alert_cause_13_wd = reg_wdata[0];
  assign alert_cause_14_we = addr_hit[215] & reg_we & !reg_error;

  assign alert_cause_14_wd = reg_wdata[0];
  assign alert_cause_15_we = addr_hit[216] & reg_we & !reg_error;

  assign alert_cause_15_wd = reg_wdata[0];
  assign alert_cause_16_we = addr_hit[217] & reg_we & !reg_error;

  assign alert_cause_16_wd = reg_wdata[0];
  assign alert_cause_17_we = addr_hit[218] & reg_we & !reg_error;

  assign alert_cause_17_wd = reg_wdata[0];
  assign alert_cause_18_we = addr_hit[219] & reg_we & !reg_error;

  assign alert_cause_18_wd = reg_wdata[0];
  assign alert_cause_19_we = addr_hit[220] & reg_we & !reg_error;

  assign alert_cause_19_wd = reg_wdata[0];
  assign alert_cause_20_we = addr_hit[221] & reg_we & !reg_error;

  assign alert_cause_20_wd = reg_wdata[0];
  assign alert_cause_21_we = addr_hit[222] & reg_we & !reg_error;

  assign alert_cause_21_wd = reg_wdata[0];
  assign alert_cause_22_we = addr_hit[223] & reg_we & !reg_error;

  assign alert_cause_22_wd = reg_wdata[0];
  assign alert_cause_23_we = addr_hit[224] & reg_we & !reg_error;

  assign alert_cause_23_wd = reg_wdata[0];
  assign alert_cause_24_we = addr_hit[225] & reg_we & !reg_error;

  assign alert_cause_24_wd = reg_wdata[0];
  assign alert_cause_25_we = addr_hit[226] & reg_we & !reg_error;

  assign alert_cause_25_wd = reg_wdata[0];
  assign alert_cause_26_we = addr_hit[227] & reg_we & !reg_error;

  assign alert_cause_26_wd = reg_wdata[0];
  assign alert_cause_27_we = addr_hit[228] & reg_we & !reg_error;

  assign alert_cause_27_wd = reg_wdata[0];
  assign alert_cause_28_we = addr_hit[229] & reg_we & !reg_error;

  assign alert_cause_28_wd = reg_wdata[0];
  assign alert_cause_29_we = addr_hit[230] & reg_we & !reg_error;

  assign alert_cause_29_wd = reg_wdata[0];
  assign alert_cause_30_we = addr_hit[231] & reg_we & !reg_error;

  assign alert_cause_30_wd = reg_wdata[0];
  assign alert_cause_31_we = addr_hit[232] & reg_we & !reg_error;

  assign alert_cause_31_wd = reg_wdata[0];
  assign alert_cause_32_we = addr_hit[233] & reg_we & !reg_error;

  assign alert_cause_32_wd = reg_wdata[0];
  assign alert_cause_33_we = addr_hit[234] & reg_we & !reg_error;

  assign alert_cause_33_wd = reg_wdata[0];
  assign alert_cause_34_we = addr_hit[235] & reg_we & !reg_error;

  assign alert_cause_34_wd = reg_wdata[0];
  assign alert_cause_35_we = addr_hit[236] & reg_we & !reg_error;

  assign alert_cause_35_wd = reg_wdata[0];
  assign alert_cause_36_we = addr_hit[237] & reg_we & !reg_error;

  assign alert_cause_36_wd = reg_wdata[0];
  assign alert_cause_37_we = addr_hit[238] & reg_we & !reg_error;

  assign alert_cause_37_wd = reg_wdata[0];
  assign alert_cause_38_we = addr_hit[239] & reg_we & !reg_error;

  assign alert_cause_38_wd = reg_wdata[0];
  assign alert_cause_39_we = addr_hit[240] & reg_we & !reg_error;

  assign alert_cause_39_wd = reg_wdata[0];
  assign alert_cause_40_we = addr_hit[241] & reg_we & !reg_error;

  assign alert_cause_40_wd = reg_wdata[0];
  assign alert_cause_41_we = addr_hit[242] & reg_we & !reg_error;

  assign alert_cause_41_wd = reg_wdata[0];
  assign alert_cause_42_we = addr_hit[243] & reg_we & !reg_error;

  assign alert_cause_42_wd = reg_wdata[0];
  assign alert_cause_43_we = addr_hit[244] & reg_we & !reg_error;

  assign alert_cause_43_wd = reg_wdata[0];
  assign alert_cause_44_we = addr_hit[245] & reg_we & !reg_error;

  assign alert_cause_44_wd = reg_wdata[0];
  assign alert_cause_45_we = addr_hit[246] & reg_we & !reg_error;

  assign alert_cause_45_wd = reg_wdata[0];
  assign alert_cause_46_we = addr_hit[247] & reg_we & !reg_error;

  assign alert_cause_46_wd = reg_wdata[0];
  assign alert_cause_47_we = addr_hit[248] & reg_we & !reg_error;

  assign alert_cause_47_wd = reg_wdata[0];
  assign alert_cause_48_we = addr_hit[249] & reg_we & !reg_error;

  assign alert_cause_48_wd = reg_wdata[0];
  assign alert_cause_49_we = addr_hit[250] & reg_we & !reg_error;

  assign alert_cause_49_wd = reg_wdata[0];
  assign alert_cause_50_we = addr_hit[251] & reg_we & !reg_error;

  assign alert_cause_50_wd = reg_wdata[0];
  assign alert_cause_51_we = addr_hit[252] & reg_we & !reg_error;

  assign alert_cause_51_wd = reg_wdata[0];
  assign alert_cause_52_we = addr_hit[253] & reg_we & !reg_error;

  assign alert_cause_52_wd = reg_wdata[0];
  assign alert_cause_53_we = addr_hit[254] & reg_we & !reg_error;

  assign alert_cause_53_wd = reg_wdata[0];
  assign alert_cause_54_we = addr_hit[255] & reg_we & !reg_error;

  assign alert_cause_54_wd = reg_wdata[0];
  assign alert_cause_55_we = addr_hit[256] & reg_we & !reg_error;

  assign alert_cause_55_wd = reg_wdata[0];
  assign alert_cause_56_we = addr_hit[257] & reg_we & !reg_error;

  assign alert_cause_56_wd = reg_wdata[0];
  assign alert_cause_57_we = addr_hit[258] & reg_we & !reg_error;

  assign alert_cause_57_wd = reg_wdata[0];
  assign alert_cause_58_we = addr_hit[259] & reg_we & !reg_error;

  assign alert_cause_58_wd = reg_wdata[0];
  assign alert_cause_59_we = addr_hit[260] & reg_we & !reg_error;

  assign alert_cause_59_wd = reg_wdata[0];
  assign alert_cause_60_we = addr_hit[261] & reg_we & !reg_error;

  assign alert_cause_60_wd = reg_wdata[0];
  assign alert_cause_61_we = addr_hit[262] & reg_we & !reg_error;

  assign alert_cause_61_wd = reg_wdata[0];
  assign alert_cause_62_we = addr_hit[263] & reg_we & !reg_error;

  assign alert_cause_62_wd = reg_wdata[0];
  assign alert_cause_63_we = addr_hit[264] & reg_we & !reg_error;

  assign alert_cause_63_wd = reg_wdata[0];
  assign alert_cause_64_we = addr_hit[265] & reg_we & !reg_error;

  assign alert_cause_64_wd = reg_wdata[0];
  assign loc_alert_regwen_0_we = addr_hit[266] & reg_we & !reg_error;

  assign loc_alert_regwen_0_wd = reg_wdata[0];
  assign loc_alert_regwen_1_we = addr_hit[267] & reg_we & !reg_error;

  assign loc_alert_regwen_1_wd = reg_wdata[0];
  assign loc_alert_regwen_2_we = addr_hit[268] & reg_we & !reg_error;

  assign loc_alert_regwen_2_wd = reg_wdata[0];
  assign loc_alert_regwen_3_we = addr_hit[269] & reg_we & !reg_error;

  assign loc_alert_regwen_3_wd = reg_wdata[0];
  assign loc_alert_regwen_4_we = addr_hit[270] & reg_we & !reg_error;

  assign loc_alert_regwen_4_wd = reg_wdata[0];
  assign loc_alert_regwen_5_we = addr_hit[271] & reg_we & !reg_error;

  assign loc_alert_regwen_5_wd = reg_wdata[0];
  assign loc_alert_regwen_6_we = addr_hit[272] & reg_we & !reg_error;

  assign loc_alert_regwen_6_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_0_re = addr_hit[273] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_0_we = addr_hit[273] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_0_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_1_re = addr_hit[274] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_1_we = addr_hit[274] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_1_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_2_re = addr_hit[275] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_2_we = addr_hit[275] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_2_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_3_re = addr_hit[276] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_3_we = addr_hit[276] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_3_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_4_re = addr_hit[277] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_4_we = addr_hit[277] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_4_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_5_re = addr_hit[278] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_5_we = addr_hit[278] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_5_wd = reg_wdata[0];
  assign loc_alert_en_shadowed_6_re = addr_hit[279] & reg_re & !reg_error;
  assign loc_alert_en_shadowed_6_we = addr_hit[279] & reg_we & !reg_error;

  assign loc_alert_en_shadowed_6_wd = reg_wdata[0];
  assign loc_alert_class_shadowed_0_re = addr_hit[280] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_0_we = addr_hit[280] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_0_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_1_re = addr_hit[281] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_1_we = addr_hit[281] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_1_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_2_re = addr_hit[282] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_2_we = addr_hit[282] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_2_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_3_re = addr_hit[283] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_3_we = addr_hit[283] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_3_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_4_re = addr_hit[284] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_4_we = addr_hit[284] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_4_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_5_re = addr_hit[285] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_5_we = addr_hit[285] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_5_wd = reg_wdata[1:0];
  assign loc_alert_class_shadowed_6_re = addr_hit[286] & reg_re & !reg_error;
  assign loc_alert_class_shadowed_6_we = addr_hit[286] & reg_we & !reg_error;

  assign loc_alert_class_shadowed_6_wd = reg_wdata[1:0];
  assign loc_alert_cause_0_we = addr_hit[287] & reg_we & !reg_error;

  assign loc_alert_cause_0_wd = reg_wdata[0];
  assign loc_alert_cause_1_we = addr_hit[288] & reg_we & !reg_error;

  assign loc_alert_cause_1_wd = reg_wdata[0];
  assign loc_alert_cause_2_we = addr_hit[289] & reg_we & !reg_error;

  assign loc_alert_cause_2_wd = reg_wdata[0];
  assign loc_alert_cause_3_we = addr_hit[290] & reg_we & !reg_error;

  assign loc_alert_cause_3_wd = reg_wdata[0];
  assign loc_alert_cause_4_we = addr_hit[291] & reg_we & !reg_error;

  assign loc_alert_cause_4_wd = reg_wdata[0];
  assign loc_alert_cause_5_we = addr_hit[292] & reg_we & !reg_error;

  assign loc_alert_cause_5_wd = reg_wdata[0];
  assign loc_alert_cause_6_we = addr_hit[293] & reg_we & !reg_error;

  assign loc_alert_cause_6_wd = reg_wdata[0];
  assign classa_regwen_we = addr_hit[294] & reg_we & !reg_error;

  assign classa_regwen_wd = reg_wdata[0];
  assign classa_ctrl_shadowed_re = addr_hit[295] & reg_re & !reg_error;
  assign classa_ctrl_shadowed_we = addr_hit[295] & reg_we & !reg_error;

  assign classa_ctrl_shadowed_en_wd = reg_wdata[0];

  assign classa_ctrl_shadowed_lock_wd = reg_wdata[1];

  assign classa_ctrl_shadowed_en_e0_wd = reg_wdata[2];

  assign classa_ctrl_shadowed_en_e1_wd = reg_wdata[3];

  assign classa_ctrl_shadowed_en_e2_wd = reg_wdata[4];

  assign classa_ctrl_shadowed_en_e3_wd = reg_wdata[5];

  assign classa_ctrl_shadowed_map_e0_wd = reg_wdata[7:6];

  assign classa_ctrl_shadowed_map_e1_wd = reg_wdata[9:8];

  assign classa_ctrl_shadowed_map_e2_wd = reg_wdata[11:10];

  assign classa_ctrl_shadowed_map_e3_wd = reg_wdata[13:12];
  assign classa_clr_regwen_we = addr_hit[296] & reg_we & !reg_error;

  assign classa_clr_regwen_wd = reg_wdata[0];
  assign classa_clr_shadowed_re = addr_hit[297] & reg_re & !reg_error;
  assign classa_clr_shadowed_we = addr_hit[297] & reg_we & !reg_error;

  assign classa_clr_shadowed_wd = reg_wdata[0];
  assign classa_accum_cnt_re = addr_hit[298] & reg_re & !reg_error;
  assign classa_accum_thresh_shadowed_re = addr_hit[299] & reg_re & !reg_error;
  assign classa_accum_thresh_shadowed_we = addr_hit[299] & reg_we & !reg_error;

  assign classa_accum_thresh_shadowed_wd = reg_wdata[15:0];
  assign classa_timeout_cyc_shadowed_re = addr_hit[300] & reg_re & !reg_error;
  assign classa_timeout_cyc_shadowed_we = addr_hit[300] & reg_we & !reg_error;

  assign classa_timeout_cyc_shadowed_wd = reg_wdata[31:0];
  assign classa_crashdump_trigger_shadowed_re = addr_hit[301] & reg_re & !reg_error;
  assign classa_crashdump_trigger_shadowed_we = addr_hit[301] & reg_we & !reg_error;

  assign classa_crashdump_trigger_shadowed_wd = reg_wdata[1:0];
  assign classa_phase0_cyc_shadowed_re = addr_hit[302] & reg_re & !reg_error;
  assign classa_phase0_cyc_shadowed_we = addr_hit[302] & reg_we & !reg_error;

  assign classa_phase0_cyc_shadowed_wd = reg_wdata[31:0];
  assign classa_phase1_cyc_shadowed_re = addr_hit[303] & reg_re & !reg_error;
  assign classa_phase1_cyc_shadowed_we = addr_hit[303] & reg_we & !reg_error;

  assign classa_phase1_cyc_shadowed_wd = reg_wdata[31:0];
  assign classa_phase2_cyc_shadowed_re = addr_hit[304] & reg_re & !reg_error;
  assign classa_phase2_cyc_shadowed_we = addr_hit[304] & reg_we & !reg_error;

  assign classa_phase2_cyc_shadowed_wd = reg_wdata[31:0];
  assign classa_phase3_cyc_shadowed_re = addr_hit[305] & reg_re & !reg_error;
  assign classa_phase3_cyc_shadowed_we = addr_hit[305] & reg_we & !reg_error;

  assign classa_phase3_cyc_shadowed_wd = reg_wdata[31:0];
  assign classa_esc_cnt_re = addr_hit[306] & reg_re & !reg_error;
  assign classa_state_re = addr_hit[307] & reg_re & !reg_error;
  assign classb_regwen_we = addr_hit[308] & reg_we & !reg_error;

  assign classb_regwen_wd = reg_wdata[0];
  assign classb_ctrl_shadowed_re = addr_hit[309] & reg_re & !reg_error;
  assign classb_ctrl_shadowed_we = addr_hit[309] & reg_we & !reg_error;

  assign classb_ctrl_shadowed_en_wd = reg_wdata[0];

  assign classb_ctrl_shadowed_lock_wd = reg_wdata[1];

  assign classb_ctrl_shadowed_en_e0_wd = reg_wdata[2];

  assign classb_ctrl_shadowed_en_e1_wd = reg_wdata[3];

  assign classb_ctrl_shadowed_en_e2_wd = reg_wdata[4];

  assign classb_ctrl_shadowed_en_e3_wd = reg_wdata[5];

  assign classb_ctrl_shadowed_map_e0_wd = reg_wdata[7:6];

  assign classb_ctrl_shadowed_map_e1_wd = reg_wdata[9:8];

  assign classb_ctrl_shadowed_map_e2_wd = reg_wdata[11:10];

  assign classb_ctrl_shadowed_map_e3_wd = reg_wdata[13:12];
  assign classb_clr_regwen_we = addr_hit[310] & reg_we & !reg_error;

  assign classb_clr_regwen_wd = reg_wdata[0];
  assign classb_clr_shadowed_re = addr_hit[311] & reg_re & !reg_error;
  assign classb_clr_shadowed_we = addr_hit[311] & reg_we & !reg_error;

  assign classb_clr_shadowed_wd = reg_wdata[0];
  assign classb_accum_cnt_re = addr_hit[312] & reg_re & !reg_error;
  assign classb_accum_thresh_shadowed_re = addr_hit[313] & reg_re & !reg_error;
  assign classb_accum_thresh_shadowed_we = addr_hit[313] & reg_we & !reg_error;

  assign classb_accum_thresh_shadowed_wd = reg_wdata[15:0];
  assign classb_timeout_cyc_shadowed_re = addr_hit[314] & reg_re & !reg_error;
  assign classb_timeout_cyc_shadowed_we = addr_hit[314] & reg_we & !reg_error;

  assign classb_timeout_cyc_shadowed_wd = reg_wdata[31:0];
  assign classb_crashdump_trigger_shadowed_re = addr_hit[315] & reg_re & !reg_error;
  assign classb_crashdump_trigger_shadowed_we = addr_hit[315] & reg_we & !reg_error;

  assign classb_crashdump_trigger_shadowed_wd = reg_wdata[1:0];
  assign classb_phase0_cyc_shadowed_re = addr_hit[316] & reg_re & !reg_error;
  assign classb_phase0_cyc_shadowed_we = addr_hit[316] & reg_we & !reg_error;

  assign classb_phase0_cyc_shadowed_wd = reg_wdata[31:0];
  assign classb_phase1_cyc_shadowed_re = addr_hit[317] & reg_re & !reg_error;
  assign classb_phase1_cyc_shadowed_we = addr_hit[317] & reg_we & !reg_error;

  assign classb_phase1_cyc_shadowed_wd = reg_wdata[31:0];
  assign classb_phase2_cyc_shadowed_re = addr_hit[318] & reg_re & !reg_error;
  assign classb_phase2_cyc_shadowed_we = addr_hit[318] & reg_we & !reg_error;

  assign classb_phase2_cyc_shadowed_wd = reg_wdata[31:0];
  assign classb_phase3_cyc_shadowed_re = addr_hit[319] & reg_re & !reg_error;
  assign classb_phase3_cyc_shadowed_we = addr_hit[319] & reg_we & !reg_error;

  assign classb_phase3_cyc_shadowed_wd = reg_wdata[31:0];
  assign classb_esc_cnt_re = addr_hit[320] & reg_re & !reg_error;
  assign classb_state_re = addr_hit[321] & reg_re & !reg_error;
  assign classc_regwen_we = addr_hit[322] & reg_we & !reg_error;

  assign classc_regwen_wd = reg_wdata[0];
  assign classc_ctrl_shadowed_re = addr_hit[323] & reg_re & !reg_error;
  assign classc_ctrl_shadowed_we = addr_hit[323] & reg_we & !reg_error;

  assign classc_ctrl_shadowed_en_wd = reg_wdata[0];

  assign classc_ctrl_shadowed_lock_wd = reg_wdata[1];

  assign classc_ctrl_shadowed_en_e0_wd = reg_wdata[2];

  assign classc_ctrl_shadowed_en_e1_wd = reg_wdata[3];

  assign classc_ctrl_shadowed_en_e2_wd = reg_wdata[4];

  assign classc_ctrl_shadowed_en_e3_wd = reg_wdata[5];

  assign classc_ctrl_shadowed_map_e0_wd = reg_wdata[7:6];

  assign classc_ctrl_shadowed_map_e1_wd = reg_wdata[9:8];

  assign classc_ctrl_shadowed_map_e2_wd = reg_wdata[11:10];

  assign classc_ctrl_shadowed_map_e3_wd = reg_wdata[13:12];
  assign classc_clr_regwen_we = addr_hit[324] & reg_we & !reg_error;

  assign classc_clr_regwen_wd = reg_wdata[0];
  assign classc_clr_shadowed_re = addr_hit[325] & reg_re & !reg_error;
  assign classc_clr_shadowed_we = addr_hit[325] & reg_we & !reg_error;

  assign classc_clr_shadowed_wd = reg_wdata[0];
  assign classc_accum_cnt_re = addr_hit[326] & reg_re & !reg_error;
  assign classc_accum_thresh_shadowed_re = addr_hit[327] & reg_re & !reg_error;
  assign classc_accum_thresh_shadowed_we = addr_hit[327] & reg_we & !reg_error;

  assign classc_accum_thresh_shadowed_wd = reg_wdata[15:0];
  assign classc_timeout_cyc_shadowed_re = addr_hit[328] & reg_re & !reg_error;
  assign classc_timeout_cyc_shadowed_we = addr_hit[328] & reg_we & !reg_error;

  assign classc_timeout_cyc_shadowed_wd = reg_wdata[31:0];
  assign classc_crashdump_trigger_shadowed_re = addr_hit[329] & reg_re & !reg_error;
  assign classc_crashdump_trigger_shadowed_we = addr_hit[329] & reg_we & !reg_error;

  assign classc_crashdump_trigger_shadowed_wd = reg_wdata[1:0];
  assign classc_phase0_cyc_shadowed_re = addr_hit[330] & reg_re & !reg_error;
  assign classc_phase0_cyc_shadowed_we = addr_hit[330] & reg_we & !reg_error;

  assign classc_phase0_cyc_shadowed_wd = reg_wdata[31:0];
  assign classc_phase1_cyc_shadowed_re = addr_hit[331] & reg_re & !reg_error;
  assign classc_phase1_cyc_shadowed_we = addr_hit[331] & reg_we & !reg_error;

  assign classc_phase1_cyc_shadowed_wd = reg_wdata[31:0];
  assign classc_phase2_cyc_shadowed_re = addr_hit[332] & reg_re & !reg_error;
  assign classc_phase2_cyc_shadowed_we = addr_hit[332] & reg_we & !reg_error;

  assign classc_phase2_cyc_shadowed_wd = reg_wdata[31:0];
  assign classc_phase3_cyc_shadowed_re = addr_hit[333] & reg_re & !reg_error;
  assign classc_phase3_cyc_shadowed_we = addr_hit[333] & reg_we & !reg_error;

  assign classc_phase3_cyc_shadowed_wd = reg_wdata[31:0];
  assign classc_esc_cnt_re = addr_hit[334] & reg_re & !reg_error;
  assign classc_state_re = addr_hit[335] & reg_re & !reg_error;
  assign classd_regwen_we = addr_hit[336] & reg_we & !reg_error;

  assign classd_regwen_wd = reg_wdata[0];
  assign classd_ctrl_shadowed_re = addr_hit[337] & reg_re & !reg_error;
  assign classd_ctrl_shadowed_we = addr_hit[337] & reg_we & !reg_error;

  assign classd_ctrl_shadowed_en_wd = reg_wdata[0];

  assign classd_ctrl_shadowed_lock_wd = reg_wdata[1];

  assign classd_ctrl_shadowed_en_e0_wd = reg_wdata[2];

  assign classd_ctrl_shadowed_en_e1_wd = reg_wdata[3];

  assign classd_ctrl_shadowed_en_e2_wd = reg_wdata[4];

  assign classd_ctrl_shadowed_en_e3_wd = reg_wdata[5];

  assign classd_ctrl_shadowed_map_e0_wd = reg_wdata[7:6];

  assign classd_ctrl_shadowed_map_e1_wd = reg_wdata[9:8];

  assign classd_ctrl_shadowed_map_e2_wd = reg_wdata[11:10];

  assign classd_ctrl_shadowed_map_e3_wd = reg_wdata[13:12];
  assign classd_clr_regwen_we = addr_hit[338] & reg_we & !reg_error;

  assign classd_clr_regwen_wd = reg_wdata[0];
  assign classd_clr_shadowed_re = addr_hit[339] & reg_re & !reg_error;
  assign classd_clr_shadowed_we = addr_hit[339] & reg_we & !reg_error;

  assign classd_clr_shadowed_wd = reg_wdata[0];
  assign classd_accum_cnt_re = addr_hit[340] & reg_re & !reg_error;
  assign classd_accum_thresh_shadowed_re = addr_hit[341] & reg_re & !reg_error;
  assign classd_accum_thresh_shadowed_we = addr_hit[341] & reg_we & !reg_error;

  assign classd_accum_thresh_shadowed_wd = reg_wdata[15:0];
  assign classd_timeout_cyc_shadowed_re = addr_hit[342] & reg_re & !reg_error;
  assign classd_timeout_cyc_shadowed_we = addr_hit[342] & reg_we & !reg_error;

  assign classd_timeout_cyc_shadowed_wd = reg_wdata[31:0];
  assign classd_crashdump_trigger_shadowed_re = addr_hit[343] & reg_re & !reg_error;
  assign classd_crashdump_trigger_shadowed_we = addr_hit[343] & reg_we & !reg_error;

  assign classd_crashdump_trigger_shadowed_wd = reg_wdata[1:0];
  assign classd_phase0_cyc_shadowed_re = addr_hit[344] & reg_re & !reg_error;
  assign classd_phase0_cyc_shadowed_we = addr_hit[344] & reg_we & !reg_error;

  assign classd_phase0_cyc_shadowed_wd = reg_wdata[31:0];
  assign classd_phase1_cyc_shadowed_re = addr_hit[345] & reg_re & !reg_error;
  assign classd_phase1_cyc_shadowed_we = addr_hit[345] & reg_we & !reg_error;

  assign classd_phase1_cyc_shadowed_wd = reg_wdata[31:0];
  assign classd_phase2_cyc_shadowed_re = addr_hit[346] & reg_re & !reg_error;
  assign classd_phase2_cyc_shadowed_we = addr_hit[346] & reg_we & !reg_error;

  assign classd_phase2_cyc_shadowed_wd = reg_wdata[31:0];
  assign classd_phase3_cyc_shadowed_re = addr_hit[347] & reg_re & !reg_error;
  assign classd_phase3_cyc_shadowed_we = addr_hit[347] & reg_we & !reg_error;

  assign classd_phase3_cyc_shadowed_wd = reg_wdata[31:0];
  assign classd_esc_cnt_re = addr_hit[348] & reg_re & !reg_error;
  assign classd_state_re = addr_hit[349] & reg_re & !reg_error;

  // Assign write-enables to checker logic vector.
  always_comb begin
    reg_we_check = '0;
    reg_we_check[0] = intr_state_we;
    reg_we_check[1] = intr_enable_we;
    reg_we_check[2] = intr_test_we;
    reg_we_check[3] = ping_timer_regwen_we;
    reg_we_check[4] = ping_timeout_cyc_shadowed_gated_we;
    reg_we_check[5] = ping_timer_en_shadowed_gated_we;
    reg_we_check[6] = alert_regwen_0_we;
    reg_we_check[7] = alert_regwen_1_we;
    reg_we_check[8] = alert_regwen_2_we;
    reg_we_check[9] = alert_regwen_3_we;
    reg_we_check[10] = alert_regwen_4_we;
    reg_we_check[11] = alert_regwen_5_we;
    reg_we_check[12] = alert_regwen_6_we;
    reg_we_check[13] = alert_regwen_7_we;
    reg_we_check[14] = alert_regwen_8_we;
    reg_we_check[15] = alert_regwen_9_we;
    reg_we_check[16] = alert_regwen_10_we;
    reg_we_check[17] = alert_regwen_11_we;
    reg_we_check[18] = alert_regwen_12_we;
    reg_we_check[19] = alert_regwen_13_we;
    reg_we_check[20] = alert_regwen_14_we;
    reg_we_check[21] = alert_regwen_15_we;
    reg_we_check[22] = alert_regwen_16_we;
    reg_we_check[23] = alert_regwen_17_we;
    reg_we_check[24] = alert_regwen_18_we;
    reg_we_check[25] = alert_regwen_19_we;
    reg_we_check[26] = alert_regwen_20_we;
    reg_we_check[27] = alert_regwen_21_we;
    reg_we_check[28] = alert_regwen_22_we;
    reg_we_check[29] = alert_regwen_23_we;
    reg_we_check[30] = alert_regwen_24_we;
    reg_we_check[31] = alert_regwen_25_we;
    reg_we_check[32] = alert_regwen_26_we;
    reg_we_check[33] = alert_regwen_27_we;
    reg_we_check[34] = alert_regwen_28_we;
    reg_we_check[35] = alert_regwen_29_we;
    reg_we_check[36] = alert_regwen_30_we;
    reg_we_check[37] = alert_regwen_31_we;
    reg_we_check[38] = alert_regwen_32_we;
    reg_we_check[39] = alert_regwen_33_we;
    reg_we_check[40] = alert_regwen_34_we;
    reg_we_check[41] = alert_regwen_35_we;
    reg_we_check[42] = alert_regwen_36_we;
    reg_we_check[43] = alert_regwen_37_we;
    reg_we_check[44] = alert_regwen_38_we;
    reg_we_check[45] = alert_regwen_39_we;
    reg_we_check[46] = alert_regwen_40_we;
    reg_we_check[47] = alert_regwen_41_we;
    reg_we_check[48] = alert_regwen_42_we;
    reg_we_check[49] = alert_regwen_43_we;
    reg_we_check[50] = alert_regwen_44_we;
    reg_we_check[51] = alert_regwen_45_we;
    reg_we_check[52] = alert_regwen_46_we;
    reg_we_check[53] = alert_regwen_47_we;
    reg_we_check[54] = alert_regwen_48_we;
    reg_we_check[55] = alert_regwen_49_we;
    reg_we_check[56] = alert_regwen_50_we;
    reg_we_check[57] = alert_regwen_51_we;
    reg_we_check[58] = alert_regwen_52_we;
    reg_we_check[59] = alert_regwen_53_we;
    reg_we_check[60] = alert_regwen_54_we;
    reg_we_check[61] = alert_regwen_55_we;
    reg_we_check[62] = alert_regwen_56_we;
    reg_we_check[63] = alert_regwen_57_we;
    reg_we_check[64] = alert_regwen_58_we;
    reg_we_check[65] = alert_regwen_59_we;
    reg_we_check[66] = alert_regwen_60_we;
    reg_we_check[67] = alert_regwen_61_we;
    reg_we_check[68] = alert_regwen_62_we;
    reg_we_check[69] = alert_regwen_63_we;
    reg_we_check[70] = alert_regwen_64_we;
    reg_we_check[71] = alert_en_shadowed_0_gated_we;
    reg_we_check[72] = alert_en_shadowed_1_gated_we;
    reg_we_check[73] = alert_en_shadowed_2_gated_we;
    reg_we_check[74] = alert_en_shadowed_3_gated_we;
    reg_we_check[75] = alert_en_shadowed_4_gated_we;
    reg_we_check[76] = alert_en_shadowed_5_gated_we;
    reg_we_check[77] = alert_en_shadowed_6_gated_we;
    reg_we_check[78] = alert_en_shadowed_7_gated_we;
    reg_we_check[79] = alert_en_shadowed_8_gated_we;
    reg_we_check[80] = alert_en_shadowed_9_gated_we;
    reg_we_check[81] = alert_en_shadowed_10_gated_we;
    reg_we_check[82] = alert_en_shadowed_11_gated_we;
    reg_we_check[83] = alert_en_shadowed_12_gated_we;
    reg_we_check[84] = alert_en_shadowed_13_gated_we;
    reg_we_check[85] = alert_en_shadowed_14_gated_we;
    reg_we_check[86] = alert_en_shadowed_15_gated_we;
    reg_we_check[87] = alert_en_shadowed_16_gated_we;
    reg_we_check[88] = alert_en_shadowed_17_gated_we;
    reg_we_check[89] = alert_en_shadowed_18_gated_we;
    reg_we_check[90] = alert_en_shadowed_19_gated_we;
    reg_we_check[91] = alert_en_shadowed_20_gated_we;
    reg_we_check[92] = alert_en_shadowed_21_gated_we;
    reg_we_check[93] = alert_en_shadowed_22_gated_we;
    reg_we_check[94] = alert_en_shadowed_23_gated_we;
    reg_we_check[95] = alert_en_shadowed_24_gated_we;
    reg_we_check[96] = alert_en_shadowed_25_gated_we;
    reg_we_check[97] = alert_en_shadowed_26_gated_we;
    reg_we_check[98] = alert_en_shadowed_27_gated_we;
    reg_we_check[99] = alert_en_shadowed_28_gated_we;
    reg_we_check[100] = alert_en_shadowed_29_gated_we;
    reg_we_check[101] = alert_en_shadowed_30_gated_we;
    reg_we_check[102] = alert_en_shadowed_31_gated_we;
    reg_we_check[103] = alert_en_shadowed_32_gated_we;
    reg_we_check[104] = alert_en_shadowed_33_gated_we;
    reg_we_check[105] = alert_en_shadowed_34_gated_we;
    reg_we_check[106] = alert_en_shadowed_35_gated_we;
    reg_we_check[107] = alert_en_shadowed_36_gated_we;
    reg_we_check[108] = alert_en_shadowed_37_gated_we;
    reg_we_check[109] = alert_en_shadowed_38_gated_we;
    reg_we_check[110] = alert_en_shadowed_39_gated_we;
    reg_we_check[111] = alert_en_shadowed_40_gated_we;
    reg_we_check[112] = alert_en_shadowed_41_gated_we;
    reg_we_check[113] = alert_en_shadowed_42_gated_we;
    reg_we_check[114] = alert_en_shadowed_43_gated_we;
    reg_we_check[115] = alert_en_shadowed_44_gated_we;
    reg_we_check[116] = alert_en_shadowed_45_gated_we;
    reg_we_check[117] = alert_en_shadowed_46_gated_we;
    reg_we_check[118] = alert_en_shadowed_47_gated_we;
    reg_we_check[119] = alert_en_shadowed_48_gated_we;
    reg_we_check[120] = alert_en_shadowed_49_gated_we;
    reg_we_check[121] = alert_en_shadowed_50_gated_we;
    reg_we_check[122] = alert_en_shadowed_51_gated_we;
    reg_we_check[123] = alert_en_shadowed_52_gated_we;
    reg_we_check[124] = alert_en_shadowed_53_gated_we;
    reg_we_check[125] = alert_en_shadowed_54_gated_we;
    reg_we_check[126] = alert_en_shadowed_55_gated_we;
    reg_we_check[127] = alert_en_shadowed_56_gated_we;
    reg_we_check[128] = alert_en_shadowed_57_gated_we;
    reg_we_check[129] = alert_en_shadowed_58_gated_we;
    reg_we_check[130] = alert_en_shadowed_59_gated_we;
    reg_we_check[131] = alert_en_shadowed_60_gated_we;
    reg_we_check[132] = alert_en_shadowed_61_gated_we;
    reg_we_check[133] = alert_en_shadowed_62_gated_we;
    reg_we_check[134] = alert_en_shadowed_63_gated_we;
    reg_we_check[135] = alert_en_shadowed_64_gated_we;
    reg_we_check[136] = alert_class_shadowed_0_gated_we;
    reg_we_check[137] = alert_class_shadowed_1_gated_we;
    reg_we_check[138] = alert_class_shadowed_2_gated_we;
    reg_we_check[139] = alert_class_shadowed_3_gated_we;
    reg_we_check[140] = alert_class_shadowed_4_gated_we;
    reg_we_check[141] = alert_class_shadowed_5_gated_we;
    reg_we_check[142] = alert_class_shadowed_6_gated_we;
    reg_we_check[143] = alert_class_shadowed_7_gated_we;
    reg_we_check[144] = alert_class_shadowed_8_gated_we;
    reg_we_check[145] = alert_class_shadowed_9_gated_we;
    reg_we_check[146] = alert_class_shadowed_10_gated_we;
    reg_we_check[147] = alert_class_shadowed_11_gated_we;
    reg_we_check[148] = alert_class_shadowed_12_gated_we;
    reg_we_check[149] = alert_class_shadowed_13_gated_we;
    reg_we_check[150] = alert_class_shadowed_14_gated_we;
    reg_we_check[151] = alert_class_shadowed_15_gated_we;
    reg_we_check[152] = alert_class_shadowed_16_gated_we;
    reg_we_check[153] = alert_class_shadowed_17_gated_we;
    reg_we_check[154] = alert_class_shadowed_18_gated_we;
    reg_we_check[155] = alert_class_shadowed_19_gated_we;
    reg_we_check[156] = alert_class_shadowed_20_gated_we;
    reg_we_check[157] = alert_class_shadowed_21_gated_we;
    reg_we_check[158] = alert_class_shadowed_22_gated_we;
    reg_we_check[159] = alert_class_shadowed_23_gated_we;
    reg_we_check[160] = alert_class_shadowed_24_gated_we;
    reg_we_check[161] = alert_class_shadowed_25_gated_we;
    reg_we_check[162] = alert_class_shadowed_26_gated_we;
    reg_we_check[163] = alert_class_shadowed_27_gated_we;
    reg_we_check[164] = alert_class_shadowed_28_gated_we;
    reg_we_check[165] = alert_class_shadowed_29_gated_we;
    reg_we_check[166] = alert_class_shadowed_30_gated_we;
    reg_we_check[167] = alert_class_shadowed_31_gated_we;
    reg_we_check[168] = alert_class_shadowed_32_gated_we;
    reg_we_check[169] = alert_class_shadowed_33_gated_we;
    reg_we_check[170] = alert_class_shadowed_34_gated_we;
    reg_we_check[171] = alert_class_shadowed_35_gated_we;
    reg_we_check[172] = alert_class_shadowed_36_gated_we;
    reg_we_check[173] = alert_class_shadowed_37_gated_we;
    reg_we_check[174] = alert_class_shadowed_38_gated_we;
    reg_we_check[175] = alert_class_shadowed_39_gated_we;
    reg_we_check[176] = alert_class_shadowed_40_gated_we;
    reg_we_check[177] = alert_class_shadowed_41_gated_we;
    reg_we_check[178] = alert_class_shadowed_42_gated_we;
    reg_we_check[179] = alert_class_shadowed_43_gated_we;
    reg_we_check[180] = alert_class_shadowed_44_gated_we;
    reg_we_check[181] = alert_class_shadowed_45_gated_we;
    reg_we_check[182] = alert_class_shadowed_46_gated_we;
    reg_we_check[183] = alert_class_shadowed_47_gated_we;
    reg_we_check[184] = alert_class_shadowed_48_gated_we;
    reg_we_check[185] = alert_class_shadowed_49_gated_we;
    reg_we_check[186] = alert_class_shadowed_50_gated_we;
    reg_we_check[187] = alert_class_shadowed_51_gated_we;
    reg_we_check[188] = alert_class_shadowed_52_gated_we;
    reg_we_check[189] = alert_class_shadowed_53_gated_we;
    reg_we_check[190] = alert_class_shadowed_54_gated_we;
    reg_we_check[191] = alert_class_shadowed_55_gated_we;
    reg_we_check[192] = alert_class_shadowed_56_gated_we;
    reg_we_check[193] = alert_class_shadowed_57_gated_we;
    reg_we_check[194] = alert_class_shadowed_58_gated_we;
    reg_we_check[195] = alert_class_shadowed_59_gated_we;
    reg_we_check[196] = alert_class_shadowed_60_gated_we;
    reg_we_check[197] = alert_class_shadowed_61_gated_we;
    reg_we_check[198] = alert_class_shadowed_62_gated_we;
    reg_we_check[199] = alert_class_shadowed_63_gated_we;
    reg_we_check[200] = alert_class_shadowed_64_gated_we;
    reg_we_check[201] = alert_cause_0_we;
    reg_we_check[202] = alert_cause_1_we;
    reg_we_check[203] = alert_cause_2_we;
    reg_we_check[204] = alert_cause_3_we;
    reg_we_check[205] = alert_cause_4_we;
    reg_we_check[206] = alert_cause_5_we;
    reg_we_check[207] = alert_cause_6_we;
    reg_we_check[208] = alert_cause_7_we;
    reg_we_check[209] = alert_cause_8_we;
    reg_we_check[210] = alert_cause_9_we;
    reg_we_check[211] = alert_cause_10_we;
    reg_we_check[212] = alert_cause_11_we;
    reg_we_check[213] = alert_cause_12_we;
    reg_we_check[214] = alert_cause_13_we;
    reg_we_check[215] = alert_cause_14_we;
    reg_we_check[216] = alert_cause_15_we;
    reg_we_check[217] = alert_cause_16_we;
    reg_we_check[218] = alert_cause_17_we;
    reg_we_check[219] = alert_cause_18_we;
    reg_we_check[220] = alert_cause_19_we;
    reg_we_check[221] = alert_cause_20_we;
    reg_we_check[222] = alert_cause_21_we;
    reg_we_check[223] = alert_cause_22_we;
    reg_we_check[224] = alert_cause_23_we;
    reg_we_check[225] = alert_cause_24_we;
    reg_we_check[226] = alert_cause_25_we;
    reg_we_check[227] = alert_cause_26_we;
    reg_we_check[228] = alert_cause_27_we;
    reg_we_check[229] = alert_cause_28_we;
    reg_we_check[230] = alert_cause_29_we;
    reg_we_check[231] = alert_cause_30_we;
    reg_we_check[232] = alert_cause_31_we;
    reg_we_check[233] = alert_cause_32_we;
    reg_we_check[234] = alert_cause_33_we;
    reg_we_check[235] = alert_cause_34_we;
    reg_we_check[236] = alert_cause_35_we;
    reg_we_check[237] = alert_cause_36_we;
    reg_we_check[238] = alert_cause_37_we;
    reg_we_check[239] = alert_cause_38_we;
    reg_we_check[240] = alert_cause_39_we;
    reg_we_check[241] = alert_cause_40_we;
    reg_we_check[242] = alert_cause_41_we;
    reg_we_check[243] = alert_cause_42_we;
    reg_we_check[244] = alert_cause_43_we;
    reg_we_check[245] = alert_cause_44_we;
    reg_we_check[246] = alert_cause_45_we;
    reg_we_check[247] = alert_cause_46_we;
    reg_we_check[248] = alert_cause_47_we;
    reg_we_check[249] = alert_cause_48_we;
    reg_we_check[250] = alert_cause_49_we;
    reg_we_check[251] = alert_cause_50_we;
    reg_we_check[252] = alert_cause_51_we;
    reg_we_check[253] = alert_cause_52_we;
    reg_we_check[254] = alert_cause_53_we;
    reg_we_check[255] = alert_cause_54_we;
    reg_we_check[256] = alert_cause_55_we;
    reg_we_check[257] = alert_cause_56_we;
    reg_we_check[258] = alert_cause_57_we;
    reg_we_check[259] = alert_cause_58_we;
    reg_we_check[260] = alert_cause_59_we;
    reg_we_check[261] = alert_cause_60_we;
    reg_we_check[262] = alert_cause_61_we;
    reg_we_check[263] = alert_cause_62_we;
    reg_we_check[264] = alert_cause_63_we;
    reg_we_check[265] = alert_cause_64_we;
    reg_we_check[266] = loc_alert_regwen_0_we;
    reg_we_check[267] = loc_alert_regwen_1_we;
    reg_we_check[268] = loc_alert_regwen_2_we;
    reg_we_check[269] = loc_alert_regwen_3_we;
    reg_we_check[270] = loc_alert_regwen_4_we;
    reg_we_check[271] = loc_alert_regwen_5_we;
    reg_we_check[272] = loc_alert_regwen_6_we;
    reg_we_check[273] = loc_alert_en_shadowed_0_gated_we;
    reg_we_check[274] = loc_alert_en_shadowed_1_gated_we;
    reg_we_check[275] = loc_alert_en_shadowed_2_gated_we;
    reg_we_check[276] = loc_alert_en_shadowed_3_gated_we;
    reg_we_check[277] = loc_alert_en_shadowed_4_gated_we;
    reg_we_check[278] = loc_alert_en_shadowed_5_gated_we;
    reg_we_check[279] = loc_alert_en_shadowed_6_gated_we;
    reg_we_check[280] = loc_alert_class_shadowed_0_gated_we;
    reg_we_check[281] = loc_alert_class_shadowed_1_gated_we;
    reg_we_check[282] = loc_alert_class_shadowed_2_gated_we;
    reg_we_check[283] = loc_alert_class_shadowed_3_gated_we;
    reg_we_check[284] = loc_alert_class_shadowed_4_gated_we;
    reg_we_check[285] = loc_alert_class_shadowed_5_gated_we;
    reg_we_check[286] = loc_alert_class_shadowed_6_gated_we;
    reg_we_check[287] = loc_alert_cause_0_we;
    reg_we_check[288] = loc_alert_cause_1_we;
    reg_we_check[289] = loc_alert_cause_2_we;
    reg_we_check[290] = loc_alert_cause_3_we;
    reg_we_check[291] = loc_alert_cause_4_we;
    reg_we_check[292] = loc_alert_cause_5_we;
    reg_we_check[293] = loc_alert_cause_6_we;
    reg_we_check[294] = classa_regwen_we;
    reg_we_check[295] = classa_ctrl_shadowed_gated_we;
    reg_we_check[296] = classa_clr_regwen_we;
    reg_we_check[297] = classa_clr_shadowed_gated_we;
    reg_we_check[298] = 1'b0;
    reg_we_check[299] = classa_accum_thresh_shadowed_gated_we;
    reg_we_check[300] = classa_timeout_cyc_shadowed_gated_we;
    reg_we_check[301] = classa_crashdump_trigger_shadowed_gated_we;
    reg_we_check[302] = classa_phase0_cyc_shadowed_gated_we;
    reg_we_check[303] = classa_phase1_cyc_shadowed_gated_we;
    reg_we_check[304] = classa_phase2_cyc_shadowed_gated_we;
    reg_we_check[305] = classa_phase3_cyc_shadowed_gated_we;
    reg_we_check[306] = 1'b0;
    reg_we_check[307] = 1'b0;
    reg_we_check[308] = classb_regwen_we;
    reg_we_check[309] = classb_ctrl_shadowed_gated_we;
    reg_we_check[310] = classb_clr_regwen_we;
    reg_we_check[311] = classb_clr_shadowed_gated_we;
    reg_we_check[312] = 1'b0;
    reg_we_check[313] = classb_accum_thresh_shadowed_gated_we;
    reg_we_check[314] = classb_timeout_cyc_shadowed_gated_we;
    reg_we_check[315] = classb_crashdump_trigger_shadowed_gated_we;
    reg_we_check[316] = classb_phase0_cyc_shadowed_gated_we;
    reg_we_check[317] = classb_phase1_cyc_shadowed_gated_we;
    reg_we_check[318] = classb_phase2_cyc_shadowed_gated_we;
    reg_we_check[319] = classb_phase3_cyc_shadowed_gated_we;
    reg_we_check[320] = 1'b0;
    reg_we_check[321] = 1'b0;
    reg_we_check[322] = classc_regwen_we;
    reg_we_check[323] = classc_ctrl_shadowed_gated_we;
    reg_we_check[324] = classc_clr_regwen_we;
    reg_we_check[325] = classc_clr_shadowed_gated_we;
    reg_we_check[326] = 1'b0;
    reg_we_check[327] = classc_accum_thresh_shadowed_gated_we;
    reg_we_check[328] = classc_timeout_cyc_shadowed_gated_we;
    reg_we_check[329] = classc_crashdump_trigger_shadowed_gated_we;
    reg_we_check[330] = classc_phase0_cyc_shadowed_gated_we;
    reg_we_check[331] = classc_phase1_cyc_shadowed_gated_we;
    reg_we_check[332] = classc_phase2_cyc_shadowed_gated_we;
    reg_we_check[333] = classc_phase3_cyc_shadowed_gated_we;
    reg_we_check[334] = 1'b0;
    reg_we_check[335] = 1'b0;
    reg_we_check[336] = classd_regwen_we;
    reg_we_check[337] = classd_ctrl_shadowed_gated_we;
    reg_we_check[338] = classd_clr_regwen_we;
    reg_we_check[339] = classd_clr_shadowed_gated_we;
    reg_we_check[340] = 1'b0;
    reg_we_check[341] = classd_accum_thresh_shadowed_gated_we;
    reg_we_check[342] = classd_timeout_cyc_shadowed_gated_we;
    reg_we_check[343] = classd_crashdump_trigger_shadowed_gated_we;
    reg_we_check[344] = classd_phase0_cyc_shadowed_gated_we;
    reg_we_check[345] = classd_phase1_cyc_shadowed_gated_we;
    reg_we_check[346] = classd_phase2_cyc_shadowed_gated_we;
    reg_we_check[347] = classd_phase3_cyc_shadowed_gated_we;
    reg_we_check[348] = 1'b0;
    reg_we_check[349] = 1'b0;
  end

  // Read data return
  always_comb begin
    reg_rdata_next = '0;
    unique case (1'b1)
      addr_hit[0]: begin
        reg_rdata_next[0] = intr_state_classa_qs;
        reg_rdata_next[1] = intr_state_classb_qs;
        reg_rdata_next[2] = intr_state_classc_qs;
        reg_rdata_next[3] = intr_state_classd_qs;
      end

      addr_hit[1]: begin
        reg_rdata_next[0] = intr_enable_classa_qs;
        reg_rdata_next[1] = intr_enable_classb_qs;
        reg_rdata_next[2] = intr_enable_classc_qs;
        reg_rdata_next[3] = intr_enable_classd_qs;
      end

      addr_hit[2]: begin
        reg_rdata_next[0] = '0;
        reg_rdata_next[1] = '0;
        reg_rdata_next[2] = '0;
        reg_rdata_next[3] = '0;
      end

      addr_hit[3]: begin
        reg_rdata_next[0] = ping_timer_regwen_qs;
      end

      addr_hit[4]: begin
        reg_rdata_next[15:0] = ping_timeout_cyc_shadowed_qs;
      end

      addr_hit[5]: begin
        reg_rdata_next[0] = ping_timer_en_shadowed_qs;
      end

      addr_hit[6]: begin
        reg_rdata_next[0] = alert_regwen_0_qs;
      end

      addr_hit[7]: begin
        reg_rdata_next[0] = alert_regwen_1_qs;
      end

      addr_hit[8]: begin
        reg_rdata_next[0] = alert_regwen_2_qs;
      end

      addr_hit[9]: begin
        reg_rdata_next[0] = alert_regwen_3_qs;
      end

      addr_hit[10]: begin
        reg_rdata_next[0] = alert_regwen_4_qs;
      end

      addr_hit[11]: begin
        reg_rdata_next[0] = alert_regwen_5_qs;
      end

      addr_hit[12]: begin
        reg_rdata_next[0] = alert_regwen_6_qs;
      end

      addr_hit[13]: begin
        reg_rdata_next[0] = alert_regwen_7_qs;
      end

      addr_hit[14]: begin
        reg_rdata_next[0] = alert_regwen_8_qs;
      end

      addr_hit[15]: begin
        reg_rdata_next[0] = alert_regwen_9_qs;
      end

      addr_hit[16]: begin
        reg_rdata_next[0] = alert_regwen_10_qs;
      end

      addr_hit[17]: begin
        reg_rdata_next[0] = alert_regwen_11_qs;
      end

      addr_hit[18]: begin
        reg_rdata_next[0] = alert_regwen_12_qs;
      end

      addr_hit[19]: begin
        reg_rdata_next[0] = alert_regwen_13_qs;
      end

      addr_hit[20]: begin
        reg_rdata_next[0] = alert_regwen_14_qs;
      end

      addr_hit[21]: begin
        reg_rdata_next[0] = alert_regwen_15_qs;
      end

      addr_hit[22]: begin
        reg_rdata_next[0] = alert_regwen_16_qs;
      end

      addr_hit[23]: begin
        reg_rdata_next[0] = alert_regwen_17_qs;
      end

      addr_hit[24]: begin
        reg_rdata_next[0] = alert_regwen_18_qs;
      end

      addr_hit[25]: begin
        reg_rdata_next[0] = alert_regwen_19_qs;
      end

      addr_hit[26]: begin
        reg_rdata_next[0] = alert_regwen_20_qs;
      end

      addr_hit[27]: begin
        reg_rdata_next[0] = alert_regwen_21_qs;
      end

      addr_hit[28]: begin
        reg_rdata_next[0] = alert_regwen_22_qs;
      end

      addr_hit[29]: begin
        reg_rdata_next[0] = alert_regwen_23_qs;
      end

      addr_hit[30]: begin
        reg_rdata_next[0] = alert_regwen_24_qs;
      end

      addr_hit[31]: begin
        reg_rdata_next[0] = alert_regwen_25_qs;
      end

      addr_hit[32]: begin
        reg_rdata_next[0] = alert_regwen_26_qs;
      end

      addr_hit[33]: begin
        reg_rdata_next[0] = alert_regwen_27_qs;
      end

      addr_hit[34]: begin
        reg_rdata_next[0] = alert_regwen_28_qs;
      end

      addr_hit[35]: begin
        reg_rdata_next[0] = alert_regwen_29_qs;
      end

      addr_hit[36]: begin
        reg_rdata_next[0] = alert_regwen_30_qs;
      end

      addr_hit[37]: begin
        reg_rdata_next[0] = alert_regwen_31_qs;
      end

      addr_hit[38]: begin
        reg_rdata_next[0] = alert_regwen_32_qs;
      end

      addr_hit[39]: begin
        reg_rdata_next[0] = alert_regwen_33_qs;
      end

      addr_hit[40]: begin
        reg_rdata_next[0] = alert_regwen_34_qs;
      end

      addr_hit[41]: begin
        reg_rdata_next[0] = alert_regwen_35_qs;
      end

      addr_hit[42]: begin
        reg_rdata_next[0] = alert_regwen_36_qs;
      end

      addr_hit[43]: begin
        reg_rdata_next[0] = alert_regwen_37_qs;
      end

      addr_hit[44]: begin
        reg_rdata_next[0] = alert_regwen_38_qs;
      end

      addr_hit[45]: begin
        reg_rdata_next[0] = alert_regwen_39_qs;
      end

      addr_hit[46]: begin
        reg_rdata_next[0] = alert_regwen_40_qs;
      end

      addr_hit[47]: begin
        reg_rdata_next[0] = alert_regwen_41_qs;
      end

      addr_hit[48]: begin
        reg_rdata_next[0] = alert_regwen_42_qs;
      end

      addr_hit[49]: begin
        reg_rdata_next[0] = alert_regwen_43_qs;
      end

      addr_hit[50]: begin
        reg_rdata_next[0] = alert_regwen_44_qs;
      end

      addr_hit[51]: begin
        reg_rdata_next[0] = alert_regwen_45_qs;
      end

      addr_hit[52]: begin
        reg_rdata_next[0] = alert_regwen_46_qs;
      end

      addr_hit[53]: begin
        reg_rdata_next[0] = alert_regwen_47_qs;
      end

      addr_hit[54]: begin
        reg_rdata_next[0] = alert_regwen_48_qs;
      end

      addr_hit[55]: begin
        reg_rdata_next[0] = alert_regwen_49_qs;
      end

      addr_hit[56]: begin
        reg_rdata_next[0] = alert_regwen_50_qs;
      end

      addr_hit[57]: begin
        reg_rdata_next[0] = alert_regwen_51_qs;
      end

      addr_hit[58]: begin
        reg_rdata_next[0] = alert_regwen_52_qs;
      end

      addr_hit[59]: begin
        reg_rdata_next[0] = alert_regwen_53_qs;
      end

      addr_hit[60]: begin
        reg_rdata_next[0] = alert_regwen_54_qs;
      end

      addr_hit[61]: begin
        reg_rdata_next[0] = alert_regwen_55_qs;
      end

      addr_hit[62]: begin
        reg_rdata_next[0] = alert_regwen_56_qs;
      end

      addr_hit[63]: begin
        reg_rdata_next[0] = alert_regwen_57_qs;
      end

      addr_hit[64]: begin
        reg_rdata_next[0] = alert_regwen_58_qs;
      end

      addr_hit[65]: begin
        reg_rdata_next[0] = alert_regwen_59_qs;
      end

      addr_hit[66]: begin
        reg_rdata_next[0] = alert_regwen_60_qs;
      end

      addr_hit[67]: begin
        reg_rdata_next[0] = alert_regwen_61_qs;
      end

      addr_hit[68]: begin
        reg_rdata_next[0] = alert_regwen_62_qs;
      end

      addr_hit[69]: begin
        reg_rdata_next[0] = alert_regwen_63_qs;
      end

      addr_hit[70]: begin
        reg_rdata_next[0] = alert_regwen_64_qs;
      end

      addr_hit[71]: begin
        reg_rdata_next[0] = alert_en_shadowed_0_qs;
      end

      addr_hit[72]: begin
        reg_rdata_next[0] = alert_en_shadowed_1_qs;
      end

      addr_hit[73]: begin
        reg_rdata_next[0] = alert_en_shadowed_2_qs;
      end

      addr_hit[74]: begin
        reg_rdata_next[0] = alert_en_shadowed_3_qs;
      end

      addr_hit[75]: begin
        reg_rdata_next[0] = alert_en_shadowed_4_qs;
      end

      addr_hit[76]: begin
        reg_rdata_next[0] = alert_en_shadowed_5_qs;
      end

      addr_hit[77]: begin
        reg_rdata_next[0] = alert_en_shadowed_6_qs;
      end

      addr_hit[78]: begin
        reg_rdata_next[0] = alert_en_shadowed_7_qs;
      end

      addr_hit[79]: begin
        reg_rdata_next[0] = alert_en_shadowed_8_qs;
      end

      addr_hit[80]: begin
        reg_rdata_next[0] = alert_en_shadowed_9_qs;
      end

      addr_hit[81]: begin
        reg_rdata_next[0] = alert_en_shadowed_10_qs;
      end

      addr_hit[82]: begin
        reg_rdata_next[0] = alert_en_shadowed_11_qs;
      end

      addr_hit[83]: begin
        reg_rdata_next[0] = alert_en_shadowed_12_qs;
      end

      addr_hit[84]: begin
        reg_rdata_next[0] = alert_en_shadowed_13_qs;
      end

      addr_hit[85]: begin
        reg_rdata_next[0] = alert_en_shadowed_14_qs;
      end

      addr_hit[86]: begin
        reg_rdata_next[0] = alert_en_shadowed_15_qs;
      end

      addr_hit[87]: begin
        reg_rdata_next[0] = alert_en_shadowed_16_qs;
      end

      addr_hit[88]: begin
        reg_rdata_next[0] = alert_en_shadowed_17_qs;
      end

      addr_hit[89]: begin
        reg_rdata_next[0] = alert_en_shadowed_18_qs;
      end

      addr_hit[90]: begin
        reg_rdata_next[0] = alert_en_shadowed_19_qs;
      end

      addr_hit[91]: begin
        reg_rdata_next[0] = alert_en_shadowed_20_qs;
      end

      addr_hit[92]: begin
        reg_rdata_next[0] = alert_en_shadowed_21_qs;
      end

      addr_hit[93]: begin
        reg_rdata_next[0] = alert_en_shadowed_22_qs;
      end

      addr_hit[94]: begin
        reg_rdata_next[0] = alert_en_shadowed_23_qs;
      end

      addr_hit[95]: begin
        reg_rdata_next[0] = alert_en_shadowed_24_qs;
      end

      addr_hit[96]: begin
        reg_rdata_next[0] = alert_en_shadowed_25_qs;
      end

      addr_hit[97]: begin
        reg_rdata_next[0] = alert_en_shadowed_26_qs;
      end

      addr_hit[98]: begin
        reg_rdata_next[0] = alert_en_shadowed_27_qs;
      end

      addr_hit[99]: begin
        reg_rdata_next[0] = alert_en_shadowed_28_qs;
      end

      addr_hit[100]: begin
        reg_rdata_next[0] = alert_en_shadowed_29_qs;
      end

      addr_hit[101]: begin
        reg_rdata_next[0] = alert_en_shadowed_30_qs;
      end

      addr_hit[102]: begin
        reg_rdata_next[0] = alert_en_shadowed_31_qs;
      end

      addr_hit[103]: begin
        reg_rdata_next[0] = alert_en_shadowed_32_qs;
      end

      addr_hit[104]: begin
        reg_rdata_next[0] = alert_en_shadowed_33_qs;
      end

      addr_hit[105]: begin
        reg_rdata_next[0] = alert_en_shadowed_34_qs;
      end

      addr_hit[106]: begin
        reg_rdata_next[0] = alert_en_shadowed_35_qs;
      end

      addr_hit[107]: begin
        reg_rdata_next[0] = alert_en_shadowed_36_qs;
      end

      addr_hit[108]: begin
        reg_rdata_next[0] = alert_en_shadowed_37_qs;
      end

      addr_hit[109]: begin
        reg_rdata_next[0] = alert_en_shadowed_38_qs;
      end

      addr_hit[110]: begin
        reg_rdata_next[0] = alert_en_shadowed_39_qs;
      end

      addr_hit[111]: begin
        reg_rdata_next[0] = alert_en_shadowed_40_qs;
      end

      addr_hit[112]: begin
        reg_rdata_next[0] = alert_en_shadowed_41_qs;
      end

      addr_hit[113]: begin
        reg_rdata_next[0] = alert_en_shadowed_42_qs;
      end

      addr_hit[114]: begin
        reg_rdata_next[0] = alert_en_shadowed_43_qs;
      end

      addr_hit[115]: begin
        reg_rdata_next[0] = alert_en_shadowed_44_qs;
      end

      addr_hit[116]: begin
        reg_rdata_next[0] = alert_en_shadowed_45_qs;
      end

      addr_hit[117]: begin
        reg_rdata_next[0] = alert_en_shadowed_46_qs;
      end

      addr_hit[118]: begin
        reg_rdata_next[0] = alert_en_shadowed_47_qs;
      end

      addr_hit[119]: begin
        reg_rdata_next[0] = alert_en_shadowed_48_qs;
      end

      addr_hit[120]: begin
        reg_rdata_next[0] = alert_en_shadowed_49_qs;
      end

      addr_hit[121]: begin
        reg_rdata_next[0] = alert_en_shadowed_50_qs;
      end

      addr_hit[122]: begin
        reg_rdata_next[0] = alert_en_shadowed_51_qs;
      end

      addr_hit[123]: begin
        reg_rdata_next[0] = alert_en_shadowed_52_qs;
      end

      addr_hit[124]: begin
        reg_rdata_next[0] = alert_en_shadowed_53_qs;
      end

      addr_hit[125]: begin
        reg_rdata_next[0] = alert_en_shadowed_54_qs;
      end

      addr_hit[126]: begin
        reg_rdata_next[0] = alert_en_shadowed_55_qs;
      end

      addr_hit[127]: begin
        reg_rdata_next[0] = alert_en_shadowed_56_qs;
      end

      addr_hit[128]: begin
        reg_rdata_next[0] = alert_en_shadowed_57_qs;
      end

      addr_hit[129]: begin
        reg_rdata_next[0] = alert_en_shadowed_58_qs;
      end

      addr_hit[130]: begin
        reg_rdata_next[0] = alert_en_shadowed_59_qs;
      end

      addr_hit[131]: begin
        reg_rdata_next[0] = alert_en_shadowed_60_qs;
      end

      addr_hit[132]: begin
        reg_rdata_next[0] = alert_en_shadowed_61_qs;
      end

      addr_hit[133]: begin
        reg_rdata_next[0] = alert_en_shadowed_62_qs;
      end

      addr_hit[134]: begin
        reg_rdata_next[0] = alert_en_shadowed_63_qs;
      end

      addr_hit[135]: begin
        reg_rdata_next[0] = alert_en_shadowed_64_qs;
      end

      addr_hit[136]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_0_qs;
      end

      addr_hit[137]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_1_qs;
      end

      addr_hit[138]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_2_qs;
      end

      addr_hit[139]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_3_qs;
      end

      addr_hit[140]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_4_qs;
      end

      addr_hit[141]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_5_qs;
      end

      addr_hit[142]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_6_qs;
      end

      addr_hit[143]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_7_qs;
      end

      addr_hit[144]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_8_qs;
      end

      addr_hit[145]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_9_qs;
      end

      addr_hit[146]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_10_qs;
      end

      addr_hit[147]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_11_qs;
      end

      addr_hit[148]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_12_qs;
      end

      addr_hit[149]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_13_qs;
      end

      addr_hit[150]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_14_qs;
      end

      addr_hit[151]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_15_qs;
      end

      addr_hit[152]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_16_qs;
      end

      addr_hit[153]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_17_qs;
      end

      addr_hit[154]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_18_qs;
      end

      addr_hit[155]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_19_qs;
      end

      addr_hit[156]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_20_qs;
      end

      addr_hit[157]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_21_qs;
      end

      addr_hit[158]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_22_qs;
      end

      addr_hit[159]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_23_qs;
      end

      addr_hit[160]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_24_qs;
      end

      addr_hit[161]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_25_qs;
      end

      addr_hit[162]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_26_qs;
      end

      addr_hit[163]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_27_qs;
      end

      addr_hit[164]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_28_qs;
      end

      addr_hit[165]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_29_qs;
      end

      addr_hit[166]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_30_qs;
      end

      addr_hit[167]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_31_qs;
      end

      addr_hit[168]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_32_qs;
      end

      addr_hit[169]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_33_qs;
      end

      addr_hit[170]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_34_qs;
      end

      addr_hit[171]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_35_qs;
      end

      addr_hit[172]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_36_qs;
      end

      addr_hit[173]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_37_qs;
      end

      addr_hit[174]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_38_qs;
      end

      addr_hit[175]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_39_qs;
      end

      addr_hit[176]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_40_qs;
      end

      addr_hit[177]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_41_qs;
      end

      addr_hit[178]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_42_qs;
      end

      addr_hit[179]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_43_qs;
      end

      addr_hit[180]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_44_qs;
      end

      addr_hit[181]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_45_qs;
      end

      addr_hit[182]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_46_qs;
      end

      addr_hit[183]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_47_qs;
      end

      addr_hit[184]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_48_qs;
      end

      addr_hit[185]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_49_qs;
      end

      addr_hit[186]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_50_qs;
      end

      addr_hit[187]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_51_qs;
      end

      addr_hit[188]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_52_qs;
      end

      addr_hit[189]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_53_qs;
      end

      addr_hit[190]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_54_qs;
      end

      addr_hit[191]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_55_qs;
      end

      addr_hit[192]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_56_qs;
      end

      addr_hit[193]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_57_qs;
      end

      addr_hit[194]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_58_qs;
      end

      addr_hit[195]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_59_qs;
      end

      addr_hit[196]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_60_qs;
      end

      addr_hit[197]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_61_qs;
      end

      addr_hit[198]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_62_qs;
      end

      addr_hit[199]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_63_qs;
      end

      addr_hit[200]: begin
        reg_rdata_next[1:0] = alert_class_shadowed_64_qs;
      end

      addr_hit[201]: begin
        reg_rdata_next[0] = alert_cause_0_qs;
      end

      addr_hit[202]: begin
        reg_rdata_next[0] = alert_cause_1_qs;
      end

      addr_hit[203]: begin
        reg_rdata_next[0] = alert_cause_2_qs;
      end

      addr_hit[204]: begin
        reg_rdata_next[0] = alert_cause_3_qs;
      end

      addr_hit[205]: begin
        reg_rdata_next[0] = alert_cause_4_qs;
      end

      addr_hit[206]: begin
        reg_rdata_next[0] = alert_cause_5_qs;
      end

      addr_hit[207]: begin
        reg_rdata_next[0] = alert_cause_6_qs;
      end

      addr_hit[208]: begin
        reg_rdata_next[0] = alert_cause_7_qs;
      end

      addr_hit[209]: begin
        reg_rdata_next[0] = alert_cause_8_qs;
      end

      addr_hit[210]: begin
        reg_rdata_next[0] = alert_cause_9_qs;
      end

      addr_hit[211]: begin
        reg_rdata_next[0] = alert_cause_10_qs;
      end

      addr_hit[212]: begin
        reg_rdata_next[0] = alert_cause_11_qs;
      end

      addr_hit[213]: begin
        reg_rdata_next[0] = alert_cause_12_qs;
      end

      addr_hit[214]: begin
        reg_rdata_next[0] = alert_cause_13_qs;
      end

      addr_hit[215]: begin
        reg_rdata_next[0] = alert_cause_14_qs;
      end

      addr_hit[216]: begin
        reg_rdata_next[0] = alert_cause_15_qs;
      end

      addr_hit[217]: begin
        reg_rdata_next[0] = alert_cause_16_qs;
      end

      addr_hit[218]: begin
        reg_rdata_next[0] = alert_cause_17_qs;
      end

      addr_hit[219]: begin
        reg_rdata_next[0] = alert_cause_18_qs;
      end

      addr_hit[220]: begin
        reg_rdata_next[0] = alert_cause_19_qs;
      end

      addr_hit[221]: begin
        reg_rdata_next[0] = alert_cause_20_qs;
      end

      addr_hit[222]: begin
        reg_rdata_next[0] = alert_cause_21_qs;
      end

      addr_hit[223]: begin
        reg_rdata_next[0] = alert_cause_22_qs;
      end

      addr_hit[224]: begin
        reg_rdata_next[0] = alert_cause_23_qs;
      end

      addr_hit[225]: begin
        reg_rdata_next[0] = alert_cause_24_qs;
      end

      addr_hit[226]: begin
        reg_rdata_next[0] = alert_cause_25_qs;
      end

      addr_hit[227]: begin
        reg_rdata_next[0] = alert_cause_26_qs;
      end

      addr_hit[228]: begin
        reg_rdata_next[0] = alert_cause_27_qs;
      end

      addr_hit[229]: begin
        reg_rdata_next[0] = alert_cause_28_qs;
      end

      addr_hit[230]: begin
        reg_rdata_next[0] = alert_cause_29_qs;
      end

      addr_hit[231]: begin
        reg_rdata_next[0] = alert_cause_30_qs;
      end

      addr_hit[232]: begin
        reg_rdata_next[0] = alert_cause_31_qs;
      end

      addr_hit[233]: begin
        reg_rdata_next[0] = alert_cause_32_qs;
      end

      addr_hit[234]: begin
        reg_rdata_next[0] = alert_cause_33_qs;
      end

      addr_hit[235]: begin
        reg_rdata_next[0] = alert_cause_34_qs;
      end

      addr_hit[236]: begin
        reg_rdata_next[0] = alert_cause_35_qs;
      end

      addr_hit[237]: begin
        reg_rdata_next[0] = alert_cause_36_qs;
      end

      addr_hit[238]: begin
        reg_rdata_next[0] = alert_cause_37_qs;
      end

      addr_hit[239]: begin
        reg_rdata_next[0] = alert_cause_38_qs;
      end

      addr_hit[240]: begin
        reg_rdata_next[0] = alert_cause_39_qs;
      end

      addr_hit[241]: begin
        reg_rdata_next[0] = alert_cause_40_qs;
      end

      addr_hit[242]: begin
        reg_rdata_next[0] = alert_cause_41_qs;
      end

      addr_hit[243]: begin
        reg_rdata_next[0] = alert_cause_42_qs;
      end

      addr_hit[244]: begin
        reg_rdata_next[0] = alert_cause_43_qs;
      end

      addr_hit[245]: begin
        reg_rdata_next[0] = alert_cause_44_qs;
      end

      addr_hit[246]: begin
        reg_rdata_next[0] = alert_cause_45_qs;
      end

      addr_hit[247]: begin
        reg_rdata_next[0] = alert_cause_46_qs;
      end

      addr_hit[248]: begin
        reg_rdata_next[0] = alert_cause_47_qs;
      end

      addr_hit[249]: begin
        reg_rdata_next[0] = alert_cause_48_qs;
      end

      addr_hit[250]: begin
        reg_rdata_next[0] = alert_cause_49_qs;
      end

      addr_hit[251]: begin
        reg_rdata_next[0] = alert_cause_50_qs;
      end

      addr_hit[252]: begin
        reg_rdata_next[0] = alert_cause_51_qs;
      end

      addr_hit[253]: begin
        reg_rdata_next[0] = alert_cause_52_qs;
      end

      addr_hit[254]: begin
        reg_rdata_next[0] = alert_cause_53_qs;
      end

      addr_hit[255]: begin
        reg_rdata_next[0] = alert_cause_54_qs;
      end

      addr_hit[256]: begin
        reg_rdata_next[0] = alert_cause_55_qs;
      end

      addr_hit[257]: begin
        reg_rdata_next[0] = alert_cause_56_qs;
      end

      addr_hit[258]: begin
        reg_rdata_next[0] = alert_cause_57_qs;
      end

      addr_hit[259]: begin
        reg_rdata_next[0] = alert_cause_58_qs;
      end

      addr_hit[260]: begin
        reg_rdata_next[0] = alert_cause_59_qs;
      end

      addr_hit[261]: begin
        reg_rdata_next[0] = alert_cause_60_qs;
      end

      addr_hit[262]: begin
        reg_rdata_next[0] = alert_cause_61_qs;
      end

      addr_hit[263]: begin
        reg_rdata_next[0] = alert_cause_62_qs;
      end

      addr_hit[264]: begin
        reg_rdata_next[0] = alert_cause_63_qs;
      end

      addr_hit[265]: begin
        reg_rdata_next[0] = alert_cause_64_qs;
      end

      addr_hit[266]: begin
        reg_rdata_next[0] = loc_alert_regwen_0_qs;
      end

      addr_hit[267]: begin
        reg_rdata_next[0] = loc_alert_regwen_1_qs;
      end

      addr_hit[268]: begin
        reg_rdata_next[0] = loc_alert_regwen_2_qs;
      end

      addr_hit[269]: begin
        reg_rdata_next[0] = loc_alert_regwen_3_qs;
      end

      addr_hit[270]: begin
        reg_rdata_next[0] = loc_alert_regwen_4_qs;
      end

      addr_hit[271]: begin
        reg_rdata_next[0] = loc_alert_regwen_5_qs;
      end

      addr_hit[272]: begin
        reg_rdata_next[0] = loc_alert_regwen_6_qs;
      end

      addr_hit[273]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_0_qs;
      end

      addr_hit[274]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_1_qs;
      end

      addr_hit[275]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_2_qs;
      end

      addr_hit[276]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_3_qs;
      end

      addr_hit[277]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_4_qs;
      end

      addr_hit[278]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_5_qs;
      end

      addr_hit[279]: begin
        reg_rdata_next[0] = loc_alert_en_shadowed_6_qs;
      end

      addr_hit[280]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_0_qs;
      end

      addr_hit[281]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_1_qs;
      end

      addr_hit[282]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_2_qs;
      end

      addr_hit[283]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_3_qs;
      end

      addr_hit[284]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_4_qs;
      end

      addr_hit[285]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_5_qs;
      end

      addr_hit[286]: begin
        reg_rdata_next[1:0] = loc_alert_class_shadowed_6_qs;
      end

      addr_hit[287]: begin
        reg_rdata_next[0] = loc_alert_cause_0_qs;
      end

      addr_hit[288]: begin
        reg_rdata_next[0] = loc_alert_cause_1_qs;
      end

      addr_hit[289]: begin
        reg_rdata_next[0] = loc_alert_cause_2_qs;
      end

      addr_hit[290]: begin
        reg_rdata_next[0] = loc_alert_cause_3_qs;
      end

      addr_hit[291]: begin
        reg_rdata_next[0] = loc_alert_cause_4_qs;
      end

      addr_hit[292]: begin
        reg_rdata_next[0] = loc_alert_cause_5_qs;
      end

      addr_hit[293]: begin
        reg_rdata_next[0] = loc_alert_cause_6_qs;
      end

      addr_hit[294]: begin
        reg_rdata_next[0] = classa_regwen_qs;
      end

      addr_hit[295]: begin
        reg_rdata_next[0] = classa_ctrl_shadowed_en_qs;
        reg_rdata_next[1] = classa_ctrl_shadowed_lock_qs;
        reg_rdata_next[2] = classa_ctrl_shadowed_en_e0_qs;
        reg_rdata_next[3] = classa_ctrl_shadowed_en_e1_qs;
        reg_rdata_next[4] = classa_ctrl_shadowed_en_e2_qs;
        reg_rdata_next[5] = classa_ctrl_shadowed_en_e3_qs;
        reg_rdata_next[7:6] = classa_ctrl_shadowed_map_e0_qs;
        reg_rdata_next[9:8] = classa_ctrl_shadowed_map_e1_qs;
        reg_rdata_next[11:10] = classa_ctrl_shadowed_map_e2_qs;
        reg_rdata_next[13:12] = classa_ctrl_shadowed_map_e3_qs;
      end

      addr_hit[296]: begin
        reg_rdata_next[0] = classa_clr_regwen_qs;
      end

      addr_hit[297]: begin
        reg_rdata_next[0] = classa_clr_shadowed_qs;
      end

      addr_hit[298]: begin
        reg_rdata_next[15:0] = classa_accum_cnt_qs;
      end

      addr_hit[299]: begin
        reg_rdata_next[15:0] = classa_accum_thresh_shadowed_qs;
      end

      addr_hit[300]: begin
        reg_rdata_next[31:0] = classa_timeout_cyc_shadowed_qs;
      end

      addr_hit[301]: begin
        reg_rdata_next[1:0] = classa_crashdump_trigger_shadowed_qs;
      end

      addr_hit[302]: begin
        reg_rdata_next[31:0] = classa_phase0_cyc_shadowed_qs;
      end

      addr_hit[303]: begin
        reg_rdata_next[31:0] = classa_phase1_cyc_shadowed_qs;
      end

      addr_hit[304]: begin
        reg_rdata_next[31:0] = classa_phase2_cyc_shadowed_qs;
      end

      addr_hit[305]: begin
        reg_rdata_next[31:0] = classa_phase3_cyc_shadowed_qs;
      end

      addr_hit[306]: begin
        reg_rdata_next[31:0] = classa_esc_cnt_qs;
      end

      addr_hit[307]: begin
        reg_rdata_next[2:0] = classa_state_qs;
      end

      addr_hit[308]: begin
        reg_rdata_next[0] = classb_regwen_qs;
      end

      addr_hit[309]: begin
        reg_rdata_next[0] = classb_ctrl_shadowed_en_qs;
        reg_rdata_next[1] = classb_ctrl_shadowed_lock_qs;
        reg_rdata_next[2] = classb_ctrl_shadowed_en_e0_qs;
        reg_rdata_next[3] = classb_ctrl_shadowed_en_e1_qs;
        reg_rdata_next[4] = classb_ctrl_shadowed_en_e2_qs;
        reg_rdata_next[5] = classb_ctrl_shadowed_en_e3_qs;
        reg_rdata_next[7:6] = classb_ctrl_shadowed_map_e0_qs;
        reg_rdata_next[9:8] = classb_ctrl_shadowed_map_e1_qs;
        reg_rdata_next[11:10] = classb_ctrl_shadowed_map_e2_qs;
        reg_rdata_next[13:12] = classb_ctrl_shadowed_map_e3_qs;
      end

      addr_hit[310]: begin
        reg_rdata_next[0] = classb_clr_regwen_qs;
      end

      addr_hit[311]: begin
        reg_rdata_next[0] = classb_clr_shadowed_qs;
      end

      addr_hit[312]: begin
        reg_rdata_next[15:0] = classb_accum_cnt_qs;
      end

      addr_hit[313]: begin
        reg_rdata_next[15:0] = classb_accum_thresh_shadowed_qs;
      end

      addr_hit[314]: begin
        reg_rdata_next[31:0] = classb_timeout_cyc_shadowed_qs;
      end

      addr_hit[315]: begin
        reg_rdata_next[1:0] = classb_crashdump_trigger_shadowed_qs;
      end

      addr_hit[316]: begin
        reg_rdata_next[31:0] = classb_phase0_cyc_shadowed_qs;
      end

      addr_hit[317]: begin
        reg_rdata_next[31:0] = classb_phase1_cyc_shadowed_qs;
      end

      addr_hit[318]: begin
        reg_rdata_next[31:0] = classb_phase2_cyc_shadowed_qs;
      end

      addr_hit[319]: begin
        reg_rdata_next[31:0] = classb_phase3_cyc_shadowed_qs;
      end

      addr_hit[320]: begin
        reg_rdata_next[31:0] = classb_esc_cnt_qs;
      end

      addr_hit[321]: begin
        reg_rdata_next[2:0] = classb_state_qs;
      end

      addr_hit[322]: begin
        reg_rdata_next[0] = classc_regwen_qs;
      end

      addr_hit[323]: begin
        reg_rdata_next[0] = classc_ctrl_shadowed_en_qs;
        reg_rdata_next[1] = classc_ctrl_shadowed_lock_qs;
        reg_rdata_next[2] = classc_ctrl_shadowed_en_e0_qs;
        reg_rdata_next[3] = classc_ctrl_shadowed_en_e1_qs;
        reg_rdata_next[4] = classc_ctrl_shadowed_en_e2_qs;
        reg_rdata_next[5] = classc_ctrl_shadowed_en_e3_qs;
        reg_rdata_next[7:6] = classc_ctrl_shadowed_map_e0_qs;
        reg_rdata_next[9:8] = classc_ctrl_shadowed_map_e1_qs;
        reg_rdata_next[11:10] = classc_ctrl_shadowed_map_e2_qs;
        reg_rdata_next[13:12] = classc_ctrl_shadowed_map_e3_qs;
      end

      addr_hit[324]: begin
        reg_rdata_next[0] = classc_clr_regwen_qs;
      end

      addr_hit[325]: begin
        reg_rdata_next[0] = classc_clr_shadowed_qs;
      end

      addr_hit[326]: begin
        reg_rdata_next[15:0] = classc_accum_cnt_qs;
      end

      addr_hit[327]: begin
        reg_rdata_next[15:0] = classc_accum_thresh_shadowed_qs;
      end

      addr_hit[328]: begin
        reg_rdata_next[31:0] = classc_timeout_cyc_shadowed_qs;
      end

      addr_hit[329]: begin
        reg_rdata_next[1:0] = classc_crashdump_trigger_shadowed_qs;
      end

      addr_hit[330]: begin
        reg_rdata_next[31:0] = classc_phase0_cyc_shadowed_qs;
      end

      addr_hit[331]: begin
        reg_rdata_next[31:0] = classc_phase1_cyc_shadowed_qs;
      end

      addr_hit[332]: begin
        reg_rdata_next[31:0] = classc_phase2_cyc_shadowed_qs;
      end

      addr_hit[333]: begin
        reg_rdata_next[31:0] = classc_phase3_cyc_shadowed_qs;
      end

      addr_hit[334]: begin
        reg_rdata_next[31:0] = classc_esc_cnt_qs;
      end

      addr_hit[335]: begin
        reg_rdata_next[2:0] = classc_state_qs;
      end

      addr_hit[336]: begin
        reg_rdata_next[0] = classd_regwen_qs;
      end

      addr_hit[337]: begin
        reg_rdata_next[0] = classd_ctrl_shadowed_en_qs;
        reg_rdata_next[1] = classd_ctrl_shadowed_lock_qs;
        reg_rdata_next[2] = classd_ctrl_shadowed_en_e0_qs;
        reg_rdata_next[3] = classd_ctrl_shadowed_en_e1_qs;
        reg_rdata_next[4] = classd_ctrl_shadowed_en_e2_qs;
        reg_rdata_next[5] = classd_ctrl_shadowed_en_e3_qs;
        reg_rdata_next[7:6] = classd_ctrl_shadowed_map_e0_qs;
        reg_rdata_next[9:8] = classd_ctrl_shadowed_map_e1_qs;
        reg_rdata_next[11:10] = classd_ctrl_shadowed_map_e2_qs;
        reg_rdata_next[13:12] = classd_ctrl_shadowed_map_e3_qs;
      end

      addr_hit[338]: begin
        reg_rdata_next[0] = classd_clr_regwen_qs;
      end

      addr_hit[339]: begin
        reg_rdata_next[0] = classd_clr_shadowed_qs;
      end

      addr_hit[340]: begin
        reg_rdata_next[15:0] = classd_accum_cnt_qs;
      end

      addr_hit[341]: begin
        reg_rdata_next[15:0] = classd_accum_thresh_shadowed_qs;
      end

      addr_hit[342]: begin
        reg_rdata_next[31:0] = classd_timeout_cyc_shadowed_qs;
      end

      addr_hit[343]: begin
        reg_rdata_next[1:0] = classd_crashdump_trigger_shadowed_qs;
      end

      addr_hit[344]: begin
        reg_rdata_next[31:0] = classd_phase0_cyc_shadowed_qs;
      end

      addr_hit[345]: begin
        reg_rdata_next[31:0] = classd_phase1_cyc_shadowed_qs;
      end

      addr_hit[346]: begin
        reg_rdata_next[31:0] = classd_phase2_cyc_shadowed_qs;
      end

      addr_hit[347]: begin
        reg_rdata_next[31:0] = classd_phase3_cyc_shadowed_qs;
      end

      addr_hit[348]: begin
        reg_rdata_next[31:0] = classd_esc_cnt_qs;
      end

      addr_hit[349]: begin
        reg_rdata_next[2:0] = classd_state_qs;
      end

      default: begin
        reg_rdata_next = '1;
      end
    endcase
  end

  // shadow busy
  logic shadow_busy;
  logic rst_done;
  logic shadow_rst_done;
  always_ff @(posedge clk_i or negedge rst_ni) begin
    if (!rst_ni) begin
      rst_done <= '0;
    end else begin
      rst_done <= 1'b1;
    end
  end

  always_ff @(posedge clk_i or negedge rst_shadowed_ni) begin
    if (!rst_shadowed_ni) begin
      shadow_rst_done <= '0;
    end else begin
      shadow_rst_done <= 1'b1;
    end
  end

  // both shadow and normal resets have been released
  assign shadow_busy = ~(rst_done & shadow_rst_done);

  // Collect up storage and update errors
  assign shadowed_storage_err_o = |{
    ping_timeout_cyc_shadowed_storage_err,
    ping_timer_en_shadowed_storage_err,
    alert_en_shadowed_0_storage_err,
    alert_en_shadowed_1_storage_err,
    alert_en_shadowed_2_storage_err,
    alert_en_shadowed_3_storage_err,
    alert_en_shadowed_4_storage_err,
    alert_en_shadowed_5_storage_err,
    alert_en_shadowed_6_storage_err,
    alert_en_shadowed_7_storage_err,
    alert_en_shadowed_8_storage_err,
    alert_en_shadowed_9_storage_err,
    alert_en_shadowed_10_storage_err,
    alert_en_shadowed_11_storage_err,
    alert_en_shadowed_12_storage_err,
    alert_en_shadowed_13_storage_err,
    alert_en_shadowed_14_storage_err,
    alert_en_shadowed_15_storage_err,
    alert_en_shadowed_16_storage_err,
    alert_en_shadowed_17_storage_err,
    alert_en_shadowed_18_storage_err,
    alert_en_shadowed_19_storage_err,
    alert_en_shadowed_20_storage_err,
    alert_en_shadowed_21_storage_err,
    alert_en_shadowed_22_storage_err,
    alert_en_shadowed_23_storage_err,
    alert_en_shadowed_24_storage_err,
    alert_en_shadowed_25_storage_err,
    alert_en_shadowed_26_storage_err,
    alert_en_shadowed_27_storage_err,
    alert_en_shadowed_28_storage_err,
    alert_en_shadowed_29_storage_err,
    alert_en_shadowed_30_storage_err,
    alert_en_shadowed_31_storage_err,
    alert_en_shadowed_32_storage_err,
    alert_en_shadowed_33_storage_err,
    alert_en_shadowed_34_storage_err,
    alert_en_shadowed_35_storage_err,
    alert_en_shadowed_36_storage_err,
    alert_en_shadowed_37_storage_err,
    alert_en_shadowed_38_storage_err,
    alert_en_shadowed_39_storage_err,
    alert_en_shadowed_40_storage_err,
    alert_en_shadowed_41_storage_err,
    alert_en_shadowed_42_storage_err,
    alert_en_shadowed_43_storage_err,
    alert_en_shadowed_44_storage_err,
    alert_en_shadowed_45_storage_err,
    alert_en_shadowed_46_storage_err,
    alert_en_shadowed_47_storage_err,
    alert_en_shadowed_48_storage_err,
    alert_en_shadowed_49_storage_err,
    alert_en_shadowed_50_storage_err,
    alert_en_shadowed_51_storage_err,
    alert_en_shadowed_52_storage_err,
    alert_en_shadowed_53_storage_err,
    alert_en_shadowed_54_storage_err,
    alert_en_shadowed_55_storage_err,
    alert_en_shadowed_56_storage_err,
    alert_en_shadowed_57_storage_err,
    alert_en_shadowed_58_storage_err,
    alert_en_shadowed_59_storage_err,
    alert_en_shadowed_60_storage_err,
    alert_en_shadowed_61_storage_err,
    alert_en_shadowed_62_storage_err,
    alert_en_shadowed_63_storage_err,
    alert_en_shadowed_64_storage_err,
    alert_class_shadowed_0_storage_err,
    alert_class_shadowed_1_storage_err,
    alert_class_shadowed_2_storage_err,
    alert_class_shadowed_3_storage_err,
    alert_class_shadowed_4_storage_err,
    alert_class_shadowed_5_storage_err,
    alert_class_shadowed_6_storage_err,
    alert_class_shadowed_7_storage_err,
    alert_class_shadowed_8_storage_err,
    alert_class_shadowed_9_storage_err,
    alert_class_shadowed_10_storage_err,
    alert_class_shadowed_11_storage_err,
    alert_class_shadowed_12_storage_err,
    alert_class_shadowed_13_storage_err,
    alert_class_shadowed_14_storage_err,
    alert_class_shadowed_15_storage_err,
    alert_class_shadowed_16_storage_err,
    alert_class_shadowed_17_storage_err,
    alert_class_shadowed_18_storage_err,
    alert_class_shadowed_19_storage_err,
    alert_class_shadowed_20_storage_err,
    alert_class_shadowed_21_storage_err,
    alert_class_shadowed_22_storage_err,
    alert_class_shadowed_23_storage_err,
    alert_class_shadowed_24_storage_err,
    alert_class_shadowed_25_storage_err,
    alert_class_shadowed_26_storage_err,
    alert_class_shadowed_27_storage_err,
    alert_class_shadowed_28_storage_err,
    alert_class_shadowed_29_storage_err,
    alert_class_shadowed_30_storage_err,
    alert_class_shadowed_31_storage_err,
    alert_class_shadowed_32_storage_err,
    alert_class_shadowed_33_storage_err,
    alert_class_shadowed_34_storage_err,
    alert_class_shadowed_35_storage_err,
    alert_class_shadowed_36_storage_err,
    alert_class_shadowed_37_storage_err,
    alert_class_shadowed_38_storage_err,
    alert_class_shadowed_39_storage_err,
    alert_class_shadowed_40_storage_err,
    alert_class_shadowed_41_storage_err,
    alert_class_shadowed_42_storage_err,
    alert_class_shadowed_43_storage_err,
    alert_class_shadowed_44_storage_err,
    alert_class_shadowed_45_storage_err,
    alert_class_shadowed_46_storage_err,
    alert_class_shadowed_47_storage_err,
    alert_class_shadowed_48_storage_err,
    alert_class_shadowed_49_storage_err,
    alert_class_shadowed_50_storage_err,
    alert_class_shadowed_51_storage_err,
    alert_class_shadowed_52_storage_err,
    alert_class_shadowed_53_storage_err,
    alert_class_shadowed_54_storage_err,
    alert_class_shadowed_55_storage_err,
    alert_class_shadowed_56_storage_err,
    alert_class_shadowed_57_storage_err,
    alert_class_shadowed_58_storage_err,
    alert_class_shadowed_59_storage_err,
    alert_class_shadowed_60_storage_err,
    alert_class_shadowed_61_storage_err,
    alert_class_shadowed_62_storage_err,
    alert_class_shadowed_63_storage_err,
    alert_class_shadowed_64_storage_err,
    loc_alert_en_shadowed_0_storage_err,
    loc_alert_en_shadowed_1_storage_err,
    loc_alert_en_shadowed_2_storage_err,
    loc_alert_en_shadowed_3_storage_err,
    loc_alert_en_shadowed_4_storage_err,
    loc_alert_en_shadowed_5_storage_err,
    loc_alert_en_shadowed_6_storage_err,
    loc_alert_class_shadowed_0_storage_err,
    loc_alert_class_shadowed_1_storage_err,
    loc_alert_class_shadowed_2_storage_err,
    loc_alert_class_shadowed_3_storage_err,
    loc_alert_class_shadowed_4_storage_err,
    loc_alert_class_shadowed_5_storage_err,
    loc_alert_class_shadowed_6_storage_err,
    classa_ctrl_shadowed_en_storage_err,
    classa_ctrl_shadowed_lock_storage_err,
    classa_ctrl_shadowed_en_e0_storage_err,
    classa_ctrl_shadowed_en_e1_storage_err,
    classa_ctrl_shadowed_en_e2_storage_err,
    classa_ctrl_shadowed_en_e3_storage_err,
    classa_ctrl_shadowed_map_e0_storage_err,
    classa_ctrl_shadowed_map_e1_storage_err,
    classa_ctrl_shadowed_map_e2_storage_err,
    classa_ctrl_shadowed_map_e3_storage_err,
    classa_clr_shadowed_storage_err,
    classa_accum_thresh_shadowed_storage_err,
    classa_timeout_cyc_shadowed_storage_err,
    classa_crashdump_trigger_shadowed_storage_err,
    classa_phase0_cyc_shadowed_storage_err,
    classa_phase1_cyc_shadowed_storage_err,
    classa_phase2_cyc_shadowed_storage_err,
    classa_phase3_cyc_shadowed_storage_err,
    classb_ctrl_shadowed_en_storage_err,
    classb_ctrl_shadowed_lock_storage_err,
    classb_ctrl_shadowed_en_e0_storage_err,
    classb_ctrl_shadowed_en_e1_storage_err,
    classb_ctrl_shadowed_en_e2_storage_err,
    classb_ctrl_shadowed_en_e3_storage_err,
    classb_ctrl_shadowed_map_e0_storage_err,
    classb_ctrl_shadowed_map_e1_storage_err,
    classb_ctrl_shadowed_map_e2_storage_err,
    classb_ctrl_shadowed_map_e3_storage_err,
    classb_clr_shadowed_storage_err,
    classb_accum_thresh_shadowed_storage_err,
    classb_timeout_cyc_shadowed_storage_err,
    classb_crashdump_trigger_shadowed_storage_err,
    classb_phase0_cyc_shadowed_storage_err,
    classb_phase1_cyc_shadowed_storage_err,
    classb_phase2_cyc_shadowed_storage_err,
    classb_phase3_cyc_shadowed_storage_err,
    classc_ctrl_shadowed_en_storage_err,
    classc_ctrl_shadowed_lock_storage_err,
    classc_ctrl_shadowed_en_e0_storage_err,
    classc_ctrl_shadowed_en_e1_storage_err,
    classc_ctrl_shadowed_en_e2_storage_err,
    classc_ctrl_shadowed_en_e3_storage_err,
    classc_ctrl_shadowed_map_e0_storage_err,
    classc_ctrl_shadowed_map_e1_storage_err,
    classc_ctrl_shadowed_map_e2_storage_err,
    classc_ctrl_shadowed_map_e3_storage_err,
    classc_clr_shadowed_storage_err,
    classc_accum_thresh_shadowed_storage_err,
    classc_timeout_cyc_shadowed_storage_err,
    classc_crashdump_trigger_shadowed_storage_err,
    classc_phase0_cyc_shadowed_storage_err,
    classc_phase1_cyc_shadowed_storage_err,
    classc_phase2_cyc_shadowed_storage_err,
    classc_phase3_cyc_shadowed_storage_err,
    classd_ctrl_shadowed_en_storage_err,
    classd_ctrl_shadowed_lock_storage_err,
    classd_ctrl_shadowed_en_e0_storage_err,
    classd_ctrl_shadowed_en_e1_storage_err,
    classd_ctrl_shadowed_en_e2_storage_err,
    classd_ctrl_shadowed_en_e3_storage_err,
    classd_ctrl_shadowed_map_e0_storage_err,
    classd_ctrl_shadowed_map_e1_storage_err,
    classd_ctrl_shadowed_map_e2_storage_err,
    classd_ctrl_shadowed_map_e3_storage_err,
    classd_clr_shadowed_storage_err,
    classd_accum_thresh_shadowed_storage_err,
    classd_timeout_cyc_shadowed_storage_err,
    classd_crashdump_trigger_shadowed_storage_err,
    classd_phase0_cyc_shadowed_storage_err,
    classd_phase1_cyc_shadowed_storage_err,
    classd_phase2_cyc_shadowed_storage_err,
    classd_phase3_cyc_shadowed_storage_err
  };
  assign shadowed_update_err_o = |{
    ping_timeout_cyc_shadowed_update_err,
    ping_timer_en_shadowed_update_err,
    alert_en_shadowed_0_update_err,
    alert_en_shadowed_1_update_err,
    alert_en_shadowed_2_update_err,
    alert_en_shadowed_3_update_err,
    alert_en_shadowed_4_update_err,
    alert_en_shadowed_5_update_err,
    alert_en_shadowed_6_update_err,
    alert_en_shadowed_7_update_err,
    alert_en_shadowed_8_update_err,
    alert_en_shadowed_9_update_err,
    alert_en_shadowed_10_update_err,
    alert_en_shadowed_11_update_err,
    alert_en_shadowed_12_update_err,
    alert_en_shadowed_13_update_err,
    alert_en_shadowed_14_update_err,
    alert_en_shadowed_15_update_err,
    alert_en_shadowed_16_update_err,
    alert_en_shadowed_17_update_err,
    alert_en_shadowed_18_update_err,
    alert_en_shadowed_19_update_err,
    alert_en_shadowed_20_update_err,
    alert_en_shadowed_21_update_err,
    alert_en_shadowed_22_update_err,
    alert_en_shadowed_23_update_err,
    alert_en_shadowed_24_update_err,
    alert_en_shadowed_25_update_err,
    alert_en_shadowed_26_update_err,
    alert_en_shadowed_27_update_err,
    alert_en_shadowed_28_update_err,
    alert_en_shadowed_29_update_err,
    alert_en_shadowed_30_update_err,
    alert_en_shadowed_31_update_err,
    alert_en_shadowed_32_update_err,
    alert_en_shadowed_33_update_err,
    alert_en_shadowed_34_update_err,
    alert_en_shadowed_35_update_err,
    alert_en_shadowed_36_update_err,
    alert_en_shadowed_37_update_err,
    alert_en_shadowed_38_update_err,
    alert_en_shadowed_39_update_err,
    alert_en_shadowed_40_update_err,
    alert_en_shadowed_41_update_err,
    alert_en_shadowed_42_update_err,
    alert_en_shadowed_43_update_err,
    alert_en_shadowed_44_update_err,
    alert_en_shadowed_45_update_err,
    alert_en_shadowed_46_update_err,
    alert_en_shadowed_47_update_err,
    alert_en_shadowed_48_update_err,
    alert_en_shadowed_49_update_err,
    alert_en_shadowed_50_update_err,
    alert_en_shadowed_51_update_err,
    alert_en_shadowed_52_update_err,
    alert_en_shadowed_53_update_err,
    alert_en_shadowed_54_update_err,
    alert_en_shadowed_55_update_err,
    alert_en_shadowed_56_update_err,
    alert_en_shadowed_57_update_err,
    alert_en_shadowed_58_update_err,
    alert_en_shadowed_59_update_err,
    alert_en_shadowed_60_update_err,
    alert_en_shadowed_61_update_err,
    alert_en_shadowed_62_update_err,
    alert_en_shadowed_63_update_err,
    alert_en_shadowed_64_update_err,
    alert_class_shadowed_0_update_err,
    alert_class_shadowed_1_update_err,
    alert_class_shadowed_2_update_err,
    alert_class_shadowed_3_update_err,
    alert_class_shadowed_4_update_err,
    alert_class_shadowed_5_update_err,
    alert_class_shadowed_6_update_err,
    alert_class_shadowed_7_update_err,
    alert_class_shadowed_8_update_err,
    alert_class_shadowed_9_update_err,
    alert_class_shadowed_10_update_err,
    alert_class_shadowed_11_update_err,
    alert_class_shadowed_12_update_err,
    alert_class_shadowed_13_update_err,
    alert_class_shadowed_14_update_err,
    alert_class_shadowed_15_update_err,
    alert_class_shadowed_16_update_err,
    alert_class_shadowed_17_update_err,
    alert_class_shadowed_18_update_err,
    alert_class_shadowed_19_update_err,
    alert_class_shadowed_20_update_err,
    alert_class_shadowed_21_update_err,
    alert_class_shadowed_22_update_err,
    alert_class_shadowed_23_update_err,
    alert_class_shadowed_24_update_err,
    alert_class_shadowed_25_update_err,
    alert_class_shadowed_26_update_err,
    alert_class_shadowed_27_update_err,
    alert_class_shadowed_28_update_err,
    alert_class_shadowed_29_update_err,
    alert_class_shadowed_30_update_err,
    alert_class_shadowed_31_update_err,
    alert_class_shadowed_32_update_err,
    alert_class_shadowed_33_update_err,
    alert_class_shadowed_34_update_err,
    alert_class_shadowed_35_update_err,
    alert_class_shadowed_36_update_err,
    alert_class_shadowed_37_update_err,
    alert_class_shadowed_38_update_err,
    alert_class_shadowed_39_update_err,
    alert_class_shadowed_40_update_err,
    alert_class_shadowed_41_update_err,
    alert_class_shadowed_42_update_err,
    alert_class_shadowed_43_update_err,
    alert_class_shadowed_44_update_err,
    alert_class_shadowed_45_update_err,
    alert_class_shadowed_46_update_err,
    alert_class_shadowed_47_update_err,
    alert_class_shadowed_48_update_err,
    alert_class_shadowed_49_update_err,
    alert_class_shadowed_50_update_err,
    alert_class_shadowed_51_update_err,
    alert_class_shadowed_52_update_err,
    alert_class_shadowed_53_update_err,
    alert_class_shadowed_54_update_err,
    alert_class_shadowed_55_update_err,
    alert_class_shadowed_56_update_err,
    alert_class_shadowed_57_update_err,
    alert_class_shadowed_58_update_err,
    alert_class_shadowed_59_update_err,
    alert_class_shadowed_60_update_err,
    alert_class_shadowed_61_update_err,
    alert_class_shadowed_62_update_err,
    alert_class_shadowed_63_update_err,
    alert_class_shadowed_64_update_err,
    loc_alert_en_shadowed_0_update_err,
    loc_alert_en_shadowed_1_update_err,
    loc_alert_en_shadowed_2_update_err,
    loc_alert_en_shadowed_3_update_err,
    loc_alert_en_shadowed_4_update_err,
    loc_alert_en_shadowed_5_update_err,
    loc_alert_en_shadowed_6_update_err,
    loc_alert_class_shadowed_0_update_err,
    loc_alert_class_shadowed_1_update_err,
    loc_alert_class_shadowed_2_update_err,
    loc_alert_class_shadowed_3_update_err,
    loc_alert_class_shadowed_4_update_err,
    loc_alert_class_shadowed_5_update_err,
    loc_alert_class_shadowed_6_update_err,
    classa_ctrl_shadowed_en_update_err,
    classa_ctrl_shadowed_lock_update_err,
    classa_ctrl_shadowed_en_e0_update_err,
    classa_ctrl_shadowed_en_e1_update_err,
    classa_ctrl_shadowed_en_e2_update_err,
    classa_ctrl_shadowed_en_e3_update_err,
    classa_ctrl_shadowed_map_e0_update_err,
    classa_ctrl_shadowed_map_e1_update_err,
    classa_ctrl_shadowed_map_e2_update_err,
    classa_ctrl_shadowed_map_e3_update_err,
    classa_clr_shadowed_update_err,
    classa_accum_thresh_shadowed_update_err,
    classa_timeout_cyc_shadowed_update_err,
    classa_crashdump_trigger_shadowed_update_err,
    classa_phase0_cyc_shadowed_update_err,
    classa_phase1_cyc_shadowed_update_err,
    classa_phase2_cyc_shadowed_update_err,
    classa_phase3_cyc_shadowed_update_err,
    classb_ctrl_shadowed_en_update_err,
    classb_ctrl_shadowed_lock_update_err,
    classb_ctrl_shadowed_en_e0_update_err,
    classb_ctrl_shadowed_en_e1_update_err,
    classb_ctrl_shadowed_en_e2_update_err,
    classb_ctrl_shadowed_en_e3_update_err,
    classb_ctrl_shadowed_map_e0_update_err,
    classb_ctrl_shadowed_map_e1_update_err,
    classb_ctrl_shadowed_map_e2_update_err,
    classb_ctrl_shadowed_map_e3_update_err,
    classb_clr_shadowed_update_err,
    classb_accum_thresh_shadowed_update_err,
    classb_timeout_cyc_shadowed_update_err,
    classb_crashdump_trigger_shadowed_update_err,
    classb_phase0_cyc_shadowed_update_err,
    classb_phase1_cyc_shadowed_update_err,
    classb_phase2_cyc_shadowed_update_err,
    classb_phase3_cyc_shadowed_update_err,
    classc_ctrl_shadowed_en_update_err,
    classc_ctrl_shadowed_lock_update_err,
    classc_ctrl_shadowed_en_e0_update_err,
    classc_ctrl_shadowed_en_e1_update_err,
    classc_ctrl_shadowed_en_e2_update_err,
    classc_ctrl_shadowed_en_e3_update_err,
    classc_ctrl_shadowed_map_e0_update_err,
    classc_ctrl_shadowed_map_e1_update_err,
    classc_ctrl_shadowed_map_e2_update_err,
    classc_ctrl_shadowed_map_e3_update_err,
    classc_clr_shadowed_update_err,
    classc_accum_thresh_shadowed_update_err,
    classc_timeout_cyc_shadowed_update_err,
    classc_crashdump_trigger_shadowed_update_err,
    classc_phase0_cyc_shadowed_update_err,
    classc_phase1_cyc_shadowed_update_err,
    classc_phase2_cyc_shadowed_update_err,
    classc_phase3_cyc_shadowed_update_err,
    classd_ctrl_shadowed_en_update_err,
    classd_ctrl_shadowed_lock_update_err,
    classd_ctrl_shadowed_en_e0_update_err,
    classd_ctrl_shadowed_en_e1_update_err,
    classd_ctrl_shadowed_en_e2_update_err,
    classd_ctrl_shadowed_en_e3_update_err,
    classd_ctrl_shadowed_map_e0_update_err,
    classd_ctrl_shadowed_map_e1_update_err,
    classd_ctrl_shadowed_map_e2_update_err,
    classd_ctrl_shadowed_map_e3_update_err,
    classd_clr_shadowed_update_err,
    classd_accum_thresh_shadowed_update_err,
    classd_timeout_cyc_shadowed_update_err,
    classd_crashdump_trigger_shadowed_update_err,
    classd_phase0_cyc_shadowed_update_err,
    classd_phase1_cyc_shadowed_update_err,
    classd_phase2_cyc_shadowed_update_err,
    classd_phase3_cyc_shadowed_update_err
  };

  // register busy
  assign reg_busy = shadow_busy;

  // Unused signal tieoff

  // wdata / byte enable are not always fully used
  // add a blanket unused statement to handle lint waivers
  logic unused_wdata;
  logic unused_be;
  assign unused_wdata = ^reg_wdata;
  assign unused_be = ^reg_be;

  // Assertions for Register Interface
  `ASSERT_PULSE(wePulse, reg_we, clk_i, !rst_ni)
  `ASSERT_PULSE(rePulse, reg_re, clk_i, !rst_ni)

  `ASSERT(reAfterRv, $rose(reg_re || reg_we) |=> tl_o_pre.d_valid, clk_i, !rst_ni)

  `ASSERT(en2addrHit, (reg_we || reg_re) |-> $onehot0(addr_hit), clk_i, !rst_ni)

  // this is formulated as an assumption such that the FPV testbenches do disprove this
  // property by mistake
  //`ASSUME(reqParity, tl_reg_h2d.a_valid |-> tl_reg_h2d.a_user.chk_en == tlul_pkg::CheckDis)

endmodule
